@@ -8,11 +8,15 @@
 
 (* Certification of Imperative Programs / Jean-Christophe Fillitre *)
 
-(*i $Id: pcic.mli,v 1.2 2001/04/04 14:13:23 filliatr Exp $ i*)
+(*i $Id: pcic.mli,v 1.3 2001/06/27 15:12:30 filliatr Exp $ i*)
 
 open Past
 open Rawterm
 
+(* On-the-fly generation of needed (possibly dependent) tuples. *)
+
+val check_product_n : int -> unit
+val check_dep_product_n : int -> unit
 
 (* transforms intermediate functional programs into (raw) CIC terms *)
 
