@@ -8,7 +8,7 @@
 
 (* Certification of Imperative Programs / Jean-Christophe Fillitre *)
 
-(* $Id: pdb.mli,v 1.1 2001/03/29 14:27:09 filliatr Exp $ *)
+(* $Id: pdb.mli,v 1.2 2001/06/27 15:12:30 filliatr Exp $ *)
 
 open Ptype
 open Past
@@ -19,7 +19,7 @@ open Past
  * These functions directly raise UserError exceptions on bad programs.
  *)
 
-val db_type_v : Names.identifier list -> 'a ml_type_v -> 'a ml_type_v
+val check_type_v : Names.identifier list -> 'a ml_type_v -> unit
 
 val db_prog : program -> program
 
