@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: extraction.mli,v 1.13 2001/04/12 16:32:43 letouzey Exp $ i*)
+(*i $Id: extraction.mli,v 1.16 2001/11/14 13:36:50 letouzey Exp $ i*)
 
 (*s Extraction from Coq terms to Miniml. *)
 
@@ -14,6 +14,7 @@ open Names
 open Term
 open Miniml
 open Environ
+open Nametab
 
 (*s Result of an extraction. *)
 
@@ -25,18 +26,14 @@ type type_var = info * arity
 
 type signature = type_var list
 
-type extraction_context = bool list
-
 type extraction_result =
   | Emltype of ml_type * signature * identifier list
   | Emlterm of ml_ast
 
-(*s Extraction functions. *)
+(*s Extraction function. *)
 
-val extract_constr : env -> extraction_context -> constr -> extraction_result
+val extract_constr : env -> constr -> extraction_result
 
 (*s ML declaration corresponding to a Coq reference. *)
 
 val extract_declaration : global_reference -> ml_decl
-
-
