@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: fourierR.ml,v 1.1 2001/04/20 19:51:04 mayero Exp $ *)
+(* $Id: fourierR.ml,v 1.4 2001/11/05 16:47:05 barras Exp $ *)
 
 
 
@@ -75,18 +75,18 @@ let pf_parse_constr gl s =
 
 let rec string_of_constr c =
  match kind_of_term c with
-   IsCast (c,t) -> string_of_constr c
-  |IsConst (c,l) -> string_of_path c
-  |IsVar(c) -> string_of_id c
+   Cast (c,t) -> string_of_constr c
+  |Const c -> string_of_path c
+  |Var(c) -> string_of_id c
   | _ -> "not_of_constant"
 ;;
 
 let rec rational_of_constr c =
   match kind_of_term c with
-  | IsCast (c,t) -> (rational_of_constr c)
-  | IsApp (c,args) ->
+  | Cast (c,t) -> (rational_of_constr c)
+  | App (c,args) ->
         (match kind_of_term c with
-           IsConst (c,l) ->
+           Const c ->
                (match (string_of_path c) with
 		 "Coq.Reals.Rdefinitions.Ropp" -> 
 		      rop (rational_of_constr args.(0))
@@ -95,6 +95,9 @@ let rec rational_of_constr c =
                 |"Coq.Reals.Rdefinitions.Rmult" -> 
                       rmult (rational_of_constr args.(0))
                             (rational_of_constr args.(1))
+                |"Coq.Reals.Rdefinitions.Rdiv" -> 
+                      rdiv (rational_of_constr args.(0))
+                            (rational_of_constr args.(1))
                 |"Coq.Reals.Rdefinitions.Rplus" -> 
                       rplus (rational_of_constr args.(0))
                             (rational_of_constr args.(1))
@@ -103,7 +106,7 @@ let rec rational_of_constr c =
                              (rational_of_constr args.(1))
                 | _ -> failwith "not a rational")
           | _ -> failwith "not a rational")
-  | IsConst (c,l) ->
+  | Const c ->
         (match (string_of_path c) with
 	       "Coq.Reals.Rdefinitions.R1" -> r1
               |"Coq.Reals.Rdefinitions.R0" -> r0
@@ -114,10 +117,10 @@ let rec rational_of_constr c =
 let rec flin_of_constr c =
   try(
     match kind_of_term c with
-  | IsCast (c,t) -> (flin_of_constr c)
-  | IsApp (c,args) ->
+  | Cast (c,t) -> (flin_of_constr c)
+  | App (c,args) ->
         (match kind_of_term c with
-           IsConst (c,l) ->
+           Const c ->
             (match (string_of_path c) with
 	     "Coq.Reals.Rdefinitions.Ropp" -> 
                   flin_emult (rop r1) (flin_of_constr args.(0))
@@ -137,9 +140,19 @@ let rec flin_of_constr c =
 	      with _-> (flin_add (flin_zero())
 				 args.(0) 
 				 (rational_of_constr args.(1))))
+	    |"Coq.Reals.Rdefinitions.Rinv"->
+	       let a=(rational_of_constr args.(0)) in
+	       flin_add_cste (flin_zero()) (rinv a)
+	    |"Coq.Reals.Rdefinitions.Rdiv"->
+	      (let b=(rational_of_constr args.(1)) in
+	       try (let a = (rational_of_constr args.(0)) in
+		       (flin_add_cste (flin_zero()) (rdiv a b)))
+	       with _-> (flin_add (flin_zero())
+		             args.(0) 
+                             (rinv b)))
             |_->assert false)
            |_ -> assert false)
-  | IsConst (c,l) ->
+  | Const c ->
         (match (string_of_path c) with
 	       "Coq.Reals.Rdefinitions.R1" -> flin_one ()
               |"Coq.Reals.Rdefinitions.R0" -> flin_zero ()
@@ -170,11 +183,11 @@ type hineq={hname:constr; (* le nom de l'hypothse *)
 *)
 let ineq1_of_constr (h,t) =
     match (kind_of_term t) with
-       IsApp (f,args) ->
+       App (f,args) ->
          let t1= args.(0) in
          let t2= args.(1) in
          (match kind_of_term f with
-           IsConst (c,l) ->
+           Const c ->
             (match (string_of_path c) with
 		 "Coq.Reals.Rdefinitions.Rlt" -> [{hname=h;
                            htype="Rlt";
@@ -205,13 +218,13 @@ let ineq1_of_constr (h,t) =
                                              (flin_of_constr t1);
 			   hstrict=false}]
                 |_->assert false)
-          | IsMutInd ((sp,i),l) ->
+          | Ind (sp,i) ->
               (match (string_of_path sp) with 
 		 "Coq.Init.Logic_Type.eqT" ->  let t0= args.(0) in
                            let t1= args.(1) in
                            let t2= args.(2) in
 		    (match (kind_of_term t0) with
-                         IsConst (c,l) ->
+                         Const c ->
 			   (match (string_of_path c) with
 			      "Coq.Reals.Rdefinitions.R"->
                          [{hname=h;
@@ -357,7 +370,7 @@ let tac_use h = match h.htype with
 
 let is_ineq (h,t) =
     match (kind_of_term t) with
-       IsApp (f,args) ->
+       App (f,args) ->
          (match (string_of_constr f) with
 		 "Coq.Reals.Rdefinitions.Rlt" -> true
 		|"Coq.Reals.Rdefinitions.Rgt" -> true
@@ -386,7 +399,7 @@ let rec fourier gl=
        et le but  prouver devient False *)
     try (let tac =
      match (kind_of_term goal) with
-      IsApp (f,args) ->
+      App (f,args) ->
       (match (string_of_constr f) with
 	     "Coq.Reals.Rdefinitions.Rlt" -> 
 	       (tclTHEN
@@ -423,7 +436,7 @@ let rec fourier gl=
     let res=fourier_lineq (!lineq) in
     let tac=ref tclIDTAC in
     if res=[]
-    then (print_string "Tactic Fourier fails, perhaps because it cannot prove some equality on rationnal numbers with the tactic Ring.";
+    then (print_string "Tactic Fourier fails.";
 		       flush stdout)
     (* l'algorithme de Fourier a russi: on va en tirer une preuve Coq *)
     else (match res with
@@ -517,7 +530,9 @@ let rec fourier gl=
 							   (parse "R1"))
 (* en attendant Field, a peut aider Ring de remplacer 1/1 par 1 ... *)	
 
-      			        [(Ring.polynom []);
+      			        [tclORELSE
+                                   (Ring.polynom [])
+                                   tclIDTAC;
 					  (tclTHEN (apply (parse "sym_eqT"))
 						(apply (parse "Rinv_R1")))]
                                
@@ -530,7 +545,8 @@ let rec fourier gl=
       |_-> assert false) |_-> assert false
 	  );
 (*    ((tclTHEN !tac (tclFAIL 1 (* 1 au hasard... *))) gl) *)
-      ((tclABSTRACT None !tac) gl)
+      (!tac gl) 
+(*      ((tclABSTRACT None !tac) gl) *)
 
 ;;
 
