@@ -1,6 +1,7 @@
 
 (*Toplevel loop for the communication between Coq and Centaur *)
 open Names;;
+open Nameops;;
 open Util;;
 open Ast;;
 open Term;;
@@ -33,6 +34,7 @@ open Ascent;;
 open Translate;;
 open Name_to_ast;;
 open Pbp;;
+open Blast;;
 open Dad;;
 open Debug_tac;;
 open Search;;
@@ -53,11 +55,11 @@ set_flags := (function () ->
                   (g_nat_syntax_flag := true; ())
                 else ());;
 
-let guarded_force_eval_stream s = 
+let guarded_force_eval_stream (s : std_ppcmds) = 
   let l = ref [] in
   let f elt = l:= elt :: !l in 
   (try  Stream.iter f s with
-  | _ -> f (sTR "error guarded_force_eval_stream"));
+  | _ -> f (Stream.next (str "error guarded_force_eval_stream")));
   Stream.of_list (List.rev !l);;
 
 
@@ -66,7 +68,7 @@ let rec string_of_path p =
               | i::p -> (string_of_int i)^" "^ (string_of_path p)
 ;;
 let print_path p =
-    output_results_nl [< 'sTR "Path:"; 'sTR  (string_of_path p)>]
+    output_results_nl (str "Path:" ++ str  (string_of_path p))
 ;;
 
 let kill_proof_node index =
@@ -82,7 +84,8 @@ let kill_proof_node index =
 (*Message functions, the text of these messages is recognized by the protocols *)
 (*of CtCoq                                                                     *)
 let ctf_header message_name request_id =
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR message_name; 'fNL; 'iNT request_id; 'fNL >];;
+ fnl () ++ str "message" ++ fnl() ++ str message_name ++ fnl() ++
+ int request_id ++ fnl();;
 
 let ctf_acknowledge_command request_id command_count opt_exn =
   let goal_count, goal_index = 
@@ -93,14 +96,14 @@ let ctf_acknowledge_command request_id command_count opt_exn =
         g_count, (min g_count !current_goal_index)
     else
       (0, 0) in
-  [< ctf_header "acknowledge" request_id;
-     'iNT command_count; 'fNL; 
-    'iNT goal_count; 'fNL; 
-    'iNT goal_index; 'fNL; 
-    'sTR !current_proof_name; 'fNL;
+   (ctf_header "acknowledge" request_id ++
+    int command_count ++ fnl() ++
+    int goal_count ++ fnl () ++
+    int goal_index ++ fnl () ++
+    str !current_proof_name ++ fnl() ++
     (match opt_exn with
       Some e -> Errors.explain_exn e
-    | None -> [< >]); 'fNL; 'sTR "E-n-d---M-e-s-s-a-g-e"; 'fNL >];;
+    | None -> mt ()) ++ fnl() ++ str "E-n-d---M-e-s-s-a-g-e" ++ fnl ());;
 
 let ctf_undoResults = ctf_header "undo_results";;
 
@@ -115,35 +118,36 @@ let ctf_Location = ctf_header "location";;
 let ctf_StateMessage = ctf_header "state";;
 
 let ctf_PathGoalMessage () =
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR "single_goal"; 'fNL >];;
+ fnl () ++ str "message" ++ fnl () ++ str "single_goal" ++ fnl ();;
 
 let ctf_GoalReqIdMessage = ctf_header "single_goal_state";;
 
 let ctf_NewStateMessage = ctf_header "fresh_state";;
 
-let ctf_SavedMessage () = [< 'fNL; 'sTR "message"; 'fNL; 'sTR "saved"; 'fNL >];;
+let ctf_SavedMessage () = fnl () ++ str "message" ++ fnl () ++
+			  str "saved" ++ fnl();;
 
 let ctf_KilledMessage req_id ngoals =
- [< ctf_header "killed" req_id; 'iNT ngoals; 'fNL >];;
+ ctf_header "killed" req_id ++ int ngoals ++ fnl ();;
 
 let ctf_AbortedAllMessage () =
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR "aborted_all"; 'fNL >];;
+  fnl() ++ str "message" ++ fnl() ++ str "aborted_all" ++ fnl();;
 
 let ctf_AbortedMessage request_id na =
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR "aborted_proof"; 'fNL; 'iNT request_id; 'fNL;
- 'sTR na; 'fNL; 'sTR "E-n-d---M-e-s-s-a-g-e"; 'fNL >];;
+  ctf_header "aborted_proof" request_id ++ str na ++ fnl () ++ 
+  str "E-n-d---M-e-s-s-a-g-e" ++ fnl ();;
 
 let ctf_UserErrorMessage request_id stream =
  let stream = guarded_force_eval_stream stream in
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR "user_error"; 'fNL; 'iNT request_id; 'fNL;
-  stream; 'fNL; 'sTR "E-n-d---M-e-s-s-a-g-e"; 'fNL >];;
+ ctf_header "user_error" request_id ++ stream ++ fnl() ++
+ str "E-n-d---M-e-s-s-a-g-e" ++ fnl();;
 
 let ctf_ResetInitialMessage () =
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR "reset_initial"; 'fNL >];;
+ fnl () ++ str "message" ++ fnl () ++ str "reset_initial" ++ fnl ();;
 
-let ctf_ResetIdentMessage request_id str =
- [< 'fNL; 'sTR "message"; 'fNL; 'sTR "reset_ident"; 'fNL; 'iNT request_id; 'fNL;
- 'sTR str; 'fNL; 'sTR "E-n-d---M-e-s-s-a-g-e"; 'fNL >];;
+let ctf_ResetIdentMessage request_id s =
+ ctf_header "reset_ident" request_id ++ str s ++ fnl () ++
+  str  "E-n-d---M-e-s-s-a-g-e" ++ fnl();;
 
 type vtp_tree =
   | P_rl of ct_RULE_LIST
@@ -174,7 +178,7 @@ let break_happened = ref false;;
 let output_results stream vtp_tree =
     let _ = Sys.signal Sys.sigint
        (Sys.Signal_handle(fun i -> (break_happened := true;()))) in
-    mSG stream;
+    msg stream;
     match vtp_tree with
       Some t -> print_tree t
     | None -> ();;
@@ -183,7 +187,7 @@ let output_results_nl stream =
     let _ = Sys.signal Sys.sigint
        (Sys.Signal_handle(fun i -> break_happened := true;()))
     in
-    mSGNL stream;;
+    msgnl stream;;
 
 
 let rearm_break () =
@@ -215,7 +219,6 @@ let show_nth n =
   try
     let pf = proof_of_pftreestate (get_pftreestate()) in
     if (!text_proof_flag<>"off") then
-(*      errorlabstrm "debug" [< 'sTR "text printing unplugged" >]*)
        (if n=0
 	   then output_results (ctf_TextMessage !global_request_id)
                                (Some (P_text (show_proof !text_proof_flag [])))
@@ -243,7 +246,10 @@ let filter_by_module_from_varg_list (l:vernac_arg list) =
 
 let add_search (global_reference:global_reference) assumptions cstr =
   try 
-  let id_string = string_of_qualid (Global.qualid_of_global global_reference) in
+    let env = Global.env() in
+  let id_string =
+    string_of_qualid (Nametab.shortest_qualid_of_global env 
+			global_reference) in
   let ast = 
     try
       CT_premise (CT_ident id_string, translate_constr assumptions cstr)
@@ -252,14 +258,14 @@ let add_search (global_reference:global_reference) assumptions cstr =
                   CT_coerce_ID_to_FORMULA(
                     CT_ident ("Error printing" ^ id_string))) in
     ctv_SEARCH_LIST:= ast::!ctv_SEARCH_LIST
-  with e -> mSGNL [< 'sTR "add_search raised an exception" >]; raise e;;
+  with e -> msgnl (str "add_search raised an exception"); raise e;;
 
 let make_error_stream node_string =
- [< 'sTR "The syntax of "; 'sTR node_string;
- 'sTR " is inconsistent with the vernac interpreter entry" >];;
+  str "The syntax of " ++ str node_string ++
+  str " is inconsistent with the vernac interpreter entry";;
 
 let ctf_EmptyGoalMessage id =
- [< 'fNL; 'sTR "Empty Goal is a no-op.  Fun oh fun."; 'fNL >];;
+  fnl () ++ str "Empty Goal is a no-op.  Fun oh fun." ++ fnl ();;
 
 
 let print_check (ast, judg) =
@@ -268,15 +274,14 @@ let print_check (ast, judg) =
      (try translate_constr (Global.env()) value 
       with UserError(f,str) ->
            raise(UserError(f,
-			   [< Ast.print_ast 
-				(ast_of_constr true (Global.env()) value);
-			      'fNL; str >]))) in
+			   Ast.print_ast 
+				(ast_of_constr true (Global.env()) value) ++
+			      fnl () ++ str ))) in
  let type_ct_ast =
      (try translate_constr (Global.env()) typ
       with UserError(f,str) ->
-           raise(UserError(f, [< Ast.print_ast (ast_of_constr true (Global.env())
-					       value);
-				 'fNL; str >]))) in
+           raise(UserError(f, Ast.print_ast (ast_of_constr true (Global.env())
+					       value) ++ fnl() ++ str))) in
  ((ctf_SearchResults !global_request_id),
  (Some  (P_pl
   (CT_premises_list
@@ -289,7 +294,7 @@ let ct_print_eval ast red_fun env evd judg =
 let {uj_val=value; uj_type=typ} = judg in
 let nvalue = red_fun value
 (* // Attention , ici il faut peut tre utiliser des environnemenst locaux *)
-and ntyp = nf_betaiota env evd typ in
+and ntyp = nf_betaiota typ in
 (ctf_SearchResults !global_request_id,
  Some (P_pl
   (CT_premises_list
@@ -301,28 +306,36 @@ and ntyp = nf_betaiota env evd typ in
 
 
 (* The following function is copied from globpr in env/printer.ml *)
-let globcv = function
-  | Node(_,"MUTIND", (Path(_,sl,s))::(Num(_,tyi))::_) ->
-       convert_qualid
-	 (Global.qualid_of_global (IndRef(section_path sl s,tyi)))
-  | Node(_,"MUTCONSTRUCT",(Path(_,sl,s))::(Num(_,tyi))::(Num(_,i))::_) ->
-       convert_qualid
-          (Global.qualid_of_global (ConstructRef ((section_path sl s, tyi), i)))
+let globcv x =
+  let env = Global.env() in 
+    match x with
+      | Node(_,"MUTIND", (Path(_,sp))::(Num(_,tyi))::_) ->
+	  let env = Global.env() in
+	    convert_qualid
+	      (Nametab.shortest_qualid_of_global env (IndRef(sp,tyi)))
+      | Node(_,"MUTCONSTRUCT",(Path(_,sp))::(Num(_,tyi))::(Num(_,i))::_) ->
+	  convert_qualid
+            (Nametab.shortest_qualid_of_global env
+	       (ConstructRef ((sp, tyi), i)))
   | _ -> failwith "globcv : unexpected value";;
 
 let pbp_tac_pcoq =
-    pbp_tac (function x -> 
+    pbp_tac (function (x:Ctast.t) -> 
       output_results
-  	[< 'fNL; 'sTR "message"; 'fNL; 'sTR "pbp_results"; 'fNL;
-	  'iNT !global_request_id;  'fNL>]
+        (ctf_header "pbp_results" !global_request_id)
+       (Some (P_t(xlate_tactic x))));;
+
+let blast_tac_pcoq =
+    blast_tac (function (x:Ctast.t) -> 
+      output_results
+	(ctf_header "pbp_results" !global_request_id)
        (Some (P_t(xlate_tactic x))));;
 
 
 let dad_tac_pcoq =
   dad_tac(function x -> 
     output_results
-    [< 'fNL; 'sTR "message"; 'fNL; 'sTR "pbp_results"; 'fNL;
-       'iNT !global_request_id;  'fNL >]
+    (ctf_header "pbp_results" !global_request_id)
     (Some (P_t(xlate_tactic x))));;
 
 let search_output_results () =
@@ -341,8 +354,9 @@ let debug_tac2_pcoq = function
 	try
 	  let result = report_error ast the_goal the_ast the_path [] g in
 	  (errorlabstrm "DEBUG TACTIC"
-	    [< 'sTR "no error here "; 'fNL; pr_goal (sig_it g);
-	      'fNL; 'sTR "the tactic is"; 'fNL ; Printer.gentacpr ast >];
+	     (str "no error here " ++ fnl () ++ pr_goal (sig_it g) ++
+	      fnl () ++ str "the tactic is" ++ fnl () ++
+	      Printer.gentacpr ast);
 	   result)
 	with
 	  e ->
@@ -389,7 +403,7 @@ let inspect n =
              	sp, Lib.Leaf lobj ->
 		  (match sp, object_tag lobj with
                        _, "VARIABLE" ->
-			 let ((_, _, v), _, _) = get_variable sp in
+			 let ((_, _, v), _) = get_variable (basename sp) in
 			   add_search2 (Nametab.locate (qualid_of_sp sp)) v
 		     | sp, ("CONSTANT"|"PARAMETER") ->
 			 let {const_type=typ} = Global.lookup_constant sp in
@@ -446,7 +460,7 @@ let logical_kill n =
 let command_changes = [
   ("TEXT_MODE",
    (function
-     |  [VARG_AST (Id(_,x))] ->
+     |  [VARG_AST (Str(_,x))] ->
 	 (match x with
 	   "fr" -> (function () -> text_proof_flag := "fr")
 	 | "en" -> (function () -> text_proof_flag := "en")
@@ -476,9 +490,9 @@ let command_changes = [
 	    begin
 	      if kind = "LETTOP" && not(refining ()) then
 		errorlabstrm "StartProof"
-		  [< 'sTR
-                       "Let declarations can only be used in proof editing mode"
-		  >];
+		  (str
+		     "Let declarations can only be used in proof editing mode"
+		  );
 		let str = (string_of_id s) in
 		start_proof_com (Some s) stre c;
 		History.start_proof str;
@@ -558,7 +572,7 @@ let command_changes = [
 	 (function
 	     () ->
 	       errorlabstrm "Begin Silent" 
-		 [< 'sTR "not available in Centaur mode" >])
+		 (str "not available in Centaur mode"))
      | _ -> errorlabstrm "BeginSilent" (make_error_stream "BeginSilent")));
   
   ("EndSilent", 
@@ -567,7 +581,7 @@ let command_changes = [
 	 (function
 	     () ->
 	       errorlabstrm "End Silent"
-		 [< 'sTR "not available in Centaur mode" >])
+		 (str "not available in Centaur mode"))
      | _ -> errorlabstrm "EndSilent" (make_error_stream "EndSilent")));
   
   ("ABORT", 
@@ -591,7 +605,7 @@ let command_changes = [
        (VARG_QUALID qid)::l ->
          (fun () -> 
            ctv_SEARCH_LIST:=[];
-	    let global_ref = Vernacentries.global dummy_loc qid in
+	    let global_ref = Nametab.global dummy_loc qid in
            filtered_search
              (filter_by_module_from_varg_list l)
 	     add_search (Nametab.locate qid);
@@ -626,9 +640,9 @@ let command_changes = [
    (function
       | [VARG_QUALID qid] ->
 	 (function () -> 
-	   let results = xlate_vernac_list (name_to_ast qid) in
+	   let results = xlate_vernac_list (Ctast.ast_to_ct (name_to_ast qid)) in
 	   output_results 
-	     [<'fNL; 'sTR "message"; 'fNL; 'sTR "PRINT_VALUE"; 'fNL >]
+	     (fnl () ++ str "message" ++ fnl () ++ str "PRINT_VALUE" ++ fnl ())
 	     (Some (P_cl results)))
      | _ -> errorlabstrm "PrintId" (make_error_stream "PrintId")));
   
@@ -641,7 +655,7 @@ let command_changes = [
 	   match kind with
 	   | "CHECK" -> print_check
 	   | "PRINTTYPE" ->
-	       errorlabstrm "PrintType" [< 'sTR "Not yet supported in CtCoq" >]
+	       errorlabstrm "PrintType" (str "Not yet supported in CtCoq")
 	   | _ -> errorlabstrm "CHECK" (make_error_stream "CHECK") in
 	 (function () -> 
 	   let a,b = f (c, judgment_of_rawconstr evmap env c) in
@@ -653,7 +667,7 @@ let command_changes = [
      | VARG_TACTIC_ARG(Redexp redexp):: VARG_CONSTR c :: g ->
 	 let evmap, env = Vernacentries.get_current_context_of_args g in
 	 let redfun =
-	   ct_print_eval c (Tacred.reduction_of_redexp redexp env evmap) env in
+	   ct_print_eval (Ctast.ast_to_ct c) (Tacred.reduction_of_redexp redexp env evmap) env in
 	 fun () -> 
 	   let strm, vtp = redfun evmap (judgment_of_rawconstr evmap env c) in
 	   output_results strm vtp
@@ -687,8 +701,7 @@ let command_changes = [
 	 (fun () -> 
 	   let results = dad_rule_names() in
 	   output_results
-	     [< 'fNL; 'sTR "message"; 'fNL; 'sTR "dad_rule_names"; 'fNL;
-	       'iNT !global_request_id; 'fNL >]
+	     (ctf_header "dad_rule_names" !global_request_id)
 	     (Some (P_ids
 		      (CT_id_list
 			 (List.map (fun s -> CT_ident s) results)))))
@@ -728,9 +741,10 @@ let command_creations = [
 let start_pcoq_mode debug = 
   begin
     start_dad();
-    set_xlate_mut_stuff globcv;
+    set_xlate_mut_stuff (fun x ->Ctast.ast_to_ct (globcv (Ctast.ct_to_ast x)));
     declare_in_coq();
     add_tactic "PcoqPbp" pbp_tac_pcoq;
+    add_tactic "PcoqBlast" blast_tac_pcoq;
     add_tactic "Dad" dad_tac_pcoq;
     add_tactic "CtDebugTac" debug_tac2_pcoq;
     add_tactic "CtDebugTac2" debug_tac2_pcoq;
