@@ -6,16 +6,14 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: closure.ml,v 1.34 2001/04/15 01:08:29 herbelin Exp $ *)
+(* $Id: closure.ml,v 1.45 2001/12/13 09:03:03 filliatr Exp $ *)
 
 open Util
 open Pp
 open Term
 open Names
 open Environ
-open Instantiate
 open Univ
-open Evd
 open Esubst
 
 
@@ -34,9 +32,16 @@ let reset () =
   beta := 0; delta := 0; zeta := 0; evar := 0; iota := 0; prune := 0
 
 let stop() =
-  mSGNL [< 'sTR"[Reds: beta=";'iNT !beta; 'sTR" delta="; 'iNT !delta;
-	   'sTR" zeta="; 'iNT !zeta; 'sTR" evar="; 'iNT !evar;
-           'sTR" iota="; 'iNT !iota; 'sTR" prune="; 'iNT !prune; 'sTR"]" >]
+  msgnl (str "[Reds: beta=" ++ int !beta ++ str" delta=" ++ int !delta ++
+	 str" zeta=" ++ int !zeta ++ str" evar=" ++ int !evar ++
+         str" iota=" ++ int !iota ++ str" prune=" ++ int !prune ++ str"]")
+
+let incr_cnt red cnt =
+  if red then begin
+    if !stats then incr cnt;
+    true
+  end else 
+    false
 
 let with_stats c =
   if !stats then begin
@@ -51,6 +56,130 @@ type evaluable_global_reference =
   | EvalVarRef of identifier
   | EvalConstRef of section_path
 
+type transparent_state = Idpred.t * Sppred.t
+
+let all_opaque = (Idpred.empty, Sppred.empty)
+let all_transparent = (Idpred.full, Sppred.full)
+
+module type RedFlagsSig = sig
+  type reds
+  type red_kind
+  val fBETA : red_kind
+  val fDELTA : red_kind
+  val fIOTA : red_kind
+  val fZETA : red_kind
+  val fCONST : section_path -> red_kind
+  val fVAR : identifier -> red_kind
+  val no_red : reds
+  val red_add : reds -> red_kind -> reds
+  val red_sub : reds -> red_kind -> reds
+  val red_add_transparent : reds -> transparent_state -> reds
+  val mkflags : red_kind list -> reds
+  val red_set : reds -> red_kind -> bool
+  val red_get_const : reds -> bool * evaluable_global_reference list
+end
+
+module RedFlags = (struct
+
+  (* [r_const=(true,cl)] means all constants but those in [cl] *)
+  (* [r_const=(false,cl)] means only those in [cl] *)
+  (* [r_delta=true] just mean [r_const=(true,[])] *)
+
+  type reds = {
+    r_beta : bool;
+    r_delta : bool;
+    r_const : transparent_state;
+    r_zeta : bool;
+    r_evar : bool;
+    r_iota : bool }
+
+  type red_kind = BETA | DELTA | IOTA | ZETA
+	      | CONST of constant | VAR of identifier
+  let fBETA = BETA
+  let fDELTA = DELTA
+  let fIOTA = IOTA
+  let fZETA = ZETA
+  let fCONST sp  = CONST sp
+  let fVAR id  = VAR id
+  let no_red = {
+    r_beta = false;
+    r_delta = false;
+    r_const = all_opaque;
+    r_zeta = false;
+    r_evar = false;
+    r_iota = false }
+
+  let red_add red = function
+    | BETA -> { red with r_beta = true }
+    | DELTA -> { red with r_delta = true; r_const = all_transparent }
+    | CONST sp ->
+	let (l1,l2) = red.r_const in
+	{ red with r_const = l1, Sppred.add sp l2 }
+    | IOTA -> { red with r_iota = true }
+    | ZETA -> { red with r_zeta = true }
+    | VAR id ->
+	let (l1,l2) = red.r_const in
+	{ red with r_const = Idpred.add id l1, l2 }
+
+  let red_sub red = function
+    | BETA -> { red with r_beta = false }
+    | DELTA -> { red with r_delta = false }
+    | CONST sp ->
+ 	let (l1,l2) = red.r_const in
+	{ red with r_const = l1, Sppred.remove sp l2 }
+    | IOTA -> { red with r_iota = false }
+    | ZETA -> { red with r_zeta = false }
+    | VAR id ->
+	let (l1,l2) = red.r_const in
+	{ red with r_const = Idpred.remove id l1, l2 }
+
+  let red_add_transparent red tr =
+    { red with r_const = tr } 
+
+  let mkflags = List.fold_left red_add no_red
+
+  let red_set red = function
+    | BETA -> incr_cnt red.r_beta beta
+    | CONST sp -> 
+	let (_,l) = red.r_const in
+	let c = Sppred.mem sp l in
+	incr_cnt c delta
+    | VAR id -> (* En attendant d'avoir des sp pour les Var *)
+	let (l,_) = red.r_const in
+	let c = Idpred.mem id l in
+	incr_cnt c delta
+    | ZETA -> incr_cnt red.r_zeta zeta
+    | IOTA -> incr_cnt red.r_iota iota
+    | DELTA -> (* Used for Rel/Var defined in context *)
+	incr_cnt red.r_delta delta
+
+  let red_get_const red =
+    let p1,p2 = red.r_const in
+    let (b1,l1) = Idpred.elements p1 in
+    let (b2,l2) = Sppred.elements p2 in
+    if b1=b2 then
+      let l1' = List.map (fun x -> EvalVarRef x) l1 in
+      let l2' = List.map (fun x -> EvalConstRef x) l2 in
+      (b1, l1' @ l2')
+    else error "unrepresentable pair of predicate"
+
+end : RedFlagsSig)
+
+open RedFlags
+
+let betadeltaiota = mkflags [fBETA;fDELTA;fZETA;fIOTA]
+let betadeltaiotanolet = mkflags [fBETA;fDELTA;fIOTA]
+let betaiota = mkflags [fBETA;fIOTA]
+let beta = mkflags [fBETA]
+let betaiotazeta = mkflags [fBETA;fIOTA;fZETA]
+let unfold_red sp = 
+  let flag = match sp with
+    | EvalVarRef id -> fVAR id
+    | EvalConstRef sp -> fCONST sp
+  in (* Remove fZETA for finer behaviour ? *)
+  mkflags [fBETA;flag;fIOTA;fZETA]
+
+(************************* Obsolte
 (* [r_const=(true,cl)] means all constants but those in [cl] *)
 (* [r_const=(false,cl)] means only those in [cl] *)
 type reds = {
@@ -113,7 +242,7 @@ let unfold_red sp =
    a LetIn expression is Letin reduction *)
 
 type red_kind =
-    BETA | DELTA | ZETA | EVAR | IOTA
+    BETA | DELTA | ZETA | IOTA
   | CONST of constant_path list | CONSTBUT of constant_path list
   | VAR of identifier | VARBUT of identifier
 
@@ -135,7 +264,6 @@ let rec red_add red = function
 	    { red with r_const = true, list_union cl l1, l2;
 		r_zeta = true; r_evar = true })
   | IOTA -> { red with r_iota = true }
-  | EVAR -> { red with r_evar = true }
   | ZETA -> { red with r_zeta = true }
   | VAR id ->
       (match red.r_const with
@@ -149,14 +277,6 @@ let rec red_add red = function
 	    { red with r_const = true, l1, list_union [cl] l2;
 		r_zeta = true; r_evar = true })
 
-
-let incr_cnt red cnt =
-  if red then begin
-    if !stats then incr cnt;
-    true
-  end else 
-    false
-
 let red_delta_set red =
   let b,_,_ = red.r_const in b
 
@@ -186,56 +306,14 @@ let red_get_const red =
   let l1' = List.map (fun x -> EvalConstRef x) l1 in
   let l2' = List.map (fun x -> EvalVarRef x) l2 in
   b, l1' @ l2'
-
+fin obsolte **************)
 (* specification of the reduction function *)
 
-type red_mode = UNIFORM | SIMPL | WITHBACK
-
-type flags = red_mode * reds
-
-(* (UNIFORM,r)  == r-reduce in any context
- * (SIMPL,r)    == bdi-reduce under cases or fix, r otherwise (like hnf does)
- * (WITHBACK,r) == internal use: means we are under a case or in rec. arg. of
- *                 fix
- *)
-
-(* Examples *)
-let no_flag = (UNIFORM,no_red)
-let beta = (UNIFORM,beta_red)
-let betaiota = (UNIFORM,betaiota_red)
-let betadeltaiota = (UNIFORM,betadeltaiota_red)
-
-let hnf_flags = (SIMPL,betaiotazeta_red)
-let unfold_flags sp = (UNIFORM, unfold_red sp)
-
-let flags_under = function
-  | (SIMPL,r) -> (WITHBACK,r)
-  | fl -> fl
-
-
-(* Reductions allowed in "normal" circumstances: reduce only what is
- * specified by r *)
-
-let red_top (_,r) rk = red_set r rk
-
-(* Sometimes, we may want to perform a bdi reduction, to generate new redexes.
- * Typically: in the Simpl reduction, terms in recursive position of a fixpoint
- * are bdi-reduced, even if r is weaker.
- *
- * It is important to keep in mind that when we talk of "normal" or
- * "head normal" forms, it always refer to the reduction specified by r,
- * whatever the term context. *)
-
-let red_under (md,r) rk =
-  match md with
-    | WITHBACK -> true
-    | _ -> red_set r rk
 
 (* Flags of reduction and cache of constants: 'a is a type that may be
  * mapped to constr. 'a infos implements a cache for constants and
  * abstractions, storing a representation (of type 'a) of the body of
  * this constant or abstraction.
- *  * i_evc is the set of constraints for existential variables
  *  * i_tab is the cache table of the results
  *  * i_repr is the function to get the representation from the current
  *         state of the cache and the body of the constant. The result
@@ -252,20 +330,19 @@ let red_under (md,r) rk =
  * instantiations (cbv or lazy) are.
  *)
 
-type 'a table_key =
-  | ConstBinding of constant
-  | EvarBinding of (existential * 'a subs)
-  | VarBinding of identifier
-  | FarRelBinding of int
+type table_key =
+  | ConstKey of constant
+  | VarKey of identifier
+  | FarRelKey of int
+   (* FarRel: index in the rel_context part of _initial_ environment *)
 
-type ('a, 'b) infos = {
-  i_flags : flags;
-  i_repr : ('a, 'b) infos -> 'a subs -> constr -> 'a;
+type 'a infos = {
+  i_flags : reds;
+  i_repr : 'a infos -> constr -> 'a;
   i_env : env;
-  i_evc : 'b evar_map;
   i_rels : int * (int * constr) list;
   i_vars : (identifier * constr) list;
-  i_tab : ('a table_key, 'a) Hashtbl.t }
+  i_tab : (table_key, 'a) Hashtbl.t }
 
 let info_flags info = info.i_flags
 
@@ -274,20 +351,18 @@ let ref_value_cache info ref =
     Some (Hashtbl.find info.i_tab ref)
   with Not_found ->
   try
-    let body,subs =
+    let body =
       match ref with
-	| FarRelBinding n ->
-	    let (s,l) = info.i_rels in lift n (List.assoc (s-n) l), ESID 0
-	| VarBinding id -> List.assoc id info.i_vars, ESID 0
-	| EvarBinding (evc,subs) -> existential_value info.i_evc evc, subs
-	| ConstBinding cst -> constant_value info.i_env cst, ESID 0
+	| FarRelKey n ->
+	    let (s,l) = info.i_rels in lift n (List.assoc (s-n) l)
+	| VarKey id -> List.assoc id info.i_vars
+	| ConstKey cst -> constant_value info.i_env cst
     in
-    let v = info.i_repr info subs body in
+    let v = info.i_repr info body in
     Hashtbl.add info.i_tab ref v;
     Some v
   with
     | Not_found (* List.assoc *)
-    | NotInstantiatedEvar (* Evar *)
     | NotEvaluableConst _ (* Const *)
       -> None
 
@@ -298,7 +373,7 @@ let defined_vars flags env =
 	 match b with
 	   | None -> e
 	   | Some body -> (id, body)::e)
-      env []
+      env ~init:[]
 (*  else []*)
 
 let defined_rels flags env =
@@ -308,27 +383,23 @@ let defined_rels flags env =
 	 match b with
 	   | None -> (i+1, subs)
 	   | Some body -> (i+1, (i,body) :: subs))
-      env (0,[])
+      env ~init:(0,[])
 (*  else (0,[])*)
 
-let create mk_cl flgs env sigma =
+let create mk_cl flgs env =
   { i_flags = flgs;
     i_repr = mk_cl;
     i_env = env;
-    i_evc = sigma;
     i_rels = defined_rels flgs env;
     i_vars = defined_vars flgs env;
     i_tab = Hashtbl.create 17 }
 
 
-let infos_under infos = { infos with i_flags = flags_under infos.i_flags }
-
-
 (**********************************************************************)
 (* The type of (machine) stacks (= lambda-bar-calculus' contexts)     *) 
 
 type 'a stack_member =
-  | Zapp of 'a array * int
+  | Zapp of 'a list
   | Zcase of case_info * 'a * 'a array
   | Zfix of 'a * 'a stack
   | Zshift of int
@@ -337,9 +408,11 @@ type 'a stack_member =
 and 'a stack = 'a stack_member list
 
 let empty_stack = []
-
-let append_stackn v p s = if Array.length v = p then s else Zapp(v,p)::s
-let append_stack v s = append_stackn v 0 s
+let append_stack_list = function
+  | ([],s) -> s
+  | (l1, Zapp l :: s) -> Zapp (l1@l) :: s
+  | (l1, s) -> Zapp l1 :: s
+let append_stack v s = append_stack_list (Array.to_list v, s)
 
 (* Collapse the shifts in the stack *)
 let zshift n s =
@@ -349,68 +422,62 @@ let zshift n s =
     | _ -> Zshift(n)::s
 
 let rec stack_args_size = function
-  | Zapp(v,n)::s -> Array.length v - n + stack_args_size s
+  | Zapp l::s -> List.length l + stack_args_size s
   | Zshift(_)::s -> stack_args_size s
   | Zupdate(_)::s -> stack_args_size s
   | _ -> 0
 
-(* Parameterization: check the a given reduction is allowed in the
-   context of the stack *)
-let can_red info stk r =
-  red_top info.i_flags r ||
-  (fst info.i_flags = SIMPL &&
-    List.exists (function (Zcase _|Zfix _) -> true | _ -> false) stk)
-
-
 (* When used as an argument stack (only Zapp can appear) *)
-let decomp_stack = function
-  | Zapp(v,n)::s -> Some (v.(n), append_stackn v (n+1) s)
-  | [] -> None
-  | _ -> assert false
-let decomp_stackn = function
-  | Zapp(v,n)::s ->
-      ((if n = 0 then v else Array.sub v n (Array.length v - n)), s)
+let rec decomp_stack = function
+  | Zapp[v]::s -> Some (v, s)
+  | Zapp(v::l)::s -> Some (v, (Zapp l :: s))
+  | Zapp [] :: s -> decomp_stack s
+  | _ -> None
+let rec decomp_stackn = function
+  | Zapp [] :: s -> decomp_stackn s
+  | Zapp l :: s -> (Array.of_list l, s)
   | _ -> assert false
 let array_of_stack s =
   let rec stackrec = function
   | [] -> []
-  | stk ->
-      let (args,s) = decomp_stackn stk in
-      args :: (stackrec s)
-  in Array.concat (stackrec s)
+  | Zapp args :: s -> args :: (stackrec s)
+  | _ -> assert false
+  in Array.of_list (List.concat (stackrec s))
 let rec list_of_stack = function
   | [] -> []
-  | stk ->
-      let (args,s) = decomp_stackn stk in
-      Array.fold_right (fun a l -> a::l) args (list_of_stack s)
+  | Zapp args :: s -> args @ (list_of_stack s)
+  | _ -> assert false
 let rec app_stack = function
   | f, [] -> f
-  | f, stk -> 
-      let (args,s) = decomp_stackn stk in
-      app_stack (appvect (f, args), s)
+  | f, (Zapp [] :: s) -> app_stack (f, s)
+  | f, (Zapp args :: s) -> 
+      app_stack (applist (f, args), s)
+  | _ -> assert false
 let rec stack_assign s p c = match s with
-  | Zapp(v,n)::s ->
-      let q = Array.length v - n in 
+  | Zapp args :: s ->
+      let q = List.length args in 
       if p >= q then
-	Zapp (v, n) :: stack_assign s (p-q) c
+	Zapp args :: stack_assign s (p-q) c
       else
-	let v' = Array.sub v n q in
-	v'.(p) <- c;
-	Zapp (v',0) :: s
+        (match list_chop p args with
+            (bef, _::aft) -> Zapp (bef@c::aft) :: s
+          | _ -> assert false)
   | _ -> s
 let rec stack_tail p s =
   if p = 0 then s else
     match s with
-      | Zapp (v, n) :: s ->
-	  let q = Array.length v - n in
+      | Zapp args :: s ->
+	  let q = List.length args in
 	  if p >= q then stack_tail (p-q) s
-	  else Zapp (v, n+p) :: s
+	  else
+            let (_,aft) = list_chop p args in
+            Zapp aft :: s
       | _ -> failwith "stack_tail"
 let rec stack_nth s p = match s with
-  | Zapp (v, n) :: s ->
-      let q = Array.length v - n in
+  | Zapp args :: s ->
+      let q = List.length args in
       if p >= q then stack_nth s (p-q)
-      else v.(p+n)
+      else List.nth args p
   | _ -> raise Not_found
 
 
@@ -418,7 +485,7 @@ let rec stack_nth s p = match s with
 (* Lazy reduction: the one used in kernel operations                  *)
 
 (* type of shared terms. fconstr and frterm are mutually recursive.
- * Clone of the Generic.term structure, but completely mutable, and
+ * Clone of the constr structure, but completely mutable, and
  * annotated with booleans (true when we noticed that the term is
  * normal and neutral) FLIFT is a delayed shift; allows sharing
  * between 2 lifted copies of a given term FCLOS is a delayed
@@ -434,27 +501,22 @@ and fterm =
   | FRel of int
   | FAtom of constr
   | FCast of fconstr * fconstr
-  | FFlex of freference
-  | FInd of inductive_path * fconstr array
-  | FConstruct of constructor_path * fconstr array
+  | FFlex of table_key
+  | FInd of inductive
+  | FConstruct of constructor
   | FApp of fconstr * fconstr array
-  | FFix of (int array * int) * (fconstr array * name list * fconstr array)
+  | FFix of (int array * int) * (name array * fconstr array * fconstr array)
       * constr array * fconstr subs
-  | FCoFix of int * (fconstr array * name list * fconstr array)
+  | FCoFix of int * (name array * fconstr array * fconstr array)
       * constr array * fconstr subs
   | FCases of case_info * fconstr * fconstr * fconstr array
   | FLambda of name * fconstr * fconstr * constr * fconstr subs
   | FProd of name * fconstr * fconstr * constr * fconstr subs
   | FLetIn of name * fconstr * fconstr * fconstr * constr * fconstr subs
+  | FEvar of existential_key * fconstr array
   | FLIFT of int * fconstr
   | FCLOS of constr * fconstr subs
-
-and freference =
-  (* only vars as args of FConst ... exploited for caching *)
-  | FConst of section_path * fconstr array
-  | FEvar of (existential * fconstr subs)
-  | FVar of identifier
-  | FFarRel of int (* index in the rel_context part of _initial_ environment *)
+  | FLOCKED
 
 let fterm_of v = v.term
 let set_whnf v = if v.norm = Red then v.norm <- Whnf
@@ -462,10 +524,6 @@ let set_cstr v = if v.norm = Red then v.norm <- Cstr
 let set_norm v = v.norm <- Norm
 let is_val v = v.norm = Norm
 
-(* Put an update mark in the stack, only if needed *)
-let zupdate m s =
-  if !share & m.norm = Red then Zupdate(m)::s else s
-
 (* Could issue a warning if no is still Red, pointing out that we loose
    sharing. *)
 let update v1 (no,t) =
@@ -475,6 +533,9 @@ let update v1 (no,t) =
      v1)
   else {norm=no;term=t}
 
+(* Lifting. Preserves sharing.
+   lft_fconstr always create a new cell, while lift_fconstr avoids it
+   when the lift is 0. *)
 let rec lft_fconstr n ft =
   match ft.term with
     FLIFT(k,m) -> lft_fconstr (n+k) m
@@ -483,25 +544,54 @@ let lift_fconstr k f =
   if k=0 then f else lft_fconstr k f
 let lift_fconstr_vect k v =
   if k=0 then v else Array.map (fun f -> lft_fconstr k f) v
+let lift_fconstr_list k l =
+  if k=0 then l else List.map (fun f -> lft_fconstr k f) l
+
+
+(* since the head may be reducible, we might introduce lifts of 0 *)
+let compact_stack head stk =
+  let rec strip_rec depth = function
+    | Zshift(k)::s -> strip_rec (depth+k) s
+    | Zupdate(m)::s ->
+        (* Be sure to create a new cell otherwise sharing would be
+           lost by the update operation *)
+        let h' = lft_fconstr depth head in
+        let _ = update m (h'.norm,h'.term) in
+        strip_rec depth s
+    | stk -> zshift depth stk in
+  strip_rec 0 stk
+
+(* Put an update mark in the stack, only if needed *)
+let zupdate m s =
+  if !share & m.norm = Red
+  then
+    let s' = compact_stack m s in
+    let _ = m.term <- FLOCKED in
+    Zupdate(m)::s'
+  else s
+
 
 let clos_rel e i =
   match expand_rel i e with
     | Inl(n,mt) -> lift_fconstr n mt
     | Inr(k,None) -> {norm=Red; term= FRel k}
     | Inr(k,Some p) ->
-        lift_fconstr (k-p) {norm=Norm;term=FFlex(FFarRel p)}
+        lift_fconstr (k-p) {norm=Norm;term=FFlex(FarRelKey p)}
 
 (* Optimization: do not enclose variables in a closure.
    Makes variable access much faster *)
-let mk_clos e t =
+let rec mk_clos e t =
   match kind_of_term t with
-    | IsRel i -> clos_rel e i
-    | IsVar x -> { norm = Red; term = FFlex (FVar x) }
-    | IsMeta _ | IsSort _ ->  { norm = Norm; term = FAtom t }
-    | (IsMutInd _|IsMutConstruct _|IsFix _|IsCoFix _
-      |IsLambda _|IsProd _) ->
+    | Rel i -> clos_rel e i
+    | Var x -> { norm = Red; term = FFlex (VarKey x) }
+    | Meta _ | Sort _ ->  { norm = Norm; term = FAtom t }
+    | Ind sp -> { norm = Norm; term = FInd sp }
+    | Construct sp -> { norm = Cstr; term = FConstruct sp }
+    | Evar (ev,args) ->
+        { norm = Cstr; term = FEvar (ev,Array.map (mk_clos e) args) }
+    | (Fix _|CoFix _|Lambda _|Prod _) ->
         {norm = Cstr; term = FCLOS(t,e)}
-    | (IsApp _|IsMutCase _|IsCast _|IsConst _|IsEvar _|IsLetIn _) ->
+    | (App _|Case _|Cast _|Const _|LetIn _) ->
         {norm = Red; term = FCLOS(t,e)}
 
 let mk_clos_vect env v = Array.map (mk_clos env) v
@@ -512,53 +602,46 @@ let mk_clos_vect env v = Array.map (mk_clos env) v
    Could be used insted of mk_clos. *)
 let mk_clos_deep clos_fun env t =
   match kind_of_term t with
-    | IsRel i -> clos_rel env i
-    | (IsVar _|IsMeta _ | IsSort _) -> mk_clos env t
-    | IsCast (a,b) ->
+    | (Rel _|Ind _|Construct _|Var _|Meta _ | Sort _|Evar _) ->
+        mk_clos env t
+    | Cast (a,b) ->
         { norm = Red;
           term = FCast (clos_fun env a, clos_fun env b)}
-    | IsApp (f,v) ->
+    | App (f,v) ->
         { norm = Red;
 	  term = FApp (clos_fun env f, Array.map (clos_fun env) v) }
-    | IsMutInd (sp,v) ->
-        { norm = Cstr; term = FInd (sp, Array.map (clos_fun env) v) }
-    | IsMutConstruct (sp,v) ->
-        { norm = Cstr; term = FConstruct (sp,Array.map (clos_fun env) v)}
-    | IsConst (sp,v) ->
+    | Const sp ->
         { norm = Red;
-	  term = FFlex (FConst (sp,Array.map (clos_fun env) v)) }
-    | IsEvar (_,v as ev) ->
-        { norm = Red;
-	  term = FFlex (FEvar (ev, env)) }
-
-    | IsMutCase (ci,p,c,v) ->
+	  term = FFlex (ConstKey sp) }
+    | Case (ci,p,c,v) ->
         { norm = Red;
 	  term = FCases (ci, clos_fun env p, clos_fun env c,
 			 Array.map (clos_fun env) v) }
-    | IsFix (op,(tys,lna,bds)) ->
+    | Fix (op,(lna,tys,bds)) ->
 	let env' = subs_liftn (Array.length bds) env in
         { norm = Cstr;
-	  term = FFix (op, (Array.map (clos_fun env) tys, lna,
-			    Array.map (clos_fun env') bds),
-		       bds, env) }
-    | IsCoFix (op,(tys,lna,bds)) ->
+	  term = FFix
+                   (op,(lna, Array.map (clos_fun env) tys,
+                             Array.map (clos_fun env') bds),
+                    bds, env) }
+    | CoFix (op,(lna,tys,bds)) ->
 	let env' = subs_liftn (Array.length bds) env in
         { norm = Cstr;
-	  term = FCoFix (op, (Array.map (clos_fun env) tys, lna,
-			      Array.map (clos_fun env') bds),
-			 bds, env) }
-
-    | IsLambda (n,t,c) ->
+	  term = FCoFix
+                   (op,(lna, Array.map (clos_fun env) tys,
+	                     Array.map (clos_fun env') bds),
+	            bds, env) }
+    | Lambda (n,t,c) ->
         { norm = Cstr;
 	  term = FLambda (n, clos_fun env t,
 			  clos_fun (subs_lift env) c,
 			  c, env) }
-    | IsProd (n,t,c)   ->
+    | Prod (n,t,c)   ->
         { norm = Cstr;
 	  term = FProd (n, clos_fun env t, 
 			clos_fun (subs_lift env) c,
 			c, env) }
-    | IsLetIn (n,b,t,c) ->
+    | LetIn (n,b,t,c) ->
         { norm = Red;
 	  term = FLetIn (n, clos_fun env b, clos_fun env t,
 			 clos_fun (subs_lift env) c,
@@ -568,36 +651,30 @@ let mk_clos_deep clos_fun env t =
 let rec to_constr constr_fun lfts v =
   match v.term with
     | FRel i -> mkRel (reloc_rel i lfts)
-    | FFlex (FFarRel p) -> mkRel (reloc_rel p lfts)
-    | FFlex (FVar x) -> mkVar x
+    | FFlex (FarRelKey p) -> mkRel (reloc_rel p lfts)
+    | FFlex (VarKey x) -> mkVar x
     | FAtom c ->
         (match kind_of_term c with
-          | IsSort s -> mkSort s
-          | IsMeta m -> mkMeta m
+          | Sort s -> mkSort s
+          | Meta m -> mkMeta m
           | _ -> assert false)
     | FCast (a,b) ->
         mkCast (constr_fun lfts a, constr_fun lfts b)
-    | FFlex (FConst (op,ve)) ->
-	mkConst (op, Array.map (constr_fun lfts) ve)
-    | FFlex (FEvar ((n,args),env)) ->
-	let f a = constr_fun lfts (mk_clos env a) in
-	mkEvar (n, Array.map f args)
-    | FInd (op,ve) ->
-	mkMutInd (op, Array.map (constr_fun lfts) ve)
-    | FConstruct (op,ve) -> 
-	mkMutConstruct (op, Array.map (constr_fun lfts) ve)
+    | FFlex (ConstKey op) -> mkConst op
+    | FInd op -> mkInd op
+    | FConstruct op -> mkConstruct op
     | FCases (ci,p,c,ve) ->
-	mkMutCase (ci, constr_fun lfts p,
-		   constr_fun lfts c,
-		   Array.map (constr_fun lfts) ve)
-    | FFix (op,(tys,lna,bds),_,_) ->
+	mkCase (ci, constr_fun lfts p,
+                constr_fun lfts c,
+		Array.map (constr_fun lfts) ve)
+    | FFix (op,(lna,tys,bds),_,_) ->
 	let lfts' = el_liftn (Array.length bds) lfts in
-	mkFix (op, (Array.map (constr_fun lfts) tys, lna,
-		    Array.map (constr_fun lfts') bds))
-    | FCoFix (op,(tys,lna,bds),_,_) ->
+	mkFix (op, (lna, Array.map (constr_fun lfts) tys,
+		         Array.map (constr_fun lfts') bds))
+    | FCoFix (op,(lna,tys,bds),_,_) ->
 	let lfts' = el_liftn (Array.length bds) lfts in
-	mkCoFix (op, (Array.map (constr_fun lfts) tys, lna,
-		      Array.map (constr_fun lfts') bds))
+	mkCoFix (op, (lna, Array.map (constr_fun lfts) tys,
+		           Array.map (constr_fun lfts') bds))
     | FApp (f,ve) ->
 	mkApp (constr_fun lfts f,
 	       Array.map (constr_fun lfts) ve)
@@ -611,11 +688,13 @@ let rec to_constr constr_fun lfts v =
 	mkLetIn (n, constr_fun lfts b,
 	      constr_fun lfts t,
 	      constr_fun (el_lift lfts) c)
+    | FEvar (ev,args) -> mkEvar(ev,Array.map (constr_fun lfts) args)
     | FLIFT (k,a) -> to_constr constr_fun (el_shft k lfts) a
     | FCLOS (t,env) ->
         let fr = mk_clos_deep mk_clos env t in
         let unfv = update v (fr.norm,fr.term) in
         to_constr constr_fun lfts unfv
+    | FLOCKED -> anomaly "Closure.to_constr: found locked term"
 
 (* This function defines the correspondance between constr and
    fconstr. When we find a closure whose substitution is the identity,
@@ -641,14 +720,14 @@ let rec fstrong unfreeze_fun lfts v =
 let rec zip zfun m stk =
   match stk with
     | [] -> m
-    | Zapp(_)::_ ->
-        let (args,s) = decomp_stackn stk in
+    | Zapp args :: s ->
+        let args = Array.of_list args in
         zip zfun {norm=m.norm; term=FApp(m, Array.map zfun args)} s
     | Zcase(ci,p,br)::s ->
         let t = FCases(ci, zfun p, m, Array.map zfun br) in
         zip zfun {norm=m.norm; term=t} s
     | Zfix(fx,par)::s ->
-        zip zfun fx (par @ append_stack [|m|] s)
+        zip zfun fx (par @ append_stack_list ([m], s))
     | Zshift(n)::s ->
         zip zfun (lift_fconstr n m) s
     | Zupdate(rf)::s ->
@@ -679,10 +758,9 @@ let strip_update_shift_app head stk =
   let rec strip_rec rstk h depth = function
     | Zshift(k) as e :: s ->
         strip_rec (e::rstk) (lift_fconstr k h) (depth+k) s
-    | (Zapp(_)::_) as stk ->
-        let (args,s) = decomp_stackn stk in
-        strip_rec (Zapp(args,0)::rstk)
-          {norm=h.norm;term=FApp(h,args)} depth s
+    | (Zapp args :: s) as stk ->
+        strip_rec (Zapp args :: rstk)
+          {norm=h.norm;term=FApp(h,Array.of_list args)} depth s
     | Zupdate(m)::s ->
         strip_rec rstk (update m (h.norm,h.term)) depth s
     | stk -> (depth,List.rev rstk, stk) in
@@ -694,17 +772,19 @@ let rec get_nth_arg head n stk =
   let rec strip_rec rstk h depth n = function
     | Zshift(k) as e :: s ->
         strip_rec (e::rstk) (lift_fconstr k h) (depth+k) n s
-    | Zapp(v, p)::s' ->
-        let q = Array.length v - p in
+    | Zapp args::s' ->
+        let q = List.length args in
         if n >= q
         then
-          let v' = if p = 0 then v else Array.sub v p q in
-          strip_rec (Zapp(v',0)::rstk)
-            {norm=h.norm;term=FApp(h,v')} depth (n-q) s'
+          strip_rec (Zapp args::rstk)
+            {norm=h.norm;term=FApp(h,Array.of_list args)} depth (n-q) s'
         else
-          let rstk' =
-            if n = 0 then rstk else Zapp(Array.sub v p n,0)::rstk in
-          (Some (List.rev rstk', v.(p+n)), append_stackn v (p+n+1) s')
+          (match list_chop n args with
+              (bef, v::aft) -> 
+                let stk' =
+                  List.rev (if n = 0 then rstk else (Zapp bef :: rstk)) in
+                (Some (stk', v), append_stack_list (aft,s'))
+            | _ -> assert false)
     | Zupdate(m)::s ->
         strip_rec rstk (update m (h.norm,h.term)) depth n s
     | s -> (None, List.rev rstk @ s) in
@@ -714,18 +794,17 @@ let rec get_nth_arg head n stk =
    Since the encountered update marks are removed, h must be a whnf *)
 let get_arg h stk =
   let (depth,stk') = strip_update_shift h stk in
-  match stk' with
-      Zapp(v,p)::s -> (Some(depth,v.(p)), append_stackn v (p+1) s)
-    | _ -> (None, zshift depth stk')
+  match  decomp_stack stk' with
+      Some (v, s') -> (Some (depth,v), s')
+    | None -> (None, zshift depth stk')
 
 
 (* Iota reduction: extract the arguments to be passed to the Case
    branches *)
 let rec reloc_rargs_rec depth stk =
   match stk with
-      Zapp(_)::_ ->
-        let (args,s) = decomp_stackn stk in
-        Zapp(lift_fconstr_vect depth args,0) :: reloc_rargs_rec depth s
+      Zapp args :: s ->
+        Zapp (lift_fconstr_list depth args) :: reloc_rargs_rec depth s
     | Zshift(k)::s -> if k=depth then s else reloc_rargs_rec (depth-k) s
     | _ -> stk
 
@@ -734,11 +813,13 @@ let reloc_rargs depth stk =
 
 let rec drop_parameters depth n stk =
   match stk with
-      Zapp(v,p)::s ->
-        let q = Array.length v - p in
+      Zapp args::s ->
+        let q = List.length args in
         if n > q then drop_parameters depth (n-q) s
         else if n = q then reloc_rargs depth s
-        else reloc_rargs depth (Zapp(v,p+n)::s)
+        else
+          let (_,aft) = list_chop n args in
+          reloc_rargs depth (append_stack_list (aft,s))
     | Zshift(k)::s -> drop_parameters (depth-k) n s
     | [] -> assert (n=0); []
     | _ -> assert false (* we know that n < stack_args_size(stk) *)
@@ -784,6 +865,7 @@ let rec knh m stk =
   match m.term with
     | FLIFT(k,a) -> knh a (zshift k stk)
     | FCLOS(t,e) -> knht e t (Zupdate(m)::stk)
+    | FLOCKED -> anomaly "Closure.knh: found lock"
     | FApp(a,b) -> knh a (append_stack b (zupdate m stk))
     | FCases(ci,p,t,br) -> knh t (Zcase(ci,p,br)::zupdate m stk)
     | FFix((ri,n),_,_,_) ->
@@ -793,23 +875,23 @@ let rec knh m stk =
            | (None, stk') -> (m,stk'))
     | FCast(t,_) -> knh t stk
 (* cases where knh stops *)
-    | (FFlex _|FLetIn _) -> (m, stk)
-    | (FRel _|FAtom _) -> (set_norm m; (m, stk))
-    | (FLambda _|FConstruct _|FCoFix _|FInd _|FProd _) ->
+    | (FFlex _|FLetIn _|FInd _|FConstruct _|FEvar _) -> (m, stk)
+    | (FRel _|FAtom _|FInd _) -> (set_norm m; (m, stk))
+    | (FLambda _|FCoFix _|FProd _) ->
         (set_whnf m; (m, stk))
 
 (* The same for pure terms *)
 and knht e t stk =
   match kind_of_term t with
-    | IsApp(a,b) ->
+    | App(a,b) ->
         knht e a (append_stack (mk_clos_vect e b) stk)
-    | IsMutCase(ci,p,t,br) ->
+    | Case(ci,p,t,br) ->
         knht e t (Zcase(ci, mk_clos e p, mk_clos_vect e br)::stk)
-    | IsFix _ -> knh (mk_clos_deep mk_clos e t) stk
-    | IsCast(a,b) -> knht e a stk
-    | IsRel n -> knh (clos_rel e n) stk
-    | (IsLambda _|IsProd _|IsMutConstruct _|IsCoFix _|IsMutInd _|
-       IsLetIn _|IsConst _|IsVar _|IsEvar _|IsMeta _|IsSort _) ->
+    | Fix _ -> knh (mk_clos_deep mk_clos e t) stk
+    | Cast(a,b) -> knht e a stk
+    | Rel n -> knh (clos_rel e n) stk
+    | (Lambda _|Prod _|Construct _|CoFix _|Ind _|
+       LetIn _|Const _|Var _|Evar _|Meta _|Sort _) ->
         (mk_clos_deep mk_clos e t, stk)
 
 
@@ -818,35 +900,27 @@ and knht e t stk =
 (* Computes a normal form from the result of knh. *)
 let rec knr info m stk =
   match m.term with
-  | FLambda(_,_,_,f,e) when can_red info stk BETA ->
+  | FLambda(_,_,_,f,e) when red_set info.i_flags fBETA ->
       (match get_arg m stk with
           (Some(depth,arg),s) -> knit info (subs_shift_cons(depth,e,arg)) f s
         | (None,s) -> (m,s))
-  | FFlex(FConst(sp,args)) when can_red info stk (CONST [sp]) ->
-      let cst = (sp, Array.map term_of_fconstr args) in
-      (match ref_value_cache info (ConstBinding cst) with
+  | FFlex(ConstKey sp) when red_set info.i_flags (fCONST sp) ->
+      (match ref_value_cache info (ConstKey sp) with
           Some v -> kni info v stk
         | None -> (set_norm m; (m,stk)))
-  | FFlex(FEvar ev) when can_red info stk EVAR ->
-(* In the case of evars, if it is not defined, then we do not set the
-   flag to Norm, because it may be instantiated later on *)
-      (match ref_value_cache info (EvarBinding ev) with
-          Some v -> kni info v stk
-        | None -> (m,stk))
-  | FFlex(FVar id) when can_red info stk (VAR id) ->
-      (match ref_value_cache info (VarBinding id) with
+  | FFlex(VarKey id) when red_set info.i_flags (fVAR id) ->
+      (match ref_value_cache info (VarKey id) with
           Some v -> kni info v stk
         | None -> (set_norm m; (m,stk)))
-  | FFlex(FFarRel k) when can_red info stk DELTA ->
-      (match ref_value_cache info (FarRelBinding k) with
+  | FFlex(FarRelKey k) when red_set info.i_flags fDELTA ->
+      (match ref_value_cache info (FarRelKey k) with
           Some v -> kni info v stk
         | None -> (set_norm m; (m,stk)))
-  | FConstruct((sp,c),args) when can_red info stk IOTA ->
+  | FConstruct(ind,c) when red_set info.i_flags fIOTA ->
       (match strip_update_shift_app m stk with
-          (depth, args, Zcase((cn,_),_,br)::s) ->
-            let npar = stack_args_size args - cn.(c-1) in
-            assert (npar>=0);
-            let rargs = drop_parameters depth npar args in
+          (depth, args, Zcase(ci,_,br)::s) ->
+            assert (ci.ci_npar>=0);
+            let rargs = drop_parameters depth ci.ci_npar args in
             kni info br.(c-1) (rargs@s)
         | (_, cargs, Zfix(fx,par)::s) ->
             let rarg = fapp_stack(m,cargs) in
@@ -854,13 +928,13 @@ let rec knr info m stk =
             let efx = contract_fix_vect fx.term in
             kni info efx stk'
         | (_,args,s) -> (m,args@s))
-  | FCoFix _ when can_red info stk IOTA ->
+  | FCoFix _ when red_set info.i_flags fIOTA ->
       (match strip_update_shift_app m stk with
-          (_, args, ((Zcase((cn,_),_,br)::_) as stk')) ->
+          (_, args, ((Zcase _::_) as stk')) ->
             let efx = contract_fix_vect m.term in
             kni info efx (args@stk')
         | (_,args,s) -> (m,args@s))
-  | FLetIn (_,v,_,_,bd,e) when can_red info stk ZETA ->
+  | FLetIn (_,v,_,_,bd,e) when red_set info.i_flags fZETA ->
       knit info (subs_cons(v,e)) bd stk
   | _ -> (m,stk)
 
@@ -899,17 +973,10 @@ and down_then_up info m stk =
             FLetIn(na, kl info a, kl info b, kl info c, f, e)
         | FProd(na,dom,rng,f,e) ->
             FProd(na, kl info dom, kl info rng, f, e)
-        | FInd(ind,args) ->
-            FInd(ind, Array.map (kl info) args)
-        | FConstruct(c,args) ->
-            FConstruct(c, Array.map (kl info) args)
-        | FCoFix(n,(ftys,na,fbds),bds,e) ->
-            FCoFix(n,(Array.map (kl info) ftys, na,
-                      Array.map (kl info) fbds),bds,e)
-        | FFlex(FConst(sp,args)) ->
-            FFlex(FConst(sp, Array.map (kl info) args))
-        | FFlex(FEvar((i,args),e)) ->
-            FFlex(FEvar((i,args),e))
+        | FCoFix(n,(na,ftys,fbds),bds,e) ->
+            FCoFix(n,(na, Array.map (kl info) ftys,
+                          Array.map (kl info) fbds),bds,e)
+        | FEvar(i,args) -> FEvar(i, Array.map (kl info) args)
         | t -> t in
       {norm=Norm;term=nt} in
 (* Precondition: m.norm = Norm *)
@@ -928,37 +995,12 @@ let norm_val info v =
 
 let inject = mk_clos (ESID 0)
 
+let whd_stack = kni
+
 (* cache of constants: the body is computed only when needed. *)
-type 'a clos_infos = (fconstr, 'a) infos
+type clos_infos = fconstr infos
 
-let create_clos_infos flgs env sigma =
-  create (fun _ -> mk_clos) flgs env sigma
+let create_clos_infos flgs env =
+  create (fun _ -> inject) flgs env
 
-let unfold_reference info = function
-  | FConst (op,v) -> 
-      let cst = (op, Array.map (norm_val info) v) in
-      ref_value_cache info (ConstBinding cst)
-  | FEvar ev -> ref_value_cache info (EvarBinding ev)
-  | FVar id -> ref_value_cache info (VarBinding id)
-  | FFarRel p -> ref_value_cache info (FarRelBinding p)
-
-(* Head normal form. *)
-
-(* TODO: optimise *)
-let rec strip_applstack k acc m =
-  match m.term with
-      FApp(a,b) ->
-        strip_applstack k (append_stack (lift_fconstr_vect k b) acc) a
-    | FLIFT(n,a) ->
-        strip_applstack (k+n) acc a
-    | FCLOS _ -> assert false
-    | _ -> (k,m,acc)
-
-
-let fhnf info v =
-  strip_applstack 0 [] (kh info v [])
-
-
-let fhnf_apply info k head appl =
-  let stk = zshift k appl in
-  strip_applstack 0 [] (kh info head stk)
+let unfold_reference = ref_value_cache
