@@ -6,37 +6,37 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: cooking.mli,v 1.3 2001/03/15 13:38:21 filliatr Exp $ i*)
+(*i $Id: cooking.mli,v 1.7 2001/10/09 16:38:58 barras Exp $ i*)
 
 open Names
 open Term
 open Declarations
 open Environ
+open Univ
 
 (*s Cooking the constants. *)
 
-type modification_action = ABSTRACT | ERASE
-
 type 'a modification =
   | NOT_OCCUR
-  | DO_ABSTRACT of 'a * modification_action list
-  | DO_REPLACE
+  | DO_ABSTRACT of 'a * constr array
+  | DO_REPLACE of constant_body
 
 type work_list =
-    (section_path * section_path modification) list
-    * (inductive_path * inductive_path modification) list
-    * (constructor_path * constructor_path modification) list
+    (constant * constant modification) list
+    * (inductive * inductive modification) list
+    * (constructor * constructor modification) list
 
 type recipe = {
-  d_from : section_path;
+  d_from : constant_body;
   d_abstract : identifier list;
   d_modlist : work_list }
 
-val cook_constant : env -> recipe -> constr option * constr
+val cook_constant :
+  env -> recipe -> constr option * constr * constraints * bool
 
 (*s Utility functions used in module [Discharge]. *)
 
-val expmod_constr : env -> work_list -> constr -> constr
-val expmod_type : env -> work_list -> types -> types
+val expmod_constr : work_list -> constr -> constr
+val expmod_type : work_list -> types -> types
 
 
