@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: options.mli,v 1.11 2001/04/19 06:49:49 courant Exp $ i*)
+(*i $Id: options.mli,v 1.13 2001/10/11 17:27:20 herbelin Exp $ i*)
 
 (* Global options of the system. *)
 
@@ -19,6 +19,8 @@ val debug : bool ref
 val print_emacs : bool ref
 val emacs_str : string -> string
 
+val term_quality : bool ref
+
 val make_silent : bool -> unit
 val is_silent : unit -> bool
 val is_verbose : unit -> bool
@@ -36,5 +38,3 @@ val without_mes_ambig : ('a -> 'b) -> 'a -> 'b
 
 val add_unsafe : string -> unit
 val is_unsafe : string -> bool
-
-val immediate_discharge : bool
