@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: goptions.ml,v 1.10 2001/03/15 13:38:27 filliatr Exp $ *)
+(* $Id: goptions.ml,v 1.16 2001/12/13 09:03:04 filliatr Exp $ *)
 
 (* This module manages customization parameters at the vernacular level     *)
 
@@ -14,6 +14,8 @@ open Pp
 open Util
 open Libobject
 open Names
+open Term
+open Nametab
 
 (****************************************************************************)
 (* 0- Common things                                                         *)
@@ -33,7 +35,7 @@ type value =
   | BoolValue   of bool
   | IntValue    of int
   | StringValue of string
-  | IdentValue  of identifier
+  | IdentValue  of global_reference
 
 (****************************************************************************)
 (* 1- Tables                                                                *)
@@ -107,10 +109,11 @@ module MakeTable =
          (fun c -> t := MySet.remove c !t))
 
     let print_table table_name printer table =
-      mSG ([< 'sTR table_name ; (hOV 0 
-           (if MySet.is_empty table then [< 'sTR "None" ; 'fNL >]
-            else MySet.fold
-	      (fun a b -> [< printer a; 'sPC; b >]) table [<>])) >])
+      msg (str table_name ++ 
+	   (hov 0 
+	      (if MySet.is_empty table then str "None" ++ fnl ()
+               else MySet.fold
+		 (fun a b -> printer a ++ spc () ++ b) table (mt ()))))
 
     class table_of_A () =
     object
@@ -121,7 +124,7 @@ module MakeTable =
       method remove x = remove_option (A.encode x)
       method mem x = 
         let answer = MySet.mem (A.encode x) !t in
-        mSG [< 'sTR (A.member_message x answer) >]
+        msg (str (A.member_message x answer))
       method print = print_table A.title A.printer !t 
     end
 
@@ -150,7 +153,7 @@ struct
   let table = string_table
   let encode x = x
   let check = A.check
-  let printer s = [< 'sTR s >]
+  let printer s = (str s)
   let key = A.key
   let title = A.title
   let member_message = A.member_message
@@ -167,19 +170,19 @@ let get_ident_table k = List.assoc (nickname k) !ident_table
 module type IdentConvertArg =
 sig
   type t
-  val encode : identifier -> t
+  val encode : global_reference -> t
   val check : t -> unit
   val printer : t -> std_ppcmds
   val key : option_name
   val title : string
-  val member_message : identifier -> bool -> string
+  val member_message : global_reference -> bool -> string
   val synchronous : bool
 end
 
 module IdentConvert = functor (A : IdentConvertArg) -> 
 struct
   type t = A.t
-  type key = identifier
+  type key = global_reference
   let table = ident_table
   let encode = A.encode
   let check = A.check
@@ -196,135 +199,86 @@ module MakeIdentTable =
 (****************************************************************************)
 (* 2- Options                                                               *)
 
-type option_value_ref =
-  | OptionBoolRef   of bool   ref
-  | OptionIntRef    of int    ref
-  | OptionStringRef of string ref
+type 'a option_sig = {
+  optsync  : bool;
+  optname  : string;
+  optkey   : option_name;
+  optread  : unit -> 'a;
+  optwrite : 'a -> unit }
+
+type option_type = bool * (unit -> value) -> (value -> unit) 
 
 module Key = struct type t = option_name let compare = Pervasives.compare end
 module OptionMap = Map.Make(Key)
 
-let sync_value_tab = ref OptionMap.empty
-let async_value_tab = ref OptionMap.empty
-
-(* Tools for synchronous options *)
-
-let load_sync_value _ = ()
-let cache_sync_value (_,(k,v)) =
-  sync_value_tab := OptionMap.add k v !sync_value_tab
-let export_sync_value fp = Some fp
-let (inOptVal,_) =
-  Libobject.declare_object ("Sync_option_value",
-    { Libobject.load_function = load_sync_value;
-      Libobject.open_function = cache_sync_value;
-      Libobject.cache_function = cache_sync_value;
-      Libobject.export_function = export_sync_value })
-
-let freeze_sync_table () = !sync_value_tab
-let unfreeze_sync_table l = sync_value_tab := l
-let init_sync_table () = sync_value_tab := OptionMap.empty
-let _ = 
-  Summary.declare_summary "Sync_option"
-    { Summary.freeze_function = freeze_sync_table;
-      Summary.unfreeze_function = unfreeze_sync_table;
-      Summary.init_function = init_sync_table;
-      Summary.survive_section = true }
-
-(* Tools for handling options *)
-
-type option_type =
-  | Sync of  value
-  | Async of (unit -> value) * (value -> unit)
+let value_tab = ref OptionMap.empty
 
 (* This raises Not_found if option of key [key] is unknown *)
-let get_option key =
-  try 
-    let (name,(r,w)) = OptionMap.find key !async_value_tab in 
-    (name,Async (r,w))
-  with Not_found ->
-    let (name,i) = OptionMap.find key !sync_value_tab in (name, Sync i)
 
+let get_option key = OptionMap.find key !value_tab
 
-(* 2-a. Declaring synchronous options *)
+let check_key key = try 
+  let _ = get_option key in
+  error "Sorry, this option name is already used"
+with Not_found ->
+  if List.mem_assoc (nickname key) !string_table
+    or List.mem_assoc (nickname key) !ident_table
+  then error "Sorry, this option name is already used"
 
-type 'a sync_option_sig = {
-  optsyncname    : string;
-  optsynckey     : option_name;
-  optsyncdefault : 'a }
+open Summary
+open Libobject
+open Lib
 
-let declare_sync_option (cast,uncast) 
-  { optsyncname=name; optsynckey=key; optsyncdefault=default } =
-  try 
-    let _ = get_option key in
-    error "Sorry, this option name is already used"
-  with Not_found ->
-    if List.mem_assoc (nickname key) !string_table
-      or List.mem_assoc (nickname key) !ident_table
-    then error "Sorry, this option name is already used";
-    sync_value_tab := OptionMap.add key (name,(cast default)) !sync_value_tab;
-    fun () -> uncast (snd (OptionMap.find key !sync_value_tab))
+let declare_option cast uncast 
+  { optsync=sync; optname=name; optkey=key; optread=read; optwrite=write } =
+  check_key key; 
+  let write = if sync then 
+    let (decl_obj,_) = 
+      declare_object ((nickname key),
+		      {load_function = (fun _ -> ());
+		       open_function = (fun _ -> ());
+		       cache_function = (fun (_,v) -> write v);
+		       export_function = (fun x -> None)})
+    in 
+    let _ = declare_summary (nickname key) 
+	      {freeze_function = read;
+	       unfreeze_function = write;
+	       init_function = (let default = read() in fun () -> write default);
+	       survive_section = true}
+    in 
+    fun v -> add_anonymous_leaf (decl_obj v)
+  else write
+  in 
+  let cread () = cast (read ()) in
+  let cwrite v = write (uncast v) in 
+  value_tab := OptionMap.add key (name,(sync,cread,cwrite)) !value_tab; 
+  write
 
-type 'a value_function = unit -> 'a
+type 'a write_function = 'a -> unit
 
-let declare_sync_int_option = declare_sync_option
-  ((function vr -> IntValue vr),
-   function IntValue x -> x | _ -> anomaly "MakeOption")
-let declare_sync_bool_option = declare_sync_option
-  ((function vr -> BoolValue vr),
-   function BoolValue x -> x | _ -> anomaly "MakeOption")
-let declare_sync_string_option = declare_sync_option
-  ((function vr -> StringValue vr),
-   function StringValue x -> x | _ -> anomaly "MakeOption")
-
-(* 2-b. Declaring synchronous options *)
-
-type 'a async_option_sig = {
-  optasyncname  : string;
-  optasynckey   : option_name;
-  optasyncread  : unit -> 'a;
-  optasyncwrite : 'a -> unit }
-
-let declare_async_option cast uncast
-  {optasyncname=name;optasynckey=key;optasyncread=read;optasyncwrite=write} =
-  try 
-    let _ = get_option key in
-    error "Sorry, this option name is already used"
-  with Not_found ->
-    if List.mem_assoc (nickname key) !string_table
-      or List.mem_assoc (nickname key) !ident_table then 
-      error "Sorry, this option name is already used";
-    let cread () = cast (read ()) in
-    let cwrite v = write (uncast v) in
-    async_value_tab := OptionMap.add key (name,(cread,cwrite)) !async_value_tab
-
-let declare_async_int_option =
-  declare_async_option 
+let declare_int_option =
+  declare_option 
     (fun v -> IntValue v)
     (function IntValue v -> v | _ -> anomaly "async_option")
-let declare_async_bool_option =
-  declare_async_option
+let declare_bool_option =
+  declare_option
     (fun v -> BoolValue v)
     (function BoolValue v -> v | _ -> anomaly "async_option")
-let declare_async_string_option =
-  declare_async_option
+let declare_string_option =
+  declare_option
     (fun v -> StringValue v)
     (function StringValue v -> v | _ -> anomaly "async_option")
 
-
 (* 3- User accessible commands *)
 
 (* Setting values of options *)
 
 let set_option_value check_and_cast key v =
-  let (name,info) =
+  let (name,(_,read,write)) =
     try get_option key
     with Not_found -> error ("There is no option "^(nickname key)^".")
   in
-  match info with
-    | Sync  current  ->
-      	Lib.add_anonymous_leaf
-	  (inOptVal (key,(name,check_and_cast v current)))
-    | Async (read,write) -> write (check_and_cast v (read ()))
+  write (check_and_cast v (read ()))
 
 let bad_type_error () = error "Bad type of value for this option"
 
@@ -343,47 +297,54 @@ let set_string_option_value = set_option_value
 
 (* Printing options/tables *)
 
-let msg_sync_option_value (name,v) =
+let msg_option_value (name,v) =
   match v with
-    | BoolValue true  -> [< 'sTR "true" >]
-    | BoolValue false -> [< 'sTR "false" >]
-    | IntValue n      -> [< 'iNT n >]
-    | StringValue s   -> [< 'sTR s >]
-    | IdentValue id    -> [< pr_id id >]
-
-let msg_async_option_value (name,vref) =
-  match vref with
-    | OptionBoolRef {contents=true}  -> [< 'sTR "true" >]
-    | OptionBoolRef {contents=false} -> [< 'sTR "false" >]
-    | OptionIntRef  r     -> [< 'iNT !r >]
-    | OptionStringRef r   -> [< 'sTR !r >]
+    | BoolValue true  -> str "true"
+    | BoolValue false -> str "false"
+    | IntValue n      -> int n
+    | StringValue s   -> str s
+    | IdentValue r    -> pr_global_env (Global.env()) r
 
 let print_option_value key =
-  let (name,info) = get_option key in
-  let s = match info with
-    | Sync v -> msg_sync_option_value (name,v)
-      | Async (read,write) -> msg_sync_option_value (name,read ())
-  in mSG [< 'sTR ("Current value of "^name^" is "); s; 'fNL >]
+  let (name,(_,read,_)) = get_option key in
+  let s = read () in 
+  match s with
+    | BoolValue b -> 
+	msg (str ("The "^name^" mode is "^(if b then "on" else "off")) ++ 
+	     fnl ())
+    | _ ->
+	msg (str ("Current value of "^name^" is ") ++
+	     msg_option_value (name,s) ++ fnl ())
+
 
 let print_tables () =
-  mSG
-  [< 'sTR "Synchronous options:"; 'fNL; 
+  msg
+    (str "Synchronous options:" ++ fnl () ++
      OptionMap.fold 
-       (fun key (name,v) p -> [< p; 'sTR ("  "^(nickname key)^": ");
-				 msg_sync_option_value (name,v); 'fNL >])
-       !sync_value_tab [<>];
-     'sTR "Asynchronous options:"; 'fNL;
+       (fun key (name,(sync,read,write)) p -> 
+	  if sync then 
+	    p ++ str ("  "^(nickname key)^": ") ++
+	    msg_option_value (name,read()) ++ fnl ()
+	  else 
+	    p)
+       !value_tab (mt ()) ++
+     str "Asynchronous options:" ++ fnl () ++
      OptionMap.fold 
-       (fun key (name,(read,write)) p -> [< p; 'sTR ("  "^(nickname key)^": ");
-                                  msg_sync_option_value (name,read()); 'fNL >])
-	    !async_value_tab [<>];
-     'sTR "Tables:"; 'fNL;
+       (fun key (name,(sync,read,write)) p -> 
+	  if sync then 
+	    p
+	  else 
+	    p ++ str ("  "^(nickname key)^": ") ++
+            msg_option_value (name,read()) ++ fnl ())
+       !value_tab (mt ()) ++
+     str "Tables:" ++ fnl () ++
      List.fold_right
-       (fun (nickkey,_) p -> [< p; 'sTR ("  "^nickkey); 'fNL >])
-       !string_table [<>];
+       (fun (nickkey,_) p -> p ++ str ("  "^nickkey) ++ fnl ())
+       !string_table (mt ()) ++
      List.fold_right
-       (fun (nickkey,_) p -> [< p; 'sTR ("  "^nickkey); 'fNL >])
-       !ident_table [<>];
-     'fNL;
- >]
+       (fun (nickkey,_) p -> p ++ str ("  "^nickkey) ++ fnl ())
+       !ident_table (mt ()) ++
+     fnl ()
+    )
+
 
