@@ -6,14 +6,14 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: goptions.mli,v 1.4 2001/03/15 13:38:27 filliatr Exp $ i*)
+(*i $Id: goptions.mli,v 1.7 2001/11/05 16:47:41 barras Exp $ i*)
 
 (* This module manages customization parameters at the vernacular level     *)
 
 (* Two kinds of things are managed : tables and options value
    - Tables are created by applying the [MakeTable] functor.
    - Variables storing options value are created by applying one of the
-   [declare_sync_int_option], [declare_async_bool_option], ... functions.
+   [declare_int_option], [declare_bool_option], ... functions.
 
    Each table/option is uniquely identified by a key of type [option_name].
    There are two kinds of table/option idenfiers: the primary ones
@@ -32,20 +32,20 @@
    following vernacular commands:
 
        Set Tata Tutu val.
-       Print Tata Tutu.
+       Print Table Tata Tutu.
 
    If it is the declaration of a boolean value, the following
    vernacular commands are made available:
 
        Set Tata Tutu.
        Unset Tata Tutu.
-       Print Tata Tutu.
+       Print Table Tata Tutu. (* synonym: Test Table Tata Tutu. *)
 
    For a primary table, say of name [PrimaryTable("Bidule")], the
    vernacular commands look like
 
       Add Bidule foo.
-      Print Bidule foo.
+      Print Table Bidule foo.
       Set Bidule foo.
       ...
 
@@ -55,6 +55,8 @@
 (*i*)
 open Pp
 open Names
+open Term
+open Nametab
 (*i*)
 
 (*s Things common to tables and options. *)
@@ -93,11 +95,11 @@ sig
   val elements : unit -> string list
 end
 
-(* The functor [MakeIdentTable] declares a new table of [identifier];
+(* The functor [MakeIdentTable] declares a new table of [global_reference];
    for generality, identifiers may be internally encode in other type
-   which is [A.t] through an encoding function [encode : identifier ->
-   A.t] (typically, [A.t] is the persistent [section_path] associated
-   to the currentdenotation of the [identifier] and the encoding function
+   which is [A.t] through an encoding function [encode : global_reference ->
+   A.t] (typically, [A.t] is the persistent [global_reference] associated
+   to the currentdenotation of the [global_reference] and the encoding function
    is the globalization function); the function [check] is for testing
    if a given object is allowed to be added to the table; the function
    [member_message] say what to print when invoking the "Test Toto
@@ -110,12 +112,12 @@ module MakeIdentTable :
   functor
     (A : sig
            type t
-           val encode : identifier -> t
+           val encode : global_reference -> t
            val check : t -> unit
            val printer : t -> std_ppcmds
            val key : option_name
            val title : string
-           val member_message : identifier -> bool -> string
+           val member_message : global_reference -> bool -> string
            val synchronous : bool
          end) ->
     sig
@@ -126,36 +128,26 @@ module MakeIdentTable :
 
 (*s Options. *)
 
-(*s a. Synchronous options. *)
-
 (* These types and function are for declaring a new option of name [key]
-   and default value [default]; the parameter [name] is the option name 
+   and access functions [read] and [write]; the parameter [name] is the option name 
    used when printing the option value (command "Print Toto Titi." *)
 
-type 'a sync_option_sig = {
-  optsyncname    : string;
-  optsynckey     : option_name;
-  optsyncdefault : 'a
+type 'a option_sig = {
+  optsync    : bool; 
+  optname    : string;
+  optkey     : option_name;
+  optread    : unit -> 'a;
+  optwrite   : 'a -> unit
 }
 
-type 'a value_function = unit -> 'a
+(* When an option is declared synchronous ([optsync] is [true]), the output is 
+   a synchronous write function. Otherwise it is [optwrite] *)
 
-val declare_sync_int_option   : int sync_option_sig    -> int value_function
-val declare_sync_bool_option  : bool sync_option_sig   -> bool value_function
-val declare_sync_string_option: string sync_option_sig -> string value_function
+type 'a write_function = 'a -> unit
 
-
-(*s b. Asynchronous options. *)
-
-type 'a async_option_sig = {
-  optasyncname  : string;
-  optasynckey   : option_name;
-  optasyncread  : unit -> 'a;
-  optasyncwrite : 'a -> unit }
-
-val declare_async_int_option    : int async_option_sig    -> unit
-val declare_async_bool_option   : bool async_option_sig   -> unit
-val declare_async_string_option : string async_option_sig -> unit
+val declare_int_option   : int option_sig    -> int write_function
+val declare_bool_option  : bool option_sig   -> bool write_function
+val declare_string_option: string option_sig -> string write_function
 
 
 (*s Special functions supposed to be used only in vernacentries.ml *)
@@ -169,9 +161,9 @@ val get_string_table :
 
 val get_ident_table :
   option_name ->
-    < add : identifier -> unit;
-      remove : identifier -> unit;
-      mem : identifier -> unit;
+    < add : global_reference -> unit;
+      remove : global_reference -> unit;
+      mem : global_reference -> unit;
       print : unit >
 
 val set_int_option_value    : option_name -> int    -> unit
