@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: summary.ml,v 1.4 2001/03/15 13:38:30 filliatr Exp $ *)
+(* $Id: summary.ml,v 1.5 2001/12/13 09:03:05 filliatr Exp $ *)
 
 open Pp
 open Util
@@ -34,7 +34,7 @@ let declare_summary sumname sdecl =
   in
   if Hashtbl.mem summaries sumname then
     anomalylabstrm "Summary.declare_summary"
-      [< 'sTR "Cannot declare a summary twice: " ; 'sTR sumname >];
+      (str "Cannot declare a summary twice: " ++ str sumname);
   Hashtbl.add summaries sumname ddecl
 
 type frozen = Dyn.t Stringmap.t
