@@ -6,16 +6,18 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: astterm.ml,v 1.75 2001/04/25 13:55:31 barras Exp $ *)
+(* $Id: astterm.ml,v 1.91 2001/12/13 09:03:05 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Sign
 open Term
+open Termops
 open Environ
 open Evd
-open Reduction
+open Reductionops
 open Impargs
 open Declare
 open Rawterm
@@ -27,41 +29,33 @@ open Ast
 open Coqast
 open Nametab
 
-type extended_global_reference =
-  | TrueGlobal of global_reference
-  | SyntacticDef of section_path
-
 (*Takes a list of variables which must not be globalized*)
 let from_list l = List.fold_right Idset.add l Idset.empty
 
-let rec adjust_implicits n = function
-  | p::l -> if p<=n then adjust_implicits n l else (p-n)::adjust_implicits n l
-  | [] -> []
-
 (* when an head ident is not a constructor in pattern *)
 let mssg_hd_is_not_constructor s =
-  [< 'sTR ("The symbol "^s^" should be a constructor") >]
+  (str "The symbol " ++ pr_id s ++ str " should be a constructor")
 
 (* checking linearity of a list of ids in patterns *)
 let non_linearl_mssg id =
-  [< 'sTR "The variable " ; 'sTR(string_of_id id);
-     'sTR " is bound several times in pattern" >] 
+  (str "The variable " ++ str(string_of_id id) ++
+     str " is bound several times in pattern") 
 
 let error_capture_loc loc s =
   user_err_loc
     (loc,"ast_to_rawconstr", 
-     [< 'sTR ("The variable "^s^" occurs in its type") >])
+     (str "The variable " ++ pr_id s ++ str " occurs in its type"))
 
 let error_expl_impl_loc loc =
   user_err_loc
     (loc,"ast_to_rawconstr", 
-     [< 'sTR "Found an explicitely given implicit argument but was expecting";
-	'fNL; 'sTR "a regular one" >])
+     (str "Found an explicitely given implicit argument but was expecting" ++
+	fnl () ++ str "a regular one"))
 
 let error_metavar_loc loc =
   user_err_loc
     (loc,"ast_to_rawconstr", 
-     [< 'sTR "Metavariable numbers must be positive" >])
+     (str "Metavariable numbers must be positive"))
 
 let rec has_duplicate = function 
   | [] -> None
@@ -76,17 +70,17 @@ let check_linearity lhs ids =
     | None -> ()
 
 let mal_formed_mssg () =
-  [<'sTR "malformed macro of multiple case" >]
+  (str "malformed macro of multiple case")
 
 (* determines if some pattern variable starts with uppercase *)
 let warning_uppercase loc uplid = (* Comment afficher loc ?? *)
   let vars =
     prlist_with_sep 
-      (fun () -> [< 'sTR ", " >]) (* We avoid 'sPC, else it breaks the line *)
-      (fun v -> [< 'sTR (string_of_id v) >]) uplid in
+      (fun () -> (str ", ")) (* We avoid spc (), else it breaks the line *)
+      (fun v -> (str (string_of_id v))) uplid in
   let (s1,s2) = if List.length uplid = 1 then (" ","s ") else ("s "," ") in
-  wARN [<'sTR ("the variable"^s1); vars;
-	 'sTR (" start"^s2^"with an upper case letter in pattern") >]
+  warn (str ("the variable"^s1) ++ vars ++
+	str (" start"^s2^"with an upper case letter in pattern"))
 
 let is_uppercase_var v =
  match (string_of_id v).[0] with
@@ -102,8 +96,8 @@ let check_uppercase loc ids =
 
 (* check that the number of pattern matches the number of matched args *)
 let mssg_number_of_patterns n pl =
-  [< 'sTR"Expecting ";'iNT n ; 'sTR" pattern(s) but found ";
-    'iNT (List.length pl); 'sTR" in " >]
+  str"Expecting " ++ int n  ++ str" pattern(s) but found " ++
+  int (List.length pl) ++ str" in "
 
 let check_number_of_pattern loc n l =
   if n<>(List.length l) then
@@ -116,34 +110,34 @@ let check_number_of_pattern loc n l =
 (* Translation of references *)
 
 let ast_to_sp = function
-  | Path(loc,sl,s) ->
+  | Path(loc,sp) ->
       (try 
-	 section_path sl s
+	 section_path sp
        with Invalid_argument _ | Failure _ ->
          anomaly_loc(loc,"Astterm.ast_to_sp",
-                     [< 'sTR"malformed section-path" >]))
+                     (str"ill-formed section-path")))
   | ast -> anomaly_loc(Ast.loc ast,"Astterm.ast_to_sp",
-                     [< 'sTR"not a section-path" >])
+                     (str"not a section-path"))
 
-let is_underscore id = (id = "_")
+let is_underscore id = (id = wildcard)
 
 let name_of_nvar s =
-  if is_underscore s then Anonymous else Name (id_of_string s)
+  if is_underscore s then Anonymous else Name s
 
 let ident_of_nvar loc s =
   if is_underscore s then
-    user_err_loc (loc,"ident_of_nvar", [< 'sTR "Unexpected wildcard" >])
-  else (id_of_string s)
+    user_err_loc (loc,"ident_of_nvar", (str "Unexpected wildcard"))
+  else s
 
 let interp_qualid p =
   let outnvar = function
     | Nvar (loc,s) -> s
-    | _ -> anomaly "interp_qualid: bad-formed qualified identifier" in
+    | _ -> anomaly "interp_qualid: ill-formed qualified identifier" in
   match p with
     | [] -> anomaly "interp_qualid: empty qualified identifier"
     | l -> 
 	let p, r = list_chop (List.length l -1) (List.map outnvar l) in
-	make_qualid p (id_of_string (List.hd r))
+	make_qualid (make_dirpath (List.rev p)) (List.hd r)
 
 let maybe_variable = function
   | [Nvar (_,s)] -> Some s
@@ -153,46 +147,51 @@ let ids_of_ctxt ctxt =
   Array.to_list
     (Array.map
        (function c -> match kind_of_term c with
-	  | IsVar id -> id
+	  | Var id -> id
 	  | _ ->
       error
 	"Astterm: arbitrary substitution of references not yet implemented")
     ctxt)
 
 type pattern_qualid_kind =
-  | IsConstrPat of loc * (constructor_path * identifier list)
-  | IsVarPat of loc * string
+  | ConstrPat of loc * constructor
+  | VarPat of loc * identifier
 
 let maybe_constructor env = function
   | Node(loc,"QUALID",l) ->
       let qid = interp_qualid l in
       (try 
 	match kind_of_term (global_qualified_reference qid) with 
-	  | IsMutConstruct ((spi,j),cl) -> 
-	      IsConstrPat (loc,((spi,j),ids_of_ctxt cl))
+	  | Construct c -> ConstrPat (loc,c)
 	  | _ -> 
 	      (match maybe_variable l with
 		 | Some s -> 
 		     warning ("Defined reference "^(string_of_qualid qid)
 			      ^" is here considered as a matching variable");
-		     IsVarPat (loc,s)
+		     VarPat (loc,s)
 		 | _ -> error ("This reference does not denote a constructor: "
 			       ^(string_of_qualid qid)))
       with Not_found -> 
 	match maybe_variable l with
-	  | Some s -> IsVarPat (loc,s)
+	  | Some s -> VarPat (loc,s)
 	  | _ -> error ("Unknown qualified constructor: "
 			^(string_of_qualid qid)))
 
   (* This may happen in quotations *)
   | Node(loc,"MUTCONSTRUCT",[sp;Num(_,ti);Num(_,n)]) ->
       (* Buggy: needs to compute the context *)
-      IsConstrPat (loc,(((ast_to_sp sp,ti),n),[]))
-     
+      ConstrPat (loc,((ast_to_sp sp,ti),n))
+
+  | Path(loc,sp) ->
+      (match absolute_reference sp with 
+	| ConstructRef c -> ConstrPat (loc,c)
+	| _ ->
+            error ("Unknown absolute constructor name: "^(string_of_path sp)))
+
   | Node(loc,("CONST"|"EVAR"|"MUTIND"|"SYNCONST" as key), l) ->
       user_err_loc (loc,"ast_to_pattern",
-   [< 'sTR "Found a pattern involving global references which are not constructors"
-   >])
+   (str "Found a pattern involving global references which are not constructors"
+))
 
   | _ -> anomaly "ast_to_pattern: badly-formed ast for Cases pattern"
 
@@ -200,36 +199,30 @@ let ast_to_global loc c =
   match c with
     | ("CONST", [sp]) ->
 	let ref = ConstRef (ast_to_sp sp) in
-	let hyps = implicit_section_args ref in
-	let section_args = List.map (fun id -> RRef (loc, VarRef id)) hyps in
 	let imps = implicits_of_global ref in
-	RRef (loc, ref), section_args, adjust_implicits (List.length hyps) imps
+	RRef (loc, ref), imps
     | ("MUTIND", [sp;Num(_,tyi)]) -> 
 	let ref = IndRef (ast_to_sp sp, tyi) in
-	let hyps = implicit_section_args ref in
-	let section_args = List.map (fun id -> RRef (loc, VarRef id)) hyps in
 	let imps = implicits_of_global ref in
-	RRef (loc, ref), section_args, adjust_implicits (List.length hyps) imps
+	RRef (loc, ref), imps
     | ("MUTCONSTRUCT", [sp;Num(_,ti);Num(_,n)]) ->
 	let ref = ConstructRef ((ast_to_sp sp,ti),n) in
-	let hyps = implicit_section_args ref in
-	let section_args = List.map (fun id -> RRef (loc, VarRef id)) hyps in
 	let imps = implicits_of_global ref in
-	RRef (loc, ref), section_args, adjust_implicits (List.length hyps) imps
+	RRef (loc, ref), imps
     | ("EVAR", [(Num (_,ev))]) ->
-	REvar (loc, ev), [], []
+	REvar (loc, ev), []
     | ("SYNCONST", [sp]) ->
-	Syntax_def.search_syntactic_definition (ast_to_sp sp), [], []
+	Syntax_def.search_syntactic_definition (ast_to_sp sp), []
     | _ -> anomaly_loc (loc,"ast_to_global",
-			[< 'sTR "Bad ast for this global a reference">])
+			(str "Bad ast for this global a reference"))
 
 (*
 let ref_from_constr c = match kind_of_term c with
-  | IsConst (sp,ctxt) -> RConst (sp, ast_to_constr_ctxt ctxt)
-  | IsEvar (ev,ctxt) -> REVar (ev, ast_to_constr_ctxt ctxt) 
-  | IsMutConstruct (csp,ctxt) -> RConstruct (csp, ast_to_constr_ctxt ctxt)
-  | IsMutInd (isp,ctxt) -> RInd (isp, ast_to_constr_ctxt ctxt)
-  | IsVar id -> RVar id  (* utilis pour coercion_value (tmp) *)
+  | Const (sp,ctxt) -> RConst (sp, ast_to_constr_ctxt ctxt)
+  | Evar (ev,ctxt) -> REVar (ev, ast_to_constr_ctxt ctxt) 
+  | Construct (csp,ctxt) -> RConstruct (csp, ast_to_constr_ctxt ctxt)
+  | Ind (isp,ctxt) -> RInd (isp, ast_to_constr_ctxt ctxt)
+  | Var id -> RVar id  (* utilis pour coercion_value (tmp) *)
   | _ -> anomaly "Not a reference"
 *)
 
@@ -237,88 +230,53 @@ let ref_from_constr c = match kind_of_term c with
    [vars2] is the set of global variables, env is the set of variables
    abstracted until this point *)
 
-let ast_to_var (env,impls) (vars1,vars2) loc s =
-  let id = id_of_string s in
+let ast_to_var (env,impls) (vars1,vars2) loc id =
   let imps =
-    if Idset.mem id env or List.mem s vars1
+    if Idset.mem id env or List.mem id vars1
     then
       try List.assoc id impls
       with Not_found -> []
     else
-      let _ = lookup_id id vars2 in
+      let _ = lookup_named id vars2 in
       (* Car Fixpoint met les fns dfinies tmporairement comme vars de sect *)
       try
-	let ref = Nametab.locate (make_qualid [] (id_of_string s)) in
+	let ref = VarRef id in
 	implicits_of_global ref
       with _ -> []
-  in RVar (loc, id), [], imps
-
-(********************************************************************)
-(* This is generic code to deal with globalization                  *)
-
-type 'a globalization_action = {
-  parse_var : identifier -> 'a;
-  parse_ref : extended_global_reference -> 'a;
-  fail : qualid -> 'a * int list;
-}
-
-let translate_qualid act qid =
-  (* Is it a bound variable? *)
-  try
-    match repr_qualid qid with
-      | [],id -> act.parse_var id, []
-      | _ -> raise Not_found
-  with Not_found ->
-  (* Is it a global reference? *)
-  try
-    let ref = Nametab.locate qid in
-    act.parse_ref (TrueGlobal ref), implicits_of_global ref
-  with Not_found ->
-  (* Is it a reference to a syntactic definition? *)
-  try
-    let sp = Syntax_def.locate_syntactic_definition qid in
-    act.parse_ref (SyntacticDef sp), []
-  with Not_found ->
-    act.fail qid
+  in RVar (loc, id), imps
 
 (**********************************************************************)
 
-let rawconstr_of_var env vars loc s =
+let rawconstr_of_var env vars loc id =
   try
-    ast_to_var env vars loc s
+    ast_to_var env vars loc id
   with Not_found ->
-    Pretype_errors.error_var_not_found_loc loc (id_of_string s)
+    Pretype_errors.error_var_not_found_loc loc id
 
 let rawconstr_of_qualid env vars loc qid =
   (* Is it a bound variable? *)
   try
     match repr_qualid qid with
-      | [],s -> ast_to_var env vars loc (string_of_id s)
+      | d,s when repr_dirpath d = [] -> ast_to_var env vars loc s
       | _ -> raise Not_found
   with Not_found ->
-  (* Is it a global reference? *)
-  try
-    let ref = Nametab.locate qid in
-    let hyps = implicit_section_args ref in
-    let section_args = List.map (fun id -> RRef (loc, VarRef id)) hyps in
+  (* Is it a global reference or a syntactic definition? *)
+  try match Nametab.extended_locate qid with
+  | TrueGlobal ref ->
     let imps = implicits_of_global ref in
-    RRef (loc, ref), section_args, adjust_implicits (List.length hyps) imps
-  with Not_found ->
-  (* Is it a reference to a syntactic definition? *)
-  try
-    let sp = Syntax_def.locate_syntactic_definition qid in
-    set_loc_of_rawconstr loc (Syntax_def.search_syntactic_definition sp),[],[]
+    RRef (loc, ref), imps
+  | SyntacticDef sp ->
+    set_loc_of_rawconstr loc (Syntax_def.search_syntactic_definition sp), []
   with Not_found ->
     error_global_not_found_loc loc qid
 
-let mkLambdaC (x,a,b) = ope("LAMBDA",[a;slam(Some (string_of_id x),b)])
+let mkLambdaC (x,a,b) = ope("LAMBDA",[a;slam(Some x,b)])
 let mkLambdaCit = List.fold_right (fun (x,a) b -> mkLambdaC(x,a,b))
-let mkProdC (x,a,b) = ope("PROD",[a;slam(Some (string_of_id x),b)])
+let mkProdC (x,a,b) = ope("PROD",[a;slam(Some x,b)])
 let mkProdCit = List.fold_right (fun (x,a) b -> mkProdC(x,a,b))
 
 let destruct_binder = function
-  | Node(_,"BINDER",c::idl) ->
-      List.map (fun id -> (id_of_string (nvar_of_ast id),c)) idl
+  | Node(_,"BINDER",c::idl) -> List.map (fun id -> (nvar_of_ast id,c)) idl
   | _ -> anomaly "BINDER is expected"
 
 (* [merge_aliases] returns the sets of all aliases encountered at this
@@ -328,14 +286,15 @@ let merge_aliases (ids,subst as aliases) = function
   | Name id   ->
       ids@[id],
       if ids=[] then subst 
-      else (string_of_id id,string_of_id (List.hd ids))::subst
+      else (id, List.hd ids)::subst
 
 let alias_of = function
   | ([],_) -> Anonymous
   | (id::_,_) -> Name id
 
-let message_redondant_alias (s1,s2) =
-  warning ("Alias variable "^s1^" is merged with "^s2)
+let message_redundant_alias (s1,s2) =
+  warning ("Alias variable "^(string_of_id s1)
+	   ^" is merged with "^(string_of_id s2))
 
 let rec ast_to_pattern env aliases = function
   | Node(_,"PATTAS",[Nvar (loc,s); p]) ->
@@ -344,18 +303,18 @@ let rec ast_to_pattern env aliases = function
 
   | Node(_,"PATTCONSTRUCT", head::((_::_) as pl)) ->
       (match maybe_constructor env head with
-	 | IsConstrPat (loc,c) ->
+	 | ConstrPat (loc,c) ->
 	     let (idsl,pl') =
 	       List.split (List.map (ast_to_pattern env ([],[])) pl) in
 	     (aliases::(List.flatten idsl),
 	      PatCstr (loc,c,pl',alias_of aliases))
-	 | IsVarPat (loc,s) ->
+	 | VarPat (loc,s) ->
 	     user_err_loc (loc,"ast_to_pattern",mssg_hd_is_not_constructor s))
 
   | ast ->
       (match maybe_constructor env ast with
-	 | IsConstrPat (loc,c) -> ([aliases], PatCstr (loc,c,[],alias_of aliases))
-	 | IsVarPat (loc,s) ->
+	 | ConstrPat (loc,c) -> ([aliases], PatCstr (loc,c,[],alias_of aliases))
+	 | VarPat (loc,s) ->
 	     let aliases = merge_aliases aliases (name_of_nvar s) in
 	     ([aliases], PatVar (loc,alias_of aliases)))
 
@@ -363,12 +322,12 @@ let rec ast_to_fix = function
   | [] -> ([],[],[],[])
   | Node(_,"NUMFDECL", [Nvar(_,fi); Num(_,ni); astA; astT])::rest ->
       let (lf,ln,lA,lt) = ast_to_fix rest in
-      ((id_of_string fi)::lf, (ni-1)::ln, astA::lA, astT::lt)
-  | Node(_,"FDECL", [Nvar(_,fi); Node(_,"BINDERS",bl); astA; astT])::rest ->
+      (fi::lf, (ni-1)::ln, astA::lA, astT::lt)
+  | Node(_,"FDECL", [Nvar(_,fi); Node(_,"BINDERS",bl); astA; astT])::rest->
       let binders = List.flatten (List.map destruct_binder bl) in
       let ni = List.length binders - 1 in
       let (lf,ln,lA,lt) = ast_to_fix rest in
-      ((id_of_string fi)::lf, ni::ln, (mkProdCit binders astA)::lA,
+      (fi::lf, ni::ln, (mkProdCit binders astA)::lA,
        (mkLambdaCit binders astT)::lt)
   | _ -> anomaly "FDECL or NUMFDECL is expected"
 
@@ -376,15 +335,15 @@ let rec ast_to_cofix = function
   | [] -> ([],[],[])
   | Node(_,"CFDECL", [Nvar(_,fi); astA; astT])::rest -> 
       let (lf,lA,lt) = ast_to_cofix rest in
-      ((id_of_string fi)::lf, astA::lA, astT::lt)
+      (fi::lf, astA::lA, astT::lt)
   | _ -> anomaly "CFDECL is expected"
 
-let error_fixname_unbound str is_cofix loc name = 
+let error_fixname_unbound s is_cofix loc name = 
   user_err_loc 
     (loc,"ast_to (COFIX)",
-       [< 'sTR "The name"; 'sPC ; 'sTR name ; 
-	  'sPC ; 'sTR "is not bound in the corresponding"; 'sPC ;
-	  'sTR ((if is_cofix then "co" else "")^"fixpoint definition") >])
+     str "The name" ++ spc () ++ pr_id name ++ 
+     spc () ++ str "is not bound in the corresponding" ++ spc () ++
+     str ((if is_cofix then "co" else "")^"fixpoint definition"))
 (*
 let rec collapse_env n env = if n=0 then env else
   add_rel_decl (Anonymous,()) (collapse_env (n-1) (snd (uncons_rel_env env)))
@@ -397,12 +356,10 @@ let check_capture loc s ty = function
 let ast_to_rawconstr sigma env allow_soapp lvar =
   let rec dbrec (ids,impls as env) = function
     | Nvar(loc,s) ->
-	let f, hyps, _ = rawconstr_of_var env lvar loc s in
-	if hyps = [] then f else RApp (loc, f, hyps)
+	fst (rawconstr_of_var env lvar loc s)
 
     | Node(loc,"QUALID", l) ->
-	let f, hyps, _ = rawconstr_of_qualid env lvar loc (interp_qualid l) in
-	if hyps = [] then f else RApp (loc, f, hyps)
+        fst (rawconstr_of_qualid env lvar loc (interp_qualid l))
   
     | Node(loc,"FIX", (Nvar (locid,iddef))::ldecl) ->
 	let (lf,ln,lA,lt) = ast_to_fix ldecl in
@@ -431,7 +388,7 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	  
     | Node(loc,("PROD"|"LAMBDA"|"LETIN" as k), [c1;Slam(_,ona,c2)]) ->
 	let na,ids' = match ona with
-	  | Some s -> let id = id_of_string s in Name id, Idset.add id ids
+	  | Some id -> Name id, Idset.add id ids
 	  | _ -> Anonymous, ids in
 	let c1' = dbrec env c1 and c2' = dbrec (ids',impls) c2 in
 	(match k with
@@ -447,7 +404,7 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	RApp (loc,dbrec env f,ast_to_args env args)
 
     | Node(loc,"APPLIST", f::args) ->
-	let (c, hyps, impargs) =
+	let (c, impargs) =
 	  match f with
 	    | Node(locs,"QUALID",p) ->
 		rawconstr_of_qualid env lvar locs (interp_qualid p)
@@ -455,9 +412,9 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 		   ("CONST"|"EVAR"|"MUTIND"|"MUTCONSTRUCT"|"SYNCONST" as key),
 		   l) ->
 		ast_to_global loc (key,l)
-	    | _ -> (dbrec env f, [], [])
+	    | _ -> (dbrec env f, [])
         in
-	  RApp (loc, c, hyps @ (ast_to_impargs env impargs args))
+	  RApp (loc, c, ast_to_impargs env impargs args)
 
     | Node(loc,"CASES", p:: Node(_,"TOMATCH",tms):: eqns) ->
 	let po = match p with 
@@ -467,13 +424,13 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 		List.map (dbrec env) tms,
 		List.map (ast_to_eqn (List.length tms) env) eqns)
 
-    | Node(loc,"CASE",Str(_,isrectag)::p::c::cl) ->
+    | Node(loc,(("CASE"|"IF"|"LET"|"MATCH")as tag), p::c::cl) ->
 	let po = match p with 
 	  | Str(_,"SYNTH") -> None 
 	  | _ -> Some(dbrec env p) in
-	let isrec = match isrectag with
-	  | "REC" -> true | "NOREC" -> false 
-	  | _ -> anomaly "ast_to: wrong REC tag in CASE" in
+	let isrec = match tag with
+	  | "MATCH" -> true | ("LET"|"CASE"|"IF") -> false 
+	  | _ -> anomaly "ast_to: wrong tag in old case expression" in
 	ROldCase (loc,isrec,po,dbrec env c,
 		  Array.of_list (List.map (dbrec env) cl))
 
@@ -483,12 +440,11 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 
     | Node(loc,"PROP", []) -> RSort(loc,RProp Null)
     | Node(loc,"SET", [])  -> RSort(loc,RProp Pos)
-    | Node(loc,"TYPE", []) -> RSort(loc,RType)
+    | Node(loc,"TYPE", _) -> RSort(loc,RType None)
 	  
     (* This case mainly parses things build in a quotation *)
     | Node(loc,("CONST"|"EVAR"|"MUTIND"|"MUTCONSTRUCT"|"SYNCONST" as key),l) ->
-	let f, hyps, _ = ast_to_global loc (key,l) in
-	if hyps = [] then f else RApp (loc, f, hyps)	
+	fst (ast_to_global loc (key,l))
 
     | Node(loc,"CAST", [c1;c2]) ->	   
 	RCast (loc,dbrec env c1,dbrec env c2)
@@ -502,12 +458,14 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 
     | Node(loc,"SQUASH",_) ->
         user_err_loc(loc,"ast_to_rawconstr",
-                     [< 'sTR "Ill-formed specification" >])
+                     (str "Ill-formed specification"))
 
     | Node(loc,opn,tl) -> 
 	anomaly ("ast_to_rawconstr found operator "^opn^" with "^
 		 (string_of_int (List.length tl))^" arguments")
 
+    | Dynamic (loc,d) -> RDynamic (loc,d)
+
     | _ -> anomaly "ast_to_rawconstr: unexpected ast"
 
   and ast_to_eqn n (ids,impls as env) = function
@@ -522,17 +480,17 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 	check_uppercase loc eqn_ids;
 	check_number_of_pattern loc n pl;
 	let rhs = replace_vars_ast subst rhs in
-	List.iter message_redondant_alias subst;
+	List.iter message_redundant_alias subst;
 	let env_ids = List.fold_right Idset.add eqn_ids ids in
 	(loc, eqn_ids,pl,dbrec (env_ids,impls) rhs)
-    | _ -> anomaly "ast_to_rawconstr: badly-formed ast for Cases equation"
+    | _ -> anomaly "ast_to_rawconstr: ill-formed ast for Cases equation"
 
   and iterated_binder oper n ty (ids,impls as env) = function
     | Slam(loc,ona,body) ->
 	let na,ids' = match ona with 
-	  | Some s ->
-	      if n>0 then check_capture loc s ty body;
-	      let id = id_of_string s in Name id, Idset.add id ids
+	  | Some id ->
+	      if n>0 then check_capture loc id ty body;
+	      Name id, Idset.add id ids
 	  | _ -> Anonymous, ids
 	in
 	let r = iterated_binder oper (n+1) ty (ids',impls) body in
@@ -581,15 +539,7 @@ let ast_to_rawconstr sigma env allow_soapp lvar =
 (* Globalization of AST quotations (mainly used to get statically         *)
 (* bound idents in grammar or pretty-printing rules)                      *)
 (**************************************************************************)
-(*
-(* A brancher ultrieurement sur Termast.ast_of_ref *)
-let ast_of_ref loc = function
-  | ConstRef sp -> Node (loc, "CONST", [path_section loc sp])
-  | ConstructRef ((sp, i), j) ->
-        Node (loc, "MUTCONSTRUCT", [path_section loc sp; num i; num j])
-  | IndRef (sp, i) -> Node (loc, "MUTIND", [path_section loc sp; num i])
-  | VarRef sp -> failwith "ast_of_ref: TODO"
-*)
+
 let ast_of_ref_loc loc ref = set_loc loc (Termast.ast_of_ref ref)
 
 let ast_of_syndef loc sp = Node (loc, "SYNCONST", [path_section loc sp])
@@ -606,35 +556,54 @@ let ast_of_var env ast id =
 
 let ast_hole = Node (dummy_loc, "ISEVAR", [])
 
-let warning_globalize ast qid =
-  warning ("Could not globalize " ^ (string_of_qualid qid)); ast, []
+let implicits_of_extended_reference = function
+  | TrueGlobal ref -> implicits_of_global ref
+  | SyntacticDef _ -> []
+
+let warning_globalize qid =
+  warning ("Could not globalize " ^ (string_of_qualid qid))
+
+let globalize_qualid qid =
+  try
+    let ref = Nametab.extended_locate qid in
+    ast_of_extended_ref_loc dummy_loc ref
+  with Not_found ->
+    Options.if_verbose warning_globalize qid;
+    Termast.ast_of_qualid qid
+
+let adjust_qualid env loc ast qid =
+  (* Is it a bound variable? *)
+  try
+    match repr_qualid qid with
+      | d,id when repr_dirpath d = [] -> ast_of_var env ast id
+      | _ -> raise Not_found
+  with Not_found ->
+  (* Is it a global reference or a syntactic definition? *)
+  try
+    let ref = Nametab.extended_locate qid in
+    ast_of_extended_ref_loc loc ref
+  with Not_found ->
+    Options.if_verbose warning_globalize qid;
+    ast
 
 let ast_adjust_consts sigma =
   let rec dbrec env = function
     | Node(loc, ("APPLIST" as key), (Node(locs,"QUALID",p) as ast)::args) ->
 	let f = adjust_qualid env loc ast (interp_qualid p) in
 	Node(loc, key, f :: List.map (dbrec env) args)
-    | Nvar (loc, s) as ast ->
-        let id = id_of_string s in
-        if isMeta s then ast
-        else if Idset.mem id env then ast
-        else adjust_qualid env loc ast (make_qualid [] (id_of_string s))
+    | Nmeta (loc, s) as ast -> ast
+    | Nvar (loc, id) as ast ->
+        if Idset.mem id env then ast
+        else adjust_qualid env loc ast (make_short_qualid id)
     | Node (loc, "QUALID", p) as ast ->
 	adjust_qualid env loc ast (interp_qualid p)
     | Slam (loc, None, t) -> Slam (loc, None, dbrec env t)
     | Slam (loc, Some na, t) ->
-        let env' = Idset.add (id_of_string na) env in
+        let env' = Idset.add na env in
         Slam (loc, Some na, dbrec env' t)
     | Node (loc, opn, tl) -> Node (loc, opn, List.map (dbrec env) tl)
     | x -> x
 
-  and adjust_qualid env loc ast sp =
-    let act = {
-      parse_var = ast_of_var env ast;
-      parse_ref = ast_of_extended_ref_loc loc;
-      fail = warning_globalize ast } in
-    fst (translate_qualid act sp)
-
   in
   dbrec
 
@@ -653,7 +622,7 @@ let rec glob_ast sigma env =
       Node (loc, "CONSTRLIST", List.map (ast_adjust_consts sigma env) l)
   | Slam (loc, None, t) -> Slam (loc, None, glob_ast sigma env t)
   | Slam (loc, Some na, t) ->
-      let env' = Idset.add (id_of_string na) env in
+      let env' = Idset.add na env in
       Slam (loc, Some na, glob_ast sigma env' t)
   | Node (loc, opn, tl) -> Node (loc, opn, List.map (glob_ast sigma env) tl)
   | x -> x
@@ -669,7 +638,7 @@ let globalize_ast ast =
 let interp_rawconstr_gen sigma env impls allow_soapp lvar com =
   ast_to_rawconstr sigma
     (from_list (ids_of_rel_context (rel_context env)), impls)
-    allow_soapp (lvar,named_context env) com
+    allow_soapp (lvar,env) com
 
 let interp_rawconstr sigma env com =
   interp_rawconstr_gen sigma env [] false [] com
@@ -688,6 +657,18 @@ let interp_rawconstr_wo_glob sigma env lvar com =
 
 (* Functions to parse and interpret constructions *)
 
+(* To embed constr in Coqast.t *)
+let constrIn t = Dynamic (dummy_loc,constr_in t)
+let constrOut = function
+  | Dynamic (_,d) ->
+    if (Dyn.tag d) = "constr" then
+      constr_out d
+    else
+      anomalylabstrm "constrOut" (str "Dynamic tag should be constr")
+  | ast ->
+    anomalylabstrm "constrOut"
+      (str "Not a Dynamic ast: " ++ print_ast ast)
+
 let interp_constr sigma env c =
   understand sigma env (interp_rawconstr sigma env c)
 
@@ -706,8 +687,14 @@ let interp_type_with_implicits sigma env impls c =
 let interp_sort = function
   | Node(loc,"PROP", []) -> Prop Null
   | Node(loc,"SET", [])  -> Prop Pos
-  | Node(loc,"TYPE", []) -> Type Univ.dummy_univ
-  | a -> user_err_loc (Ast.loc a,"interp_sort", [< 'sTR "Not a sort" >])
+  | Node(loc,"TYPE", _) -> new_Type_sort ()
+  | a -> user_err_loc (Ast.loc a,"interp_sort", (str "Not a sort"))
+
+let interp_elimination_sort = function
+  | Node(loc,"PROP", []) -> InProp
+  | Node(loc,"SET", [])  -> InSet
+  | Node(loc,"TYPE", _) -> InType
+  | a -> user_err_loc (Ast.loc a,"interp_sort", (str "Not a sort"))
 
 let judgment_of_rawconstr sigma env c =
   understand_judgment sigma env (interp_rawconstr sigma env c)
@@ -716,27 +703,25 @@ let type_judgment_of_rawconstr sigma env c =
   understand_type_judgment sigma env (interp_rawconstr sigma env c)
 
 (*To retype a list of key*constr with undefined key*)
-let retype_list sigma env lst=
-  List.map (fun (x,csr) -> (x,Retyping.get_judgment_of env sigma csr)) lst
+let retype_list sigma env lst =
+  List.fold_right (fun (x,csr) a ->
+    try (x,Retyping.get_judgment_of env sigma csr)::a with
+    | Anomaly _ -> a) lst []
+
+(*  List.map (fun (x,csr) -> (x,Retyping.get_judgment_of env sigma csr)) lst*)
 
 (* Interprets a constr according to two lists *)
 (* of instantiations (variables and metas)    *)
 (* Note: typ is retyped *)
 let interp_constr_gen sigma env lvar lmeta com exptyp =
-  let c =
-    interp_rawconstr_gen sigma env [] false
-      (List.map (fun x -> string_of_id (fst x)) lvar)
-      com
+  let c = interp_rawconstr_gen sigma env [] false (List.map fst lvar) com
   and rtype lst = retype_list sigma env lst in
   understand_gen sigma env (rtype lvar) (rtype lmeta) exptyp c;;
 
 (*Interprets a casted constr according to two lists of instantiations
   (variables and metas)*)
 let interp_openconstr_gen sigma env lvar lmeta com exptyp =
-  let c =
-    interp_rawconstr_gen sigma env [] false
-      (List.map (fun x -> string_of_id (fst x)) lvar)
-      com
+  let c = interp_rawconstr_gen sigma env [] false (List.map fst lvar) com
   and rtype lst = retype_list sigma env lst in
   understand_gen_tcc sigma env (rtype lvar) (rtype lmeta) exptyp c;;
 
@@ -803,9 +788,7 @@ let interp_constrpattern_gen sigma env lvar com =
   let c =
     ast_to_rawconstr sigma
       (from_list (ids_of_rel_context (rel_context env)), [])
-      true (List.map
-	      (fun x ->
-		 string_of_id (fst x)) lvar,named_context env) com
+      true (List.map fst lvar,env) com
   and nlvar = List.map (fun (id,c) -> (id,pattern_of_constr c)) lvar in
   try 
     pattern_of_rawconstr nlvar c
