@@ -6,7 +6,11 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: coqast.mli,v 1.4 2001/03/15 13:38:31 filliatr Exp $ i*)
+(*i $Id: coqast.mli,v 1.5 2001/08/10 14:42:19 herbelin Exp $ i*)
+
+(*i*)
+open Names
+(*i*)
 
 (* Abstract syntax trees. *)
 
@@ -14,12 +18,14 @@ type loc = int * int
 
 type t =
   | Node of loc * string * t list
-  | Nvar of loc * string
-  | Slam of loc * string option * t
+  | Nmeta of loc * string
+  | Nvar of loc * identifier
+  | Slam of loc * identifier option * t
+  | Smetalam of loc * string * t
   | Num of loc * int
-  | Id of loc * string
   | Str of loc * string
-  | Path of loc * string list* string
+  | Id of loc * string
+  | Path of loc * section_path
   | Dynamic of loc * Dyn.t
 
 (* returns the list of metas occuring in the ast *)
@@ -30,5 +36,7 @@ val collect_metas : t -> int list
 val subst_meta : (int * t) list -> t -> t
 
 (* hash-consing function *)
-val hcons_ast: (string -> string) -> (t -> t) * (loc -> loc)
-
+val hcons_ast: 
+  (string -> string) * (Names.identifier -> Names.identifier)
+  * (section_path -> section_path)
+  -> (t -> t) * (loc -> loc)
