@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: pcoq.mli,v 1.24 2001/04/14 03:16:36 delahaye Exp $ i*)
+(*i $Id: pcoq.mli,v 1.29 2001/09/20 18:10:47 barras Exp $ i*)
 
 (* The lexer and parser of Coq. *)
 
@@ -34,7 +34,7 @@ val map_entry : ('a -> 'b) -> 'a Gram.Entry.e -> 'b Gram.Entry.e
 
 val slam_ast : Coqast.loc -> Coqast.t -> Coqast.t -> Coqast.t
 val abstract_binders_ast :
-  Coqast.loc -> string -> Coqast.t list -> Coqast.t -> Coqast.t
+  Coqast.loc -> string -> Coqast.t -> Coqast.t -> Coqast.t
 
 (* Entry types *)
 
@@ -114,10 +114,13 @@ module Tactic :
     val constrarg_binding_list : Coqast.t list Gram.Entry.e
     val constrarg_list : Coqast.t list Gram.Entry.e
     val fixdecl : Coqast.t list Gram.Entry.e
+    val ident_or_numarg : Coqast.t Gram.Entry.e
     val ident_or_constrarg : Coqast.t Gram.Entry.e
     val identarg : Coqast.t Gram.Entry.e
+    val hypident : Coqast.t Gram.Entry.e
+    val idmeta_arg : Coqast.t Gram.Entry.e
+    val idmetahyp : Coqast.t Gram.Entry.e
     val qualidarg : Coqast.t Gram.Entry.e
-    val qualidconstarg : Coqast.t Gram.Entry.e
     val input_fun : Coqast.t Gram.Entry.e
     val intropattern : Coqast.t Gram.Entry.e
     val lconstrarg : Coqast.t Gram.Entry.e
@@ -131,8 +134,9 @@ module Tactic :
     val match_rule : Coqast.t Gram.Entry.e
     val match_list : Coqast.t list Gram.Entry.e
     val ne_identarg_list : Coqast.t list Gram.Entry.e
+    val ne_hyp_list : Coqast.t list Gram.Entry.e
+    val ne_idmetahyp_list : Coqast.t list Gram.Entry.e
     val ne_qualidarg_list : Coqast.t list Gram.Entry.e
-    val ne_qualidconstarg_list : Coqast.t list Gram.Entry.e
     val ne_intropattern : Coqast.t Gram.Entry.e
     val ne_pattern_list : Coqast.t list Gram.Entry.e
     val clause_pattern : Coqast.t Gram.Entry.e
@@ -166,7 +170,6 @@ module Vernac_ :
     val identarg : Coqast.t Gram.Entry.e
     val ne_identarg_list : Coqast.t list Gram.Entry.e
     val qualidarg : Coqast.t Gram.Entry.e
-    val qualidconstarg : Coqast.t Gram.Entry.e
     val commentarg : Coqast.t Gram.Entry.e
     val commentarg_list : Coqast.t list Gram.Entry.e
     val ne_qualidarg_list : Coqast.t list Gram.Entry.e
