@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: cbv.ml,v 1.2 2001/03/15 13:38:34 filliatr Exp $ *)
+(* $Id: cbv.ml,v 1.9 2001/11/29 09:21:16 barras Exp $ *)
 
 open Util
 open Pp
@@ -34,8 +34,7 @@ open Esubst
  *  FIXP(op,bd,S,args) is the fixpoint (Fix or Cofix) of bodies bd under
  *          the substitution S, and then applied to args. Here again,
  *          weak reduction.
- *  CONSTR(n,(sp,i),vars,args) is the n-th constructor of the i-th
- *          inductive type sp.
+ *  CONSTR(c,args) is the constructor [c] applied to [args].
  *
  * Note that any term has not an equivalent in cbv_value: for example,
  * a product (x:A)B must be in normal form because only VAL may
@@ -49,7 +48,7 @@ type cbv_value =
   | LAM of name * constr * constr * cbv_value subs
   | FIXP of fixpoint * cbv_value subs * cbv_value list
   | COFIXP of cofixpoint * cbv_value subs * cbv_value list
-  | CONSTR of int * inductive_path * cbv_value array * cbv_value list
+  | CONSTR of constructor * cbv_value list
 
 (* les vars pourraient etre des constr,
    cela permet de retarder les lift: utile ?? *) 
@@ -64,9 +63,8 @@ let rec shift_value n = function
       FIXP (fix,subs_shft (n,s), List.map (shift_value n) args)
   | COFIXP (cofix,s,args) ->
       COFIXP (cofix,subs_shft (n,s), List.map (shift_value n) args)
-  | CONSTR (i,spi,vars,args) ->
-      CONSTR (i, spi, Array.map (shift_value n) vars,
-              List.map (shift_value n) args)
+  | CONSTR (c,args) ->
+      CONSTR (c, List.map (shift_value n) args)
 	
 
 (* Contracts a fixpoint: given a fixpoint and a substitution,
@@ -93,6 +91,11 @@ let contract_cofixp env (i,(_,_,bds as bodies)) =
   in       
   subst_bodies_from_i 0 env, bds.(i)
 
+let make_constr_ref n = function
+  | FarRelKey p -> mkRel (n+p)
+  | VarKey id -> mkVar id
+  | ConstKey cst -> mkConst cst
+
 
 (* type of terms with a hole. This hole can appear only under App or Case.
  *   TOP means the term is considered without context
@@ -121,26 +124,13 @@ let stack_app appl stack =
     | (_, APP(args,stk)) -> APP(appl@args,stk)
     | _                  -> APP(appl, stack)
 
-(* Tests if we are in a case (modulo some applications) *)
-let under_case_stack = function
-  | (CASE _ | APP(_,CASE _)) -> true
-  | _ -> false
 
-(* Tells if the reduction rk is allowed by flags under a given stack.
- * The stack is useful when flags is (SIMPL,r) because in that case,
- * we perform bdi-reduction under the Case, or r-reduction otherwise
- *)
-let red_allowed flags stack rk =
-  if under_case_stack stack then 
-    red_under flags rk
-  else 
-    red_top flags rk
+open RedFlags
 
-let red_allowed_ref flags stack = function
-  | FarRelBinding _ -> red_allowed flags stack DELTA
-  | VarBinding id -> red_allowed flags stack (VAR id)
-  | EvarBinding _ -> red_allowed flags stack EVAR
-  | ConstBinding (sp,_) -> red_allowed flags stack (CONST [sp])
+let red_set_ref flags = function
+  | FarRelKey _ -> red_set flags fDELTA
+  | VarKey id -> red_set flags (fVAR id)
+  | ConstKey sp -> red_set flags (fCONST sp)
 
 (* Transfer application lists from a value to the stack
  * useful because fixpoints may be totally applied in several times
@@ -149,49 +139,33 @@ let strip_appl head stack =
   match head with
     | FIXP (fix,env,app) -> (FIXP(fix,env,[]), stack_app app stack)
     | COFIXP (cofix,env,app) -> (COFIXP(cofix,env,[]), stack_app app stack)
-    | CONSTR (i,spi,vars,app) -> (CONSTR(i,spi,vars,[]), stack_app app stack)
+    | CONSTR (c,app) -> (CONSTR(c,[]), stack_app app stack)
     | _ -> (head, stack)
 
 
-(* Invariant: if the result of norm_head is CONSTR or (CO)FIXP, its last
- * argument is [].
- * Because we must put all the applied terms in the stack.
- *)
-let reduce_const_body redfun v stk =
-  if under_case_stack stk then strip_appl (redfun v) stk else strip_appl v stk
- 
-
 (* Tests if fixpoint reduction is possible. A reduction function is given as
    argument *)
-let rec check_app_constr redfun = function
+let rec check_app_constr = function
   | ([], _) -> false
   | ((CONSTR _)::_, 0) -> true
-  | (t::_, 0) -> (* TODO: partager ce calcul *)
-      (match redfun t with
-         | CONSTR _ -> true
-         | _ -> false)
-  | (_::l, n) -> check_app_constr redfun (l,(pred n))
+  | (_::l, n) -> check_app_constr (l,(pred n))
 	
-let fixp_reducible redfun flgs ((reci,i),_) stk =
-  if red_allowed flgs stk IOTA then
+let fixp_reducible flgs ((reci,i),_) stk =
+  if red_set flgs fIOTA then
     match stk with               (* !!! for Acc_rec: reci.(i) = -2 *)
-      | APP(appl,_) -> reci.(i) >=0 & check_app_constr redfun (appl, reci.(i))
+      | APP(appl,_) -> reci.(i) >=0 & check_app_constr (appl, reci.(i))
       | _ -> false
   else 
     false
 
-let cofixp_reducible redfun flgs _ stk =
-  if red_allowed flgs stk IOTA then
+let cofixp_reducible flgs _ stk =
+  if red_set flgs fIOTA then
     match stk with
       | (CASE _ | APP(_,CASE _)) -> true
       | _ -> false
   else 
     false
 
-let mindsp_nparams env (sp,i) =
-  let mib = lookup_mind sp env in
-  (Declarations.mind_nth_type_packet mib i).Declarations.mind_nparams
-
 (* The main recursive functions
  *
  * Go under applications and cases (pushed in the stack), expand head
@@ -207,41 +181,33 @@ let rec norm_head info env t stack =
   (* no reduction under binders *)
   match kind_of_term t with
   (* stack grows (remove casts) *)
-  | IsApp (head,args) -> (* Applied terms are normalized immediately;
+  | App (head,args) -> (* Applied terms are normalized immediately;
                         they could be computed when getting out of the stack *)
       let nargs = Array.map (cbv_stack_term info TOP env) args in
       norm_head info env head (stack_app (Array.to_list nargs) stack)
-  | IsMutCase (ci,p,c,v) -> norm_head info env c (CASE(p,v,ci,env,stack))
-  | IsCast (ct,_) -> norm_head info env ct stack
+  | Case (ci,p,c,v) -> norm_head info env c (CASE(p,v,ci,env,stack))
+  | Cast (ct,_) -> norm_head info env ct stack
 
   (* constants, axioms
    * the first pattern is CRUCIAL, n=0 happens very often:
    * when reducing closed terms, n is always 0 *)
-  | IsRel i -> (match expand_rel i env with
-                | Inl (0,v) ->
-                    reduce_const_body (cbv_norm_more info env) v stack
-                | Inl (n,v) ->
-                    reduce_const_body
-                      (cbv_norm_more info env) (shift_value n v) stack
-                | Inr (n,None) ->
-		    (VAL(0, mkRel n), stack)
-                | Inr (n,Some p) ->
-		    norm_head_ref (n-p) info env stack (FarRelBinding p))
+  | Rel i ->
+      (match expand_rel i env with
+        | Inl (0,v)      -> strip_appl v stack
+        | Inl (n,v)      -> strip_appl (shift_value n v) stack
+        | Inr (n,None)   -> (VAL(0, mkRel n), stack)
+        | Inr (n,Some p) -> norm_head_ref (n-p) info env stack (FarRelKey p))
 
-  | IsVar id -> norm_head_ref 0 info env stack (VarBinding id)
+  | Var id -> norm_head_ref 0 info env stack (VarKey id)
 
-  | IsConst (sp,vars) ->
-      let normt = (sp,Array.map (cbv_norm_term info env) vars) in
-      norm_head_ref 0 info env stack (ConstBinding normt)
+  | Const sp -> norm_head_ref 0 info env stack (ConstKey sp)
 
-  | IsEvar ev -> norm_head_ref 0 info env stack (EvarBinding (ev,env))
-
-  | IsLetIn (x, b, t, c) ->
+  | LetIn (x, b, t, c) ->
       (* zeta means letin are contracted; delta without zeta means we *)
       (* allow substitution but leave let's in place *)
-      let zeta = red_allowed (info_flags info) stack ZETA in
+      let zeta = red_set (info_flags info) fZETA in
       let env' =
-	if zeta or red_allowed (info_flags info) stack DELTA then 
+	if zeta or red_set (info_flags info) fDELTA then 
 	  subs_cons (cbv_stack_term info TOP env b,env)
 	else
 	  subs_lift env in
@@ -255,35 +221,24 @@ let rec norm_head info env t stack =
 	(VAL(0,normt), stack) (* Considrer une coupure commutative ? *)
 
   (* non-neutral cases *)
-  | IsLambda (x,a,b) -> (LAM(x,a,b,env), stack)
-  | IsFix fix -> (FIXP(fix,env,[]), stack)
-  | IsCoFix cofix -> (COFIXP(cofix,env,[]), stack)
-  | IsMutConstruct ((spi,i),vars) ->
-      (CONSTR(i,spi, Array.map (cbv_stack_term info TOP env) vars,[]), stack)
+  | Lambda (x,a,b) -> (LAM(x,a,b,env), stack)
+  | Fix fix -> (FIXP(fix,env,[]), stack)
+  | CoFix cofix -> (COFIXP(cofix,env,[]), stack)
+  | Construct c -> (CONSTR(c, []), stack)
 
   (* neutral cases *)
-  | (IsSort _ | IsMeta _) -> (VAL(0, t), stack)
-  | IsMutInd (sp,vars) -> 
-      (VAL(0, mkMutInd (sp, Array.map (cbv_norm_term info env) vars)), stack)
-  | IsProd (x,t,c) -> 
+  | (Sort _ | Meta _ | Ind _|Evar _) -> (VAL(0, t), stack)
+  | Prod (x,t,c) -> 
       (VAL(0, mkProd (x, cbv_norm_term info env t,
 		      cbv_norm_term info (subs_lift env) c)),
 	     stack)
 
 and norm_head_ref k info env stack normt =
-  if red_allowed_ref (info_flags info) stack normt then
+  if red_set_ref (info_flags info) normt then
     match ref_value_cache info normt with
-      | Some body ->
-	  reduce_const_body (cbv_norm_more info env) (shift_value k body) stack
-      | None -> (VAL(0,make_constr_ref k info normt), stack)
-  else (VAL(0,make_constr_ref k info normt), stack)
-
-and make_constr_ref n info = function
-  | FarRelBinding p -> mkRel (n+p)
-  | VarBinding id -> mkVar id
-  | EvarBinding ((ev,args),env) ->
-      mkEvar (ev,Array.map (cbv_norm_term info env) args)
-  | ConstBinding cst -> mkConst cst
+      | Some body -> strip_appl (shift_value k body) stack
+      | None -> (VAL(0,make_constr_ref k normt), stack)
+  else (VAL(0,make_constr_ref k normt), stack)
 
 (* cbv_stack_term performs weak reduction on constr t under the subs
  * env, with context stack, i.e. ([env]t stack).  First computes weak
@@ -295,53 +250,43 @@ and cbv_stack_term info stack env t =
   match norm_head info env t stack with
     (* a lambda meets an application -> BETA *)
     | (LAM (x,a,b,env), APP (arg::args, stk))
-      when red_allowed (info_flags info) stk BETA ->
+      when red_set (info_flags info) fBETA ->
         let subs = subs_cons (arg,env) in
           cbv_stack_term info (stack_app args stk) subs b
 
     (* a Fix applied enough -> IOTA *)
     | (FIXP(fix,env,_), stk)
-        when fixp_reducible (cbv_norm_more info env) (info_flags info) fix stk ->
+        when fixp_reducible (info_flags info) fix stk ->
         let (envf,redfix) = contract_fixp env fix in
         cbv_stack_term info stk envf redfix
 
     (* constructor guard satisfied or Cofix in a Case -> IOTA *)
     | (COFIXP(cofix,env,_), stk)
-        when cofixp_reducible (cbv_norm_more info env) (info_flags info) cofix stk->
+        when cofixp_reducible (info_flags info) cofix stk->
         let (envf,redfix) = contract_cofixp env cofix in
         cbv_stack_term info stk envf redfix
 
-    (* constructor in a Case -> IOTA
-       (use red_under because we know there is a Case) *)
-    | (CONSTR(n,sp,_,_), APP(args,CASE(_,br,(arity,_),env,stk)))
-            when red_under (info_flags info) IOTA ->
-	let ncargs = arity.(n-1) in
-	let real_args = list_lastn ncargs args in
+    (* constructor in a Case -> IOTA *)
+    | (CONSTR((sp,n),_), APP(args,CASE(_,br,ci,env,stk)))
+            when red_set (info_flags info) fIOTA ->
+	let real_args = snd (list_chop ci.ci_npar args) in
         cbv_stack_term info (stack_app real_args stk) env br.(n-1)
          
-    (* constructor of arity 0 in a Case -> IOTA ( "   " )*)
-    | (CONSTR(n,_,_,_), CASE(_,br,_,env,stk))
-                  when red_under (info_flags info) IOTA ->
+    (* constructor of arity 0 in a Case -> IOTA *)
+    | (CONSTR((_,n),_), CASE(_,br,_,env,stk))
+            when red_set (info_flags info) fIOTA ->
                     cbv_stack_term info stk env br.(n-1)
 
     (* may be reduced later by application *)  
     | (head, TOP) -> head
     | (FIXP(fix,env,_), APP(appl,TOP)) -> FIXP(fix,env,appl) 
     | (COFIXP(cofix,env,_), APP(appl,TOP)) -> COFIXP(cofix,env,appl) 
-    | (CONSTR(n,spi,vars,_), APP(appl,TOP)) -> CONSTR(n,spi,vars,appl)
+    | (CONSTR(c,_), APP(appl,TOP)) -> CONSTR(c,appl)
 
     (* definitely a value *)
     | (head,stk) -> VAL(0,apply_stack info (cbv_norm_value info head) stk)
 
 
-(* if we are in SIMPL mode, maybe v isn't reduced enough *)
-and cbv_norm_more info env v =
-  match (v, (info_flags info)) with
-    | (VAL(k,t), ((SIMPL|WITHBACK),_)) ->
-        cbv_stack_term (infos_under info) TOP (subs_shft (k,env)) t
-    | _ -> v
-
-
 (* When we are sure t will never produce a redex with its stack, we
  * normalize (even under binders) the applied terms and we build the
  * final term
@@ -352,7 +297,7 @@ and apply_stack info t = function
       apply_stack info (applistc t (List.map (cbv_norm_value info) args)) st
   | CASE (ty,br,ci,env,st) ->
       apply_stack info
-        (mkMutCase (ci, cbv_norm_term info env ty, t,
+        (mkCase (ci, cbv_norm_term info env ty, t,
 		    Array.map (cbv_norm_term info env) br))
         st
 
@@ -368,23 +313,24 @@ and cbv_norm_value info = function (* reduction under binders *)
   | LAM (x,a,b,env) ->
       mkLambda (x, cbv_norm_term info env a,
 		cbv_norm_term info (subs_lift env) b)
-  | FIXP ((lij,(lty,lna,bds)),env,args) ->
+  | FIXP ((lij,(names,lty,bds)),env,args) ->
       applistc
         (mkFix (lij,
-		(Array.map (cbv_norm_term info env) lty, lna, 
+		(names,
+                 Array.map (cbv_norm_term info env) lty,
 		 Array.map (cbv_norm_term info 
 			      (subs_liftn (Array.length lty) env)) bds)))
         (List.map (cbv_norm_value info) args)
-  | COFIXP ((j,(lty,lna,bds)),env,args) ->
+  | COFIXP ((j,(names,lty,bds)),env,args) ->
       applistc
         (mkCoFix (j,
-		  (Array.map (cbv_norm_term info env) lty, lna, 
+		  (names,Array.map (cbv_norm_term info env) lty,
 		   Array.map (cbv_norm_term info 
 				(subs_liftn (Array.length lty) env)) bds)))
         (List.map (cbv_norm_value info) args)
-  | CONSTR (n,spi,vars,args) ->
+  | CONSTR (c,args) ->
       applistc
-        (mkMutConstruct ((spi,n), Array.map (cbv_norm_value info) vars))
+        (mkConstruct c)
         (List.map (cbv_norm_value info) args)
 
 (* with profiling *)
@@ -392,12 +338,11 @@ let cbv_norm infos constr =
   with_stats (lazy (cbv_norm_term infos (ESID 0) constr))
 
 
-type 'a cbv_infos = (cbv_value, 'a) infos
+type cbv_infos = cbv_value infos
 
 (* constant bodies are normalized at the first expansion *)
-let create_cbv_infos flgs env sigma =
+let create_cbv_infos flgs env =
   create
-    (fun old_info s c -> cbv_stack_term old_info TOP s c)
+    (fun old_info c -> cbv_stack_term old_info TOP (ESID 0) c)
     flgs
     env
-    sigma
