@@ -6,13 +6,11 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: cbv.mli,v 1.2 2001/03/15 13:38:34 filliatr Exp $ i*)
+(*i $Id: cbv.mli,v 1.6 2001/11/29 09:21:16 barras Exp $ i*)
 
 (*i*)
-open Pp
 open Names
 open Term
-open Evd
 open Environ
 open Closure
 open Esubst
@@ -22,9 +20,10 @@ open Esubst
 (*s Call-by-value reduction *)
 
 (* Entry point for cbv normalization of a constr *)
-type 'a cbv_infos
-val create_cbv_infos : flags -> env -> 'a evar_map -> 'a cbv_infos
-val cbv_norm : 'a cbv_infos -> constr -> constr
+type cbv_infos
+
+val create_cbv_infos : RedFlags.reds -> env -> cbv_infos
+val cbv_norm         : cbv_infos -> constr -> constr
 
 (***********************************************************************)
 (*i This is for cbv debug *)
@@ -33,7 +32,7 @@ type cbv_value =
   | LAM of name * constr * constr * cbv_value subs
   | FIXP of fixpoint * cbv_value subs * cbv_value list
   | COFIXP of cofixpoint * cbv_value subs * cbv_value list
-  | CONSTR of int * (section_path * int) * cbv_value array * cbv_value list
+  | CONSTR of constructor * cbv_value list
 
 val shift_value : int -> cbv_value -> cbv_value
 
@@ -43,20 +42,14 @@ type cbv_stack =
   | CASE of constr * constr array * case_info * cbv_value subs * cbv_stack
 
 val stack_app : cbv_value list -> cbv_stack -> cbv_stack
-val under_case_stack : cbv_stack -> bool
 val strip_appl : cbv_value -> cbv_stack -> cbv_value * cbv_stack
 
-val red_allowed : flags -> cbv_stack -> red_kind -> bool
-val reduce_const_body :
-  (cbv_value -> cbv_value) -> cbv_value -> cbv_stack -> cbv_value * cbv_stack
-
 (* recursive functions... *)
-val cbv_stack_term : 'a cbv_infos ->
+val cbv_stack_term : cbv_infos ->
   cbv_stack -> cbv_value subs -> constr -> cbv_value
-val cbv_norm_term : 'a cbv_infos -> cbv_value subs -> constr -> constr
-val cbv_norm_more : 'a cbv_infos -> cbv_value subs -> cbv_value -> cbv_value
-val norm_head : 'a cbv_infos ->
+val cbv_norm_term : cbv_infos -> cbv_value subs -> constr -> constr
+val norm_head : cbv_infos ->
   cbv_value subs -> constr -> cbv_stack -> cbv_value * cbv_stack
-val apply_stack : 'a cbv_infos -> constr -> cbv_stack -> constr
-val cbv_norm_value : 'a cbv_infos -> cbv_value -> constr
+val apply_stack : cbv_infos -> constr -> cbv_stack -> constr
+val cbv_norm_value : cbv_infos -> cbv_value -> constr
 (* End of cbv debug section i*)
