@@ -6,18 +6,20 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: evarutil.ml,v 1.43 2001/03/28 15:11:17 barras Exp $ *)
+(* $Id: evarutil.ml,v 1.55 2001/12/13 09:03:06 filliatr Exp $ *)
 
 open Util
 open Pp
 open Names
+open Nameops
 open Univ
 open Term
+open Termops
 open Sign
 open Environ
 open Evd
 open Instantiate
-open Reduction
+open Reductionops
 open Indrec
 open Pretype_errors
 
@@ -47,6 +49,44 @@ let filter_sign p sign x =
     (x,[],nil_sign)
 *)
 
+(* Expanding existential variables (pretyping.ml) *)
+(* 1- whd_ise fails if an existential is undefined *)
+
+exception Uninstantiated_evar of int
+
+let rec whd_ise sigma c =
+  match kind_of_term c with
+    | Evar (ev,args) when Evd.in_dom sigma ev ->
+	if Evd.is_defined sigma ev then
+          whd_ise sigma (existential_value sigma (ev,args))
+	else raise (Uninstantiated_evar ev)
+  | _ -> c
+
+
+(* Expand evars, possibly in the head of an application *)
+let whd_castappevar_stack sigma c = 
+  let rec whrec (c, l as s) =
+    match kind_of_term c with
+      | Evar (ev,args) when Evd.in_dom sigma ev & Evd.is_defined sigma ev -> 
+	  whrec (existential_value sigma (ev,args), l)
+      | Cast (c,_) -> whrec (c, l)
+      | App (f,args) -> whrec (f, Array.fold_right (fun a l -> a::l) args l)
+      | _ -> s
+  in 
+  whrec (c, [])
+
+let whd_castappevar sigma c = applist (whd_castappevar_stack sigma c)
+
+let nf_evar = Pretype_errors.nf_evar
+let j_nf_evar = Pretype_errors.j_nf_evar
+let jl_nf_evar = Pretype_errors.jl_nf_evar
+let jv_nf_evar = Pretype_errors.jv_nf_evar
+let tj_nf_evar = Pretype_errors.tj_nf_evar
+
+(**********************)
+(* Creating new evars *)
+(**********************)
+
 let evar_env evd = Global.env_of_context evd.evar_hyps
 
 (* Generator of existential names *)
@@ -57,7 +97,7 @@ let new_evar =
 let make_evar_instance env =
   fold_named_context
     (fun env (id, b, _) l -> (*if b=None then*) mkVar id :: l (*else l*))
-    env []
+    env ~init:[]
 
 (* create an untyped existential variable *)
 let new_evar_in_sign env =
@@ -75,37 +115,70 @@ let new_isevar_sign env sigma typ instance =
     error "new_isevar_sign: two vars have the same name";
   let newev = new_evar() in
   let info = { evar_concl = typ; evar_hyps = sign; 
-	       evar_body = Evar_empty; evar_info = None } in
+	       evar_body = Evar_empty } in
   (Evd.add sigma newev info, mkEvar (newev,Array.of_list instance))
 
 (* We don't try to guess in which sort the type should be defined, since
    any type has type Type. May cause some trouble, but not so far... *)
-let dummy_sort = mkType dummy_univ
+let new_Type () = mkType (new_univ ())
+
+let new_Type_sort () = Type (new_univ ())
+
+let judge_of_new_Type () = Typeops.judge_of_type (new_univ ())
+(*
+let new_Type () = mkType dummy_univ
+
+let new_Type_sort () = Type dummy_univ
+
+let judge_of_new_Type () = 
+  { uj_val = mkSort (Type dummy_univ);
+    uj_type = mkSort (Type dummy_univ) }
+*)
+
+(* This refreshes universes in types; works only for inferred types (i.e. for
+   types of the form (x1:A1)...(xn:An)B with B a sort or an atom in
+   head normal form) *)
+let refresh_universes t =
+  let modified = ref false in
+  let rec refresh t = match kind_of_term t with
+    | Sort (Type _) -> modified := true; new_Type ()
+    | Prod (na,u,v) -> mkProd (na,u,refresh v)
+    | _ -> t in
+  let t' = refresh t in
+  if !modified then t' else t
 
 (* Declaring any type to be in the sort Type shouldn't be harmful since
    cumulativity now includes Prop and Set in Type. *)
 let new_type_var env sigma =
   let instance = make_evar_instance env in
-  let (sigma',c) = new_isevar_sign env sigma dummy_sort instance in
+  let (sigma',c) = new_isevar_sign env sigma (new_Type ()) instance in
   (sigma', c)
 
-let split_evar_to_arrow sigma c =
-  let (ev,args) = destEvar c in
+let split_evar_to_arrow sigma (ev,args) =
   let evd = Evd.map sigma ev in
   let evenv = evar_env evd in
   let (sigma1,dom) = new_type_var evenv sigma in
   let hyps = evd.evar_hyps in
   let nvar = next_ident_away (id_of_string "x") (ids_of_named_context hyps) in
-  let newenv = push_named_assum (nvar, dom) evenv in
+  let newenv = push_named_decl (nvar, None, dom) evenv in
   let (sigma2,rng) = new_type_var newenv sigma1 in
-  let prod = mkProd (named_hd newenv dom Anonymous, dom, subst_var nvar rng) in
+  let x = named_hd newenv dom Anonymous in
+  let prod = mkProd (x, dom, subst_var nvar rng) in
   let sigma3 = Evd.define sigma2 ev prod in
-  let dsp = num_of_evar dom in
-  let rsp = num_of_evar rng in
-  (sigma3,
-   mkEvar (dsp,args),
-   mkEvar (rsp, array_cons (mkRel 1) (Array.map (lift 1) args)))
+  let evdom = fst (destEvar dom), args in
+  let evrng =
+    fst (destEvar rng), array_cons (mkRel 1) (Array.map (lift 1) args) in
+  let prod' = mkProd (x, mkEvar evdom, mkEvar evrng) in
+  (sigma3, prod', evdom, evrng)
 
+let define_evar_as_arrow sigma ev =
+  let (sigma, prod, _, _) = split_evar_to_arrow sigma ev in
+  (sigma, prod)
+
+let define_evar_as_sort sigma (ev,args) =
+  let s = new_Type () in
+  let sigma = Evd.define sigma ev s in
+  (sigma, destSort s)
 
 (* Redefines an evar with a smaller context (i.e. it may depend on less
  * variables) such that c becomes closed.
@@ -131,9 +204,10 @@ let do_restrict_hyps sigma ev args =
       (hyps,([],[])) args 
   in
   let sign' = List.rev rsign in
-  let env' = change_hyps (fun _ -> sign') env in
+  let env' = reset_with_named_context sign' env in
   let instance = make_evar_instance env' in
   let (sigma',nc) = new_isevar_sign env' sigma evd.evar_concl instance in
+  let nc = refresh_universes nc in (* needed only if nc is an inferred type *)
   let sigma'' = Evd.define sigma' ev nc in
   (sigma'', nc)
 
@@ -145,8 +219,8 @@ let do_restrict_hyps sigma ev args =
  *------------------------------------*)
 
 type evar_constraint = conv_pb * constr * constr
-type 'a evar_defs =
-    { mutable evars : 'a Evd.evar_map;
+type evar_defs =
+    { mutable evars : Evd.evar_map;
       mutable conv_pbs : evar_constraint list }
 
 let create_evar_defs evd = { evars=evd; conv_pbs=[] }
@@ -178,15 +252,14 @@ let ise_map isevars sp = Evd.map isevars.evars sp
 
 (* define the existential of section path sp as the constr body *)
 let ise_define isevars sp body =
+  let body = refresh_universes body in (* needed only if an inferred type *)
   isevars.evars <- Evd.define isevars.evars sp body
 
+let is_defined_evar isevars (n,_) = Evd.is_defined isevars.evars n
+
 (* Does k corresponds to an (un)defined existential ? *)
 let ise_undefined isevars c = match kind_of_term c with
-  | IsEvar (n,_) -> not (Evd.is_defined isevars.evars n)
-  | _ -> false
-
-let ise_defined isevars c = match kind_of_term c with
-  | IsEvar (n,_) -> Evd.is_defined isevars.evars n
+  | Evar ev -> not (is_defined_evar isevars ev)
   | _ -> false
 
 let need_restriction isevars args = not (array_for_all closed0 args)
@@ -200,14 +273,14 @@ let need_restriction isevars args = not (array_for_all closed0 args)
  * false. The problem is that we won't get the right error message.
  *)
 
-let real_clean isevars sp args rhs =
+let real_clean isevars ev args rhs =
   let subst = List.map (fun (x,y) -> (y,mkVar x)) (filter_unique args) in
   let rec subs k t =
     match kind_of_term t with
-      | IsRel i ->
+      | Rel i ->
  	 if i<=k then t
  	 else (try List.assoc (mkRel (i-k)) subst with Not_found -> t)
-      | IsEvar (ev,args) ->
+      | Evar (ev,args) ->
 	  let args' = Array.map (subs k) args in
 	  if need_restriction isevars args' then
 	    if Evd.is_defined isevars.evars ev then 
@@ -220,11 +293,12 @@ let real_clean isevars sp args rhs =
 	    end
 	  else
 	    mkEvar (ev,args')
-      | IsVar _ -> (try List.assoc t subst with Not_found -> t)
+      | Var _ -> (try List.assoc t subst with Not_found -> t)
       | _ -> map_constr_with_binders succ subs k t
   in
   let body = subs 0 rhs in
-  if not (closed0 body) then error_not_clean empty_env sp body;
+  if not (closed0 body)
+  then error_not_clean empty_env isevars.evars ev body;
   body
 
 let make_evar_instance_with_rel env =
@@ -232,10 +306,11 @@ let make_evar_instance_with_rel env =
   let vars = 
     fold_named_context
       (fun env (id,b,_) l -> (* if b=None then *) mkVar id :: l (*else l*))
-      env [] in
+      env ~init:[] in
   snd (fold_rel_context
-    (fun env (_,b,_) (i,l) -> (i-1, (*if b=None then *) mkRel i :: l (*else l*)))
-    env (n,vars))
+	 (fun env (_,b,_) (i,l) -> 
+	    (i-1, (*if b=None then *) mkRel i :: l (*else l*)))
+	 env ~init:(n,vars))
 
 let make_subst env args =
   snd (fold_named_context
@@ -244,12 +319,27 @@ let make_subst env args =
 	 | (* None *) _ , a::rest -> (rest, (id,a)::l)
 (*	 | Some _, _ -> g*)
 	 | _ -> anomaly "Instance does not match its signature")
-    env (List.rev args,[]))
+    env ~init:(List.rev args,[]))
 
 (* [new_isevar] declares a new existential in an env env with type typ *)
 (* Converting the env into the sign of the evar to define *)
 
-let new_isevar isevars env typ k =
+let push_rel_context_to_named_context env =
+  let sign0 = named_context env in
+  let (subst,_,sign) =
+  Sign.fold_rel_context
+    (fun (na,c,t) (subst,avoid,sign) ->
+       let na = if na = Anonymous then Name(id_of_string"_") else na in
+       let id = next_name_away na avoid in
+       ((mkVar id)::subst,
+        id::avoid,
+	add_named_decl (id,option_app (substl subst) c,
+                        type_app (substl subst) t)
+	  sign))
+    (rel_context env) ~init:([],ids_of_named_context sign0,sign0)
+  in (subst, reset_with_named_context sign env)
+
+let new_isevar isevars env typ =
   let subst,env' = push_rel_context_to_named_context env in
   let typ' = substl subst typ in
   let instance = make_evar_instance_with_rel env in
@@ -275,13 +365,10 @@ let new_isevar isevars env typ k =
  * ?1 would be instantiated by (le y y) but y is not in the scope of ?1
  *)
 
-let keep_rels_and_vars c = match kind_of_term c with
-  | IsVar _ | IsRel _ -> c
-  | _ -> mkImplicit   (* Mettre mkMeta ?? *)
-
 let evar_define isevars (ev,argsv) rhs =
-  if occur_evar ev rhs then error_occur_check empty_env ev rhs;
-  let args = List.map keep_rels_and_vars (Array.to_list argsv) in 
+  if occur_evar ev rhs
+  then error_occur_check empty_env (evars_of isevars) ev rhs;
+  let args = Array.to_list argsv in 
   let evd = ise_map isevars ev in
   (* the substitution to invert *)
   let worklist = make_subst (evar_env evd) args in
@@ -294,22 +381,22 @@ let evar_define isevars (ev,argsv) rhs =
  *)
 
 let has_undefined_isevars isevars t = 
-  try let _ = whd_ise isevars.evars t in false
+  try let _ = local_strong (whd_ise isevars.evars) t in false
   with Uninstantiated_evar _ -> true
 
 let head_is_evar isevars = 
   let rec hrec k = match kind_of_term k with
-    | IsEvar (n,_)   -> not (Evd.is_defined isevars.evars n)
-    | IsApp (f,_) -> hrec f
-    | IsCast (c,_) -> hrec c
+    | Evar (n,_)   -> not (Evd.is_defined isevars.evars n)
+    | App (f,_) -> hrec f
+    | Cast (c,_) -> hrec c
     | _ -> false
   in 
   hrec 
 
 let rec is_eliminator c = match kind_of_term c with
-  | IsApp _    -> true
-  | IsMutCase _ -> true
-  | IsCast (c,_) -> is_eliminator c
+  | App _    -> true
+  | Case _ -> true
+  | Cast (c,_) -> is_eliminator c
   | _ -> false
 
 let head_is_embedded_evar isevars c =
@@ -317,10 +404,10 @@ let head_is_embedded_evar isevars c =
 
 let head_evar = 
   let rec hrec c = match kind_of_term c with
-    | IsEvar (ev,_)       -> ev
-    | IsMutCase (_,_,c,_) -> hrec c
-    | IsApp (c,_)        -> hrec c
-    | IsCast (c,_)        -> hrec c
+    | Evar (ev,_)       -> ev
+    | Case (_,_,c,_) -> hrec c
+    | App (c,_)        -> hrec c
+    | Cast (c,_)        -> hrec c
     | _                   -> failwith "headconstant"
   in 
   hrec 
@@ -338,7 +425,7 @@ let head_evar =
  * hyps of the existential, to do a "pop" for each Rel which is
  * not an argument of the existential, and a subst1 for each which
  * is, again, with the corresponding variable. This is done by
- * Tradevar.evar_define
+ * evar_define
  *
  * Thus, we take the arguments of the existential which we are about
  * to assign, and zip them with the identifiers in the hypotheses.
@@ -394,7 +481,7 @@ let solve_refl conv_algo env isevars ev argsv1 argsv2 =
   let nargs = (Array.of_list (List.map mkVar (ids_of_named_context nsign))) in
   let newev = new_evar () in
   let info = { evar_concl = evd.evar_concl; evar_hyps = nsign;
-	       evar_body = Evar_empty; evar_info = None } in
+	       evar_body = Evar_empty } in
   isevars.evars <-
     Evd.define (Evd.add isevars.evars newev info) ev (mkEvar (newev,nargs));
   [ev]
@@ -407,9 +494,9 @@ let solve_refl conv_algo env isevars ev argsv1 argsv2 =
 
 (* Rq: uncomplete algorithm if pbty = CONV_X_LEQ ! *)
 let solve_simple_eqn conv_algo env isevars (pbty,(n1,args1 as ev1),t2) =
-  let t2 = nf_ise1 isevars.evars t2 in
+  let t2 = nf_evar isevars.evars t2 in
   let lsp = match kind_of_term t2 with
-    | IsEvar (n2,args2 as ev2)
+    | Evar (n2,args2 as ev2)
         when not (Evd.is_defined isevars.evars n2) ->
 	if n1 = n2 then
 	  solve_refl conv_algo env isevars n1 args1 args2
@@ -462,16 +549,16 @@ let mk_valcon c = Some c
 let split_tycon loc env isevars = function
   | None -> None,None
   | Some c ->
-      let t = whd_betadeltaiota env isevars.evars c in
+      let sigma = evars_of isevars in
+      let t = whd_betadeltaiota env sigma c in
       match kind_of_term t with
-        | IsProd (na,dom,rng) -> Some dom, Some rng
-	| _ ->
-	    if ise_undefined isevars t then
-	      let (sigma,dom,rng) = split_evar_to_arrow isevars.evars t in
-	      isevars.evars <- sigma;
-	      Some dom, Some rng
-	    else
-	      error_not_product_loc loc env c
+        | Prod (na,dom,rng) -> Some dom, Some rng
+	| Evar (n,_ as ev) when not (Evd.is_defined isevars.evars n) ->
+	    let (sigma',_,evdom,evrng) = split_evar_to_arrow sigma ev in
+	    evars_reset_evd sigma' isevars;
+	    Some (mkEvar evdom), Some (mkEvar evrng)
+	| _ -> error_not_product_loc loc env sigma c
 
 let valcon_of_tycon x = x
 
+let lift_tycon = option_app (lift 1)
