@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: pretyping.ml,v 1.68 2001/04/02 08:18:05 filliatr Exp $ *)
+(* $Id: pretyping.ml,v 1.89 2001/12/18 09:15:35 herbelin Exp $ *)
 
 open Pp
 open Util
@@ -14,7 +14,8 @@ open Names
 open Sign
 open Evd
 open Term
-open Reduction
+open Termops
+open Reductionops
 open Environ
 open Type_errors
 open Typeops
@@ -26,36 +27,43 @@ open Pretype_errors
 open Rawterm
 open Evarconv
 open Coercion
+open Dyn
 
 
 (***********************************************************************)
 (* This concerns Cases *)
+open Declarations
 open Inductive
+open Inductiveops
 open Instantiate
 
 let lift_context n l = 
   let k = List.length l in 
   list_map_i (fun i (name,c) -> (name,liftn n (k-i) c)) 0 l
 
-let transform_rec loc env sigma (p,c,lf) (indt,pt) = 
-  let (indf,realargs) = dest_ind_type indt in
-  let (mispec,params) = dest_ind_family indf in
-  let mI = mkMutInd (mis_inductive mispec) in
-  let recargs = mis_recarg mispec in
-  let tyi = mis_index mispec in
-  if Array.length lf <> mis_nconstr mispec then 
-    error_number_branches_loc loc CCI env c
-      (mkAppliedInd indt) (mis_nconstr mispec);
-  if mis_is_recursive_subset [tyi] mispec then
-    let dep = find_case_dep_nparams env sigma (c,p) indf pt in 
+let transform_rec loc env sigma (pj,c,lf) indt = 
+  let p = pj.uj_val in
+  let ((ind,params) as indf,realargs) = dest_ind_type indt in
+  let (mib,mip) = lookup_mind_specif env ind in
+  let mI = mkInd ind in
+  let recargs = mip.mind_listrec in
+  let tyi = snd ind in
+  let ci = make_default_case_info env ind in
+  let nconstr = Array.length mip.mind_consnames in
+  if Array.length lf <> nconstr then 
+    (let cj = {uj_val=c; uj_type=mkAppliedInd indt} in
+     error_number_branches_loc loc env sigma cj nconstr);
+  if mis_is_recursive_subset [tyi] mip then
+    let (dep,_) =
+      find_case_dep_nparams env
+        (nf_evar sigma c, j_nf_evar sigma pj) indf in 
     let init_depFvec i = if i = tyi then Some(dep,mkRel 1) else None in
-    let depFvec = Array.init (mis_ntypes mispec) init_depFvec in
+    let depFvec = Array.init mib.mind_ntypes init_depFvec in
     (* build now the fixpoint *)
-    let lnames,_ = get_arity indf in
+    let lnames,_ = get_arity env indf in
     let nar = List.length lnames in
-    let nparams = mis_nparams mispec in
-    let constrs = get_constructors (lift_inductive_family (nar+2) indf) in
-    let ci = make_default_case_info mispec in
+    let nparams = mip.mind_nparams in
+    let constrs = get_constructors env (lift_inductive_family (nar+2) indf) in
     let branches = 
       array_map3
 	(fun f t reca -> 
@@ -70,7 +78,7 @@ let transform_rec loc env sigma (p,c,lf) (indt,pt) =
 	(lambda_create env
 	   (applist (mI,List.append (List.map (lift (nar+1)) params)
                        (extended_rel_list 0 lnames)),
-            mkMutCase (ci, lift (nar+2) p, mkRel 1, branches)))
+            mkCase (ci, lift (nar+2) p, mkRel 1, branches)))
         (lift_rel_context 1 lnames) 
     in
     if noccurn 1 deffix then 
@@ -90,61 +98,45 @@ let transform_rec loc env sigma (p,c,lf) (indt,pt) =
 	       else
 		 let args = extended_rel_list 1 lnames in
 		 whd_beta (applist (lift (nar+1) p, args)))))
-          lnames 
-      in
-      let fix = mkFix (([|nar|],0),
-		       ([|typPfix|],[Name(id_of_string "F")],[|deffix|])) in
+          lnames in
+      let fix =
+        mkFix (([|nar|],0),
+	       ([|Name(id_of_string "F")|],[|typPfix|],[|deffix|])) in
       applist (fix,realargs@[c])
   else
-    let ci = make_default_case_info mispec in
-    mkMutCase (ci, p, c, lf)
+    mkCase (ci, p, c, lf)
 
 (***********************************************************************)
 
-let ctxt_of_ids cl = cl
+(* To embed constr in rawconstr *)
+let ((constr_in : constr -> Dyn.t),
+     (constr_out : Dyn.t -> constr)) = create "constr"
 
 let mt_evd = Evd.empty
 
 let vect_lift_type = Array.mapi (fun i t -> type_app (lift i) t)
 
-let j_nf_ise sigma {uj_val=v;uj_type=t} =
-  {uj_val=nf_ise1 sigma v;uj_type=type_app (nf_ise1 sigma) t}
-
-let jv_nf_ise sigma = Array.map (j_nf_ise sigma)
-
-let tj_nf_ise sigma {utj_val=v;utj_type=t} =
-  {utj_val=type_app (nf_ise1 sigma) v;utj_type=t}
-
 (* Utilis pour infrer le prdicat des Cases *)
 (* Semble exagrement fort *)
 (* Faudra prfrer une unification entre les types de toutes les clauses *)
 (* et autoriser des ?  rester dans le rsultat de l'unification *)
 
-let evar_type_fixpoint env isevars lna lar vdefj =
+let evar_type_fixpoint loc env isevars lna lar vdefj =
   let lt = Array.length vdefj in 
     if Array.length lar = lt then 
       for i = 0 to lt-1 do 
         if not (the_conv_x_leq env isevars
 		  (vdefj.(i)).uj_type
 		  (lift lt lar.(i))) then
-          error_ill_typed_rec_body CCI env i lna 
-	    (jv_nf_ise (evars_of isevars) vdefj) 
-	    (Array.map (type_app (nf_ise1 (evars_of isevars))) lar)
+          error_ill_typed_rec_body_loc loc env (evars_of isevars)
+            i lna vdefj lar
       done
 
-let wrong_number_of_cases_message loc env isevars (c,ct) expn = 
-  let c = nf_ise1 (evars_of isevars) c
-  and ct = nf_ise1 (evars_of isevars) ct in
-  error_number_branches_loc loc CCI env c ct expn
-
 let check_branches_message loc env isevars c (explft,lft) = 
   for i = 0 to Array.length explft - 1 do
     if not (the_conv_x_leq env isevars lft.(i) explft.(i)) then 
-      let c = nf_ise1 (evars_of isevars) c
-      and lfi = nf_betaiota env (evars_of isevars)
-        (nf_ise1 (evars_of isevars) lft.(i)) in
-      error_ill_formed_branch_loc loc CCI env c i lfi 
-	(nf_betaiota env (evars_of isevars) explft.(i))
+      let sigma = evars_of isevars in
+      error_ill_formed_branch_loc loc env sigma c i lft.(i) explft.(i)
   done
 
 (* coerce to tycon if any *)
@@ -167,7 +159,7 @@ let pretype_id loc env lvar id =
     { uj_val  = mkRel n; uj_type = type_app (lift n) typ }
   with Not_found ->
   try
-    let typ = lookup_id_type id (named_context env) in
+    let (_,_,typ) = lookup_named id env in
     { uj_val  = mkVar id; uj_type = typ }
   with Not_found ->
     error_var_not_found_loc loc id
@@ -176,43 +168,12 @@ let pretype_id loc env lvar id =
 (* Main pretyping function                                               *)
 
 let pretype_ref isevars env lvar ref = 
-  let c = Declare.constr_of_reference (evars_of isevars) env ref in
+  let c = Declare.constr_of_reference ref in
   make_judge c (Retyping.get_type_of env Evd.empty c)
 
-(*
-let pretype_ref _ isevars env lvar ref =
-...
-
-| RConst (sp,ctxt) ->
-    let cst = (sp,Array.map pretype ctxt) in
-    make_judge (mkConst cst) (type_of_constant env (evars_of isevars) cst)
-*)
-(* A traiter mais les tables globales ncessaires  cela pour l'instant
-| REVar (sp,ctxt) ->
-    let ev = (sp,Array.map pretype ctxt) in
-    let body = 
-      if Evd.is_defined (evars_of isevars) sp then
-	existential_value (evars_of isevars) ev
-      else
-	mkEvar ev
-    in
-    let typ = existential_type (evars_of isevars) ev in
-    make_judge body typ
-
-| RInd (ind_sp,ctxt) ->
-    let ind = (ind_sp,Array.map pretype ctxt) in
-    make_judge (mkMutInd ind) (type_of_inductive env (evars_of isevars) ind)
- 
-| RConstruct (cstr_sp,ctxt) ->
-    let cstr = (cstr_sp,Array.map pretype ctxt) in
-    let typ = type_of_constructor env (evars_of isevars) cstr in
-    { uj_val=mkMutConstruct cstr; uj_type=typ }
-*)
 let pretype_sort = function
   | RProp c -> judge_of_prop_contents c
-  | RType ->
-      { uj_val = dummy_sort;
-	uj_type = dummy_sort }
+  | RType _ -> judge_of_new_Type ()
 
 (* [pretype tycon env isevars lvar lmeta cstr] attempts to type [cstr] *)
 (* in environment [env], with existential variables [(evars_of isevars)] and *)
@@ -234,7 +195,7 @@ let rec pretype tycon env isevars lvar lmeta = function
       sous-contexte du contexte courant, et qu'il n'y a pas de Rel "cach" *)
       let hyps = (Evd.map (evars_of isevars) ev).evar_hyps in
       let args = instance_from_named_context hyps in
-      let c = mkEvar (ev, Array.of_list args) in
+      let c = mkEvar (ev, args) in
       let j = (Retyping.get_judgment_of env (evars_of isevars) c) in
       inh_conv_coerce_to_tycon loc env isevars j tycon
 
@@ -246,45 +207,45 @@ let rec pretype tycon env isevars lvar lmeta = function
             Not_found ->
 	      user_err_loc
 		(loc,"pretype",
-		 [< 'sTR "Metavariable "; 'iNT n; 'sTR" is unbound" >])
+		 str "Metavariable " ++ int n ++ str " is unbound")
       in inh_conv_coerce_to_tycon loc env isevars j tycon
 	   
   | RHole loc ->
       if !compter then nbimpl:=!nbimpl+1;
       (match tycon with
-	 | Some ty -> { uj_val = new_isevar isevars env ty CCI; uj_type = ty }
+	 | Some ty -> { uj_val = new_isevar isevars env ty; uj_type = ty }
 	 | None ->
 	     (match loc with
-		  None -> anomaly "There is an implicit argument I cannot solve"
+		  None -> error "There is an implicit argument I cannot solve"
 		| Some loc -> 
 		    user_err_loc
 		      (loc,"pretype",
-		       [< 'sTR "Cannot infer a term for this placeholder" >])))
+		       (str "Cannot infer a term for this placeholder"))))
 
-  | RRec (loc,fixkind,lfi,lar,vdef) ->
-      let larj = Array.map (pretype_type empty_valcon env isevars lvar lmeta) lar in
+  | RRec (loc,fixkind,names,lar,vdef) ->
+      let larj =
+        Array.map (pretype_type empty_valcon env isevars lvar lmeta) lar in
       let lara = Array.map (fun a -> a.utj_val) larj in
-      let nbfix = Array.length lfi in
-      let lfi = List.map (fun id -> Name id) (Array.to_list lfi) in
-      let newenv =
-	array_fold_left2 (fun env id ar -> (push_rel_assum (id,ar) env))
-	  env (Array.of_list lfi) (vect_lift_type lara) in
+      let nbfix = Array.length lar in
+      let names = Array.map (fun id -> Name id) names in
+      let newenv = push_rec_types (names,lara,[||]) env in
       let vdefj =
 	Array.mapi 
 	  (fun i def -> (* we lift nbfix times the type in tycon, because of
 			 * the nbfix variables pushed to newenv *)
-             pretype (mk_tycon (lift nbfix (larj.(i).utj_val))) newenv isevars lvar
-	       lmeta def) vdef in
-      evar_type_fixpoint env isevars lfi lara vdefj;
+             pretype (mk_tycon (lift nbfix (larj.(i).utj_val)))
+               newenv isevars lvar lmeta def)
+          vdef in
+      evar_type_fixpoint loc env isevars names lara vdefj;
       let fixj =
 	match fixkind with
 	  | RFix (vn,i as vni) ->
-	      let fix = (vni,(lara,lfi,Array.map j_val vdefj)) in
-	      check_fix env (evars_of isevars) fix;
+	      let fix = (vni,(names,lara,Array.map j_val vdefj)) in
+	      check_fix env fix;
 	      make_judge (mkFix fix) lara.(i)
 	  | RCoFix i -> 
-	      let cofix = (i,(lara,lfi,Array.map j_val vdefj)) in
-	      check_cofix env (evars_of isevars) cofix;
+	      let cofix = (i,(names,lara,Array.map j_val vdefj)) in
+	      check_cofix env cofix;
 	      make_judge (mkCoFix cofix) lara.(i) in
       inh_conv_coerce_to_tycon loc env isevars fixj tycon
 
@@ -299,8 +260,10 @@ let rec pretype tycon env isevars lvar lmeta = function
 	| c::rest ->
 	    let argloc = loc_of_rawconstr c in
 	    let resj = inh_app_fun env isevars resj in
-      	    match kind_of_term (whd_betadeltaiota env (evars_of isevars) resj.uj_type) with
-	      | IsProd (na,c1,c2) ->
+            let resty =
+              whd_betadeltaiota env (evars_of isevars) resj.uj_type in
+      	    match kind_of_term resty with
+	      | Prod (na,c1,c2) ->
 		  let hj = pretype (mk_tycon c1) env isevars lvar lmeta c in
 		  let newresj =
       		    { uj_val = applist (j_val resj, [j_val hj]);
@@ -308,14 +271,10 @@ let rec pretype tycon env isevars lvar lmeta = function
 		  apply_rec env (n+1) newresj rest
 
 	      | _ ->
-		  let j_nf_ise env sigma {uj_val=v;uj_type=t} =
-		    { uj_val = nf_ise1 sigma v; 
-		      uj_type = nf_ise1 sigma t } in
 		  let hj = pretype empty_tycon env isevars lvar lmeta c in
 		  error_cant_apply_not_functional_loc 
-		    (Rawterm.join_loc floc argloc) env
-	      	    (j_nf_ise env (evars_of isevars) resj)
-		    [j_nf_ise env (evars_of isevars) hj]
+		    (Rawterm.join_loc floc argloc) env (evars_of isevars)
+	      	    resj [hj]
 
       in let resj = apply_rec env 1 fj args in
       (*
@@ -336,10 +295,9 @@ let rec pretype tycon env isevars lvar lmeta = function
       let (dom,rng) = split_tycon loc env isevars tycon in
       let dom_valcon = valcon_of_tycon dom in
       let j = pretype_type dom_valcon env isevars lvar lmeta c1 in
-      let var = (name,j.utj_val) in
-      let j' = pretype rng (push_rel_assum var env) isevars lvar lmeta c2 
-      in 
-      fst (abs_rel env (evars_of isevars) name j.utj_val j')
+      let var = (name,None,j.utj_val) in
+      let j' = pretype rng (push_rel var env) isevars lvar lmeta c2 in 
+      judge_of_abstraction env name j j'
 
   | RProd(loc,name,c1,c2)        ->
       let j = pretype_type empty_valcon env isevars lvar lmeta c1 in
@@ -347,81 +305,83 @@ let rec pretype tycon env isevars lvar lmeta = function
       let env' = push_rel_assum var env in
       let j' = pretype_type empty_valcon env' isevars lvar lmeta c2 in
       let resj =
-	try fst (gen_rel env (evars_of isevars) name j j')
+	try judge_of_product env name j j'
 	with TypeError _ as e -> Stdpp.raise_with_loc loc e in
       inh_conv_coerce_to_tycon loc env isevars resj tycon
 	
   | RLetIn(loc,name,c1,c2)      ->
       let j = pretype empty_tycon env isevars lvar lmeta c1 in
-      let var = (name,j.uj_val,j.uj_type) in
-      let j' = pretype tycon (push_rel_def var env) isevars lvar lmeta c2 in
-      { uj_val = mkLetIn (name, j.uj_val, j.uj_type, j'.uj_val) ;
+      let t = Evarutil.refresh_universes j.uj_type in
+      let var = (name,Some j.uj_val,t) in
+        let tycon = option_app (lift 1) tycon in
+      let j' = pretype tycon (push_rel var env) isevars lvar lmeta c2 in
+      { uj_val = mkLetIn (name, j.uj_val, t, j'.uj_val) ;
 	uj_type = type_app (subst1 j.uj_val) j'.uj_type }
       
   | ROldCase (loc,isrec,po,c,lf) ->
       let cj = pretype empty_tycon env isevars lvar lmeta c in
       let (IndType (indf,realargs) as indt) = 
-	try find_rectype env (evars_of isevars) cj.uj_type
-	with Induc -> error_case_not_inductive CCI env
-	    (nf_ise1 (evars_of isevars) cj.uj_val)
-            (nf_ise1 (evars_of isevars) cj.uj_type) in
+	try find_rectype env Evd.empty (nf_evar (evars_of isevars) cj.uj_type)
+	with Induc ->
+          error_case_not_inductive_loc loc env (evars_of isevars) cj in
       let pj = match po with
 	| Some p -> pretype empty_tycon env isevars lvar lmeta p
 	| None -> 
-	    try match tycon with
-		Some pred ->
-                  Retyping.get_judgment_of env (evars_of isevars) pred
-	      | None -> error "notype"
-	    with UserError _ -> (* get type information from type of branches*)
-	      let expbr = Cases.branch_scheme env isevars isrec indf in
-	      let rec findtype i =
-		if i >= Array.length lf
-		then error_cant_find_case_type_loc loc env cj.uj_val
-		else
-		  try
-		    let expti = expbr.(i) in
-		    let fj =
-		      pretype (mk_tycon expti) env isevars lvar lmeta lf.(i) in
-		    let efjt = nf_ise1 (evars_of isevars) fj.uj_type in 
-		    let pred = 
-		      Cases.pred_case_ml_onebranch
-                        loc env (evars_of isevars) isrec indt
-			(i,fj.uj_val,efjt) in
-		    if has_undefined_isevars isevars pred then findtype (i+1)
-		    else 
-		      let pty =
-                        Retyping.get_type_of env (evars_of isevars) pred in
-		      { uj_val = pred; 
-			uj_type = pty }
-		  with UserError _ -> findtype (i+1) in
-	      findtype 0 in
+            (* get type information from type of branches *)
+	    let expbr = Cases.branch_scheme env isevars isrec indf in
+	    let rec findtype i =
+	      if i >= Array.length lf
+	      then
+                (* get type information from constraint *)
+                (* warning: if the constraint comes from an evar type, it *)
+                (* may be Type while Prop or Set would be expected *)
+	        match tycon with
+		  | Some pred ->
+                      Retyping.get_judgment_of env (evars_of isevars) pred
+	          | None ->
+                      let sigma = evars_of isevars in
+                      error_cant_find_case_type_loc loc env sigma cj.uj_val
+	      else
+		try
+		  let expti = expbr.(i) in
+		  let fj =
+		    pretype (mk_tycon expti) env isevars lvar lmeta lf.(i) in
+		  let pred = 
+		    Cases.pred_case_ml
+                      env (evars_of isevars) isrec indt (i,fj.uj_type) in
+		  if has_undefined_isevars isevars pred then findtype (i+1)
+		  else 
+		    let pty =
+                      Retyping.get_type_of env (evars_of isevars) pred in
+		    let pj = { uj_val = pred; uj_type = pty } in
+                    let _ = option_app (the_conv_x_leq env isevars pred) tycon
+                    in pj
+		with Cases.NotInferable _ -> findtype (i+1) in
+	    findtype 0 in
+      let pj = j_nf_evar (evars_of isevars) pj in
 
-      let evalPt = nf_ise1 (evars_of isevars) pj.uj_type in
+      let (dep,_) = find_case_dep_nparams env (cj.uj_val,pj) indf in
 
-      let dep = find_case_dep_nparams env (evars_of isevars)
-        (cj.uj_val,pj.uj_val) indf evalPt in
-
-      let (p,pt) =
-	if dep then (pj.uj_val, evalPt) else
+      let pj =
+	if dep then pj else
 	  let n = List.length realargs in
 	  let rec decomp n p =
 	    if n=0 then p else
 	      match kind_of_term p with
-		| IsLambda (_,_,c) -> decomp (n-1) c
+		| Lambda (_,_,c) -> decomp (n-1) c
 		| _ -> decomp (n-1) (applist (lift 1 p, [mkRel 1])) in
-	  let sign,s = decompose_prod_n n evalPt in
-	  let ind = build_dependent_inductive indf in
+	  let sign,s = decompose_prod_n n pj.uj_type in
+	  let ind = build_dependent_inductive env indf in
 	  let s' = mkProd (Anonymous, ind, s) in
 	  let ccl = lift 1 (decomp n pj.uj_val) in
 	  let ccl' = mkLambda (Anonymous, ind, ccl) in
-	  (lam_it ccl' sign, prod_it s' sign) in
+	  {uj_val=lam_it ccl' sign; uj_type=prod_it s' sign} in
       let (bty,rsty) =
 	Indrec.type_rec_branches
-          isrec env (evars_of isevars) indt pt p cj.uj_val in
+          isrec env (evars_of isevars) indt pj cj.uj_val in
       if Array.length bty <> Array.length lf then
-	wrong_number_of_cases_message loc env isevars 
-	  (cj.uj_val,nf_ise1 (evars_of isevars) cj.uj_type)
-	  (Array.length bty)
+	error_number_branches_loc loc env (evars_of isevars) 
+	  cj (Array.length bty)
       else
 	let lfj =
 	  array_map2
@@ -433,14 +393,15 @@ let rec pretype tycon env isevars lvar lmeta = function
 	let v =
 	  if isrec
 	  then 
-	    transform_rec loc env (evars_of isevars)(p,cj.uj_val,lfv) (indt,pt)
+	    transform_rec loc env (evars_of isevars)(pj,cj.uj_val,lfv) indt
 	  else
 	    let mis,_ = dest_ind_family indf in
-	    let ci = make_default_case_info mis in
-	    mkMutCase (ci, p, cj.uj_val, Array.map (fun j-> j.uj_val) lfj)
+	    let ci = make_default_case_info env mis in
+	    mkCase (ci, (nf_betaiota pj.uj_val), cj.uj_val,
+                       Array.map (fun j-> j.uj_val) lfj)
 	in
-	{uj_val = v;
-	 uj_type = rsty }
+	{ uj_val = v;
+	  uj_type = rsty }
 
   | RCases (loc,prinfo,po,tml,eqns) ->
       Cases.compile_cases loc
@@ -448,11 +409,23 @@ let rec pretype tycon env isevars lvar lmeta = function
 	tycon env (* loc *) (po,tml,eqns)
 
   | RCast(loc,c,t) ->
-      let tj = pretype_type (valcon_of_tycon tycon) env isevars lvar lmeta t in
+      let tj = pretype_type empty_tycon env isevars lvar lmeta t in
       let cj = pretype (mk_tycon tj.utj_val) env isevars lvar lmeta c in
-      let cj = {uj_val = mkCast (cj.uj_val,tj.utj_val); uj_type=tj.utj_val} in
+      (* User Casts are for helping pretyping, experimentally not to be kept*)
+      (* ... except for Correctness *)
+      let v = mkCast (cj.uj_val, tj.utj_val) in
+      let cj = { uj_val = v; uj_type = tj.utj_val } in
       inh_conv_coerce_to_tycon loc env isevars cj tycon
 
+  | RDynamic (loc,d) ->
+    if (tag d) = "constr" then
+      let c = constr_out d in
+      let j = (Retyping.get_judgment_of env (evars_of isevars) c) in
+      j
+      (*inh_conv_coerce_to_tycon loc env isevars j tycon*)
+    else
+      user_err_loc (loc,"pretype",(str "Not a constr tagged Dynamic"))
+
 (* [pretype_type valcon env isevars lvar lmeta c] coerces [c] into a type *)
 and pretype_type valcon env isevars lvar lmeta = function
   | RHole loc ->
@@ -462,8 +435,8 @@ and pretype_type valcon env isevars lvar lmeta = function
 	     { utj_val = v;
 	       utj_type = Retyping.get_sort_of env (evars_of isevars) v }
 	 | None ->
-	     { utj_val = new_isevar isevars env dummy_sort CCI;
-	       utj_type = Type Univ.dummy_univ })
+	     let s = new_Type_sort () in
+	     { utj_val = new_isevar isevars env (mkSort s); utj_type = s})
   | c ->
       let j = pretype empty_tycon env isevars lvar lmeta c in
       let tj = inh_coerce_to_sort env isevars j in
@@ -472,16 +445,17 @@ and pretype_type valcon env isevars lvar lmeta = function
 	| Some v ->
 	    if the_conv_x_leq env isevars v tj.utj_val then tj
 	    else
-	      error_unexpected_type_loc (loc_of_rawconstr c) env tj.utj_val v
+	      error_unexpected_type_loc
+                (loc_of_rawconstr c) env (evars_of isevars) tj.utj_val v
 
 
 let unsafe_infer tycon isevars env lvar lmeta constr =
   let j = pretype tycon env isevars lvar lmeta constr in
-  j_nf_ise (evars_of isevars) j
+  j_nf_evar (evars_of isevars) j
 
 let unsafe_infer_type valcon isevars env lvar lmeta constr =
   let tj = pretype_type valcon env isevars lvar lmeta constr in
-  tj_nf_ise (evars_of isevars) tj
+  tj_nf_evar (evars_of isevars) tj
 
 (* If fail_evar is false, [process_evars] builds a meta_map with the
    unresolved Evar that were not in initial sigma; otherwise it fail
@@ -496,12 +470,12 @@ let check_evars fail_evar initial_sigma sigma c =
   let metamap = ref [] in
   let rec proc_rec c =
     match kind_of_term c with
-      | IsEvar (ev,args as k) ->
+      | Evar (ev,args as k) ->
           assert (Evd.in_dom sigma ev);
 	  if not (Evd.in_dom initial_sigma ev) then
 	    (if fail_evar then
 	      errorlabstrm "whd_ise"
-		[< 'sTR"There is an unknown subterm I cannot solve" >]
+		(str"There is an unknown subterm I cannot solve")
 	    else (* try to avoid duplication *)
               (if not (List.exists (fun (k',_) -> k=k') !metamap) then
 	        metamap := (k, existential_type sigma k) :: !metamap))
@@ -564,7 +538,7 @@ let understand_type sigma env c =
   let _,c = ise_infer_type_gen true sigma env [] [] c in
   c.utj_val
 
-let understand_gen sigma env lvar lmeta exptyp c =
+let understand_gen sigma env lvar lmeta ~expected_type:exptyp c =
   let _, c = ise_infer_gen true sigma env lvar lmeta exptyp c in
   c.uj_val
 
