@@ -6,26 +6,103 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: tacred.ml,v 1.41 2001/03/15 13:38:37 filliatr Exp $ *)
+(* $Id: tacred.ml,v 1.54 2001/12/20 08:32:14 herbelin Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Term
+open Termops
+open Declarations
 open Inductive
 open Environ
-open Reduction
+open Reductionops
 open Closure
 open Instantiate
 open Cbv
 
+exception Elimconst
+exception Redelimination
+
+let set_opaque_const = Conv_oracle.set_opaque_const
+let set_transparent_const sp =
+  let cb = Global.lookup_constant sp in
+  if cb.const_body <> None & cb.const_opaque then
+    errorlabstrm "set_transparent_const"
+      (str "Cannot make" ++ spc () ++ 
+         Nametab.pr_global_env (Global.env()) (Nametab.ConstRef sp) ++
+         spc () ++ str "transparent because it was declared opaque.");
+  Conv_oracle.set_transparent_const sp
+
+let set_opaque_var      = Conv_oracle.set_opaque_var
+let set_transparent_var = Conv_oracle.set_transparent_var
+
+let _ = 
+  Summary.declare_summary "Transparent constants and variables"
+    { Summary.freeze_function = Conv_oracle.freeze;
+      Summary.unfreeze_function = Conv_oracle.unfreeze;
+      Summary.init_function = Conv_oracle.init;
+      Summary.survive_section = false }
+
+let is_evaluable env ref =
+  match ref with
+      EvalConstRef sp ->
+        let (ids,sps) = Conv_oracle.freeze() in
+        Sppred.mem sp sps &
+        let cb = Environ.lookup_constant sp env in
+        cb.const_body <> None & not cb.const_opaque
+    | EvalVarRef id ->
+        let (ids,sps) = Conv_oracle.freeze() in
+        Idpred.mem id ids &
+        let (_,value,_) = Environ.lookup_named id env in
+        value <> None
+
+type evaluable_reference =
+  | EvalConst of constant
+  | EvalVar of identifier
+  | EvalRel of int
+  | EvalEvar of existential
+
+let mkEvalRef = function
+  | EvalConst cst -> mkConst cst
+  | EvalVar id -> mkVar id
+  | EvalRel n -> mkRel n
+  | EvalEvar ev -> mkEvar ev
+
+let isEvalRef env c = match kind_of_term c with
+  | Const sp -> is_evaluable env (EvalConstRef sp)
+  | Var id -> is_evaluable env (EvalVarRef id)
+  | Rel _ | Evar _ -> true
+  | _ -> false
+
+let destEvalRef c = match kind_of_term c with
+  | Const cst ->  EvalConst cst
+  | Var id  -> EvalVar id
+  | Rel n -> EvalRel n
+  | Evar ev -> EvalEvar ev
+  | _ -> anomaly "Not an evaluable reference"
+
+let reference_opt_value sigma env = function
+  | EvalConst cst -> constant_opt_value env cst
+  | EvalVar id ->
+      let (_,v,_) = lookup_named id env in
+      v
+  | EvalRel n ->
+      let (_,v,_) = lookup_rel n env in
+      option_app (lift n) v
+  | EvalEvar ev -> existential_opt_value sigma ev
+
+exception NotEvaluable
+let reference_value sigma env c =
+  match reference_opt_value sigma env c with
+    | None -> raise NotEvaluable
+    | Some d -> d
+
 (************************************************************************)
 (* Reduction of constant hiding fixpoints (e.g. for Simpl). The trick   *)
 (* is to reuse the name of the function after reduction of the fixpoint *)
 
-exception Elimconst
-exception Redelimination
-
 type constant_evaluation = 
   | EliminationFix of int * (int * (int * constr) list * int)
   | EliminationMutualFix of
@@ -76,7 +153,7 @@ let _ =
    == [yip:Bip]..[yi1:Bi1](Fix(f|t)[xi<-ai] (Rel 1)..(Rel p))
    with bj=aj if j<>ik and bj=(Rel c) and Bic=Aic[xn..xic-1 <- an..aic-1] *)
 
-let check_fix_reversibility labs args ((lv,i),(tys,_,bds)) =
+let check_fix_reversibility labs args ((lv,i),(_,tys,bds)) =
   let n = List.length labs in
   let nargs = List.length args in
   if nargs > n then raise Elimconst;
@@ -84,7 +161,7 @@ let check_fix_reversibility labs args ((lv,i),(tys,_,bds)) =
   let li =
     List.map
       (function d -> match kind_of_term d with
-         | IsRel k ->
+         | Rel k ->
              if
 	       array_for_all (noccurn k) tys
 	       && array_for_all (noccurn (k+nbfix)) bds
@@ -98,8 +175,11 @@ let check_fix_reversibility labs args ((lv,i),(tys,_,bds)) =
   if list_distinct (List.map fst li) then 
     let k = lv.(i) in
     if k < nargs then
+(*  Such an optimisation would need eta-expansion 
       let p = destRel (List.nth args k) in 
       EliminationFix (n-p+1,(nbfix,li,n))
+*)
+      EliminationFix (n,(nbfix,li,n))
     else
       EliminationFix (n-nargs+lv.(i)+1,(nbfix,li,n))
   else 
@@ -114,8 +194,8 @@ let invert_name labs l na0 env sigma ref = function
 	let refi = match ref with
 	  | EvalRel _ | EvalEvar _ -> None
 	  | EvalVar id' -> Some (EvalVar id)
-	  | EvalConst (sp,args) ->
-	      Some (EvalConst (make_path (dirpath sp) id CCI, args)) in
+	  | EvalConst sp ->
+	      Some (EvalConst (make_path (dirpath sp) id)) in
 	match refi with
 	  | None -> None
 	  | Some ref ->
@@ -137,12 +217,12 @@ let compute_consteval_direct sigma env ref =
   let rec srec env n labs c =
     let c',l = whd_betadeltaeta_stack env sigma c in
     match kind_of_term c' with
-      | IsLambda (id,t,g) when l=[] ->
-	  srec (push_rel_assum (id,t) env) (n+1) (t::labs) g
-      | IsFix fix ->
+      | Lambda (id,t,g) when l=[] ->
+	  srec (push_rel (id,None,t) env) (n+1) (t::labs) g
+      | Fix fix ->
 	  (try check_fix_reversibility labs l fix 
 	  with Elimconst -> NotAnElimination)
-      | IsMutCase (_,_,d,_) when isRel d -> EliminationCases n
+      | Case (_,_,d,_) when isRel d -> EliminationCases n
       | _ -> NotAnElimination
   in 
   match reference_opt_value sigma env ref with
@@ -154,22 +234,21 @@ let compute_consteval_mutual_fix sigma env ref =
     let c',l = whd_betaetalet_stack c in
     let nargs = List.length l in
     match kind_of_term c' with
-      | IsLambda (na,t,g) when l=[] ->
-	  srec (push_rel_assum (na,t) env) (minarg+1) (t::labs) ref g
-      | IsFix ((lv,i),(_,names,_) as fix) ->
+      | Lambda (na,t,g) when l=[] ->
+	  srec (push_rel (na,None,t) env) (minarg+1) (t::labs) ref g
+      | Fix ((lv,i),(names,_,_) as fix) ->
 	  (* Last known constant wrapping Fix is ref = [labs](Fix l) *)
 	  (match compute_consteval_direct sigma env ref with
 	     | NotAnElimination -> (*Above const was eliminable but this not!*)
 		 NotAnElimination
 	     | EliminationFix (minarg',infos) ->
-		 let names = Array.of_list names in
 		 let refs =
 		   Array.map
 		     (invert_name labs l names.(i) env sigma ref) names in
 		 let new_minarg = max (minarg'+minarg-nargs) minarg' in
 		 EliminationMutualFix (new_minarg,ref,(refs,infos))
 	     | _ -> assert false)
-      | _ when isEvalRef c' ->
+      | _ when isEvalRef env c' ->
 	  (* Forget all \'s and args and do as if we had started with c' *)
 	  let ref = destEvalRef c' in
 	  (match reference_opt_value sigma env ref with
@@ -272,7 +351,7 @@ let reduce_fix_use_function f whfun fix stack =
 	    whfun (recarg, empty_stack) in
         let stack' = stack_assign stack recargnum (app_stack recarg') in
 	(match kind_of_term recarg'hd with
-           | IsMutConstruct _ ->
+           | Construct _ ->
 	       Reduced (contract_fix_use_function f fix,stack')
 	   | _ -> NotReducible)
 
@@ -285,40 +364,38 @@ let contract_cofix_use_function f (bodynum,(_,names,bodies as typedbodies)) =
   let subbodies = list_tabulate make_Fi nbodies in
   substl subbodies bodies.(bodynum)
 
-let reduce_mind_case_use_function (sp,args) env mia =
+let reduce_mind_case_use_function sp env mia =
   match kind_of_term mia.mconstr with 
-    | IsMutConstruct(ind_sp,i as cstr_sp, args) ->
-	let ncargs = (fst mia.mci).(i-1) in
-	let real_cargs = list_lastn ncargs mia.mcargs in
+    | Construct(ind_sp,i as cstr_sp) ->
+	let real_cargs = snd (list_chop mia.mci.ci_npar mia.mcargs) in
 	applist (mia.mlf.(i-1), real_cargs)
-    | IsCoFix (_,(_,names,_) as cofix) ->
-	let names = Array.of_list names in
+    | CoFix (_,(names,_,_) as cofix) ->
 	let build_fix_name i =
 	  match names.(i) with 
 	    | Name id ->
-		let sp = make_path (dirpath sp) id (kind_of_path sp) in
-		(match constant_opt_value env (sp,args) with
+		let sp = make_path (dirpath sp) id in
+		(match constant_opt_value env sp with
 		  | None -> None
-		  | Some _ -> Some (mkConst (sp,args)))
+		  | Some _ -> Some (mkConst sp))
 	    | Anonymous -> None in 
 	let cofix_def = contract_cofix_use_function build_fix_name cofix in
-	mkMutCase (mia.mci, mia.mP, applist(cofix_def,mia.mcargs), mia.mlf)
+	mkCase (mia.mci, mia.mP, applist(cofix_def,mia.mcargs), mia.mlf)
     | _ -> assert false
 
 let special_red_case env whfun (ci, p, c, lf)  =
   let rec redrec s = 
     let (constr, cargs) = whfun s in 
     match kind_of_term constr with 
-      | IsConst (sp,args as cst) -> 
-	  (match constant_opt_value env cst with
-	     | Some gvalue ->
-		 if reducible_mind_case gvalue then
-		   reduce_mind_case_use_function cst env
-		     {mP=p; mconstr=gvalue; mcargs=list_of_stack cargs;
-		      mci=ci; mlf=lf}
-		 else
-		   redrec (gvalue, cargs)
-             | None -> raise Redelimination)
+      | Const cst -> 
+          (if not (is_evaluable env (EvalConstRef cst)) then
+            raise Redelimination;
+	  let gvalue = constant_value env cst in
+	  if reducible_mind_case gvalue then
+	    reduce_mind_case_use_function cst env
+	      {mP=p; mconstr=gvalue; mcargs=list_of_stack cargs;
+               mci=ci; mlf=lf}
+	  else
+	    redrec (gvalue, cargs))
       | _ ->
           if reducible_mind_case constr then
             reduce_mind_case
@@ -329,8 +406,8 @@ let special_red_case env whfun (ci, p, c, lf)  =
   in 
   redrec (c, empty_stack)
 
+
 let rec red_elim_const env sigma ref largs =
-  if not (evaluable_reference sigma env ref) then raise Redelimination;
   match reference_eval sigma env ref with
     | EliminationCases n when stack_args_size largs >= n ->
 	let c = reference_value sigma env ref in
@@ -344,7 +421,7 @@ let rec red_elim_const env sigma ref largs =
 	let co = construct_const env sigma in
 	(match reduce_fix_use_function f co (destFix d) lrest with
 	   | NotReducible -> raise Redelimination
-	   | Reduced (c,rest) -> (nf_beta env sigma c, rest))
+	   | Reduced (c,rest) -> (nf_beta c, rest))
     | EliminationMutualFix (min,refgoal,refinfos)
 	when stack_args_size largs >= min ->
 	let rec descend ref args =
@@ -360,38 +437,39 @@ let rec red_elim_const env sigma ref largs =
 	let co = construct_const env sigma in
 	(match reduce_fix_use_function f co (destFix d) lrest with
 	   | NotReducible -> raise Redelimination
-	   | Reduced (c,rest) -> (nf_beta env sigma c, rest))
+	   | Reduced (c,rest) -> (nf_beta c, rest))
     | _ -> raise Redelimination
 
 and construct_const env sigma = 
   let rec hnfstack (x, stack as s) =
     match kind_of_term x with
-      | IsCast (c,_) -> hnfstack (c, stack)
-      | IsApp (f,cl) -> hnfstack (f, append_stack cl stack)
-      | IsLambda (id,t,c) ->
+      | Cast (c,_) -> hnfstack (c, stack)
+      | App (f,cl) -> hnfstack (f, append_stack cl stack)
+      | Lambda (id,t,c) ->
           (match decomp_stack stack with 
              | None -> assert false
              | Some (c',rest) -> 
 		   stacklam hnfstack [c'] c rest)
-      | IsLetIn (n,b,t,c) -> stacklam hnfstack [b] c stack
-      | IsMutCase (ci,p,c,lf) ->
+      | LetIn (n,b,t,c) -> stacklam hnfstack [b] c stack
+      | Case (ci,p,c,lf) ->
           hnfstack 
-	    (special_red_case env (construct_const env sigma) (ci,p,c,lf), stack)
-      | IsMutConstruct _ -> s
-      | IsCoFix _ -> s
-      | IsFix fix -> 
+	    (special_red_case env
+              (construct_const env sigma) (ci,p,c,lf), stack)
+      | Construct _ -> s
+      | CoFix _ -> s
+      | Fix fix -> 
 	  (match reduce_fix hnfstack fix stack with
              | Reduced s' -> hnfstack s'
 	     | NotReducible -> raise Redelimination)
-      | _ when isEvalRef x ->
+      | _ when isEvalRef env x ->
 	  let ref = destEvalRef x in
           (try
-	     hnfstack (red_elim_const env sigma ref stack)
+	    hnfstack (red_elim_const env sigma ref stack)
            with Redelimination ->
 	     (match reference_opt_value sigma env ref with
 		| Some cval ->
 		    (match kind_of_term cval with
-                       | IsCoFix _ -> s
+                       | CoFix _ -> s
                        | _ -> hnfstack (cval, stack))
 		| None ->
 		    raise Redelimination))
@@ -405,17 +483,33 @@ and construct_const env sigma =
 (* Red reduction tactic: reduction to a product *)
 
 let internal_red_product env sigma c = 
+  let simpfun = clos_norm_flags betaiotazeta env sigma in
   let rec redrec env x =
     match kind_of_term x with
-      | IsApp (f,l) -> appvect (redrec env f, l)
-      | IsCast (c,_) -> redrec env c
-      | IsProd (x,a,b) -> mkProd (x, a, redrec (push_rel_assum (x,a) env) b)
-      | _ when isEvalRef x -> 
-	  (match reference_opt_value sigma env (destEvalRef x) with
+      | App (f,l) -> 
+          (match kind_of_term f with
+             | Fix fix ->
+                 let stack = append_stack l empty_stack in
+                 (match fix_recarg fix stack with
+                    | None -> raise Redelimination
+                    | Some (recargnum,recarg) ->
+                        let recarg' = redrec env recarg in
+                        let stack' = stack_assign stack recargnum recarg' in
+                        simpfun (app_stack (f,stack')))
+             | _ -> simpfun (appvect (redrec env f, l)))
+      | Cast (c,_) -> redrec env c
+      | Prod (x,a,b) -> mkProd (x, a, redrec (push_rel (x,None,a) env) b)
+      | LetIn (x,a,b,t) -> redrec env (subst1 a t)
+      | Case (ci,p,d,lf) -> simpfun (mkCase (ci,p,redrec env d,lf))
+      | _ when isEvalRef env x -> 
+          (* TO DO: re-fold fixpoints after expansion *)
+          (* to get true one-step reductions *)
+          let ref = destEvalRef x in
+	  (match reference_opt_value sigma env ref with
 	     | None -> raise Redelimination
 	     | Some c -> c)
       | _ -> raise Redelimination
-  in nf_betaiota env sigma (redrec env c)
+  in redrec env c
 
 let red_product env sigma c = 
   try internal_red_product env sigma c
@@ -426,36 +520,36 @@ let red_product env sigma c =
 let hnf_constr env sigma c = 
   let rec redrec (x, largs as s) =
     match kind_of_term x with
-      | IsLambda (n,t,c) ->
+      | Lambda (n,t,c) ->
           (match decomp_stack largs with
              | None      -> app_stack s
              | Some (a,rest) -> 
 		 stacklam redrec [a] c rest)
-      | IsLetIn (n,b,t,c) -> stacklam redrec [b] c largs
-      | IsApp (f,cl)   -> redrec (f, append_stack cl largs)
-      | IsCast (c,_) -> redrec (c, largs)
-      | IsMutCase (ci,p,c,lf) ->
+      | LetIn (n,b,t,c) -> stacklam redrec [b] c largs
+      | App (f,cl)   -> redrec (f, append_stack cl largs)
+      | Cast (c,_) -> redrec (c, largs)
+      | Case (ci,p,c,lf) ->
           (try
              redrec 
 	       (special_red_case env (whd_betadeltaiota_state env sigma) 
 		  (ci, p, c, lf), largs)
            with Redelimination -> 
 	     app_stack s)
-      | IsFix fix ->
+      | Fix fix ->
 	  (match reduce_fix (whd_betadeltaiota_state env sigma) fix largs with
              | Reduced s' -> redrec s'
 	     | NotReducible -> app_stack s)
-      | _ when isEvalRef x ->
+      | _ when isEvalRef env x ->
 	  let ref = destEvalRef x in
           (try
-             let (c',lrest) = red_elim_const env sigma ref largs in 
-	     redrec (c', lrest)
+            let (c',lrest) = red_elim_const env sigma ref largs in 
+	    redrec (c', lrest)
            with Redelimination ->
              match reference_opt_value sigma env ref with
 	       | Some c ->
-		   (match kind_of_term c with 
-                      | IsCoFix _ -> app_stack (x,largs)
-		      | _ ->  redrec (c, largs))
+		   (match kind_of_term (snd (decompose_lam c)) with 
+                      | CoFix _ | Fix _ -> app_stack (x,largs)
+		      | _ -> redrec (c, largs))
 	       | None -> app_stack s)
       | _ -> app_stack s
   in 
@@ -467,24 +561,24 @@ let hnf_constr env sigma c =
 let whd_nf env sigma c = 
   let rec nf_app (c, stack as s) =
     match kind_of_term c with
-      | IsLambda (name,c1,c2)    ->
+      | Lambda (name,c1,c2)    ->
           (match decomp_stack stack with
              | None -> (c,empty_stack)
              | Some (a1,rest) -> 
 		 stacklam nf_app [a1] c2 rest)
-      | IsLetIn (n,b,t,c) -> stacklam nf_app [b] c stack
-      | IsApp (f,cl) -> nf_app (f, append_stack cl stack)
-      | IsCast (c,_) -> nf_app (c, stack)
-      | IsMutCase (ci,p,d,lf) ->
+      | LetIn (n,b,t,c) -> stacklam nf_app [b] c stack
+      | App (f,cl) -> nf_app (f, append_stack cl stack)
+      | Cast (c,_) -> nf_app (c, stack)
+      | Case (ci,p,d,lf) ->
           (try
              nf_app (special_red_case env nf_app (ci,p,d,lf), stack)
            with Redelimination -> 
 	     s)
-      | IsFix fix ->
+      | Fix fix ->
 	  (match reduce_fix nf_app fix stack with
              | Reduced s' -> nf_app s'
 	     | NotReducible -> s)
-      | _ when isEvalRef c ->
+      | _ when isEvalRef env c ->
           (try
 	     nf_app (red_elim_const env sigma (destEvalRef c) stack)
            with Redelimination -> 
@@ -500,29 +594,29 @@ let nf env sigma c = strong whd_nf env sigma c
  * ol is the occurence list to find. *)
 let rec substlin env name n ol c =
   match kind_of_term c with
-    | IsConst (sp,_ as const) when EvalConstRef sp = name ->
+    | Const const when EvalConstRef const = name ->
         if List.hd ol = n then
           try 
 	    (n+1, List.tl ol, constant_value env const)
           with
 	      NotEvaluableConst _ ->
 		errorlabstrm "substlin"
-		  [< pr_sp sp; 'sTR " is not a defined constant" >]
+		  (pr_sp const ++ str " is not a defined constant")
         else 
 	  ((n+1), ol, c)
 
-    | IsVar id when EvalVarRef id = name ->
+    | Var id when EvalVarRef id = name ->
         if List.hd ol = n then
-          match lookup_named_value id env with
-	    | Some c -> (n+1, List.tl ol, c)
-	    | None -> 
+          match lookup_named id env with
+	    | (_,Some c,_) -> (n+1, List.tl ol, c)
+	    | _ -> 
 		errorlabstrm "substlin"
-		  [< pr_id id; 'sTR " is not a defined constant" >]
+		  (pr_id id ++ str " is not a defined constant")
         else 
 	  ((n+1), ol, c)
 
     (* INEFFICIENT: OPTIMIZE *)
-    | IsApp (c1,cl) ->
+    | App (c1,cl) ->
         Array.fold_left 
 	  (fun (n1,ol1,c1') c2 ->
 	     (match ol1 with 
@@ -532,7 +626,7 @@ let rec substlin env name n ol c =
                     (n2,ol2,applist(c1',[c2']))))
           (substlin env name n ol c1) cl
 
-    | IsLambda (na,c1,c2) ->
+    | Lambda (na,c1,c2) ->
         let (n1,ol1,c1') = substlin env name n ol c1 in
         (match ol1 with 
            | [] -> (n1,[],mkLambda (na,c1',c2))
@@ -540,7 +634,7 @@ let rec substlin env name n ol c =
                let (n2,ol2,c2') = substlin env name n1 ol1 c2 in
                (n2,ol2,mkLambda (na,c1',c2')))
 
-    | IsLetIn (na,c1,t,c2) ->
+    | LetIn (na,c1,t,c2) ->
         let (n1,ol1,c1') = substlin env name n ol c1 in
         (match ol1 with 
            | [] -> (n1,[],mkLetIn (na,c1',t,c2))
@@ -548,7 +642,7 @@ let rec substlin env name n ol c =
                let (n2,ol2,c2') = substlin env name n1 ol1 c2 in
                (n2,ol2,mkLetIn (na,c1',t,c2')))
 
-    | IsProd (na,c1,c2) ->
+    | Prod (na,c1,c2) ->
         let (n1,ol1,c1') = substlin env name n ol c1 in
         (match ol1 with 
            | [] -> (n1,[],mkProd (na,c1',c2))
@@ -556,7 +650,7 @@ let rec substlin env name n ol c =
                let (n2,ol2,c2') = substlin env name n1 ol1 c2 in
                (n2,ol2,mkProd (na,c1',c2')))
 	
-    | IsMutCase (ci,p,d,llf) -> 
+    | Case (ci,p,d,llf) -> 
         let rec substlist nn oll = function
           | []     -> (nn,oll,[])
           | f::lfe ->
@@ -569,16 +663,16 @@ let rec substlin env name n ol c =
 	in
 	let (n1,ol1,p') = substlin env name n ol p in  (* ATTENTION ERREUR *)
         (match ol1 with                                 (* si P pas affiche *)
-           | [] -> (n1,[],mkMutCase (ci, p', d, llf))
+           | [] -> (n1,[],mkCase (ci, p', d, llf))
            | _  ->
                let (n2,ol2,d') = substlin env name n1 ol1 d in
                (match ol2 with
-		  | [] -> (n2,[],mkMutCase (ci, p', d', llf))
+		  | [] -> (n2,[],mkCase (ci, p', d', llf))
 		  | _  -> 
 	              let (n3,ol3,lf') = substlist n2 ol2 (Array.to_list llf)
-                      in (n3,ol3,mkMutCaseL (ci, p', d', lf'))))
+                      in (n3,ol3,mkCase (ci, p', d', Array.of_list lf'))))
         
-    | IsCast (c1,c2)   ->
+    | Cast (c1,c2)   ->
         let (n1,ol1,c1') = substlin env name n ol c1 in
         (match ol1 with 
            | [] -> (n1,[],mkCast (c1',c2))
@@ -586,22 +680,25 @@ let rec substlin env name n ol c =
                let (n2,ol2,c2') = substlin env name n1 ol1 c2 in
                (n2,ol2,mkCast (c1',c2')))
 
-    | IsFix _ -> 
+    | Fix _ -> 
         (warning "do not consider occurrences inside fixpoints"; (n,ol,c))
 	
-    | IsCoFix _ -> 
+    | CoFix _ -> 
         (warning "do not consider occurrences inside cofixpoints"; (n,ol,c))
 
-    | (IsRel _|IsMeta _|IsVar _|IsSort _
-      |IsEvar _|IsConst _|IsMutInd _|IsMutConstruct _) -> (n,ol,c)
-	
-let unfold env sigma name =
-  clos_norm_flags (unfold_flags name) env sigma
+    | (Rel _|Meta _|Var _|Sort _
+      |Evar _|Const _|Ind _|Construct _) -> (n,ol,c)
 
 let string_of_evaluable_ref = function
   | EvalVarRef id -> string_of_id id
   | EvalConstRef sp -> string_of_path sp
 
+let unfold env sigma name =
+  if is_evaluable env name then
+    clos_norm_flags (unfold_red name) env sigma
+  else
+    error (string_of_evaluable_ref name^" is opaque")
+
 (* [unfoldoccs : (readable_constraints -> (int list * section_path) -> constr -> constr)]
  * Unfolds the constant name in a term c following a list of occurrences occl.
  * at the occurrences of occ_list. If occ_list is empty, unfold all occurences.
@@ -611,7 +708,7 @@ let unfoldoccs env sigma (occl,name) c =
     | []  -> unfold env sigma name c
     | l -> 
         match substlin env name 1 (Sort.list (<) l) c with
-          | (_,[],uc) -> nf_betaiota env sigma uc
+          | (_,[],uc) -> nf_betaiota uc
           | (1,_,_) -> error ((string_of_evaluable_ref name)^" does not occur")
           | _ -> error ("bad occurrence numbers of "
 			^(string_of_evaluable_ref name))
@@ -633,11 +730,11 @@ let fold_commands cl env sigma c =
 
 (* call by value reduction functions *)
 let cbv_norm_flags flags env sigma t =
-  cbv_norm (create_cbv_infos flags env sigma) t
+  cbv_norm (create_cbv_infos flags env) (nf_evar sigma t)
 
-let cbv_beta env = cbv_norm_flags beta env
-let cbv_betaiota env = cbv_norm_flags betaiota env
-let cbv_betadeltaiota env =  cbv_norm_flags betadeltaiota env
+let cbv_beta = cbv_norm_flags beta empty_env Evd.empty
+let cbv_betaiota = cbv_norm_flags betaiota empty_env Evd.empty
+let cbv_betadeltaiota env sigma =  cbv_norm_flags betadeltaiota env sigma
 
 let compute = cbv_betadeltaiota
 
@@ -665,8 +762,8 @@ type red_expr =
   | Red of bool
   | Hnf
   | Simpl
-  | Cbv of Closure.flags
-  | Lazy of Closure.flags
+  | Cbv of Closure.RedFlags.reds
+  | Lazy of Closure.RedFlags.reds
   | Unfold of (int list * evaluable_global_reference) list
   | Fold of constr list
   | Pattern of (int list * constr * constr) list
@@ -683,58 +780,65 @@ let reduction_of_redexp = function
 
 (* Used in several tactics. *)
 
+exception NotStepReducible
+
 let one_step_reduce env sigma c = 
   let rec redrec (x, largs as s) =
     match kind_of_term x with
-      | IsLambda (n,t,c)  ->
+      | Lambda (n,t,c)  ->
           (match decomp_stack largs with
-             | None      -> error "Not reducible 1"
+             | None      -> raise NotStepReducible
              | Some (a,rest) -> (subst1 a c, rest))
-      | IsApp (f,cl) -> redrec (f, append_stack cl largs)
-      | IsLetIn (_,f,_,cl) -> (subst1 f cl,largs)
-      | IsMutCase (ci,p,c,lf) ->
+      | App (f,cl) -> redrec (f, append_stack cl largs)
+      | LetIn (_,f,_,cl) -> (subst1 f cl,largs)
+      | Case (ci,p,c,lf) ->
           (try
 	     (special_red_case env (whd_betadeltaiota_state env sigma)
 	       (ci,p,c,lf), largs)
-           with Redelimination -> error "Not reducible 2")
-      | IsFix fix ->
+           with Redelimination -> raise NotStepReducible)
+      | Fix fix ->
 	  (match reduce_fix (whd_betadeltaiota_state env sigma) fix largs with
              | Reduced s' -> s'
-	     | NotReducible -> error "Not reducible 3")
-      | IsCast (c,_) -> redrec (c,largs)
-      | _ when isEvalRef x ->
-	  let ref = destEvalRef x in
+	     | NotReducible -> raise NotStepReducible)
+      | Cast (c,_) -> redrec (c,largs)
+      | _ when isEvalRef env x ->
+	  let ref =
+            try destEvalRef x
+            with Redelimination -> raise NotStepReducible in
           (try
-             red_elim_const env sigma ref largs
+            red_elim_const env sigma ref largs
            with Redelimination ->
 	     match reference_opt_value sigma env ref with
 	       | Some d -> d, largs
-	       | None -> error "Not reductible 1")
+	       | None -> raise NotStepReducible)
 
-      | _ -> error "Not reducible 3"
+      | _ -> raise NotStepReducible
   in 
   app_stack (redrec (c, empty_stack))
 
 (* put t as t'=(x1:A1)..(xn:An)B with B an inductive definition of name name
    return name, B and t' *)
 
-let reduce_to_mind env sigma t = 
+let reduce_to_ind_gen allow_product env sigma t = 
   let rec elimrec env t l = 
-    let c, _ = whd_stack t in
+    let c, _ = Reductionops.whd_stack t in
     match kind_of_term c with
-      | IsMutInd (mind,args) -> ((mind,args),t,it_mkProd_or_LetIn t l)
-      | IsProd (n,ty,t') ->
-	  elimrec (push_rel_assum (n,t) env) t' ((n,None,ty)::l)
-      | _ -> 
+      | Ind (mind,args) -> ((mind,args),it_mkProd_or_LetIn t l)
+      | Prod (n,ty,t') ->
+	  if allow_product then
+	    elimrec (push_rel (n,None,t) env) t' ((n,None,ty)::l)
+	  else
+	     errorlabstrm "tactics__reduce_to_mind"
+	       (str"Not an inductive definition")
+      | _ ->
           (try 
-	     let t' = nf_betaiota env sigma (one_step_reduce env sigma t) in 
+	     let t' = nf_betaiota (one_step_reduce env sigma t) in 
 	     elimrec env t' l
-           with UserError _ -> errorlabstrm "tactics__reduce_to_mind"
-               [< 'sTR"Not an inductive product." >])
- in
- elimrec env t []
-
-let reduce_to_ind env sigma t =
-  let ((ind_sp,_),redt,c) = reduce_to_mind env sigma t in 
-  (Declare.path_of_inductive_path ind_sp, redt, c)
+           with NotStepReducible ->
+	     errorlabstrm "tactics__reduce_to_mind"
+               (str"Not an inductive product"))
+  in
+  elimrec env t []
 
+let reduce_to_quantified_ind x = reduce_to_ind_gen true x
+let reduce_to_atomic_ind x = reduce_to_ind_gen false x
