@@ -6,14 +6,16 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: typing.ml,v 1.23 2001/03/15 13:38:37 filliatr Exp $ *)
+(* $Id: typing.ml,v 1.28 2001/11/12 12:38:04 barras Exp $ *)
 
 open Util
 open Names
 open Term
 open Environ
-open Reduction
+open Reductionops
 open Type_errors
+open Pretype_errors
+open Inductive
 open Typeops
 
 let vect_lift = Array.mapi lift
@@ -26,113 +28,111 @@ type 'a mach_flags = {
 (* The typing machine without information, without universes but with
    existential variables. *)
 
+let assumption_of_judgment env sigma j =
+  assumption_of_judgment env (j_nf_evar sigma j)
+
+let type_judgment env sigma j =
+  type_judgment env (j_nf_evar sigma j)
+
+
 let rec execute mf env sigma cstr =
   match kind_of_term cstr with
-    | IsMeta n ->
+    | Meta n ->
 	error "execute: found a non-instanciated goal"
 
-    | IsEvar ev ->
-	let ty = type_of_existential env sigma ev in
+    | Evar ev ->
+	let ty = Instantiate.existential_type sigma ev in
 	let jty = execute mf env sigma ty in
 	let jty = assumption_of_judgment env sigma jty in
 	{ uj_val = cstr; uj_type = jty }
 	
-    | IsRel n -> 
-	relative env sigma n
+    | Rel n -> 
+	judge_of_relative env n
 
-    | IsVar id -> 
-      (try
-         make_judge cstr (snd (lookup_named id env))
-       with Not_found ->
-         error ("execute: variable " ^ (string_of_id id) ^ " not defined"))
+    | Var id -> 
+        judge_of_variable env id
 	  
-    | IsConst c ->
-        make_judge cstr (type_of_constant env sigma c)
+    | Const c ->
+        make_judge cstr (constant_type env c)
 	  
-    | IsMutInd ind ->
-	make_judge cstr (type_of_inductive env sigma ind)
+    | Ind ind ->
+	make_judge cstr (type_of_inductive env ind)
 	  
-    | IsMutConstruct cstruct -> 
-	make_judge cstr (type_of_constructor env sigma cstruct)
+    | Construct cstruct -> 
+	make_judge cstr (type_of_constructor env cstruct)
 	  
-    | IsMutCase (ci,p,c,lf) ->
+    | Case (ci,p,c,lf) ->
         let cj = execute mf env sigma c in
         let pj = execute mf env sigma p in
         let lfj = execute_array mf env sigma lf in
-        type_of_case env sigma ci pj cj lfj
+        let (j,_) = judge_of_case env ci pj cj lfj in
+        j
   
-    | IsFix ((vn,i as vni),(lar,lfi,vdef)) ->
+    | Fix ((vn,i as vni),recdef) ->
         if (not mf.fix) && array_exists (fun n -> n < 0) vn then
           error "General Fixpoints not allowed";
-        let larjv,vdefv = execute_fix mf env sigma lar lfi vdef in
- 	let larv = Array.map body_of_type larjv in
-	let fix = vni,(larv,lfi,vdefv) in
-        check_fix env sigma fix;
-	make_judge (mkFix fix) larjv.(i)
+        let (_,tys,_ as recdef') = execute_recdef mf env sigma recdef in
+	let fix = (vni,recdef') in
+        check_fix env fix;
+	make_judge (mkFix fix) tys.(i)
 	  
-    | IsCoFix (i,(lar,lfi,vdef)) ->
-        let (larjv,vdefv) = execute_fix mf env sigma lar lfi vdef in
-	let larv = Array.map body_of_type larjv in
-        let cofix = i,(larv,lfi,vdefv) in
-        check_cofix env sigma cofix;
-	make_judge (mkCoFix cofix) larjv.(i)
+    | CoFix (i,recdef) ->
+        let (_,tys,_ as recdef') = execute_recdef mf env sigma recdef in
+        let cofix = (i,recdef') in
+        check_cofix env cofix;
+	make_judge (mkCoFix cofix) tys.(i)
 	  
-    | IsSort (Prop c) -> 
+    | Sort (Prop c) -> 
 	judge_of_prop_contents c
 
-    | IsSort (Type u) ->
-	let (j,_) = judge_of_type u in j
+    | Sort (Type u) ->
+	judge_of_type u
 	  
-    | IsApp (f,args) ->
+    | App (f,args) ->
 	let j = execute mf env sigma f in
-        let jl = execute_list mf env sigma (Array.to_list args) in
-	let (j,_) = apply_rel_list env sigma mf.nocheck jl j in
+        let jl = execute_array mf env sigma args in
+	let (j,_) = judge_of_apply env j jl in
 	j
 	    
-    | IsLambda (name,c1,c2) -> 
+    | Lambda (name,c1,c2) -> 
         let j = execute mf env sigma c1 in
-	let var = assumption_of_judgment env sigma j in
-	let env1 = push_rel_assum (name,var) env in
+	let var = type_judgment env sigma j in
+	let env1 = push_rel (name,None,var.utj_val) env in
         let j' = execute mf env1 sigma c2 in 
-        let (j,_) = abs_rel env1 sigma name var j' in
-	j
+        judge_of_abstraction env1 name var j'
 	  
-    | IsProd (name,c1,c2) ->
+    | Prod (name,c1,c2) ->
         let j = execute mf env sigma c1 in
         let varj = type_judgment env sigma j in
-	let env1 = push_rel_assum (name,varj.utj_val) env in
+	let env1 = push_rel (name,None,varj.utj_val) env in
         let j' = execute mf env1 sigma c2 in
         let varj' = type_judgment env sigma j' in
-	let (j,_) = gen_rel env1 sigma name varj varj' in
-	j
+	judge_of_product env1 name varj varj'
 
-     | IsLetIn (name,c1,c2,c3) ->
+     | LetIn (name,c1,c2,c3) ->
         let j1 = execute mf env sigma c1 in
         let j2 = execute mf env sigma c2 in
-	let tj2 = assumption_of_judgment env sigma j2 in
-	let { uj_val = b; uj_type = t },_ = cast_rel env sigma j1 tj2 in
-        let j3 = execute mf (push_rel_def (name,b,t) env) sigma c3 in
-	{ uj_val = mkLetIn (name, j1.uj_val, tj2, j3.uj_val) ;
-	  uj_type = type_app (subst1 j1.uj_val) j3.uj_type }
+        let j2 = type_judgment env sigma j2 in
+        let env1 = push_rel (name,Some j1.uj_val,j2.utj_val) env in
+        let j3 = execute mf env1 sigma c3 in
+        let (j,_) = judge_of_letin env name j1 j2 j3 in
+        j
   
-    | IsCast (c,t) ->
+    | Cast (c,t) ->
         let cj = execute mf env sigma c in
         let tj = execute mf env sigma t in
-	let tj = assumption_of_judgment env sigma tj in
-        let j, _ = cast_rel env sigma cj tj in
+	let tj = type_judgment env sigma tj in
+        let j, _ = judge_of_cast env cj tj in
 	j
-	  
-and execute_fix mf env sigma lar lfi vdef =
+
+and execute_recdef mf env sigma (names,lar,vdef) =
   let larj = execute_array mf env sigma lar in
   let lara = Array.map (assumption_of_judgment env sigma) larj in
-  let nlara = 
-    List.combine (List.rev lfi) (Array.to_list (vect_lift_type lara)) in
-  let env1 = 
-    List.fold_left (fun env nvar -> push_rel_assum nvar env) env nlara in
+  let env1 = push_rec_types (names,lara,vdef) env in
   let vdefj = execute_array mf env1 sigma vdef in
   let vdefv = Array.map j_val vdefj in
-  let cst3 = type_fixpoint env1 sigma lfi lara vdefj in
-  (lara,vdefv)
+  let _ = type_fixpoint env1 names lara vdefj in
+  (names,lara,vdefv)
 
 and execute_array mf env sigma v =
   let jl = execute_list mf env sigma (Array.to_list v) in
@@ -159,7 +159,7 @@ let unsafe_machine env sigma constr =
 
 let type_of env sigma c = 
   let j = safe_machine env sigma c in 
-  nf_betaiota env sigma (body_of_type j.uj_type)
+  nf_betaiota (body_of_type j.uj_type)
 
 (* The typed type of a judgment. *)
 
