@@ -6,19 +6,22 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: clenv.ml,v 1.47 2001/03/28 15:11:18 barras Exp $ *)
+(* $Id: clenv.ml,v 1.61 2001/12/17 19:35:34 herbelin Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Term
+open Termops
 open Sign
 open Instantiate
 open Environ
 open Evd
 open Proof_type
+open Proof_trees
 open Logic
-open Reduction
+open Reductionops
 open Tacmach
 open Evar_refiner
 
@@ -39,7 +42,7 @@ let exist_to_meta (emap, c) =
   List.iter add_binding emap;
   let rec replace c =
     match kind_of_term c with
-        IsEvar k -> List.assoc k !subst
+        Evar k -> List.assoc k !subst
       | _ -> map_constr replace c in
   (!mmap, replace c)
 
@@ -58,7 +61,7 @@ type 'a clausenv = {
   env : clbinding Intmap.t;
   hook : 'a }
 
-type wc = walking_constraints
+type wc = named_context sigma
 
 let applyHead n c wc = 
   let rec apprec n c cty wc =
@@ -66,22 +69,30 @@ let applyHead n c wc =
       (wc,c)
     else 
       match kind_of_term (w_whd_betadeltaiota wc cty) with
-        | IsProd (_,c1,c2) ->
-            let c1ty = w_type_of wc c1 in
+        | Prod (_,c1,c2) ->
 	    let evar = Evarutil.new_evar_in_sign (w_env wc) in
             let (evar_n, _) = destEvar evar in
 	    (compose 
 	       (apprec (n-1) (applist(c,[evar])) (subst1 evar c2))
-	       (w_Declare evar_n (c1,c1ty)))
+	       (w_Declare evar_n c1))
 	    wc
 	| _ -> error "Apply_Head_Then"
   in 
   apprec n c (w_type_of wc c) wc
 
 let mimick_evar hdc nargs sp wc =
-  (w_Focusing_THEN sp
+  let evd = Evd.map wc.sigma sp in
+  let wc' = extract_decl sp wc in
+  let (wc'', c) = applyHead nargs hdc wc' in
+    if wc'==wc''
+    then w_Define sp c wc
+    else 
+      let wc''' = restore_decl sp evd wc'' in
+	w_Define sp c {it = wc.it ; sigma = wc'''.sigma}
+
+(*  (w_Focusing_THEN sp
      (applyHead nargs hdc)
-     (fun c wc -> w_Define sp c wc)) wc
+     (fun c wc -> w_Define sp c wc)) wc *)
 
 
 (* Unification  l'ordre 0 de m et n: [unify_0 mc wc m n] renvoie deux listes:
@@ -92,91 +103,92 @@ let mimick_evar hdc nargs sp wc =
    Attention : pas d'unification entre les diffrences instances d'une
    mme meta ou evar, il peut rester des doublons *)
 
-let unify_0 mc wc m n =
+let unify_0 cv_pb mc wc m n =
   let env = w_env wc
   and sigma = w_Underlying wc in
-  let rec unirec_rec ((metasubst,evarsubst) as substn) m n =
-    let cM = whd_ise1 sigma m
-    and cN = whd_ise1 sigma n in 
+  let rec unirec_rec pb ((metasubst,evarsubst) as substn) m n =
+    let cM = Evarutil.whd_castappevar sigma m
+    and cN = Evarutil.whd_castappevar sigma n in 
     try 
       match (kind_of_term cM,kind_of_term cN) with
-	| IsCast (c,_), _ -> unirec_rec substn c cN
-	| _, IsCast (c,_) -> unirec_rec substn cM c
-	| IsMeta k1, IsMeta k2 ->
+	| Cast (c,_), _ -> unirec_rec pb substn c cN
+	| _, Cast (c,_) -> unirec_rec pb substn cM c
+	| Meta k1, Meta k2 ->
 	    if k1 < k2 then (k1,cN)::metasubst,evarsubst
 	    else if k1 = k2 then substn
 	    else (k2,cM)::metasubst,evarsubst
-	| IsMeta k, _    -> (k,cN)::metasubst,evarsubst
-	| _, IsMeta k    -> (k,cM)::metasubst,evarsubst
-	| IsLambda (_,t1,c1), IsLambda (_,t2,c2) ->
-	    unirec_rec (unirec_rec substn t1 t2) c1 c2
-	| IsProd (_,t1,c1), IsProd (_,t2,c2) ->
-	    unirec_rec (unirec_rec substn t1 t2) c1 c2
+	| Meta k, _    -> (k,cN)::metasubst,evarsubst
+	| _, Meta k    -> (k,cM)::metasubst,evarsubst
+	| Lambda (_,t1,c1), Lambda (_,t2,c2) ->
+	    unirec_rec CONV (unirec_rec CONV substn t1 t2) c1 c2
+	| Prod (_,t1,c1), Prod (_,t2,c2) ->
+	    unirec_rec pb (unirec_rec CONV substn t1 t2) c1 c2
 
-	| IsApp (f1,l1), IsApp (f2,l2) ->
+	| App (f1,l1), App (f2,l2) ->
 	    let len1 = Array.length l1
 	    and len2 = Array.length l2 in
 	    if len1 = len2 then
-              array_fold_left2 unirec_rec (unirec_rec substn f1 f2) l1 l2
+              array_fold_left2 (unirec_rec CONV)
+		(unirec_rec CONV substn f1 f2) l1 l2
 	    else if len1 < len2 then
               let extras,restl2 = array_chop (len2-len1) l2 in 
-	      array_fold_left2 unirec_rec 
-		(unirec_rec substn f1 (appvect (f2,extras)))
+	      array_fold_left2 (unirec_rec CONV)
+		(unirec_rec CONV substn f1 (appvect (f2,extras)))
                 l1 restl2
 	    else 
 	      let extras,restl1 = array_chop (len1-len2) l1 in 
-	      array_fold_left2 unirec_rec 
-		(unirec_rec substn (appvect (f1,extras)) f2)
+	      array_fold_left2 (unirec_rec CONV)
+		(unirec_rec CONV substn (appvect (f1,extras)) f2)
                 restl1 l2
 		
-	| IsMutCase (_,p1,c1,cl1), IsMutCase (_,p2,c2,cl2) ->
-            array_fold_left2 unirec_rec 
-	      (unirec_rec (unirec_rec substn p1 p2) c1 c2) cl1 cl2
+	| Case (_,p1,c1,cl1), Case (_,p2,c2,cl2) ->
+            array_fold_left2 (unirec_rec CONV)
+	      (unirec_rec CONV (unirec_rec CONV substn p1 p2) c1 c2) cl1 cl2
 		
-	| IsMutConstruct _, IsMutConstruct _ ->
+	| Construct _, Construct _ ->
 	    if is_conv env sigma cM cN then
 	      substn
 	    else
-	      error_cannot_unify CCI (m,n)
+	      error_cannot_unify (m,n)
 	      
-	| IsMutInd _, IsMutInd _ ->
+	| Ind _, Ind _ ->
 	    if is_conv env sigma  cM cN then
 	      substn
 	    else
-	      error_cannot_unify CCI (m,n)
+	      error_cannot_unify (m,n)
 	      
-	| IsEvar _, _ ->
+	| Evar _, _ ->
 	    metasubst,((cM,cN)::evarsubst)
-	| _, IsEvar _ ->
+	| _, Evar _ ->
 	    metasubst,((cN,cM)::evarsubst)
 
-	| (IsConst _ | IsVar _ | IsRel _), _ ->
+	| (Const _ | Var _ | Rel _), _ ->
 	    if is_conv env sigma cM cN then
 	      substn
 	    else 
-	      error_cannot_unify CCI (m,n)
+	      error_cannot_unify (m,n)
 		
-	| _, (IsConst _ | IsVar _| IsRel _) ->
+	| _, (Const _ | Var _| Rel _) ->
 	    if (not (occur_meta cM)) & is_conv env sigma cM cN then 
 	      substn
 	    else 
-	      error_cannot_unify CCI (m,n)
+	      error_cannot_unify (m,n)
 
-	| IsLetIn (_,b,_,c), _ -> unirec_rec substn (subst1 b c) cN
+	| LetIn (_,b,_,c), _ -> unirec_rec pb substn (subst1 b c) cN
 		
-	| _ -> error_cannot_unify CCI (m,n)
+	| _ -> error_cannot_unify (m,n)
 	      
     with ex when catchable_exception ex ->
-      if (not(occur_meta cM)) & is_conv env sigma cM cN then 
+      if (not(occur_meta cM)) & is_fconv pb env sigma cM cN then 
 	substn
       else 
 	raise ex
 
     in
-    if (not(occur_meta m)) & is_conv env sigma m n then 
+    if (not(occur_meta m)) & is_fconv cv_pb env sigma m n then 
       (mc,[])
     else 
-      unirec_rec (mc,[]) m n
+      unirec_rec cv_pb (mc,[]) m n
 
 
 (* Unification
@@ -228,8 +240,8 @@ let unify_0 mc wc m n =
  * close it off.  But this might not always work,
  * since other metavars might also need to be resolved. *)
 
-let rec w_Unify m n mc wc =
-  let (mc',ec') = unify_0 mc wc m n in 
+let rec w_Unify cv_pb m n mc wc =
+  let (mc',ec') = unify_0 cv_pb mc wc m n in 
   w_resrec mc' ec' wc
 
 and w_resrec metas evars wc =
@@ -239,21 +251,21 @@ and w_resrec metas evars wc =
     | (lhs,rhs) :: t ->
     match kind_of_term rhs with
 
-      | IsMeta k -> w_resrec ((k,lhs)::metas) t wc
+      | Meta k -> w_resrec ((k,lhs)::metas) t wc
 
       | krhs ->
       match kind_of_term lhs with
 
-	| IsEvar (evn,_) ->
+	| Evar (evn,_) ->
 	    if w_defined_evar wc evn then
-	      let (wc',metas') = w_Unify rhs lhs metas wc in 
+	      let (wc',metas') = w_Unify CONV rhs lhs metas wc in 
 	      w_resrec metas' t wc'
 	    else
 	      (try 
 		 w_resrec metas t (w_Define evn rhs wc)
                with ex when catchable_exception ex ->
 		 (match krhs with
-             	    | IsApp (f,cl) when isConst f ->
+             	    | App (f,cl) when isConst f ->
 			let wc' = mimick_evar f (Array.length cl) evn wc in
 			w_resrec metas evars wc'
 		    | _ -> error "w_Unify"))
@@ -265,7 +277,10 @@ and w_resrec metas evars wc =
    d'une mme Meta sont compatibles. D'ailleurs le "fst" jette les metas
    provenant de w_Unify. (Utilis seulement dans prolog.ml) *)
 
-let unifyTerms m n = walking (fun wc -> fst (w_Unify m n [] wc))
+(* let unifyTerms m n = walking (fun wc -> fst (w_Unify CONV m n [] wc)) *)
+let unifyTerms m n gls = 
+  tclIDTAC {it = gls.it; 
+	    sigma = (get_gc (fst (w_Unify CONV m n [] (Refiner.project_with_focus gls))))}
 
 let unify m gls =
   let n = pf_concl gls in unifyTerms m n gls
@@ -276,7 +291,7 @@ let unify m gls =
 let collect_metas c = 
   let rec collrec acc c =
     match kind_of_term c with
-      | IsMeta mv -> mv::acc
+      | Meta mv -> mv::acc
       | _         -> fold_constr collrec acc c
   in
   List.rev (collrec [] c)
@@ -284,7 +299,7 @@ let collect_metas c =
 let metavars_of c =  
   let rec collrec acc c =
     match kind_of_term c with
-      | IsMeta mv -> Intset.add mv acc
+      | Meta mv -> Intset.add mv acc
       | _         -> fold_constr collrec acc c
   in 
   collrec Intset.empty c
@@ -326,8 +341,8 @@ let clenv_environments bound c =
   let rec clrec (ne,e,metas) n c =
     match n, kind_of_term c with
       | (0, _) -> (ne, e, List.rev metas, c)
-      | (n, IsCast (c,_)) -> clrec (ne,e,metas) n c
-      | (n, IsProd (na,c1,c2)) ->
+      | (n, Cast (c,_)) -> clrec (ne,e,metas) n c
+      | (n, Prod (na,c1,c2)) ->
 	  let mv = new_meta () in
 	  let dep = dependent (mkRel 1) c2 in
 	  let ne' =
@@ -347,11 +362,19 @@ let clenv_environments bound c =
 	  let e' = Intmap.add mv (Cltyp (mk_freelisted c1)) e in 
 	  clrec (ne',e', (mkMeta mv)::metas) (n-1)
 	    (if dep then (subst1 (mkMeta mv) c2) else c2)
-      | (n, IsLetIn (na,b,_,c)) -> clrec (ne,e,metas) (n-1) (subst1 b c)
+      | (n, LetIn (na,b,_,c)) -> clrec (ne,e,metas) (n-1) (subst1 b c)
       | (n, _) -> (ne, e, List.rev metas, c)
   in 
   clrec (Intmap.empty,Intmap.empty,[]) bound c
 
+let mk_clenv_from_n wc n (c,cty) =
+  let (namenv,env,args,concl) = clenv_environments n cty in
+  { templval = mk_freelisted (match args with [] -> c | _ -> applist (c,args));
+    templtyp = mk_freelisted concl;
+    namenv = namenv;
+    env = env;
+    hook = wc }
+
 let mk_clenv_from wc (c,cty) =
   let (namenv,env,args,concl) = clenv_environments (-1) cty in
   { templval = mk_freelisted (match args with [] -> c | _ -> applist (c,args));
@@ -379,13 +402,17 @@ let mk_clenv_hnf_constr_type_of wc t =
   mk_clenv_from wc (t,w_hnf_constr wc (w_type_of wc t))
 
 let mk_clenv_rename_from wc (c,t) = 
-  mk_clenv_from wc (c,rename_bound_var [] t)
+  mk_clenv_from wc (c,rename_bound_var (w_env wc) [] t)
+
+let mk_clenv_rename_from_n wc n (c,t) = 
+  mk_clenv_from_n wc n (c,rename_bound_var (w_env wc) [] t)
     
 let mk_clenv_rename_type_of wc t =
-  mk_clenv_from wc (t,rename_bound_var [] (w_type_of wc t))
+  mk_clenv_from wc (t,rename_bound_var (w_env wc) [] (w_type_of wc t))
     
 let mk_clenv_rename_hnf_constr_type_of wc t =
-  mk_clenv_from wc (t,rename_bound_var [] (w_hnf_constr wc (w_type_of wc t)))
+  mk_clenv_from wc
+    (t,rename_bound_var (w_env wc) [] (w_hnf_constr wc (w_type_of wc t)))
 
 let mk_clenv_type_of wc t = mk_clenv_from wc (t,w_type_of wc t)
 			      
@@ -403,8 +430,8 @@ let clenv_assign mv rhs clenv =
 	       (* Streams are lazy, force evaluation of id to catch Not_found*)
 	       let id = Intmap.find mv clenv.namenv in
 	       errorlabstrm "clenv_assign"
-		 [< 'sTR "An incompatible instantiation has already been found for "; 
-		    pr_id id >]
+		 (str "An incompatible instantiation has already been found for " ++ 
+		    pr_id id)
 	     with Not_found ->
 	       anomaly "clenv_assign: non dependent metavar already assigned"
 	   else
@@ -451,13 +478,13 @@ let clenv_instance_term clenv c =
 let clenv_cast_meta clenv = 
   let rec crec u =
     match kind_of_term u with
-      | IsApp _ | IsMutCase _ -> crec_hd u
-      | IsCast (c,_) when isMeta c -> u
+      | App _ | Case _ -> crec_hd u
+      | Cast (c,_) when isMeta c -> u
       | _  -> map_constr crec u
 	    
   and crec_hd u =
     match kind_of_term (strip_outer_cast u) with
-      | IsMeta mv ->
+      | Meta mv ->
 	  (try 
 	     match Intmap.find mv clenv.env with
                | Cltyp b ->
@@ -466,9 +493,9 @@ let clenv_cast_meta clenv =
 	       | Clval(_) -> u
 	   with Not_found -> 
 	     u)
-      | IsApp(f,args) -> mkApp (crec_hd f, Array.map crec args)
-      | IsMutCase(ci,p,c,br) ->
-	  mkMutCase (ci, crec_hd p, crec_hd c, Array.map crec br)
+      | App(f,args) -> mkApp (crec_hd f, Array.map crec args)
+      | Case(ci,p,c,br) ->
+	  mkCase (ci, crec_hd p, crec_hd c, Array.map crec br)
       | _ -> u
   in 
   crec
@@ -552,14 +579,14 @@ let clenv_merge with_types =
       | ((lhs,rhs)::t, metas) ->
       (match kind_of_term rhs with
 
-	| IsMeta k -> clenv_resrec ((k,lhs)::metas) t clenv
+	| Meta k -> clenv_resrec ((k,lhs)::metas) t clenv
 
 	| krhs ->
         (match kind_of_term lhs with
 
-	  | IsEvar (evn,_) ->
+	  | Evar (evn,_) ->
     	      if w_defined_evar clenv.hook evn then
-		let (metas',evars') = unify_0 [] clenv.hook rhs lhs in
+		let (metas',evars') = unify_0 CONV [] clenv.hook rhs lhs in
 		clenv_resrec (metas'@metas) (evars'@t) clenv
     	      else begin
 		let rhs' = 
@@ -571,7 +598,7 @@ let clenv_merge with_types =
 		    (clenv_wtactic (w_Define evn rhs') clenv)
 		with ex when catchable_exception ex ->
 		  (match krhs with
-		     | IsApp (f,cl) when isConst f or isMutConstruct f ->
+		     | App (f,cl) when isConst f or isConstruct f ->
 			 clenv_resrec metas evars 
 			   (clenv_wtactic
 			      (mimick_evar f (Array.length cl) evn)
@@ -584,7 +611,7 @@ let clenv_merge with_types =
       | ([], (mv,n)::t) ->
     	  if clenv_defined clenv mv then
             let (metas',evars') =
-              unify_0 [] clenv.hook (clenv_value clenv mv).rebus n in
+              unify_0 CONV [] clenv.hook (clenv_value clenv mv).rebus n in
             clenv_resrec (metas'@t) evars' clenv
     	  else
 	    let mc,ec =
@@ -593,7 +620,7 @@ let clenv_merge with_types =
 		(try 
 		let nty = clenv_type_of clenv 
 			    (clenv_instance clenv (mk_freelisted n)) in
-		unify_0 [] clenv.hook mvty nty
+		unify_0 CUMUL [] clenv.hook nty mvty
 		with e when Logic.catchable_exception e -> ([],[]))
 	      else ([],[]) in
 	    clenv_resrec (mc@t) ec (clenv_assign mv n clenv)
@@ -610,8 +637,8 @@ let clenv_merge with_types =
    [clenv_typed_unify M N clenv] expects in addition that expected
    types of metavars are unifiable with the types of their instances    *)
 
-let clenv_unify_core with_types m n clenv =
-  let (mc,ec) = unify_0 [] clenv.hook m n in 
+let clenv_unify_core with_types cv_pb m n clenv =
+  let (mc,ec) = unify_0 cv_pb [] clenv.hook m n in 
   clenv_merge with_types mc ec clenv
     
 (* let clenv_unify = clenv_unify_core false *)
@@ -651,7 +678,7 @@ let clenv_bchain mv subclenv clenv =
   in
   (* unify the type of the template of [subclenv] with the type of [mv] *)
   let clenv'' =
-    clenv_unify (clenv_instance clenv' (clenv_template_type subclenv))
+    clenv_unify CUMUL (clenv_instance clenv' (clenv_template_type subclenv))
       (clenv_instance_type clenv' mv)
       clenv' 
   in
@@ -712,15 +739,12 @@ let constrain_clenv_to_subterm clause (op,cl) =
     let cl = strip_outer_cast cl in
     (try 
        if closed0 cl 
-       then clenv_unify op cl clause,cl
+       then clenv_unify CONV op cl clause,cl
        else error "Bound 1"
      with ex when catchable_exception ex ->
        (match kind_of_term cl with 
-	  | IsApp (f,args) ->  (try 
-		 matchrec f
-	       with ex when catchable_exception ex -> 
-		 iter_fail matchrec args)
-	(* let n = Array.length args in
+	  | App (f,args) ->
+	      let n = Array.length args in
 	      assert (n>0);
 	      let c1 = mkApp (f,Array.sub args 0 (n-1)) in
 	      let c2 = args.(n-1) in
@@ -728,36 +752,35 @@ let constrain_clenv_to_subterm clause (op,cl) =
 		 matchrec c1
 	       with ex when catchable_exception ex -> 
 		 matchrec c2)
-	   *)
-          | IsMutCase(_,_,c,lf) -> (* does not search in the predicate *)
+          | Case(_,_,c,lf) -> (* does not search in the predicate *)
 	       (try 
 		 matchrec c
 	       with ex when catchable_exception ex -> 
 		 iter_fail matchrec lf)
-	  | IsLetIn(_,c1,_,c2) -> 
+	  | LetIn(_,c1,_,c2) -> 
 	       (try 
 		 matchrec c1
 	       with ex when catchable_exception ex -> 
 		 matchrec c2)
 
-	  | IsFix(_,(types,_,terms)) -> 
+	  | Fix(_,(_,types,terms)) -> 
 	       (try 
 		 iter_fail matchrec types
 	       with ex when catchable_exception ex -> 
 		 iter_fail matchrec terms)
 	
-	  | IsCoFix(_,(types,_,terms)) -> 
+	  | CoFix(_,(_,types,terms)) -> 
 	       (try 
 		 iter_fail matchrec types
 	       with ex when catchable_exception ex -> 
 		 iter_fail matchrec terms)
 
-          | IsProd (_,t,c) ->
+          | Prod (_,t,c) ->
 	      (try 
 		 matchrec t 
 	       with ex when catchable_exception ex -> 
 		 matchrec c)
-          | IsLambda (_,t,c) ->
+          | Lambda (_,t,c) ->
 	      (try 
 		 matchrec t 
 	       with ex when catchable_exception ex -> 
@@ -848,7 +871,7 @@ let clenv_constrain_missing_args mlist clause =
     let occlist = clenv_missing clause (clenv_template clause,
                                    	(clenv_template_type clause)) in 
     if List.length occlist = List.length mlist then
-      List.fold_left2 (fun clenv occ m -> clenv_unify (mkMeta occ) m clenv)
+      List.fold_left2 (fun clenv occ m -> clenv_unify CONV (mkMeta occ) m clenv)
         clause occlist mlist
     else 
       error ("Not the right number of missing arguments (expected "
@@ -861,7 +884,7 @@ let clenv_constrain_dep_args mlist clause =
     let occlist = clenv_dependent clause (clenv_template clause,
 					  (clenv_template_type clause)) in 
     if List.length occlist = List.length mlist then
-      List.fold_left2 (fun clenv occ m -> clenv_unify (mkMeta occ) m clenv)
+      List.fold_left2 (fun clenv occ m -> clenv_unify CONV (mkMeta occ) m clenv)
         clause occlist mlist
     else 
       error ("Not the right number of missing arguments (expected "
@@ -874,7 +897,7 @@ let clenv_constrain_dep_args_of mv mlist clause =
     let occlist = clenv_dependent clause (clenv_value clause mv,
                                           clenv_type clause mv) in 
     if List.length occlist = List.length mlist then
-      List.fold_left2 (fun clenv occ m -> clenv_unify (mkMeta occ) m clenv)
+      List.fold_left2 (fun clenv occ m -> clenv_unify CONV (mkMeta occ) m clenv)
         clause occlist mlist
     else 
       error ("clenv_constrain_dep_args_of: Not the right number " ^ 
@@ -884,7 +907,7 @@ let clenv_lookup_name clenv id =
   match intmap_inv clenv.namenv id with
     | []  -> 
 	errorlabstrm "clenv_lookup_name"
-          [< 'sTR"No such bound variable "; pr_id id >]
+          (str"No such bound variable " ++ pr_id id)
     | [n] -> 
 	n
     |  _  -> 
@@ -903,23 +926,23 @@ let clenv_match_args s clause =
              | Dep s ->
 		 if List.mem_assoc b t then 
 		   errorlabstrm "clenv_match_args" 
-		     [< 'sTR "The variable "; pr_id s; 
-			'sTR " occurs more than once in binding" >]
+		     (str "The variable " ++ pr_id s ++ 
+			str " occurs more than once in binding")
 		 else
 		   clenv_lookup_name clause s
 	     | NoDep n ->
 		 if List.mem_assoc b t then errorlabstrm "clenv_match_args"
-		   [< 'sTR "The position "; 'iNT n ;
-		      'sTR " occurs more than once in binding" >];
+		   (str "The position " ++ int n ++
+		      str " occurs more than once in binding");
 		 (try 
 		    List.nth mvs (n-1)
-		  with Failure "item" -> errorlabstrm "clenv_match_args"
-                      [< 'sTR"No such binder" >])
+		  with Failure "nth" -> errorlabstrm "clenv_match_args"
+                      (str "No such binder"))
 	     | Com -> anomaly "No free term in clenv_match_args")
 	in
 	let k_typ = w_hnf_constr clause.hook (clenv_instance_type clause k)
 	and c_typ = w_hnf_constr clause.hook (w_type_of clause.hook c) in
-	matchrec (clenv_assign k c (clenv_unify k_typ c_typ clause)) t
+	matchrec (clenv_assign k c (clenv_unify CUMUL c_typ k_typ clause)) t
   in 
   matchrec clause s
 
@@ -938,9 +961,7 @@ let clenv_pose_dependent_evars clenv =
        let evar = Evarutil.new_evar_in_sign (w_env clenv.hook) in
        let (evar_n,_) = destEvar evar in
        let tY = clenv_instance_type clenv mv in
-       let tYty = w_type_of clenv.hook tY in
-       let clenv' = clenv_wtactic (w_Declare evar_n (tY,tYty))
-		      clenv in
+       let clenv' = clenv_wtactic (w_Declare evar_n tY) clenv in
        clenv_assign mv evar clenv')
     clenv
     dep_mvs
@@ -963,10 +984,10 @@ let clenv_add_sign (id,sign) clenv =
    lambda-abstraction head, then we second-order unification. *)
 
 let clenv_fo_resolver clenv gls =
-  clenv_unify (clenv_instance_template_type clenv) (pf_concl gls) clenv
+  clenv_unify CUMUL (clenv_instance_template_type clenv) (pf_concl gls) clenv
 
 let clenv_typed_fo_resolver clenv gls =
-  clenv_typed_unify (clenv_instance_template_type clenv) (pf_concl gls) clenv
+  clenv_typed_unify CUMUL (clenv_instance_template_type clenv) (pf_concl gls) clenv
 
 (* if lname_typ is [xn,An;..;x1,A1] and l is a list of terms,
    gives [x1:A1]..[xn:An]c' such that c converts to ([x1:A1]..[xn:An]c' l) *)
@@ -985,7 +1006,7 @@ let abstract_list_all env sigma typ c l =
   let lname_typ,_ = splay_prod env sigma typ in
   let p = abstract_scheme env c (List.map (function a -> [],a) l) lname_typ in 
   try 
-    if is_conv env sigma (Typing.type_of env sigma p) typ then p
+    if is_conv_leq env sigma (Typing.type_of env sigma p) typ then p
     else error "abstract_list_all"
   with UserError _ ->
     raise (RefinerError (CannotGeneralize typ))
@@ -994,14 +1015,14 @@ let secondOrderAbstraction allow_K gl p oplist clause =
   let (clause',cllist) = 
     constrain_clenv_using_subterm_list allow_K clause oplist (pf_concl gl) in
   let typp = clenv_instance_type clause' p in 
-  clenv_unify (mkMeta p)
+  clenv_unify CONV (mkMeta p)
     (abstract_list_all (pf_env gl) (project gl) typp (pf_concl gl) cllist) 
     clause'
 
 let clenv_so_resolver allow_K clause gl =
   let c, oplist = whd_stack (clenv_instance_template_type clause) in
   match kind_of_term c with
-    | IsMeta p ->
+    | Meta p ->
 	let clause' = secondOrderAbstraction allow_K gl p oplist clause in 
 	clenv_fo_resolver clause' gl
     | _ -> error "clenv_so_resolver"
@@ -1021,7 +1042,7 @@ let clenv_unique_resolver allow_K clenv gls =
   let pathd,_ = whd_stack (clenv_instance_template_type clenv) in
   let glhd,_ = whd_stack (pf_concl gls) in
   match kind_of_term pathd, kind_of_term glhd with
-    | IsMeta _, IsLambda _ ->
+    | Meta _, Lambda _ ->
 	(try 
 	   clenv_typed_fo_resolver clenv gls
 	 with ex when catchable_exception ex -> 
@@ -1030,7 +1051,7 @@ let clenv_unique_resolver allow_K clenv gls =
 	   with ex when catchable_exception ex -> 
 	     error "Cannot solve a second-order unification problem")
 
-    | IsMeta _, _ -> 
+    | Meta _, _ -> 
 	(try 
 	   clenv_so_resolver allow_K clenv gls
          with ex when catchable_exception ex -> 
@@ -1060,18 +1081,18 @@ let e_res_pf kONT clenv gls =
 let collect_com lbind = 
   map_succeed (function (Com,c)->c | _ -> failwith "Com") lbind
 
-let make_clenv_binding_apply wc (c,t) lbind = 
+let make_clenv_binding_apply wc n (c,t) lbind = 
   let largs = collect_com lbind in
   let lcomargs = List.length largs in
   if lcomargs = List.length lbind then 
-    let clause = mk_clenv_from wc (c,t) in
+    let clause = mk_clenv_from_n wc n (c,t) in
     clenv_constrain_missing_args largs clause
   else if lcomargs = 0 then 
-    let clause = mk_clenv_rename_from wc (c,t) in
+    let clause = mk_clenv_rename_from_n wc n (c,t) in
     clenv_match_args lbind clause
   else 
     errorlabstrm "make_clenv_bindings"
-      [<'sTR "Cannot mix bindings and free associations">]
+      (str "Cannot mix bindings and free associations")
 
 let make_clenv_binding wc (c,t) lbind = 
   let largs    = collect_com lbind in
@@ -1084,7 +1105,7 @@ let make_clenv_binding wc (c,t) lbind =
     clenv_match_args lbind clause
   else 
     errorlabstrm "make_clenv_bindings"
-      [<'sTR "Cannot mix bindings and free associations">]
+      (str "Cannot mix bindings and free associations")
 
 open Printer
 
@@ -1092,15 +1113,15 @@ let pr_clenv clenv =
   let pr_name mv =
     try 
       let id = Intmap.find mv clenv.namenv in
-      [< 'sTR"[" ; pr_id id ; 'sTR"]" >]
-    with Not_found -> [< >]
+      (str"[" ++ pr_id id ++ str"]")
+    with Not_found -> (mt ())
   in
   let pr_meta_binding = function
     | (mv,Cltyp b) ->
-      	hOV 0 [< 'iNT mv ; pr_name mv ; 'sTR " : " ; prterm b.rebus ; 'fNL >]
+      	hov 0 (int mv ++ pr_name mv ++ str " : " ++ prterm b.rebus ++ fnl ())
     | (mv,Clval(b,_)) ->
-      	hOV 0 [< 'iNT mv ; pr_name mv ; 'sTR " := " ; prterm b.rebus ; 'fNL >]
+      	hov 0 (int mv ++ pr_name mv ++ str " := " ++ prterm b.rebus ++ fnl ())
   in
-  [< 'sTR"TEMPL: " ; prterm clenv.templval.rebus ;
-     'sTR" : " ; prterm clenv.templtyp.rebus ; 'fNL ;
-     (prlist pr_meta_binding (intmap_to_list clenv.env)) >]
+  (str"TEMPL: " ++ prterm clenv.templval.rebus ++
+     str" : " ++ prterm clenv.templtyp.rebus ++ fnl () ++
+     (prlist pr_meta_binding (intmap_to_list clenv.env)))
