@@ -6,17 +6,20 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: logic.ml,v 1.50 2001/04/14 03:16:52 delahaye Exp $ *)
+(* $Id: logic.ml,v 1.63 2001/12/13 09:03:09 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Evd
 open Term
+open Termops
 open Sign
 open Environ
-open Reduction
+open Reductionops
 open Inductive
+open Inductiveops
 open Typing
 open Proof_trees
 open Proof_type
@@ -31,10 +34,10 @@ open Evarutil
 variables only in Application and Case *)
 
 let collect_meta_variables c = 
-  let rec collrec acc c = match splay_constr c with
-    | OpMeta mv, _ -> mv::acc
-    | OpCast, [|c;_|] -> collrec acc c
-    | (OpApp | OpMutCase _), cl -> Array.fold_left collrec acc cl
+  let rec collrec acc c = match kind_of_term c with
+    | Meta mv -> mv::acc
+    | Cast(c,_) -> collrec acc c
+    | (App _| Case _) -> fold_constr collrec acc c
     | _ -> acc
   in 
   List.rev(collrec [] c)
@@ -64,7 +67,7 @@ let catchable_exception = function
     Nametab.GlobalizationError _)) -> true
   | _ -> false
 
-let error_cannot_unify k (m,n) =
+let error_cannot_unify (m,n) =
   raise (RefinerError (CannotUnify (m,n)))
 
 let check = ref true
@@ -91,25 +94,24 @@ let rec mk_refgoals sigma goal goalacc conclty trm =
   else
 *)
   match kind_of_term trm with
-    | IsMeta _ ->
+    | Meta _ ->
 	if occur_meta conclty then
           raise (RefinerError (OccurMetaGoal conclty));
-	let ctxt = out_some goal.evar_info in 
-	(mk_goal ctxt hyps (nf_betaiota env sigma conclty))::goalacc, conclty
+	(mk_goal hyps (nf_betaiota conclty))::goalacc, conclty
 
-    | IsCast (t,ty) ->
+    | Cast (t,ty) ->
 	let _ = type_of env sigma ty in
 	conv_leq_goal env sigma trm ty conclty;
 	mk_refgoals sigma goal goalacc ty t
 
-    | IsApp (f,l) ->
+    | App (f,l) ->
 	let (acc',hdty) = mk_hdgoals sigma goal goalacc f in
 	let (acc'',conclty') =
 	  mk_arggoals sigma goal acc' hdty (Array.to_list l) in
 	let _ = conv_leq_goal env sigma trm conclty' conclty in
         (acc'',conclty')
 
-    | IsMutCase (_,p,c,lf) ->
+    | Case (_,p,c,lf) ->
 	let (acc',lbrty,conclty') = mk_casegoals sigma goal goalacc p c in
 	let acc'' = 
 	  array_fold_left2
@@ -132,16 +134,15 @@ and mk_hdgoals sigma goal goalacc trm =
   let env = evar_env goal in
   let hyps = goal.evar_hyps in
   match kind_of_term trm with
-    | IsCast (c,ty) when isMeta c ->
+    | Cast (c,ty) when isMeta c ->
 	let _ = type_of env sigma ty in
-	let ctxt = out_some goal.evar_info in  
-	(mk_goal ctxt hyps (nf_betaiota env sigma ty))::goalacc,ty
+	(mk_goal hyps (nf_betaiota ty))::goalacc,ty
 
-    | IsApp (f,l) ->
+    | App (f,l) ->
 	let (acc',hdty) = mk_hdgoals sigma goal goalacc f in
 	mk_arggoals sigma goal acc' hdty (Array.to_list l)
 
-    | IsMutCase (_,p,c,lf) ->
+    | Case (_,p,c,lf) ->
 	let (acc',lbrty,conclty') = mk_casegoals sigma goal goalacc p c in
 	let acc'' = 
 	  array_fold_left2
@@ -157,10 +158,10 @@ and mk_arggoals sigma goal goalacc funty = function
   | harg::tlargs as allargs ->
       let t = whd_betadeltaiota (evar_env goal) sigma funty in
       match kind_of_term t with
-	| IsProd (_,c1,b) ->
+	| Prod (_,c1,b) ->
 	    let (acc',hargty) = mk_refgoals sigma goal goalacc c1 harg in
 	    mk_arggoals sigma goal acc' (subst1 harg b) tlargs
-	| IsLetIn (_,c1,_,b) ->
+	| LetIn (_,c1,_,b) ->
 	    mk_arggoals sigma goal goalacc (subst1 c1 b) allargs
 	| _ -> raise (RefinerError (CannotApply (t,harg)))
 
@@ -168,17 +169,19 @@ and mk_casegoals sigma goal goalacc p c =
   let env = evar_env goal in
   let (acc',ct) = mk_hdgoals sigma goal goalacc c in 
   let (acc'',pt) = mk_hdgoals sigma goal acc' p in
+  let pj = {uj_val=p; uj_type=pt} in 
   let indspec =
-    try find_rectype env sigma ct
+    try find_mrectype env sigma ct
     with Induc -> anomaly "mk_casegoals" in
-  let (lbrty,conclty) = type_case_branches env sigma indspec pt p c in
+  let (lbrty,conclty) =
+    type_case_branches_with_names env indspec pj c in
   (acc'',lbrty,conclty)
 
 
 let error_use_instantiate () =
   errorlabstrm "Logic.prim_refiner"
-    [< 'sTR"cannot intro when there are open metavars in the domain type";
-       'sPC; 'sTR"- use Instantiate" >]
+    (str"cannot intro when there are open metavars in the domain type" ++
+       spc () ++ str"- use Instantiate")
     
 (* Auxiliary functions for primitive MOVE tactic
  *
@@ -200,10 +203,11 @@ let split_sign hfrom hto l =
   splitrec [] false l
 
 let move_after with_dep toleft (left,(idfrom,_,_ as declfrom),right) hto =
+  let env = Global.env() in
   let test_dep (hyp,c,typ as d) (hyp2,c,typ2 as d2) =
     if toleft
-    then occur_var_in_decl hyp2 d
-    else occur_var_in_decl hyp d2
+    then occur_var_in_decl env hyp2 d
+    else occur_var_in_decl env hyp d2
   in
   let rec moverec first middle = function
     | [] -> error ("No such hypothesis : " ^ (string_of_id hto))
@@ -240,14 +244,29 @@ let apply_to_hyp sign id f =
 	   found := true; f sign d tail
 	 end else 
 	   add_named_decl d sign)
-      sign empty_named_context
+      sign ~init:empty_named_context
   in
   if (not !check) || !found then sign' else error "No such assumption"
 
+let apply_to_hyp2 env id f =
+  let found = ref false in
+  let env' =
+    fold_named_context_both_sides
+      (fun env (idc,c,ct as d) tail ->
+	 if idc = id then begin
+	   found := true; f env d tail
+	 end else 
+	   push_named_decl d env)
+      (named_context env) ~init:(reset_context env)
+  in
+  if (not !check) || !found then env' else error "No such assumption"
+
 let global_vars_set_of_var = function
-  | (_,None,t) -> global_vars_set (body_of_type t)
+  | (_,None,t) -> global_vars_set (Global.env()) (body_of_type t)
   | (_,Some c,t) ->
-      Idset.union (global_vars_set (body_of_type t)) (global_vars_set c)
+      let env =Global.env() in
+      Idset.union (global_vars_set env (body_of_type t))
+        (global_vars_set env c)
 
 let check_backward_dependencies sign d =
   if not (Idset.for_all
@@ -257,20 +276,45 @@ let check_backward_dependencies sign d =
     error "Can't introduce at that location: free variable conflict"
 
 let check_forward_dependencies id tail =
+  let env = Global.env() in
   List.iter
     (function (id',_,_ as decl) ->
-       if occur_var_in_decl id decl then
+       if occur_var_in_decl env id decl then
 	 error ((string_of_id id) ^ " is used in the hypothesis " 
 		^ (string_of_id id')))
     tail
 
-let convert_hyp sign sigma id ty =
+let convert_hyp sign sigma id b =
   apply_to_hyp sign id
     (fun sign (idc,c,ct) _ ->
        let env = Global.env_of_context sign in
-       if !check && not (is_conv env sigma ty (body_of_type ct)) then
-	 error "convert-hyp rule passed non-converting term";
-       add_named_decl (idc,c,ty) sign)
+       match c with
+	 | None -> (* Change the type *)
+	     if !check && not (is_conv env sigma b ct) then
+	       error "convert-hyp rule passed non-converting term";
+	     add_named_decl (idc,c,b) sign
+	 | Some c -> (* Change the body *)
+	     if !check && not (is_conv env sigma b c) then
+	       error "convert-hyp rule passed non-converting term";
+	     add_named_decl (idc,Some b,ct) sign)
+
+let convert_def inbody sign sigma id c =
+  apply_to_hyp sign id
+    (fun sign (idc,b,t) _ ->
+       let env = Global.env_of_context sign in
+       match b with
+	 | None -> error "convert-deftype rule passed to an hyp without body" 
+	 | Some b ->
+	     let b,t = 
+	       if inbody then
+		 (if !check && not (is_conv env sigma c b) then 
+		    error "convert-deftype rule passed non-converting type";
+		  (c,t))
+	       else
+		 (if !check && not (is_conv env sigma c t) then
+		    error "convert-deftype rule passed non-converting type";
+		  (b,c)) in
+	     add_named_decl (idc,Some b,t) sign)
 
 let replace_hyp sign id d =
   apply_to_hyp sign id
@@ -292,27 +336,56 @@ let remove_hyp env id =
        if !check then check_forward_dependencies id tail;
        env)
 
+let recheck_typability (what,id) env sigma t =
+  try let _ = type_of env sigma t in ()
+  with _ ->
+    let s = match what with
+      | None -> "the conclusion"
+      | Some id -> "hypothesis "^(string_of_id id) in
+    error
+      ("The correctness of "^s^" relies on the body of "^(string_of_id id))
+  
+let remove_hyp_body env sigma id =
+  apply_to_hyp2 env id
+    (fun env (_,c,t) tail ->
+       match c with
+         | None -> error ((string_of_id id)^" is not a local definition")
+         | Some c ->
+             let env' = push_named_decl (id,None,t) env in
+             if !check then 
+               ignore
+                 (Sign.fold_named_context 
+                    (fun (id',c,t as d) env'' ->
+                       (match c with
+                          | None -> 
+                              recheck_typability (Some id',id) env'' sigma t
+                          | Some b ->
+                              let b' = mkCast (b,t) in
+                              recheck_typability (Some id',id) env'' sigma b');
+                       push_named_decl d env'')
+                    tail ~init:env');
+             env')
+
 (* Primitive tactics are handled here *)
 
 let prim_refiner r sigma goal =
   let env = evar_env goal in
   let sign = goal.evar_hyps in
   let cl = goal.evar_concl in
-  let info = out_some goal.evar_info in
   match r with
     | { name = Intro; newids = [id] } ->
     	if !check && mem_named_context id sign then
 	  error "New variable is already declared";
         (match kind_of_term (strip_outer_cast cl) with
-	   | IsProd (_,c1,b) ->
+	   | Prod (_,c1,b) ->
 	       if occur_meta c1 then error_use_instantiate();
-	       let sg = mk_goal info (add_named_assum (id,c1) sign)
+	       let sg = mk_goal (add_named_decl (id,None,c1) sign)
 			  (subst1 (mkVar id) b) in
 	       [sg]
-	   | IsLetIn (_,c1,t1,b) ->
+	   | LetIn (_,c1,t1,b) ->
 	       if occur_meta c1 or occur_meta t1 then error_use_instantiate();
 	       let sg =
-		 mk_goal info (add_named_def (id,c1,t1) sign)
+		 mk_goal (add_named_decl (id,Some c1,t1) sign)
 		   (subst1 (mkVar id) b) in
 	       [sg]
 	   | _ ->
@@ -323,15 +396,15 @@ let prim_refiner r sigma goal =
     	if !check && mem_named_context id sign then
 	  error "New variable is already declared";
         (match kind_of_term (strip_outer_cast cl) with
-	   | IsProd (_,c1,b) ->
+	   | Prod (_,c1,b) ->
 	       if occur_meta c1 then error_use_instantiate();
 	       let sign' = insert_after_hyp sign whereid (id,None,c1) in
-	       let sg = mk_goal info sign' (subst1 (mkVar id) b) in 
+	       let sg = mk_goal sign' (subst1 (mkVar id) b) in 
 	       [sg]
-	   | IsLetIn (_,c1,t1,b) ->
+	   | LetIn (_,c1,t1,b) ->
 	       if occur_meta c1 or occur_meta t1 then error_use_instantiate();
 	       let sign' = insert_after_hyp sign whereid (id,Some c1,t1) in
-	       let sg = mk_goal info sign' (subst1 (mkVar id) b) in 
+	       let sg = mk_goal sign' (subst1 (mkVar id) b) in 
 	       [sg]
 	   | _ ->
 	       if !check then error "Introduction needs a product"
@@ -339,25 +412,31 @@ let prim_refiner r sigma goal =
 	
     | { name = Intro_replacing; newids = []; hypspecs = [id] } ->
 	(match kind_of_term (strip_outer_cast cl) with
-           | IsProd (_,c1,b) ->
+           | Prod (_,c1,b) ->
 	       if occur_meta c1 then error_use_instantiate();
 	       let sign' = replace_hyp sign id (id,None,c1) in
-	       let sg = mk_goal info sign' (subst1 (mkVar id) b) in
+	       let sg = mk_goal sign' (subst1 (mkVar id) b) in
 	       [sg]
-           | IsLetIn (_,c1,t1,b) ->
+           | LetIn (_,c1,t1,b) ->
 	       if occur_meta c1 then error_use_instantiate();
 	       let sign' = replace_hyp sign id (id,Some c1,t1) in
-	       let sg = mk_goal info sign' (subst1 (mkVar id) b) in
+	       let sg = mk_goal sign' (subst1 (mkVar id) b) in
 	       [sg]
 	   | _ ->
 	       if !check then error "Introduction needs a product"
 	       else anomaly "Intro_replacing: expects a product")
 	
-    | { name = Fix; hypspecs = []; terms = []; 
+    | { name = Cut b; terms = [t]; newids = [id] } ->
+        if occur_meta t then error_use_instantiate();
+        let sg1 = mk_goal sign (nf_betaiota t) in
+        let sg2 = mk_goal (add_named_decl (id,None,t) sign) cl in
+        if b then [sg1;sg2] else [sg2;sg1]  
+
+    | { name = FixRule; hypspecs = []; terms = []; 
 	newids = [f]; params = [Num(_,n)] } ->
      	let rec check_ind k cl = 
           match kind_of_term (strip_outer_cast cl) with 
-            | IsProd (_,c1,b) -> 
+            | Prod (_,c1,b) -> 
             	if k = 1 then 
 		  try 
 		    let _ = find_inductive env sigma c1 in ()
@@ -370,13 +449,13 @@ let prim_refiner r sigma goal =
      	check_ind n cl;
 	if !check && mem_named_context f sign then
 	  error ("The name "^(string_of_id f)^" is already used");
-        let sg = mk_goal info (add_named_assum (f,cl) sign) cl in
+        let sg = mk_goal (add_named_decl (f,None,cl) sign) cl in
         [sg]
     
-    | { name = Fix; hypspecs = []; terms = lar; newids = lf; params = ln } ->
+    | { name = FixRule; hypspecs = []; terms = lar; newids = lf; params = ln } ->
      	let rec check_ind k cl = 
           match kind_of_term (strip_outer_cast cl) with 
-            | IsProd (_,c1,b) -> 
+            | Prod (_,c1,b) -> 
             	if k = 1 then 
 		  try 
 		    fst (find_inductive env sigma c1)
@@ -396,9 +475,9 @@ let prim_refiner r sigma goal =
 		       "mutual inductive declaration");
 	      if mem_named_context f sign then 
 		error "name already used in the environment";
-	      mk_sign (add_named_assum (f,ar) sign) (lar',lf',ln')
+	      mk_sign (add_named_decl (f,None,ar) sign) (lar',lf',ln')
 	  | ([],[],[]) -> 
-	      List.map (mk_goal info sign) (cl::lar)
+	      List.map (mk_goal sign) (cl::lar)
 	  | _ -> error "not the right number of arguments"
 	in 
 	mk_sign sign (cl::lar,lf,ln)
@@ -407,7 +486,7 @@ let prim_refiner r sigma goal =
      	let rec check_is_coind cl = 
 	  let b = whd_betadeltaiota env sigma cl in
           match kind_of_term b with
-            | IsProd (_,c1,b) -> check_is_coind  b
+            | Prod (_,c1,b) -> check_is_coind  b
             | _ -> 
 		try 
 		  let _ = find_coinductive env sigma b in ()
@@ -419,11 +498,12 @@ let prim_refiner r sigma goal =
         let rec mk_sign sign = function 
           | (ar::lar'),(f::lf') ->
 	      (try
-                (let _ = lookup_id f sign in
+                (let _ = Sign.lookup_named f sign in
                 error "name already used in the environment")
               with
-              |	Not_found -> mk_sign (add_named_assum (f,ar) sign) (lar',lf'))
-	  | ([],[]) -> List.map (mk_goal info sign) (cl::lar)
+              |	Not_found ->
+                  mk_sign (add_named_decl (f,None,ar) sign) (lar',lf'))
+	  | ([],[]) -> List.map (mk_goal sign) (cl::lar)
 	  | _ -> error "not the right number of arguments"
      	in 
 	mk_sign sign (cl::lar,lf)
@@ -438,22 +518,38 @@ let prim_refiner r sigma goal =
     | { name = Convert_concl; terms = [cl'] } ->
     	let cl'ty = type_of env sigma cl' in
 	if is_conv_leq env sigma cl' cl then
-          let sg = mk_goal info sign cl' in
+          let sg = mk_goal sign cl' in
           [sg]
 	else 
 	  error "convert-concl rule passed non-converting term"
 
-    | { name = Convert_hyp; hypspecs = [id]; terms = [ty'] } ->
-	[mk_goal info (convert_hyp sign sigma id ty') cl]
+    | { name = Convert_hyp; hypspecs = [id]; terms = [ty] } ->
+	[mk_goal (convert_hyp sign sigma id ty) cl]
+
+    | { name = Convert_defbody; hypspecs = [id]; terms = [c] } ->
+	[mk_goal (convert_def true sign sigma id c) cl]
+
+    | { name = Convert_deftype; hypspecs = [id]; terms = [ty] } ->
+	[mk_goal (convert_def false sign sigma id ty) cl]
 
     | { name = Thin; hypspecs = ids } ->
 	let clear_aux sign id =
-          if !check && occur_var id cl then
+          if !check && occur_var env id cl then
             error ((string_of_id id) ^ " is used in the conclusion.");
           remove_hyp sign id 
 	in
 	let sign' = List.fold_left clear_aux sign ids in
-     	let sg = mk_goal info sign' cl in
+     	let sg = mk_goal sign' cl in
+     	[sg]
+
+    | { name = ThinBody; hypspecs = ids } ->
+	let clear_aux env id =
+          let env' = remove_hyp_body env sigma id in
+          if !check then recheck_typability (None,id) env' sigma cl;
+          env'
+	in
+	let sign' = named_context (List.fold_left clear_aux env ids) in
+     	let sg = mk_goal sign' cl in
      	[sg]
 
     | { name = Move withdep; hypspecs = ids } ->
@@ -462,7 +558,7 @@ let prim_refiner r sigma goal =
   	let (left,right,declfrom,toleft) = split_sign hfrom hto sign in
   	let hyps' = 
 	  move_after withdep toleft (left,declfrom,right) hto in
-  	[mk_goal info hyps' cl]
+  	[mk_goal hyps' cl]
 	
     | _ -> anomaly "prim_refiner: Unrecognized primitive rule"
 
@@ -471,10 +567,10 @@ let prim_extractor subfun vl pft =
   match pft with
     | { ref = Some (Prim { name = Intro; newids = [id] }, [spf]) } ->
 	(match kind_of_term (strip_outer_cast cl) with
-	   | IsProd (_,ty,_) ->
+	   | Prod (_,ty,_) ->
 	       let cty = subst_vars vl ty in
 	       mkLambda (Name id, cty, subfun (id::vl) spf)
-	   | IsLetIn (_,b,ty,_) ->
+	   | LetIn (_,b,ty,_) ->
 	       let cb = subst_vars vl b in
 	       let cty = subst_vars vl ty in
 	       mkLetIn (Name id, cb, cty, subfun (id::vl) spf)
@@ -482,10 +578,10 @@ let prim_extractor subfun vl pft =
 	
     | { ref = Some (Prim { name = Intro_after; newids = [id]}, [spf]) } -> 
 	(match kind_of_term (strip_outer_cast cl) with
-	   | IsProd (_,ty,_) ->
+	   | Prod (_,ty,_) ->
 	       let cty = subst_vars vl ty in
 	       mkLambda (Name id, cty, subfun (id::vl) spf)
-	   | IsLetIn (_,b,ty,_) ->
+	   | LetIn (_,b,ty,_) ->
 	       let cb = subst_vars vl b in
 	       let cty = subst_vars vl ty in
 	       mkLetIn (Name id, cb, cty, subfun (id::vl) spf)
@@ -493,38 +589,42 @@ let prim_extractor subfun vl pft =
 	
     | {ref=Some(Prim{name=Intro_replacing;hypspecs=[id]},[spf]) } ->
 	(match kind_of_term (strip_outer_cast cl) with
-	   | IsProd (_,ty,_) ->
+	   | Prod (_,ty,_) ->
 	       let cty = subst_vars vl ty in
 	       mkLambda (Name id, cty, subfun (id::vl) spf)
-	   | IsLetIn (_,b,ty,_) ->
+	   | LetIn (_,b,ty,_) ->
 	       let cb = subst_vars vl b in
 	       let cty = subst_vars vl ty in
 	       mkLetIn (Name id, cb, cty, subfun (id::vl) spf)
 	   | _ -> error "incomplete proof!")
-	
-    | {ref=Some(Prim{name=Fix;newids=[f];params=[Num(_,n)]},[spf]) } -> 
+
+    | {ref=Some(Prim{name=Cut b;terms=[t];newids=[id]},[spf1;spf2]) } ->
+        let spf1, spf2 = if b then spf1, spf2 else spf2, spf1 in
+	mkLetIn (Name id,subfun vl spf1,subst_vars vl t,subfun (id::vl) spf2)
+
+    | {ref=Some(Prim{name=FixRule;newids=[f];params=[Num(_,n)]},[spf]) } -> 
 	let cty = subst_vars vl cl in 
 	let na = Name f in 
-	mkFix (([|n-1|],0),([|cty|], [na], [|subfun (f::vl) spf|]))
+	mkFix (([|n-1|],0),([|na|], [|cty|], [|subfun (f::vl) spf|]))
 
-    | {ref=Some(Prim{name=Fix;newids=lf;terms=lar;params=ln},spfl) } ->
+    | {ref=Some(Prim{name=FixRule;newids=lf;terms=lar;params=ln},spfl) } ->
 	let lcty = List.map (subst_vars vl) (cl::lar) in 
 	let vn = 
 	  Array.of_list (List.map (function Num(_,n) -> n-1
 				     | _ -> anomaly "mutual_fix_refiner")
 			   ln) 
 	in 
-	let lna = List.map (fun f -> Name f) lf in
+	let names = Array.map (fun f -> Name f) (Array.of_list lf) in
 	let newvl = List.fold_left (fun vl id -> (id::vl)) vl lf  in 
 	let lfix =Array.map (subfun newvl) (Array.of_list spfl) in
-	mkFix ((vn,0),(Array.of_list lcty,lna,lfix))	
+	mkFix ((vn,0),(names,Array.of_list lcty,lfix))	
 
     | {ref=Some(Prim{name=Cofix;newids=lf;terms=lar},spfl) } ->
 	let lcty = List.map (subst_vars vl) (cl::lar) in 
-	let lna  = List.map (fun f -> Name f) lf in
+	let names  = Array.map (fun f -> Name f) (Array.of_list lf) in
 	let newvl = List.fold_left (fun vl id -> (id::vl)) vl lf in 
 	let lfix =Array.map (subfun newvl) (Array.of_list spfl) in
-	mkCoFix (0,(Array.of_list lcty,lna,lfix))
+	mkCoFix (0,(names,Array.of_list lcty,lfix))
 	  
     | {ref=Some(Prim{name=Refine;terms=[c]},spfl) } ->
 	let mvl = collect_meta_variables c in
@@ -535,13 +635,22 @@ let prim_extractor subfun vl pft =
     | {ref=Some(Prim{name=Convert_concl;terms=[c]},[pf])} ->
 	subfun vl pf
 	
-    | {ref=Some(Prim{name=Convert_hyp;hypspecs=[id];terms=[c]},[pf])} ->
+    | {ref=Some(Prim{name=Convert_hyp;hypspecs=[id];terms=[_]},[pf])} ->
+	subfun vl pf
+
+    | {ref=Some(Prim{name=Convert_defbody;hypspecs=[id];terms=[_]},[pf])} ->
+	subfun vl pf
+
+    | {ref=Some(Prim{name=Convert_deftype;hypspecs=[id];terms=[_]},[pf])} ->
 	subfun vl pf
 	
     | {ref=Some(Prim{name=Thin;hypspecs=ids},[pf])} ->
      (* No need to make ids Anonymous in vl: subst_vars take the more recent *)
 	subfun vl pf
 	
+    | {ref=Some(Prim{name=ThinBody;hypspecs=ids},[pf])} ->
+	subfun vl pf
+	
     | {ref=Some(Prim{name=Move _;hypspecs=ids},[pf])} ->
 	subfun vl pf
 	  
@@ -558,51 +667,67 @@ open Printer
 
 let pr_prim_rule = function
   | {name=Intro;newids=[id]} -> 
-      [< 'sTR"Intro " ; pr_id id >]
+      str"Intro " ++ pr_id id
 	
   | {name=Intro_after;newids=[id]} -> 
-      [< 'sTR"intro after " ; pr_id id >]
+      str"intro after "  ++ pr_id id
 	
   | {name=Intro_replacing;newids=[id]} -> 
-      [< 'sTR"intro replacing " ; pr_id id >]
+      (str"intro replacing "  ++ pr_id id)
 	
-  | {name=Fix;newids=[f];params=[Num(_,n)]} -> 
-      [< 'sTR"Fix "; pr_id f; 'sTR"/"; 'iNT n>]
+  | {name=Cut b;terms=[t];newids=[id]} ->
+      if b then
+        (str"TrueCut " ++ prterm t)
+      else
+        (str"Cut " ++ prterm t ++ str ";[Intro " ++ pr_id id ++ str "|Idtac]")
+	
+  | {name=FixRule;newids=[f];params=[Num(_,n)]} -> 
+      (str"Fix " ++ pr_id f ++ str"/" ++ int n)
       
-  | {name=Fix;newids=(f::lf);params=(Num(_,n))::ln;terms=lar} -> 
+  | {name=FixRule;newids=(f::lf);params=(Num(_,n))::ln;terms=lar} -> 
       let rec print_mut = 
         function (f::lf),((Num(_,n))::ln),(ar::lar) -> 
-          [< pr_id f; 'sTR"/"; 'iNT n; 'sTR" : "; prterm ar;
-             print_mut (lf,ln,lar)>]
-          | _ -> [<>] in
-      [< 'sTR"Fix "; pr_id f; 'sTR"/"; 'iNT n;
-         'sTR" with "; print_mut (lf,ln,lar) >]
+          pr_id f ++ str"/" ++ int n ++ str" : " ++ prterm ar ++
+          print_mut (lf,ln,lar)
+          | _ -> (mt ()) in
+      (str"Fix " ++ pr_id f ++ str"/" ++ int n ++
+         str" with " ++ print_mut (lf,ln,lar))
       
   | {name=Cofix;newids=[f];terms=[]} -> 
-      [< 'sTR"Cofix "; pr_id f >]
+      (str"Cofix " ++ pr_id f)
       
   | {name=Cofix;newids=(f::lf);terms=lar} -> 
       let rec print_mut = 
         function (f::lf),(ar::lar) -> 
-	  [< pr_id f; 'sTR" : "; prterm ar; print_mut (lf,lar)>]
-          | _ -> [<>] 
+	  (pr_id f ++ str" : " ++ prterm ar ++ print_mut (lf,lar))
+          | _ -> (mt ()) 
       in
-      [< 'sTR"Cofix "; pr_id f;  'sTR" with "; print_mut (lf,lar) >]
+      (str"Cofix " ++ pr_id f ++  str" with " ++ print_mut (lf,lar))
 
   | {name=Refine;terms=[c]} -> 
-      [< 'sTR(if occur_meta c then "Refine " else "Exact ") ; prterm c >]
+      (str(if occur_meta c then "Refine " else "Exact ")  ++ prterm c)
       
   | {name=Convert_concl;terms=[c]} ->
-      [< 'sTR"Change " ; prterm c >]
+      (str"Change "  ++ prterm c)
       
   | {name=Convert_hyp;hypspecs=[id];terms=[c]} ->
-      [< 'sTR"Change " ; prterm c ; 'sPC ; 'sTR"in " ; pr_id id >]
+      (str"Change "  ++ prterm c  ++ spc ()  ++ str"in "  ++ pr_id id)
+      
+  | {name=Convert_defbody;hypspecs=[id];terms=[c]} ->
+      (str"Change "  ++ prterm c  ++ spc ()  ++ str"in "  ++ pr_id id)
+
+  | {name=Convert_deftype;hypspecs=[id];terms=[c]} ->
+      (str"Change "  ++ prterm c  ++ spc ()  ++
+	 str"in (Type of "  ++ pr_id id ++ str ")")
       
   | {name=Thin;hypspecs=ids} ->
-      [< 'sTR"Clear " ; prlist_with_sep pr_spc pr_id ids >]
+      (str"Clear "  ++ prlist_with_sep pr_spc pr_id ids)
+      
+  | {name=ThinBody;hypspecs=ids} ->
+      (str"ClearBody "  ++ prlist_with_sep pr_spc pr_id ids)
       
   | {name=Move withdep;hypspecs=[id1;id2]} ->
-      [< 'sTR (if withdep then "Dependent " else "");
-	 'sTR"Move " ; pr_id id1; 'sTR " after "; pr_id id2 >]
+      (str (if withdep then "Dependent " else "") ++
+	 str"Move "  ++ pr_id id1 ++ str " after " ++ pr_id id2)
       
   | _ -> anomaly "pr_prim_rule: Unrecognized primitive rule"
