@@ -6,11 +6,12 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: pfedit.ml,v 1.28 2001/03/28 15:11:20 barras Exp $ *)
+(* $Id: pfedit.ml,v 1.34 2001/12/13 09:03:09 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Sign
 open Term
 open Declarations
@@ -23,6 +24,7 @@ open Proof_trees
 open Proof_type
 open Lib
 open Astterm
+open Safe_typing
 
 (*********************************************************************)
 (* Managing the proofs state                                         *)
@@ -41,13 +43,13 @@ let get_all_proof_names () = Edit.dom proof_edits
 
 let msg_proofs use_resume =
   match Edit.dom proof_edits with
-    | [] -> [< 'sPC ; 'sTR"(No proof-editing in progress)." >]
-    | l ->  [< 'sTR"." ; 'fNL ; 'sTR"Proofs currently edited:" ; 'sPC ;
-               (prlist_with_sep pr_spc pr_id (get_all_proof_names ())) ;
-	       'sTR"." ;
-               (if use_resume then [< 'fNL ; 'sTR"Use \"Resume\" first." >]
-              	else [< >])
-            >]
+    | [] -> (spc () ++ str"(No proof-editing in progress).")
+    | l ->  (str"." ++ fnl () ++ str"Proofs currently edited:" ++ spc () ++
+               (prlist_with_sep pr_spc pr_id (get_all_proof_names ())) ++
+	       str"." ++
+               (if use_resume then (fnl () ++ str"Use \"Resume\" first.")
+              	else (mt ()))
+)
 
 let undo_default = 12
 let undo_limit = ref undo_default
@@ -59,7 +61,7 @@ let undo_limit = ref undo_default
 let get_state () =
   match Edit.read proof_edits with
     | None -> errorlabstrm "Pfedit.get_state"
-          [< 'sTR"No focused proof"; msg_proofs true >]
+          (str"No focused proof" ++ msg_proofs true)
     | Some(_,pfs,ts) -> (pfs,ts)
 
 let get_topstate ()    = snd(get_state())
@@ -77,7 +79,7 @@ let set_current_proof s =
     Edit.focus proof_edits s
   with Invalid_argument "Edit.focus" ->
     errorlabstrm "Pfedit.set_proof"
-      [< 'sTR"No such proof" ; (msg_proofs false) >]
+      (str"No such proof" ++ (msg_proofs false))
 
 let resume_proof = set_current_proof
 
@@ -86,12 +88,12 @@ let suspend_proof () =
     Edit.unfocus proof_edits
   with Invalid_argument "Edit.unfocus" ->
     errorlabstrm "Pfedit.suspend_current_proof"
-      [< 'sTR"No active proof" ; (msg_proofs true) >]
+      (str"No active proof" ++ (msg_proofs true))
       
 let resume_last_proof () =
   match (Edit.last_focused proof_edits) with
     | None ->
-        errorlabstrm "resume_last" [< 'sTR"No proof-editing in progress." >]
+        errorlabstrm "resume_last" (str"No proof-editing in progress.")
     | Some p -> 
 	Edit.focus proof_edits p
 	  
@@ -99,7 +101,7 @@ let get_current_proof_name () =
   match Edit.read proof_edits with
     | None ->
         errorlabstrm "Pfedit.get_proof"
-          [< 'sTR"No focused proof" ; msg_proofs true >]
+          (str"No focused proof" ++ msg_proofs true)
     | Some(na,_,_) -> na
 
 let add_proof (na,pfs,ts) =
@@ -110,7 +112,7 @@ let delete_proof na =
     Edit.delete proof_edits na
   with (UserError ("Edit.delete",_)) ->
     errorlabstrm "Pfedit.delete_proof"
-      [< 'sTR"No such proof" ; msg_proofs false >]
+      (str"No such proof" ++ msg_proofs false)
       
 let init_proofs () = Edit.clear proof_edits
 		       
@@ -119,7 +121,7 @@ let mutate f =
     Edit.mutate proof_edits (fun _ pfs -> f pfs)
   with Invalid_argument "Edit.mutate" ->
     errorlabstrm "Pfedit.mutate"
-      [< 'sTR"No focused proof" ; msg_proofs true >]
+      (str"No focused proof" ++ msg_proofs true)
 
 let start (na,ts) =
   let pfs = mk_pftreestate ts.top_goal in 
@@ -129,7 +131,7 @@ let restart_proof () =
   match Edit.read proof_edits with
     | None -> 
 	errorlabstrm "Pfedit.restart"
-          [< 'sTR"No focused proof to restart" ; msg_proofs true >]
+          (str"No focused proof to restart" ++ msg_proofs true)
     | Some(na,_,ts) -> 
 	delete_proof na;
         start (na,ts);
@@ -164,7 +166,7 @@ let undo n =
        subtree - this solution only works properly if undoing one step *)
     if subtree_solved() then  Edit.undo proof_edits 1
   with (Invalid_argument "Edit.undo") ->
-    errorlabstrm "Pfedit.undo" [< 'sTR"No focused proof"; msg_proofs true >]
+    errorlabstrm "Pfedit.undo" (str"No focused proof" ++ msg_proofs true)
 
 (*********************************************************************)
 (*                  Proof cooking                                    *)
@@ -177,7 +179,9 @@ let cook_proof () =
   and strength = ts.top_strength in
   let pfterm = extract_pftreestate pfs in
   (ident,
-   ({ const_entry_body = pfterm; const_entry_type = Some concl },
+   ({ const_entry_body = pfterm;
+      const_entry_type = Some concl;
+      const_entry_opaque = true },
     strength))
 
 (*********************************************************************)
@@ -189,8 +193,8 @@ let refining () = [] <> (Edit.dom proof_edits)
 let check_no_pending_proofs () =
   if refining () then 
     errorlabstrm "check_no_pending_proofs"
-      [< 'sTR"Proof editing in progress" ; (msg_proofs false) ;
-         'sTR"Use \"Abort All\" first or complete proof(s)." >]
+      (str"Proof editing in progress" ++ (msg_proofs false) ++
+         str"Use \"Abort All\" first or complete proof(s).")
 
 let delete_current_proof () = delete_proof (get_current_proof_name ())
 
@@ -201,7 +205,7 @@ let delete_all_proofs = init_proofs
 (*********************************************************************)
 
 let start_proof na str sign concl =
-  let top_goal = mk_goal (mt_ctxt Intset.empty) sign concl in
+  let top_goal = mk_goal sign concl in
   let ts = { 
     top_hyps = (sign,empty_named_context);
     top_goal = top_goal;
@@ -220,7 +224,7 @@ let solve_nth k tac =
 let by tac = mutate (solve_pftreestate tac)
 
 let instantiate_nth_evar_com n c =
-  mutate (instantiate_pf_com n c)
+  mutate (Evar_refiner.instantiate_pf_com n c)
 
 let traverse n = mutate (traverse n)
  
