@@ -6,17 +6,17 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: refiner.ml,v 1.38 2001/04/19 04:58:38 delahaye Exp $ *)
+(* $Id: refiner.ml,v 1.48 2001/12/13 09:03:09 filliatr Exp $ *)
 
 open Pp
 open Util
-open Stamps
 open Term
+open Termops
 open Sign
 open Evd
 open Sign
 open Environ
-open Reduction
+open Reductionops
 open Instantiate
 open Type_errors
 open Proof_trees
@@ -46,15 +46,20 @@ let rec and_status = function
   | _ -> Incomplete_proof
 
 (* Normalizing evars in a goal. Called by tactic Local_constraints
-   (i.e. when the sigma of the proof tree changes) *)
-
+   (i.e. when the sigma of the proof tree changes). Detect if the
+   goal is unchanged *)
 let norm_goal sigma gl =
-  let red_fun = nf_ise1 sigma in
+  let red_fun = Evarutil.nf_evar sigma in
   let ncl = red_fun gl.evar_concl in
-  { evar_concl = ncl;
-    evar_hyps = map_named_context red_fun gl.evar_hyps;
-    evar_body = gl.evar_body;
-    evar_info = gl.evar_info }
+  let ngl =
+    { evar_concl = ncl;
+      evar_hyps =
+      Sign.fold_named_context 
+        (fun (d,b,ty) sign ->
+          add_named_decl (d, option_app red_fun b, red_fun ty) sign)
+        gl.evar_hyps ~init:empty_named_context;
+      evar_body = gl.evar_body} in
+  if ngl = gl then None else Some ngl
 
 
 (* [mapshape [ l1 ; ... ; lk ] [ v1 ; ... ; vk ] [ p_1 ; .... ; p_(l1+...+lk) ]]
@@ -86,7 +91,7 @@ let rec frontier p =
 	      p'
             else 
 	      errorlabstrm "Refiner.frontier"
-                [< 'sTR"frontier was handed back a ill-formed proof." >]))
+                (str"frontier was handed back a ill-formed proof.")))
     | Some(r,pfl) ->
     	let gll,vl = List.split(List.map frontier pfl) in
     	(List.flatten gll,
@@ -126,7 +131,7 @@ let tac_tab = Hashtbl.create 17
 let add_tactic s t =
   if Hashtbl.mem tac_tab s then
     errorlabstrm ("Refiner.add_tactic: "^s) 
-      [<'sTR "Cannot redeclare a tactic.">];
+      (str "Cannot redeclare a tactic.");
   Hashtbl.add tac_tab s t
 
 let overwriting_add_tactic s t =
@@ -141,13 +146,13 @@ let lookup_tactic s =
     Hashtbl.find tac_tab s
   with Not_found -> 
     errorlabstrm "Refiner.lookup_tactic"
-      [< 'sTR"The tactic " ; 'sTR s ; 'sTR" is not installed" >]
+      (str"The tactic " ++ str s ++ str" is not installed")
 
 
 (* refiner r is a tactic applying the rule r *)
 
 let bad_subproof () =
-  anomalylabstrm "Refiner.refiner" [< 'sTR"Bad subproof in validation.">]
+  anomalylabstrm "Refiner.refiner" (str"Bad subproof in validation.")
 
 let check_subproof_connection gl spfl =
   if not (list_for_all2eq (fun g pf -> g=pf.goal) gl spfl)
@@ -157,7 +162,7 @@ let refiner = function
   | Prim pr as r ->
       let prim_fun = prim_refiner pr in
       (fun goal_sigma ->
-         let sgl = prim_fun (ts_it goal_sigma.sigma) goal_sigma.it in 
+         let sgl = prim_fun goal_sigma.sigma goal_sigma.it in 
 	 ({it=sgl; sigma = goal_sigma.sigma},
           (fun spfl ->
 	     assert (check_subproof_connection sgl spfl);
@@ -179,38 +184,30 @@ let refiner = function
               subproof = Some hidden_proof;
               ref = Some(r,spfl) }))
       
-  | (Context ctxt) as r ->
-      (fun goal_sigma ->
-         let gl = goal_sigma.it in
-         let sg = mk_goal ctxt gl.evar_hyps gl.evar_concl in
-         ({it=[sg];sigma=goal_sigma.sigma},
-          (fun spfl -> 
-	     assert (check_subproof_connection [sg] spfl);
-             { status = (List.hd spfl).status;
-               goal = gl;
-               subproof = None;
-               ref = Some(r,spfl) })))
-
    (* [Local_constraints lc] makes the local constraints be [lc] and
       normalizes evars *)
 
-  | (Local_constraints lc) as r ->
+  | Change_evars as r ->
       (fun goal_sigma ->
          let gl = goal_sigma.it in
-         let ctxt = set_lc lc (out_some gl.evar_info) in 
-         let sg = mk_goal ctxt gl.evar_hyps gl.evar_concl in
-	 let sgl = [norm_goal (ts_it goal_sigma.sigma) sg] in
-	 ({it=sgl;sigma=goal_sigma.sigma},
-          (fun spfl -> 
-	    assert (check_subproof_connection sgl spfl);
-             { status = (List.hd spfl).status;
-               goal = gl;
-               subproof = None;
-               ref = Some(r,spfl) })))
+         (match norm_goal goal_sigma.sigma gl with
+             Some ngl ->
+	       ({it=[ngl];sigma=goal_sigma.sigma},
+                (fun spfl -> 
+	          assert (check_subproof_connection [ngl] spfl);
+                  { status = (List.hd spfl).status;
+                    goal = gl;
+                    subproof = None;
+                    ref = Some(r,spfl) }))
+           (* if the evar change does not affect the goal, leave the
+              proof tree unchanged *)
+           | None -> ({it=[gl];sigma=goal_sigma.sigma},
+                      (fun spfl ->
+                        assert (List.length spfl = 1);
+                        List.hd spfl))))
 
-let context ctxt = refiner (Context ctxt)
 let vernac_tactic texp = refiner (Tactic texp)
-let norm_evar_tac gl = refiner (Local_constraints (get_lc gl.it)) gl
+let norm_evar_tac gl = refiner Change_evars gl
 
 (* [rc_of_pfsigma : proof sigma -> readable_constraints] *)
 let rc_of_pfsigma sigma = rc_of_gc sigma.sigma sigma.it.goal
@@ -236,9 +233,7 @@ let extract_open_proof sigma pf =
 	let flat_proof = v spfl in
 	proof_extractor vl flat_proof
 	  
-    | {ref=Some(Context ctxt,[pf])} -> (proof_extractor vl) pf
-	  
-    | {ref=Some(Local_constraints lc,[pf])} -> (proof_extractor vl) pf
+    | {ref=Some(Change_evars,[pf])} -> (proof_extractor vl) pf
 	  
     | {ref=None;goal=goal} ->
 	let visible_rels =
@@ -252,7 +247,7 @@ let extract_open_proof sigma pf =
 	let abs_concl =
           List.fold_right
             (fun (_,id) concl ->
-	       let (c,ty) = lookup_id id goal.evar_hyps in
+	       let (_,c,ty) = Sign.lookup_named id goal.evar_hyps in
 	       mkNamedProd_or_LetIn (id,c,ty) concl)
             sorted_rels goal.evar_concl	in
         incr meta_cnt;
@@ -295,7 +290,7 @@ let goal_goal_list gls = {it=[gls.it];sigma=gls.sigma}
 let tclIDTAC gls = (goal_goal_list gls, idtac_valid)
 
 (* General failure tactic *)
-let tclFAIL_s s gls = errorlabstrm "Refiner.tclFAIL_s" [< 'sTR s>]
+let tclFAIL_s s gls = errorlabstrm "Refiner.tclFAIL_s" (str s)
 
 (* A special exception for levels for the Fail tactic *)
 exception FailError of int
@@ -313,7 +308,7 @@ let thens_tac tac2l taci (sigr,gs,p) =
   let (gl,gi) =
     try list_chop (List.length tac2l) gs
     with Failure _ -> errorlabstrm "Refiner.combine_tactics"
-        [<'sTR "Wrong number of tactics.">] in
+        (str "Wrong number of tactics.") in
   let tac2gl =
     List.combine gl tac2l @ list_map_i (fun i g -> (g, taci i)) 1 gi in
   let gll,pl =
@@ -326,14 +321,14 @@ let then_tac tac = thens_tac [] (fun _ -> tac)
 
 let non_existent_goal n =
   errorlabstrm ("No such goal: "^(string_of_int n))
-    [< 'sTR"Trying to apply a tactic to a non existent goal" >]
+    (str"Trying to apply a tactic to a non existent goal")
 
 (* Apply tac on the i-th goal (if i>0). If i<0, then start counting from
    the last goal (i=-1). *)
 let theni_tac i tac ((_,gl,_) as subgoals) =
   let nsg = List.length gl in
   let k = if i < 0 then nsg + i + 1 else i in
-  if nsg < 1 then errorlabstrm "theni_tac" [< 'sTR"No more subgoals.">]
+  if nsg < 1 then errorlabstrm "theni_tac" (str"No more subgoals.")
   else if k >= 1 & k <= nsg then
     thens_tac [] (fun i -> if i = k then tac else tclIDTAC) subgoals
   else non_existent_goal k 
@@ -385,18 +380,17 @@ let rec tclTHENLIST = function
 (* various progress criterions *)
 let same_goal gl subgoal = 
   (hypotheses subgoal) = (hypotheses gl) &
-  eq_constr (conclusion subgoal) (conclusion gl) &
-  (subgoal.evar_info = gl.evar_info)
+  eq_constr (conclusion subgoal) (conclusion gl)
 
 
 let weak_progress gls ptree =
   (List.length gls.it <> 1) or 
   (not (same_goal (List.hd gls.it) ptree.it))
 
-
+(* Il y avait ici un ts_eq ........ *)
 let progress gls ptree =
   (weak_progress gls ptree) or
-  (not (ts_eq ptree.sigma gls.sigma))
+  (not (ptree.sigma == gls.sigma))
 
 
 (* PROGRESS tac ptree applies tac to the goal ptree and fails if tac leaves
@@ -404,14 +398,14 @@ the goal unchanged *)
 let tclPROGRESS tac ptree =
   let rslt = tac ptree in
   if progress (fst rslt) ptree then rslt
-  else errorlabstrm "Refiner.PROGRESS" [< 'sTR"Failed to progress.">]
+  else errorlabstrm "Refiner.PROGRESS" (str"Failed to progress.")
 
 (* weak_PROGRESS tac ptree applies tac to the goal ptree and fails 
    if tac leaves the goal unchanged, possibly modifying sigma *)
 let tclWEAK_PROGRESS tac ptree =
   let rslt = tac ptree in
   if weak_progress (fst rslt) ptree then rslt
-  else errorlabstrm "Refiner.tclWEAK_PROGRESS" [< 'sTR"Failed to progress.">]
+  else errorlabstrm "Refiner.tclWEAK_PROGRESS" (str"Failed to progress.")
 
 
 (* Same as tclWEAK_PROGRESS but fails also if tactics generates several goals,
@@ -421,7 +415,7 @@ let tclNOTSAMEGOAL (tac : tactic) goal =
   let gls = (fst rslt).it in
   if List.exists (same_goal goal.it) gls
   then errorlabstrm "Refiner.tclNOTSAMEGOAL" 
-      [< 'sTR"Tactic generated a subgoal identical to the original goal.">]
+      (str"Tactic generated a subgoal identical to the original goal.")
   else rslt
 
 
@@ -467,7 +461,7 @@ let tclSOLVE tacl = tclFIRST (List.map tclCOMPLETE tacl)
 let tclDO n t = 
   let rec dorec k = 
     if k < 0 then errorlabstrm "Refiner.tclDO"
-      [<'sTR"Wrong argument : Do needs a positive integer.">];
+      (str"Wrong argument : Do needs a positive integer.");
     if k = 0 then tclIDTAC
     else if k = 1 then t else (tclTHEN t (dorec (k-1)))
   in 
@@ -534,8 +528,8 @@ let tactic_list_tactic tac gls =
 
 
 (* solve_subgoal :
-     (global_constraints ref * goal list * validation -> 
-         global_constraints ref * goal list * validation) -> 
+     (evar_map ref * goal list * validation -> 
+         evar_map ref * goal list * validation) -> 
       (proof_tree sigma -> proof_tree sigma)
    solve_subgoal tac pf_sigma applies the tactic tac at the nth subgoal of
    pf_sigma *)
@@ -544,7 +538,7 @@ let solve_subgoal tacl pf_sigma =
   let gl,p = frontier pf in
   let r = tacl (sigr,gl,p) in
   let (sigr,gll,pl) =
-    if (ts_it !sigr) == (ts_it pf_sigma.sigma) then r
+    if !sigr == pf_sigma.sigma then r
     else then_tac norm_evar_tac r in
   repackage sigr (pl (List.map leaf gll))
 
@@ -563,13 +557,13 @@ let solve_subgoal tacl pf_sigma =
 
 type pftreestate = {
   tpf      : proof_tree ;
-  tpfsigma : global_constraints;
+  tpfsigma : evar_map;
   tstack   : (int * validation) list }
 
 let proof_of_pftreestate pts = pts.tpf
 let is_top_pftreestate pts = pts.tstack = [] 
 let cursor_of_pftreestate pts = List.map fst pts.tstack
-let evc_of_pftreestate pts = ts_it pts.tpfsigma
+let evc_of_pftreestate pts = pts.tpfsigma
 
 let top_goal_of_pftreestate pts = 
   { it = goal_of_proof pts.tpf; sigma = pts.tpfsigma }
@@ -577,7 +571,7 @@ let top_goal_of_pftreestate pts =
 let nth_goal_of_pftreestate n pts =
   let goals = fst (frontier pts.tpf) in
   try {it = List.nth goals (n-1); sigma = pts.tpfsigma }
-  with Failure "nth" -> non_existent_goal n
+  with Invalid_argument _ | Failure _ -> non_existent_goal n
 
 let descend n p =
   match p.ref with
@@ -654,35 +648,35 @@ let weak_undo_pftreestate pts =
 let mk_pftreestate g =
   { tpf      = leaf g;
     tstack   = [];
-    tpfsigma = ts_mk Evd.empty }
+    tpfsigma = Evd.empty }
 
 (* Extracts a constr from a proof-tree state ; raises an error if the
    proof is not complete or the state does not correspond to the head
    of the proof-tree *)
 
 let extract_open_pftreestate pts =
-  extract_open_proof (ts_it pts.tpfsigma) pts.tpf
+  extract_open_proof pts.tpfsigma pts.tpf
 
 let extract_pftreestate pts =
   if pts.tstack <> [] then
     errorlabstrm "extract_pftreestate"
-      [< 'sTR"Cannot extract from a proof-tree in which we have descended;" ;
-         'sPC; 'sTR"Please ascend to the root" >];
+      (str"Cannot extract from a proof-tree in which we have descended;" ++
+         spc () ++ str"Please ascend to the root");
   let pfterm,subgoals = extract_open_pftreestate pts in
   if subgoals <> [] then
    errorlabstrm "extract_proof"
-   [< 'sTR "Attempt to save an incomplete proof" >];
+   (str "Attempt to save an incomplete proof");
   let env = Global.env_of_context pts.tpf.goal.evar_hyps in
-  strong whd_betaiotaevar env (ts_it pts.tpfsigma) pfterm
-  (***  
-  local_strong (whd_ise (ts_it pts.tpfsigma)) pfterm
+  strong whd_betaiotaevar env pts.tpfsigma pfterm
+  (***
+  local_strong (Evarutil.whd_ise (ts_it pts.tpfsigma)) pfterm
   ***)
 (* Focus on the first leaf proof in a proof-tree state *)
 
 let rec first_unproven pts =
   let pf = (proof_of_pftreestate pts) in 
   if is_complete_proof pf then
-    errorlabstrm "first_unproven" [< 'sTR"No unproven subgoals" >];
+    errorlabstrm "first_unproven" (str"No unproven subgoals");
   if is_leaf_proof pf then
     pts
   else
@@ -699,7 +693,7 @@ let rec first_unproven pts =
 let rec last_unproven pts =
   let pf = proof_of_pftreestate pts in 
   if is_complete_proof pf then
-    errorlabstrm "last_unproven" [< 'sTR"No unproven subgoals" >];
+    errorlabstrm "last_unproven" (str"No unproven subgoals");
   if is_leaf_proof pf then
     pts
   else 
@@ -716,17 +710,17 @@ let rec last_unproven pts =
 let rec nth_unproven n pts =
   let pf = proof_of_pftreestate pts in 
   if is_complete_proof pf then
-    errorlabstrm "nth_unproven" [< 'sTR"No unproven subgoals" >];
+    errorlabstrm "nth_unproven" (str"No unproven subgoals");
   if is_leaf_proof pf then
     if n = 1 then 
       pts 
     else
-      errorlabstrm "nth_unproven" [< 'sTR"Not enough unproven subgoals" >]
+      errorlabstrm "nth_unproven" (str"Not enough unproven subgoals")
   else 
     let children = children_of_proof pf in
     let rec process i k = function
       | [] -> 
-	  errorlabstrm "nth_unproven" [< 'sTR"Not enough unproven subgoals" >]
+	  errorlabstrm "nth_unproven" (str"Not enough unproven subgoals")
       | pf1::rest -> 
 	  let k1 = nb_unsolved_goals pf1 in 
 	  if k1 < k then 
@@ -795,9 +789,17 @@ let pr_tactic (s,l) =
 
 let pr_rule = function
   | Prim r -> pr_prim_rule r
-  | Tactic texp -> hOV 0 (pr_tactic texp)
-  | Context ctxt -> pr_ctxt ctxt
-  | Local_constraints lc -> [< 'sTR"Local constraint change" >]
+  | Tactic texp -> hov 0 (pr_tactic texp)
+  | Change_evars ->
+      (* This is internal tactic and cannot be replayed at user-level.
+         Function pr_rule_dot below is used when we want to hide
+         Change_evars *)
+      str "Evar change"
+
+(* Does not print change of evars *)
+let pr_rule_dot = function 
+  | Change_evars -> (mt ())
+  | r -> (pr_rule r ++ str"." ++ fnl ()) 
 
 exception Different
 
@@ -806,78 +808,79 @@ let thin_sign osign sign =
   Sign.fold_named_context
     (fun (id,c,ty as d) sign ->
        try 
-	 if lookup_id id osign = (c,ty) then sign
+	 if Sign.lookup_named id osign = (id,c,ty) then sign
 	 else raise Different
        with Not_found | Different -> add_named_decl d sign)
-    sign empty_named_context
+    sign ~init:empty_named_context
 
 let rec print_proof sigma osign pf =
   let {evar_hyps=hyps; evar_concl=cl; 
-       evar_info=info; evar_body=body} = pf.goal in
+       evar_body=body} = pf.goal in
   let hyps' = thin_sign osign hyps in
   match pf.ref with
     | None -> 
-	hOV 0 [< pr_seq {evar_hyps=hyps'; evar_concl=cl;
-			 evar_info=info; evar_body=body} >]
+	hov 0 (pr_seq {evar_hyps=hyps'; evar_concl=cl; evar_body=body})
     | Some(r,spfl) ->
-    	hOV 0 [< hOV 0 (pr_seq {evar_hyps=hyps'; evar_concl=cl;
-				evar_info=info; evar_body=body});
-		 'sPC ; 'sTR" BY ";
-		 hOV 0 [< pr_rule r >]; 'fNL ;
-		 'sTR"  ";
-		 hOV 0 (prlist_with_sep pr_fnl (print_proof sigma hyps) spfl)
-              >]
+    	hov 0 
+	  (hov 0 (pr_seq {evar_hyps=hyps'; evar_concl=cl; evar_body=body}) ++
+	   spc () ++ str" BY " ++
+	   hov 0 (pr_rule r) ++ fnl () ++
+	   str"  " ++
+	   hov 0 (prlist_with_sep pr_fnl (print_proof sigma hyps) spfl)
+)
 	  
-let pr_change gl = [< 'sTR"Change " ; prterm gl.evar_concl ; 'sTR"." ; 'fNL>]
+let pr_change gl = (str"Change " ++ prterm gl.evar_concl ++ str"." ++ fnl ())
 		     
 let rec print_script nochange sigma osign pf =
   let {evar_hyps=sign; evar_concl=cl} = pf.goal in
   match pf.ref with
     | None ->
         if nochange then 
-	  [< 'sTR"<Your Tactic Text here>" >] 
+	  (str"<Your Tactic Text here>") 
 	else 
 	  pr_change pf.goal
     | Some(r,spfl) ->
-        [< (if nochange then [< >] else (pr_change pf.goal));
-           pr_rule r ; 'sTR"." ; 'fNL ;
+        ((if nochange then (mt ()) else (pr_change pf.goal)) ++
+           pr_rule_dot r ++
            prlist_with_sep pr_fnl
-             (print_script nochange sigma sign) spfl >]
+             (print_script nochange sigma sign) spfl)
 
 let rec print_treescript sigma osign pf =
   let {evar_hyps=sign; evar_concl=cl} = pf.goal in
   match pf.ref with
-    | None -> [< >]
+    | None -> (mt ())
     | Some(r,spfl) ->
-        [< pr_rule r ; 'sTR"." ; 'fNL ;
+        (pr_rule_dot r ++
            let prsub =
              prlist_with_sep pr_fnl (print_treescript sigma sign) spfl
            in
            if List.length spfl > 1 then 
-	     [< 'sTR"  "; hOV 0 prsub >]
+	     (str"  " ++ hov 0 prsub)
            else 
 	     prsub 
-	>]
+)
 
 let rec print_info_script sigma osign pf =
   let {evar_hyps=sign; evar_concl=cl} = pf.goal in
   match pf.ref with
-    | None -> [< >]
+    | None -> (mt ())
+    | Some(Change_evars,[spf]) ->
+        print_info_script sigma osign spf
     | Some(r,spfl) ->
-        [< pr_rule r ; 
+        (pr_rule r ++ 
            match spfl with
              | [pf1] ->
                  if pf1.ref = None then 
-		   [<'sTR "."; 'fNL >]
+		   (str "." ++ fnl ())
                  else 
-		   [< 'sTR";" ; 'bRK(1,3) ;
-		      print_info_script sigma sign pf1 >]
-             | _ -> [< 'sTR"." ; 'fNL ;
+		   (str";" ++ brk(1,3) ++
+		      print_info_script sigma sign pf1)
+             | _ -> (str"." ++ fnl () ++
                        prlist_with_sep pr_fnl
-                         (print_info_script sigma sign) spfl >] >]
+                         (print_info_script sigma sign) spfl))
 
 let format_print_info_script sigma osign pf = 
-  hOV 0 (print_info_script sigma osign pf)
+  hov 0 (print_info_script sigma osign pf)
     
 let print_subscript sigma sign pf = 
   if is_tactic_proof pf then 
@@ -890,10 +893,9 @@ let tclINFO (tac : tactic) gls =
   begin try 
     let pf = v (List.map leaf (sig_it sgl)) in
     let sign = (sig_it gls).evar_hyps in
-    mSGNL(hOV 0 [< 'sTR" == "; 
-                   print_subscript 
-                     ((compose ts_it sig_sig) gls) sign pf >])
+    msgnl (hov 0 (str" == " ++
+                  print_subscript (sig_sig gls) sign pf))
   with e when catchable_exception e -> 
-    mSGNL(hOV 0 [< 'sTR "Info failed to apply validation" >])
+    msgnl (hov 0 (str "Info failed to apply validation"))
   end;
   res
