@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: tactic_debug.mli,v 1.3 2001/03/15 13:38:41 filliatr Exp $ i*)
+(*i $Id: tactic_debug.mli,v 1.4 2001/10/23 22:40:38 delahaye Exp $ i*)
 
 open Proof_type
 open Term
@@ -24,6 +24,9 @@ type debug_info =
 (* Prints the state and waits *)
 val debug_prompt : goal sigma option -> Coqast.t -> debug_info
 
+(* Prints a constr *)
+val db_constr : debug_info -> Environ.env -> constr -> unit
+
 (* Prints a matched hypothesis *)
 val db_matched_hyp : debug_info -> Environ.env -> string * constr -> unit
 
