@@ -6,12 +6,14 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: auto.ml,v 1.39 2001/03/28 15:11:22 barras Exp $ *)
+(* $Id: auto.ml,v 1.45 2001/12/13 09:03:10 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Term
+open Termops
 open Sign
 open Inductive
 open Evd
@@ -32,6 +34,8 @@ open Libobject
 open Library
 open Vernacinterp
 open Printer
+open Nametab
+open Declarations
 
 (****************************************************************************)
 (*            The Type of Constructions Autotactic Hints                    *)
@@ -186,7 +190,7 @@ let (inAutoHint,outAutoHint) =
 (**************************************************************************)
 
 let rec nb_hyp c = match kind_of_term c with
-  | IsProd(_,_,c2) -> if dependent (mkRel 1) c2 then nb_hyp c2 else 1+(nb_hyp c2)
+  | Prod(_,_,c2) -> if dependent (mkRel 1) c2 then nb_hyp c2 else 1+(nb_hyp c2)
   | _ -> 0 
 
 (* adding and removing tactics in the search table *)
@@ -198,7 +202,7 @@ let try_head_pattern c =
 let make_exact_entry name (c,cty) =
   let cty = strip_outer_cast cty in
   match kind_of_term cty with
-    | IsProd (_,_,_) -> 
+    | Prod (_,_,_) -> 
 	failwith "make_exact_entry"
     | _ ->
 	(head_of_constr_reference (List.hd (head_constr cty)),
@@ -207,7 +211,7 @@ let make_exact_entry name (c,cty) =
 let make_apply_entry env sigma (eapply,verbose) name (c,cty) =
   let cty = hnf_constr env sigma cty in
   match kind_of_term cty with
-    | IsProd _ ->
+    | Prod _ ->
         let ce = mk_clenv_from () (c,cty) in
 	let c' = (clenv_template_type ce).rebus in
 	let pat = Pattern.pattern_of_constr c' in
@@ -219,8 +223,8 @@ let make_apply_entry env sigma (eapply,verbose) name (c,cty) =
         in 
 	if eapply & (nmiss <> 0) then begin
           if verbose then 
-	    wARN [< 'sTR "the hint: EApply "; prterm c;
-		    'sTR " will only be used by EAuto" >]; 
+	    warn (str "the hint: EApply " ++ prterm c ++
+		    str " will only be used by EAuto"); 
           (hd,
            { hname = name;
 	     pri = nb_hyp cty + nmiss;
@@ -245,7 +249,7 @@ let make_resolves env sigma name eap (c,cty) =
       [make_exact_entry; make_apply_entry env sigma eap]
   in 
   if ents = [] then 
-    errorlabstrm "Hint" [< prterm c; 'sPC; 'sTR "cannot be used as a hint" >];
+    errorlabstrm "Hint" (prterm c ++ spc () ++ str "cannot be used as a hint");
   ents
 
 (* used to add an hypothesis to the local hint database *)
@@ -273,10 +277,6 @@ let add_resolves env sigma clist dbnames =
 	    )))
     dbnames
 
-let global qid =
-  try Nametab.locate qid
-  with Not_found -> Nametab.error_global_not_found_loc dummy_loc qid
-
 (* REM : in most cases hintname = id *)
 let make_unfold (hintname, ref) =
   (Pattern.label_of_ref ref,
@@ -306,7 +306,7 @@ let add_extern name pri (patmetas,pat) tacast dbname =
   match (list_subtract tacmetas patmetas) with
     | i::_ ->
 	errorlabstrm "add_extern" 
-	  [< 'sTR "The meta-variable ?"; 'iNT i; 'sTR" is not bound" >]
+	  (str "The meta-variable ?" ++ int i ++ str" is not bound")
     | []  ->
 	Lib.add_anonymous_leaf
 	  (inAutoHint(dbname, [make_extern name pri pat tacast]))
@@ -316,9 +316,9 @@ let add_externs name pri pat tacast dbnames =
 
 let make_trivial (name,c) =
   let sigma = Evd.empty and env = Global.env() in
-  let t = type_of env sigma c in
+  let t = hnf_constr env sigma (type_of env sigma c) in
   let hd = head_of_constr_reference (List.hd (head_constr t)) in
-  let ce = mk_clenv_from () (c,hnf_constr env sigma t) in
+  let ce = mk_clenv_from () (c,t) in
   (hd, { hname = name;
 	       pri=1;
                pat = Some (Pattern.pattern_of_constr (clenv_template_type ce).rebus);
@@ -340,7 +340,7 @@ let _ =
 	       (function VARG_IDENTIFIER i -> string_of_id i
 		  | _ -> bad_vernac_args "HintUnfold") l in
       	   fun () -> 
-	     let ref = global qid in
+	     let ref = Nametab.global dummy_loc qid in
 	     add_unfolds [(hintname, ref)] dbnames
        | _-> bad_vernac_args "HintUnfold")
     
@@ -378,17 +378,16 @@ let _ =
 	   begin 
 	     try
 	       let env = Global.env() and sigma = Evd.empty in
-	       let (isp, _ as rectype) =
-		 destMutInd (Declare.global_qualified_reference qid) in
+	       let isp = destInd (Declare.global_qualified_reference qid) in
 	       let conspaths =
-		 mis_conspaths (Global.lookup_mind_specif rectype) in
-	       let hyps = Declare.implicit_section_args (IndRef isp) in
-	       let section_args = List.map (fun sp -> mkVar (basename sp)) hyps in
+                 let (mib,mip) = Global.lookup_inductive isp in
+                 mip.mind_consnames in
 	       let lcons = 
 		 array_map_to_list
-		   (fun sp ->
-		      let c = Declare.global_absolute_reference sp in
-		      (basename sp, applist (c, section_args)))
+		   (fun id ->
+                     let sp = make_path (dirpath (fst isp)) id in
+		     let c = Declare.global_absolute_reference sp in
+		      (id, c))
 		   conspaths in
 	       let dbnames = if l = [] then ["core"] else 
 	      	 List.map (function VARG_IDENTIFIER i -> string_of_id i
@@ -423,13 +422,11 @@ let _ =
 	     List.map
 	       (function
 		  | VARG_QUALID qid -> 
-		      let ref = global qid in
+		      let ref = Nametab.global dummy_loc qid in
 		      let env = Global.env() in
-		      let c = Declare.constr_of_reference Evd.empty env ref in
-		      let hyps = Declare.implicit_section_args ref in
-		      let section_args = List.map (fun sp -> mkVar (basename sp)) hyps in
+		      let c = Declare.constr_of_reference ref in
 		      let _,i = Nametab.repr_qualid qid in
-		      (i, applist (c,section_args))
+		      (i, c)
 		  | _-> bad_vernac_args "HintsResolve") lh in
 	   let dbnames = if l = [] then ["core"] else 
 	     List.map (function VARG_IDENTIFIER i -> string_of_id i
@@ -446,7 +443,7 @@ let _ =
 	     List.map (function
 			 | VARG_QUALID qid ->
 			     let _,n = Nametab.repr_qualid qid in
-			     (n, global qid)
+			     (n, Nametab.global dummy_loc qid)
 		      	 | _ -> bad_vernac_args "HintsUnfold") lh in
 	   let dbnames = if l = [] then ["core"] else 
 	     List.map (function 
@@ -467,10 +464,8 @@ let _ =
 		      let _,n = Nametab.repr_qualid qid in
 		      let ref = Nametab.locate qid in
 		      let env = Global.env () in
-		      let c = Declare.constr_of_reference Evd.empty env ref in
-		      let hyps = Declare.implicit_section_args ref in
-		      let section_args = List.map (fun sp -> mkVar (basename sp)) hyps in
-		      (n, applist (c, section_args))
+		      let c = Declare.constr_of_reference ref in
+		      (n, c)
 		  | _ -> bad_vernac_args "HintsImmediate") lh in
 	   let dbnames = if l = [] then ["core"] else 
 	     List.map (function 
@@ -484,20 +479,24 @@ let _ =
 (**************************************************************************)
 
 let fmt_autotactic = function
-  | Res_pf (c,clenv) -> [< 'sTR"Apply "; prterm c >]
-  | ERes_pf (c,clenv) -> [< 'sTR"EApply "; prterm c >]
-  | Give_exact c -> [< 'sTR"Exact " ; prterm c >]
+  | Res_pf (c,clenv) -> (str"Apply " ++ prterm c)
+  | ERes_pf (c,clenv) -> (str"EApply " ++ prterm c)
+  | Give_exact c -> (str"Exact " ++ prterm c)
   | Res_pf_THEN_trivial_fail (c,clenv) -> 
-      [< 'sTR"Apply "; prterm c ; 'sTR" ; Trivial" >]
-  | Unfold_nth c -> [< 'sTR"Unfold " ;  pr_global c >]
-  | Extern coqast -> [< 'sTR "Extern "; gentacpr coqast >]
+      (str"Apply " ++ prterm c ++ str" ; Trivial")
+  | Unfold_nth c -> (str"Unfold " ++  pr_global c)
+  | Extern coqast -> (str "Extern " ++ gentacpr coqast)
+
+let fmt_hint v =
+  (fmt_autotactic v.code ++ str"(" ++ int v.pri ++ str")" ++ spc ())
 
 let fmt_hint_list hintlist =
-  [< prlist 
-       (function v ->
-          [< fmt_autotactic v.code ; 'sTR"(" ; 'iNT v.pri;
-	     'sTR")"; 'sPC >]) 
-       hintlist >]
+  (str "  " ++ hov 0 (prlist fmt_hint hintlist) ++ fnl ())
+
+let fmt_hints_db (name,db,hintlist) =
+  (str "In the database " ++ str name ++ str ":" ++
+     if hintlist = [] then (str " nothing" ++ fnl ())
+     else (fnl () ++ fmt_hint_list hintlist))
 
 (* Print all hints associated to head c in any database *)
 let fmt_hint_list_for_head c = 
@@ -508,24 +507,16 @@ let fmt_hint_list_for_head c =
       dbs 
   in
   if valid_dbs = [] then 
-    [<'sTR "No hint declared for :"; pr_ref_label c >]
+    (str "No hint declared for :" ++ pr_ref_label c)
   else 
-    hOV 0 
-      [< 'sTR"For "; pr_ref_label c; 'sTR" -> "; 'fNL;
-	 prlist (fun (name,db,hintlist) ->
-	 	   [< 'sTR "  In the database "; 'sTR name; 'sTR " :"; 'fNL;
-		      fmt_hint_list hintlist >])
-	   valid_dbs >]
+    hov 0 
+      (str"For " ++ pr_ref_label c ++ str" -> " ++ fnl () ++
+	 hov 0 (prlist fmt_hints_db valid_dbs))
 
-let fmt_hint_id id = 
-  try 
-    let c = Declare.global_reference CCI id in
-    fmt_hint_list_for_head (head_of_constr_reference c)
-  with Not_found -> 
-    [< pr_id id; 'sTR " not declared" >]
+let fmt_hint_ref ref = fmt_hint_list_for_head (label_of_ref ref)
 
 (* Print all hints associated to head id in any database *)
-let print_hint_id id =  pPNL(fmt_hint_id id)
+let print_hint_qid qid =  ppnl(fmt_hint_ref (Nametab.global dummy_loc qid))
 
 let fmt_hint_term cl = 
   try 
@@ -547,18 +538,14 @@ let fmt_hint_term cl =
 	  dbs
     in 
     if valid_dbs = [] then 
-      [<'sTR "No hint applicable for current goal" >]
+      (str "No hint applicable for current goal")
     else
-      [< 'sTR "Applicable Hints :";
-	 prlist (fun (name,db,hintlist) ->
-	 	   [< 'sTR "  In the database "; 'sTR name; 
-		      'sTR " :"; 'fNL;
-		      fmt_hint_list hintlist >])
-	   valid_dbs >]
+      (str "Applicable Hints :" ++ fnl () ++
+	 hov 0 (prlist fmt_hints_db valid_dbs))
   with Bound | Match_failure _ | Failure _ -> 
-    [<'sTR "No hint applicable for current goal" >]
+    (str "No hint applicable for current goal")
 	  
-let print_hint_term cl = pPNL (fmt_hint_term cl)
+let print_hint_term cl = ppnl (fmt_hint_term cl)
 
 (* print all hints that apply to the concl of the current goal *)
 let print_applicable_hint () = 
@@ -570,9 +557,9 @@ let print_applicable_hint () =
 let print_hint_db db =
   Hint_db.iter 
     (fun head hintlist ->
-       mSG (hOV 0 
-	      [< 'sTR "For "; pr_ref_label head; 'sTR " -> ";
-		 fmt_hint_list hintlist; 'fNL >]))
+       msg (hov 0 
+	      (str "For " ++ pr_ref_label head ++ str " -> " ++
+		 fmt_hint_list hintlist)))
     db
 
 let print_hint_db_by_name dbname =
@@ -585,7 +572,7 @@ let print_hint_db_by_name dbname =
 let print_searchtable () =
   Stringmap.iter
     (fun name db ->
-       mSG [< 'sTR "In the database "; 'sTR name; 'fNL >];
+       msg (str "In the database " ++ str name ++ fnl ());
        print_hint_db db)
     !searchtable
 
@@ -611,7 +598,7 @@ let _ =
 let _ = 
   vinterp_add "PrintHintId"
     (function 
-       | [(VARG_IDENTIFIER id)] -> fun () -> print_hint_id id
+       | [(VARG_QUALID qid)] -> fun () -> print_hint_qid qid
        | _ -> bad_vernac_args "PrintHintId")
 
 (**************************************************************************)
@@ -741,15 +728,15 @@ let decomp_unary_term c gls =
   if Hipattern.is_conjunction hd then 
     simplest_case c gls 
   else 
-    errorlabstrm "Auto.decomp_unary_term" [<'sTR "not a unary type" >] 
+    errorlabstrm "Auto.decomp_unary_term" (str "not a unary type") 
 
 let decomp_empty_term c gls = 
   let typc = pf_type_of gls c in 
-  let (hd,_) = decomp_app typc in 
+  let (hd,_) = decompose_app typc in 
   if Hipattern.is_empty_type hd then 
     simplest_case c gls 
   else 
-    errorlabstrm "Auto.decomp_empty_term" [<'sTR "not an empty type" >] 
+    errorlabstrm "Auto.decomp_empty_term" (str "not an empty type") 
 
 
 (* decomp is an natural number giving an indication on decomposition 
@@ -893,7 +880,8 @@ let compileAutoArg contac = function
 	 tclFIRST 
            (List.map 
               (fun (id,_,typ) -> 
-                 if (Hipattern.is_conjunction (hd_of_prod (body_of_type typ)))
+                let cl = snd (decompose_prod (body_of_type typ)) in
+                 if (Hipattern.is_conjunction cl)
 		 then 
 		   (tclTHEN
                       (tclTHEN (simplest_elim (mkVar id)) 
@@ -937,7 +925,7 @@ let rec super_search n db_list local_db argl goal =
 let search_superauto n to_add argl g = 
   let sigma =
     List.fold_right
-      (fun (id,c) -> add_named_assum (id, pf_type_of g c))
+      (fun (id,c) -> add_named_decl (id, None, pf_type_of g c))
       to_add empty_named_context in
   let db0 = list_map_append (make_resolve_hyp (pf_env g) (project g)) sigma in
   let db = Hint_db.add_list db0 (make_local_hint_db g) in
@@ -953,20 +941,20 @@ let cvt_autoArg = function
   | "UsingTDB"     -> [UsingTDB]
   | "NoAutoArg"    -> []
   | x -> errorlabstrm "cvt_autoArg"
-        [< 'sTR "Unexpected argument for Auto!"; 'sTR x >]
+        (str "Unexpected argument for Auto!" ++ str x)
 
 let cvt_autoArgs =
   list_join_map
     (function 
        | Quoted_string s -> (cvt_autoArg s)
-       | _ -> errorlabstrm "cvt_autoArgs" [< 'sTR "String expected" >])
+       | _ -> errorlabstrm "cvt_autoArgs" (str "String expected"))
 
 let interp_to_add gl = function
   | Qualid qid ->
       let _,id = Nametab.repr_qualid qid in
       (next_ident_away id (pf_ids_of_hyps gl), 
-       Declare.constr_of_reference Evd.empty (Global.env()) (global qid))
-  | _ -> errorlabstrm "cvt_autoArgs" [< 'sTR "Qualid expected" >]
+       Declare.constr_of_reference (Nametab.global dummy_loc qid))
+  | _ -> errorlabstrm "cvt_autoArgs" (str "Qualid expected")
 
 let dyn_superauto l g = 
   match l with
