@@ -6,16 +6,19 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: equality.ml,v 1.59 2001/04/15 01:50:47 herbelin Exp $ *)
+(* $Id: equality.ml,v 1.72 2001/12/13 09:03:11 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Univ
 open Term
+open Termops
 open Inductive
+open Inductiveops
 open Environ
-open Reduction
+open Reductionops
 open Instantiate
 open Typeops
 open Typing
@@ -33,6 +36,8 @@ open Tacinterp
 open Tacred
 open Vernacinterp
 open Coqlib
+open Setoid_replace
+open Declarations
 
 (* Rewriting tactics *)
 
@@ -50,10 +55,13 @@ let general_rewrite_bindings lft2rgt (c,l) gl =
   let sigma = project gl in 
   let _,t = splay_prod env sigma ctype in
   match match_with_equation t with
-    | None -> error "The term provided does not end with an equation" 
+    | None -> 
+	if l = []
+	then general_s_rewrite lft2rgt c gl
+	else error "The term provided does not end with an equation" 
     | Some (hdcncl,_) -> 
-        let hdcncls = string_head hdcncl in 
-	let suffix = Declare.elimination_suffix (sort_of_goal gl) in
+        let hdcncls = string_of_inductive hdcncl in 
+	let suffix = Indrec.elimination_suffix (elimination_sort_of_goal gl)in
         let elim =
 	  if lft2rgt then
             pf_global gl (id_of_string (hdcncls^suffix^"_r"))
@@ -101,8 +109,8 @@ let abstract_replace (eq,sym_eq) (eqt,sym_eqt) c2 c1 unsafe gl =
   if unsafe or (pf_conv_x gl t1 t2) then
     let (e,sym) = 
       match kind_of_term (hnf_type_of gl t1) with 
-        | IsSort (Prop(Pos)) -> (eq,sym_eq)
-        | IsSort (Type(_))   -> (eqt,sym_eqt)
+        | Sort (Prop(Pos)) -> (eq,sym_eq)
+        | Sort (Type(_))   -> (eqt,sym_eqt)
         | _                  -> error "replace"
     in 
     (tclTHENL (elim_type (applist (e, [t1;c1;c2])))
@@ -111,20 +119,12 @@ let abstract_replace (eq,sym_eq) (eqt,sym_eqt) c2 c1 unsafe gl =
   else
     error "terms does not have convertible types"
 
-(* Only for internal use *)
-let unsafe_replace c2 c1 gl = 
-  let eq        = (pf_parse_const gl "eq")     in
-  let eqt       = (pf_parse_const gl "eqT")    in 
-  let sym_eq    = (pf_parse_const gl "sym_eq") in
-  let sym_eqt   = (pf_parse_const gl "sym_eqT") in
-  abstract_replace (eq,sym_eq) (eqt,sym_eqt) c2 c1 true gl
-
-let replace c2 c1 gl = 
-  let eq        = (pf_parse_const gl "eq")     in
-  let eqt       = (pf_parse_const gl "eqT")    in 
-  let sym_eq    = (pf_parse_const gl "sym_eq") in
-  let sym_eqt   = (pf_parse_const gl "sym_eqT") in
-  abstract_replace (eq,sym_eq) (eqt,sym_eqt) c2 c1 false gl
+let replace c2 c1 gl =
+  let eq = build_coq_eq_data.eq () in
+  let eq_sym = build_coq_eq_data.sym () in
+  let eqT = build_coq_eqT_data.eq () in
+  let eqT_sym = build_coq_eqT_data.sym () in
+  abstract_replace (eq,eq_sym) (eqT,eqT_sym) c2 c1 false gl
     
 let dyn_replace args gl = 
   match args with 
@@ -180,7 +180,7 @@ let v_conditional_rewriteRL =
 let find_constructor env sigma c =
   let hd,stack = whd_betadeltaiota_stack env sigma c in
   match kind_of_term hd with
-    | IsMutConstruct _ -> (hd,stack)
+    | Construct _ -> (hd,stack)
     | _ -> error "find_constructor"
 
 (* Patterns *)
@@ -208,23 +208,24 @@ type elimination_types =
 
 let necessary_elimination sort_arity sort =
   let sort_arity = mkSort sort_arity in
-  if (isType sort) then
-    if is_Set sort_arity then
-      Set_Type
-    else 
-      if is_Type sort_arity then
-	Type_Type
-      else  
-	errorlabstrm "necessary_elimination" 
-	  [< 'sTR "no primitive equality on proofs" >]  
-  else
-    if is_Set sort_arity then
-      Set_SetorProp
-    else
-      if is_Type sort_arity then
-	Type_SetorProp
-      else  errorlabstrm "necessary_elimination" 
-        [< 'sTR "no primitive equality on proofs" >]
+  match sort with
+      Type _ ->
+        if is_Set sort_arity then
+          Set_Type
+        else 
+          if is_Type sort_arity then
+	    Type_Type
+          else  
+	    errorlabstrm "necessary_elimination" 
+	      (str "no primitive equality on proofs")  
+    | _ ->
+        if is_Set sort_arity then
+          Set_SetorProp
+        else
+          if is_Type sort_arity then
+	    Type_SetorProp
+          else  errorlabstrm "necessary_elimination" 
+            (str "no primitive equality on proofs")
 
 let find_eq_pattern aritysort sort = 
   match necessary_elimination aritysort sort with
@@ -269,7 +270,7 @@ let find_eq_pattern aritysort sort =
  *)
 
 exception DiscrFound of
-  (constructor_path * int) list * constructor_path * constructor_path
+  (constructor * int) list * constructor * constructor
 
 let find_positions env sigma t1 t2 =
   let rec findrec posn t1 t2 =
@@ -277,7 +278,7 @@ let find_positions env sigma t1 t2 =
     let hd2,args2 = whd_betadeltaiota_stack env sigma t2 in
     match (kind_of_term hd1, kind_of_term hd2) with
   	
-      | IsMutConstruct (sp1,_), IsMutConstruct (sp2,_) ->
+      | Construct sp1, Construct sp2 ->
         (* both sides are constructors, so either we descend, or we can
            discriminate here. *)
 	  if sp1 = sp2 then
@@ -296,10 +297,9 @@ let find_positions env sigma t1 t2 =
 	    []
           else
 	    let ty1_0 = get_type_of env sigma t1_0 in
-	    (match get_sort_of env sigma ty1_0 with
-	       | Prop Pos -> [(List.rev posn,t1_0,t2_0)] (* Set *)
-	       | Type _ -> [(List.rev posn,t1_0,t2_0)] (* Type *)
-	       | _ -> [])
+	    match get_sort_family_of env sigma ty1_0 with
+	      | InSet | InType -> [(List.rev posn,t1_0,t2_0)]
+	      | InProp -> []
 	  in 
 	  (try 
 	     Inr(findrec [] t1 t2)
@@ -383,21 +383,24 @@ let descend_then sigma env head dirn =
   let IndType (indf,_) as indt =
     try find_rectype env sigma (get_type_of env sigma head)
     with Not_found -> assert false in
-  let mispec,_ = dest_ind_family indf in
-  let cstr = get_constructors indf in
+  let ind,_ = dest_ind_family indf in
+  let (mib,mip) = lookup_mind_specif env ind in
+  let cstr = get_constructors env indf in
   let dirn_nlams = cstr.(dirn-1).cs_nargs in
   let dirn_env = push_rels cstr.(dirn-1).cs_args env in
   (dirn_nlams,
    dirn_env,
    (fun dirnval (dfltval,resty) ->
-      let arign,_ = get_arity indf in
-      let p = it_mkLambda_or_LetIn (lift (mis_nrealargs mispec) resty) arign in
+      let arign,_ = get_arity env indf in
+      let p = it_mkLambda_or_LetIn (lift mip.mind_nrealargs resty) arign in
       let build_branch i =
 	let result = if i = dirn then dirnval else dfltval in
-	it_mkLambda_or_LetIn_name env result cstr.(i-1).cs_args
-      in
-      mkMutCaseL (make_default_case_info mispec, p, head,
-		 List.map build_branch (interval 1 (mis_nconstr mispec)))))
+	it_mkLambda_or_LetIn_name env result cstr.(i-1).cs_args in
+      let brl =
+        List.map build_branch
+          (interval 1 (Array.length mip.mind_consnames)) in
+      let ci = make_default_case_info env ind in
+      mkCase (ci, p, head, Array.of_list brl)))
   
 (* Now we need to construct the discriminator, given a discriminable
    position.  This boils down to:
@@ -417,7 +420,7 @@ let descend_then sigma env head dirn =
    giving [True], and all the rest giving False. *)
 
 let construct_discriminator sigma env dirn c sort =
-  let (IndType(IndFamily (mispec,_) as indf,_) as indt) =
+  let (IndType((ind,_) as indf,_) as indt) =
     try find_rectype env sigma (type_of env sigma c)
     with Not_found ->
        (* one can find Rel(k) in case of dependent constructors 
@@ -426,40 +429,40 @@ let construct_discriminator sigma env dirn c sort =
           CP : changed assert false in a more informative error
        *)
       errorlabstrm "Equality.construct_discriminator"
-	[< 'sTR "Cannot discriminate on inductive constructors with 
-		 dependent types" >] in
-  let arsign,arsort = get_arity indf in
+	(str "Cannot discriminate on inductive constructors with 
+		 dependent types") in
+  let (mib,mip) = lookup_mind_specif env ind in
+  let arsign,arsort = get_arity env indf in
   let (true_0,false_0,sort_0) = 
     match necessary_elimination arsort (destSort sort) with
-      | Type_Type -> build_coq_UnitT (), build_coq_EmptyT (), (Type dummy_univ)
-      | _         -> build_coq_True (),  build_coq_False (),  (Prop Null)
+      | Type_Type ->
+	  build_coq_UnitT (), build_coq_EmptyT (), Evarutil.new_Type_sort ()
+      | _         -> build_coq_True (),  build_coq_False (), (Prop Null)
   in
   let p = it_mkLambda_or_LetIn (mkSort sort_0) arsign in
-  let cstrs = get_constructors indf in
+  let cstrs = get_constructors env indf in
   let build_branch i =
     let endpt = if i = dirn then true_0 else false_0 in
-    it_mkLambda_or_LetIn endpt cstrs.(i-1).cs_args 
-  in
-  let build_match =
-    mkMutCaseL (make_default_case_info mispec, p, c,
-	       List.map build_branch (interval 1 (mis_nconstr mispec)))
-  in
-  build_match
+    it_mkLambda_or_LetIn endpt cstrs.(i-1).cs_args in
+  let brl =
+    List.map build_branch(interval 1 (Array.length mip.mind_consnames)) in
+  let ci = make_default_case_info env ind in
+  mkCase (ci, p, c, Array.of_list brl)
     
 let rec build_discriminator sigma env dirn c sort = function
   | [] -> construct_discriminator sigma env dirn c sort
   | ((sp,cnum),argnum)::l ->
       let cty = type_of env sigma c in
-      let IndType (indf,_) =
+      let IndType ((ind,_)as indf,_) =
 	try find_rectype env sigma cty with Not_found -> assert false in
-      let _,arsort = get_arity indf in
-      let nparams = mis_nparams (fst (dest_ind_family indf)) in
+      let (mib,mip) = lookup_mind_specif env ind in
+      let _,arsort = get_arity env indf in
+      let nparams = mip.mind_nparams  in
       let (cnum_nlams,cnum_env,kont) = descend_then sigma env c cnum in
       let newc = mkRel(cnum_nlams-(argnum-nparams)) in
       let subval = build_discriminator sigma cnum_env dirn newc sort l  in
       (match necessary_elimination arsort (destSort sort) with
-         | Type_Type ->
-	     kont subval (build_coq_EmptyT (),mkSort (Type(dummy_univ)))
+         | Type_Type -> kont subval (build_coq_EmptyT (),Evarutil.new_Type ())
 	 | _ -> kont subval (build_coq_False (),mkSort (Prop Null)))
 
 let find_eq_data_decompose eqn =
@@ -470,19 +473,15 @@ let find_eq_data_decompose eqn =
   else if (is_matching (build_coq_idT_pattern ()) eqn) then
     (build_coq_idT_data, match_eq (build_coq_idT_pattern ()) eqn)
   else
-    errorlabstrm  "find_eq_data_decompose" [< >]
+    errorlabstrm  "find_eq_data_decompose" (mt ())
 
 let gen_absurdity id gl =
-(*
-  if   (pf_is_matching gl (pat_False ()) (clause_type (Some id) gl)) 
-    or (pf_is_matching gl (pat_EmptyT ()) (clause_type  (Some id) gl))
-*)
   if is_empty_type (clause_type (Some id) gl)
   then
     simplest_elim (mkVar id) gl
   else
     errorlabstrm "Equality.gen_absurdity" 
-      [< 'sTR "Not the negation of an equality" >]
+      (str "Not the negation of an equality")
 
 (* Precondition: eq is leibniz equality
  
@@ -498,7 +497,8 @@ let gen_absurdity id gl =
 let discrimination_pf e (t,t1,t2) discriminator lbeq gls =
   let env = pf_env gls in
   let (indt,_) = find_mrectype env (project gls) t in 
-  let aritysort = mis_sort (Global.lookup_mind_specif indt) in
+  let (mib,mip) = lookup_mind_specif env indt in
+  let aritysort = mip.mind_sort in
   let sort = pf_type_of gls (pf_concl gls) in
   match necessary_elimination aritysort (destSort sort) with
     | Type_Type  ->
@@ -524,22 +524,22 @@ let discrimination_pf e (t,t1,t2) discriminator lbeq gls =
 exception NotDiscriminable
 
 let discr id gls =
-  let eqn = (pf_whd_betadeltaiota gls (clause_type (Some id) gls)) in
+  let eqn = pf_whd_betadeltaiota gls (clause_type (Some id) gls) in
   let sort = pf_type_of gls (pf_concl gls) in 
   let (lbeq,(t,t1,t2)) =
     try find_eq_data_decompose eqn
-    with e when catchable_exception e -> raise NotDiscriminable
+    with e when catchable_exception e ->
+      errorlabstrm "discr"  (str(string_of_id id) ++ 
+			    str" Not a primitive equality here ")
   in
-  let tj = pf_execute gls t in
   let sigma = project gls in
   let env = pf_env gls in
   (match find_positions env sigma t1 t2 with
-     | Inr _ -> raise NotDiscriminable
+     | Inr _ ->
+	 errorlabstrm "discr" (str" Not a discriminable equality")
      | Inl (cpath, (_,dirn), _) ->
 	 let e = pf_get_new_id (id_of_string "ee") gls in
-	 let e_env =
-	   push_named_assum (e,assumption_of_judgment env sigma tj) env
-	 in
+	 let e_env = push_named_decl (e,None,t) env in
 	 let discriminator =
 	   build_discriminator sigma e_env dirn (mkVar e) sort cpath in
 	 let (indt,_) = find_mrectype env sigma t in 
@@ -547,47 +547,32 @@ let discr id gls =
 	   discrimination_pf e (t,t1,t2) discriminator lbeq gls 
 	 in
 	 tclCOMPLETE((tclTHENS (cut_intro absurd_term)
-			([onLastHyp (compose gen_absurdity out_some);
+			([onLastHyp gen_absurdity;
 			  refine (mkApp (pf, [| mkVar id |]))]))) gls)
 
 
 let not_found_message id =
-  [<'sTR "the variable"; 'sPC ; 'sTR (string_of_id id) ; 'sPC;
-    'sTR" was not found in the current environment" >]
+  (str "The variable" ++ spc () ++ str (string_of_id id) ++ spc () ++
+    str" was not found in the current environment")
 
-let insatisfied_prec_message cls =
-  match cls with
-    | None -> [< 'sTR"goal does not satify the expected preconditions">] 
-    |  Some id -> [< 'sTR(string_of_id id); 'sPC;
-		     'sTR"does not satify the expected preconditions" >]
+let onNegatedEquality tac gls =
+  if is_matching (build_coq_not_pattern ()) (pf_concl gls) then
+    (tclTHEN (tclTHEN hnf_in_concl intro) (onLastHyp tac)) gls
+  else if is_matching (build_coq_imp_False_pattern ()) (pf_concl gls)then
+    (tclTHEN intro (onLastHyp tac)) gls
+  else 
+    errorlabstrm "extract_negated_equality_then"
+      (str"The goal should negate an equality")
 
-let discrOnLastHyp gls =
-  try onLastHyp (compose discr out_some) gls
-  with NotDiscriminable ->
-    errorlabstrm "DiscrConcl" [< 'sTR" Not a discriminable equality" >]
-
-let discrClause cls gls =
-  match cls with
-    | None ->
-    	if is_matching (build_coq_not_pattern ()) (pf_concl gls) then
-          (tclTHEN (tclTHEN hnf_in_concl intro) discrOnLastHyp) gls
-    	else if is_matching (build_coq_imp_False_pattern ()) (pf_concl gls)then
-	  (tclTHEN intro discrOnLastHyp) gls
-	else 
-	  errorlabstrm "DiscrClause" (insatisfied_prec_message cls)
-    | Some id ->
-	try (discr id gls)
-      	with
-	  | Not_found -> errorlabstrm "DiscrClause" (not_found_message id)
-	  | NotDiscriminable ->
-	      errorlabstrm "DiscrHyp"
-		[< 'sTR(string_of_id id);'sTR" Not a discriminable equality" >]
+let discrClause = function
+  | None -> onNegatedEquality discr
+  | Some id -> discr id
 
 let discrEverywhere = 
   tclORELSE
     (Tacticals.tryAllClauses discrClause)
     (fun gls -> 
-       errorlabstrm "DiscrEverywhere" [< 'sTR" No discriminable equalities" >])
+       errorlabstrm "DiscrEverywhere" (str" No discriminable equalities"))
 
 let discrConcl gls  = discrClause None gls
 let discrHyp id gls = discrClause (Some id) gls
@@ -595,7 +580,7 @@ let discrHyp id gls = discrClause (Some id) gls
 (**)
 let h_discr      = hide_atomic_tactic "Discr"      discrEverywhere
 let h_discrConcl = hide_atomic_tactic "DiscrConcl" discrConcl
-let h_discrHyp   = hide_ident_tactic  "DiscrHyp"   discrHyp
+let h_discrHyp   = hide_ident_or_numarg_tactic "DiscrHyp" discrHyp
 (**)
 
 (* returns the sigma type (sigS, sigT) with the respective
@@ -625,7 +610,7 @@ let make_tuple env sigma (prev_lind,rterm,rty) lind =
   let {intro = exist_term; typ = sig_term} =
     find_sigma_data (get_sort_of env sigma rty) in
   let a = type_of env sigma (mkRel lind) in
-  let na = fst (lookup_rel_type lind env) in
+  let (na,_,_) = lookup_rel lind env in
   (* If [lind] is not [prev_lind+1] then we lift down rty *)
   let rty = lift (- lind + prev_lind + 1) rty in
   (* Now [lind] is [mkRel 1] and we abstract on (na:a) *)
@@ -753,7 +738,8 @@ let rec build_injrec sigma env (t1,t2) c = function
   | ((sp,cnum),argnum)::l ->
       let cty = type_of env sigma c in
       let (ity,_) = find_mrectype env sigma cty in
-      let nparams = Global.mind_nparams ity in
+      let (mib,mip) = lookup_mind_specif env ity in
+      let nparams = mip.mind_nparams in
       let (cnum_nlams,cnum_env,kont) = descend_then sigma env c cnum in
       let newc = mkRel(cnum_nlams-(argnum-nparams)) in
       let (subval,tuplety,dfltval) =
@@ -770,9 +756,9 @@ let try_delta_expand env sigma t =
   let whdt = whd_betadeltaiota env sigma t  in 
   let rec hd_rec c  =
     match kind_of_term c with
-      | IsMutConstruct _ -> whdt
-      | IsApp (f,_)  -> hd_rec f
-      | IsCast (c,_) -> hd_rec c
+      | Construct _ -> whdt
+      | App (f,_)  -> hd_rec f
+      | Cast (c,_) -> hd_rec c
       | _  -> t
   in 
   hd_rec whdt 
@@ -782,26 +768,27 @@ let try_delta_expand env sigma t =
    in hd position, otherwise delta expansion is not done *)
 
 let inj id gls =
-  let eqn = (pf_whd_betadeltaiota gls (clause_type (Some id) gls)) in
+  let eqn = pf_whd_betadeltaiota gls (clause_type (Some id) gls) in
   let (eq,(t,t1,t2))= 
     try 
       find_eq_data_decompose eqn
     with e when catchable_exception e -> 
-      errorlabstrm "Inj"  [<'sTR(string_of_id id); 
-			    'sTR" Not a primitive  equality here " >] 
+      errorlabstrm "Inj"  (str(string_of_id id) ++ 
+			    str" Not a primitive equality here ") 
   in
-  let tj = pf_execute gls t in
   let sigma = project gls in
   let env = pf_env gls in
   match find_positions env sigma t1 t2 with
     | Inl _ ->
-	errorlabstrm "Inj" [<'sTR (string_of_id id);
-			     'sTR" is not a projectable equality" >]
+	errorlabstrm "Inj"
+	  (str (string_of_id id) ++
+	    str" is not a projectable equality but a discriminable one")
+    | Inr [] ->
+	errorlabstrm "Equality.inj" 
+	   (str"Nothing to do, it is an equality between convertible terms")
     | Inr posns ->
 	let e = pf_get_new_id (id_of_string "e") gls in
-	let e_env =
-	  push_named_assum (e,assumption_of_judgment env sigma tj) env
-	in
+	let e_env = push_named_decl (e,None,t) env in
 	let injectors =
 	  map_succeed
 	    (fun (cpath,t1_0,t2_0) ->
@@ -815,7 +802,7 @@ let inj id gls =
 	in
 	if injectors = [] then
 	  errorlabstrm "Equality.inj" 
-	    [<'sTR "Failed to decompose the equality">];
+	    (str "Failed to decompose the equality");
 	tclMAP 
 	  (fun (injfun,resty) ->
 	     let pf = applist(eq.congr (),
@@ -824,59 +811,52 @@ let inj id gls =
 			       try_delta_expand env sigma t2;
 			       mkVar id]) 
 	     in
-	     let ty = pf_type_of gls pf in
-	     ((tclTHENS  (cut  ty) ([tclIDTAC;refine pf]))))
+	     let ty =
+	       try pf_type_of gls pf
+               with
+		 | UserError("refiner__fail",_) -> 
+		     errorlabstrm "InjClause" 
+		(str (string_of_id id) ++ str" Not a projectable equality")
+	     in ((tclTHENS  (cut  ty) ([tclIDTAC;refine pf]))))
 	  injectors
 	  gls
 	    
-let injClause cls gls =
-  match cls with
-    | None ->
-    	if is_matching (build_coq_not_pattern ()) (pf_concl gls) then
-          (tclTHEN (tclTHEN hnf_in_concl intro)
-             (onLastHyp (compose inj out_some))) gls
-    	else
-	  errorlabstrm "InjClause" (insatisfied_prec_message  cls)
-    | Some id ->
-	try 
-	  inj id gls
-        with
-          | UserError("refiner__fail",_) -> 
-              errorlabstrm "InjClause" 
-		[< 'sTR (string_of_id id); 'sTR" Not a projectable equality" >]
+let injClause = function
+  | None -> onNegatedEquality inj
+  | Some id -> inj id
 
 let injConcl gls  = injClause None gls
 let injHyp id gls = injClause (Some id) gls
 
 (**)
 let h_injConcl = hide_atomic_tactic "Inj" injConcl
-let h_injHyp   = hide_ident_tactic "InjHyp" injHyp
+let h_injHyp   = hide_ident_or_numarg_tactic "InjHyp" injHyp
 (**)
 
 let decompEqThen ntac id gls =
-  let eqn = (pf_whd_betadeltaiota gls (clause_type (Some id) gls)) in
+  let eqn = pf_whd_betadeltaiota gls (clause_type (Some id) gls) in
   let (lbeq,(t,t1,t2))= find_eq_data_decompose  eqn in
   let sort = pf_type_of gls (pf_concl gls) in 
-  let tj = pf_execute gls t in
   let sigma = project gls in
   let env = pf_env gls in 
   (match find_positions env sigma t1 t2 with
      | Inl (cpath, (_,dirn), _) ->
 	 let e = pf_get_new_id (id_of_string "e") gls in
-	 let e_env =
-	   push_named_assum (e,assumption_of_judgment env sigma tj) env in
+	 let e_env = push_named_decl (e,None,t) env in
 	 let discriminator =
 	   build_discriminator sigma e_env dirn (mkVar e) sort cpath in
 	 let (pf, absurd_term) =
 	   discrimination_pf e (t,t1,t2) discriminator lbeq gls in
 	 tclCOMPLETE
 	   ((tclTHENS (cut_intro absurd_term)
-	       ([onLastHyp (compose gen_absurdity out_some);
+	       ([onLastHyp gen_absurdity;
 		 refine (mkApp (pf, [| mkVar id |]))]))) gls
+     | Inr [] ->
+	 errorlabstrm "Equality.inj" 
+	   (str"Nothing to do, it is an equality between convertible terms")
      | Inr posns ->
 	 (let e = pf_get_new_id (id_of_string "e") gls in
-	  let e_env =
-	    push_named_assum (e,assumption_of_judgment env sigma tj) env in
+	  let e_env = push_named_decl (e,None,t) env in
 	  let injectors =
 	    map_succeed
 	      (fun (cpath,t1_0,t2_0) ->
@@ -890,7 +870,7 @@ let decompEqThen ntac id gls =
 	  in
 	  if injectors = [] then
 	    errorlabstrm "Equality.decompEqThen" 
-              [<'sTR "Discriminate failed to decompose the equality">];
+              (str "Discriminate failed to decompose the equality");
 	  ((tclTHEN
 	      (tclMAP (fun (injfun,resty) ->
 			 let pf = applist(lbeq.congr (),
@@ -905,23 +885,9 @@ let decompEqThen ntac id gls =
 
 let decompEq = decompEqThen (fun x -> tclIDTAC)
 
-let dEqThen ntac cls gls =
-  match cls with
-    | None ->
-    	if is_matching (build_coq_not_pattern ()) (pf_concl gls) then
-	  (tclTHEN hnf_in_concl
-	     (tclTHEN intro
-         	(onLastHyp (compose (decompEqThen ntac) out_some)))) gls
-    	else
-	  errorlabstrm "DEqThen" (insatisfied_prec_message  cls)
-    | Some id ->
-	try 
-	  decompEqThen ntac id gls
-      	with 
-	  | Not_found -> 
-	      errorlabstrm "DEqThen" (not_found_message id)
-          | e when catchable_exception e ->
-	       errorlabstrm "DEqThen" (insatisfied_prec_message cls)
+let dEqThen ntac = function
+  | None -> onNegatedEquality (decompEqThen ntac)
+  | Some id -> decompEqThen ntac id
 
 let dEq = dEqThen (fun x -> tclIDTAC)
 
@@ -930,14 +896,14 @@ let dEqHyp id gls = dEq (Some id) gls
 
 (**)
 let dEqConcl_tac = hide_atomic_tactic "DEqConcl" dEqConcl
-let dEqHyp_tac = hide_ident_tactic "DEqHyp" dEqHyp
+let dEqHyp_tac = hide_ident_or_numarg_tactic "DEqHyp" dEqHyp
 (**)
 
 let rewrite_msg = function 
   | None ->  
-      [<'sTR "passed term is not a primitive equality">] 
+      (str "passed term is not a primitive equality") 
   | Some id ->
-      [<'sTR (string_of_id id); 'sTR "does not satisfy preconditions ">]
+      (str (string_of_id id) ++ str "does not satisfy preconditions ")
 
 let swap_equands gls eqn =
   let (lbeq,(t,e1,e2)) =
@@ -968,17 +934,17 @@ let swapEquandsInHyp id gls =
 
 let find_elim  sort_of_gl  lbeq =
   match kind_of_term sort_of_gl  with
-    | IsSort(Prop Null)  (* Prop *)  ->  (lbeq.ind (), false)  
-    | IsSort(Prop Pos)   (* Set *)   ->  
+    | Sort(Prop Null)  (* Prop *)  ->  (lbeq.ind (), false)  
+    | Sort(Prop Pos)   (* Set *)   ->  
 	(match lbeq.rrec with
            | Some eq_rec -> (eq_rec (), false) 
 	   | None -> errorlabstrm "find_elim"
-		 [< 'sTR "this type of elimination is not allowed">])
+		 (str "this type of elimination is not allowed"))
     | _ (* Type *) -> 
         (match lbeq.rect with
            | Some eq_rect -> (eq_rect (), true) 
            | None -> errorlabstrm "find_elim"
-		 [< 'sTR "this type of elimination is not allowed">])
+		 (str "this type of elimination is not allowed"))
 
 (* builds a predicate [e:t][H:(lbeq t e t1)](body e)
    to be used as an argument for equality dependent elimination principle:
@@ -1005,7 +971,7 @@ let bareRevSubstInConcl lbeq body (t,e1,e2) gls =
       find_elim (pf_type_of gls (pf_concl gls)) lbeq  
     with e when catchable_exception e -> 
       errorlabstrm "RevSubstIncConcl"
-        [< 'sTR "this type of substitution is not allowed">]  
+        (str "this type of substitution is not allowed")  
   in 
   let p =
     if dep then
@@ -1058,7 +1024,7 @@ let find_sigma_data_decompose ex =
        let subst = match_sigma ex (build_coq_existT_pattern ()) in
        (build_sigma_type (),subst)
      with PatternMatchingFailure -> 
-       errorlabstrm "find_sigma_data_decompose" [< >])
+       errorlabstrm "find_sigma_data_decompose" (mt ()))
 
 let decomp_tuple_term env c t = 
   let rec decomprec inner_code ex exty =
@@ -1106,7 +1072,7 @@ let substInConcl eqn gls =
 let substInHyp eqn id gls =
   let (lbeq,(t,e1,e2)) = (find_eq_data_decompose eqn) in 
   let body = subst_term e1 (clause_type (Some id) gls) in
-  if not (dependent (mkRel 1) body) then errorlabstrm  "SubstInHyp" [<>];
+  if not (dependent (mkRel 1) body) then errorlabstrm  "SubstInHyp" (mt ());
   (tclTHENS (cut_replacing id (subst1 e2 body))
      ([tclIDTAC;
        (tclTHENS (bareRevSubstInConcl lbeq body (t,e1,e2))
@@ -1122,15 +1088,15 @@ let try_rewrite tac gls =
     tac gls
   with 
     | UserError ("find_eq_data_decompose",_) -> errorlabstrm 
-	  "try_rewrite" [< 'sTR "Not a primitive equality here">]
+	  "try_rewrite" (str "Not a primitive equality here")
     | UserError ("swap_equamds",_) -> errorlabstrm 
-          "try_rewrite" [< 'sTR "Not a primitive equality here">]
+          "try_rewrite" (str "Not a primitive equality here")
     | UserError("find_eq_elim",s) -> errorlabstrm "try_rew" 
-          [<'sTR "This type of elimination is not allowed ">]  
+          (str "This type of elimination is not allowed ")  
     | e when catchable_exception e -> 
 	errorlabstrm "try_rewrite"
-          [< 'sTR "Cannot find a well type generalisation of the goal that";
-             'sTR " makes progress the proof.">]
+          (str "Cannot find a well type generalisation of the goal that" ++
+             str " makes progress the proof.")
 
 (* list_int n 0 [] gives the list [1;2;...;n] *)
 let rec list_int n cmr l =
@@ -1141,53 +1107,25 @@ let rec list_int n cmr l =
 
 (* Tells if two constrs are equal modulo unification *)
 
-(* Alpha-conversion *)
-let bind_eq = function
-  | (Anonymous,Anonymous) -> true
-  | (Name _,Name _) -> true
-  | _ -> false
-
-let eqop_mod_names = function
-  | OpLambda n0, OpLambda n1 -> bind_eq (n0,n1)
-  | OpProd n0, OpProd n1 -> bind_eq (n0,n1)
-  | OpLetIn n0, OpLetIn n1 -> bind_eq (n0,n1)
-  | op0, op1 -> op0 = op1
-
 exception NotEqModRel
 
-let rec eq_mod_rel l_meta t0 t1 =
-  match splay_constr_with_binders t1 with
-    | OpMeta n, [], [||] ->
-	if not (List.mem_assoc n l_meta) then
-	  [(n,t0)]@l_meta
-	else if (List.assoc n l_meta) = t0 then
-	  l_meta
-	else
-	  raise NotEqModRel
-    | op1, bd1, v1 ->
-	match splay_constr_with_binders t0 with
-	  | op0, bd0, v0
-	      when (eqop_mod_names (op0, op1)
-		    & (List.length bd0 = List.length bd1)
-		    & (Array.length v0 = Array.length v1)) ->
-	      array_fold_left2 eq_mod_rel
-		(List.fold_left2 eq_mod_rel_binders l_meta bd0 bd1)
-		v0 v1
-	    | _ -> raise NotEqModRel
-
-  and eq_mod_rel_binders l_meta t0 t1 = match (t0,t1) with
-    | (na0,Some b0,t0), (na1,Some b1,t1) when bind_eq (na0,na1) ->
-	eq_mod_rel (eq_mod_rel l_meta b0 b1) t0 t1
-    | (na0,None,t0), (na1,None,t1) when bind_eq (na0,na1) ->
-	eq_mod_rel l_meta t0 t1
-    | _ -> raise NotEqModRel
+let eq_mod_rel l_meta t0 t1 =
+  let bindings = ref l_meta in
+  let rec eq_rec t0 t1 = 
+    match kind_of_term t1 with
+      | Meta n ->
+	  if not (List.mem_assoc n !bindings) then
+	    (bindings := (n,t0) :: !bindings; true)
+	  else (List.assoc n l_meta) = t0
+      | _ -> compare_constr eq_rec t0 t1 in
+  if eq_rec t0 t1 then !bindings else raise NotEqModRel
 
 (* Verifies if the constr has an head constant *)
 
 let is_hd_const c = match kind_of_term c with
-  | IsApp (f,args) ->
+  | App (f,args) ->
       (match kind_of_term f with
-         | IsConst (c,_) -> Some (c, args)
+         | Const c -> Some (c, args)
          |_ -> None)
   | _ -> None
 
@@ -1197,10 +1135,10 @@ let is_hd_const c = match kind_of_term c with
 
 let nb_occ_term t u =
   let rec nbrec nocc u =
-    if t = u then   (* Pourquoi pas eq_constr ?? *)
+    if eq_constr t u then
       nocc + 1
     else
-      Array.fold_left nbrec nocc (snd (splay_constr u))
+      fold_constr nbrec nocc u
   in nbrec 0 u
     
 
@@ -1209,35 +1147,24 @@ let nb_occ_term t u =
    Rem: t_eq is assumed closed then there is no need to lift it
 *)
 let sub_term_with_unif cref ceq =
-  let rec find_match l_meta nb_occ hdsp t_args u = match splay_constr u with
-    | OpApp, cl -> begin
-	let f, args = destApplication u in
-	match kind_of_term f with
-          | IsConst (sp,_) when sp = hdsp -> begin
+  let rec find_match hdsp t_args (l_meta,nb_occ) u =
+    match kind_of_term u with
+    | App(f,args) ->
+	(match kind_of_term f with
+          | Const sp when sp = hdsp -> begin
 	      try (array_fold_left2 eq_mod_rel l_meta args t_args, nb_occ+1)
 	      with NotEqModRel ->
-		Array.fold_left
-                  (fun (l_meta,nb_occ) x -> find_match l_meta nb_occ
-                       hdsp t_args x) (l_meta,nb_occ) args
+		Array.fold_left (find_match hdsp t_args) (l_meta,nb_occ) args
 	    end
 
-          | IsConst _ | IsVar _ | IsMutInd _ | IsMutConstruct _
-	  | IsFix _ | IsCoFix _ ->
-               Array.fold_left 
-		 (fun (l_meta,nb_occ) x -> find_match l_meta
-		      nb_occ hdsp t_args x) (l_meta,nb_occ) cl
+          | (Const _ | Var _ | Ind _ | Construct _ | Fix _ | CoFix _) ->
+              fold_constr (find_match hdsp t_args) (l_meta,nb_occ) u
 
 	   (* Pourquoi ne rcurre-t-on pas dans f ? *)
-           | _ -> (l_meta,nb_occ)
-      end
+           | _ -> (l_meta,nb_occ))
 
-(* Le code original ne rcurrait pas sous les Cast 
-    | OpCast, _ -> (l_meta,nb_occ)
-*)
-    | _, t -> 
-	Array.fold_left 
-	  (fun (l_meta,nb_occ) x -> find_match l_meta nb_occ hdsp t_args x)
-	  (l_meta,nb_occ) t
+    | _ ->
+        fold_constr (find_match hdsp t_args) (l_meta,nb_occ) u
 
   in
   match (is_hd_const ceq) with
@@ -1251,7 +1178,7 @@ let sub_term_with_unif cref ceq =
           else
             Some (ceq,nb_occ)
     |Some (head,t_args) ->
-        let (l,nb) = find_match [] 0 head t_args cref in
+        let (l,nb) = find_match head t_args ([],0) cref in
         if nb = 0 then
           None
         else
@@ -1259,15 +1186,9 @@ let sub_term_with_unif cref ceq =
 	    
 (*The Rewrite in tactic*)
 let general_rewrite_in lft2rgt id (c,lb) gls =
-  let typ_id =
-    (try
-       let (_,ty) = lookup_named id (pf_env gls) in ty
-     with Not_found -> 
-       errorlabstrm "general_rewrite_in" 
-	 [< 'sTR"No such hypothesis : "; pr_id id >])
-  in
+  let typ_id = pf_get_hyp_typ gls id in
   let (wc,_) = startWalk gls
-  and (_,_,t) = reduce_to_ind (pf_env gls) (project gls) (pf_type_of gls c) in
+  and (_,t) = pf_reduce_to_quantified_ind gls (pf_type_of gls c) in
   let ctype = type_clenv_binding wc (c,t) lb in
   match (match_with_equation ctype) with
     | None -> error "The term provided does not end with an equation" 
@@ -1281,7 +1202,7 @@ let general_rewrite_in lft2rgt id (c,lb) gls =
         (match sub_term_with_unif typ_id mbr_eq with
            | None ->
                errorlabstrm "general_rewrite_in" 
-		 [<'sTR "Nothing to rewrite in: "; pr_id id>]
+		 (str "Nothing to rewrite in: " ++ pr_id id)
            | Some (l2,nb_occ) ->
                (tclTHENSI
 		  (tclTHEN
@@ -1327,7 +1248,7 @@ let rewrite_in lR com id gls =
   (try 
      let _ = lookup_named id (pf_env gls) in () 
    with Not_found -> 
-     errorlabstrm "rewrite_in" [< 'sTR"No such hypothesis : " ;pr_id id >]);
+     errorlabstrm "rewrite_in" (str"No such hypothesis : " ++pr_id id));
   let c = pf_interp_constr gls com in
   let eqn = pf_type_of gls c in
   try
@@ -1338,7 +1259,7 @@ let rewrite_in lR com id gls =
           ([tclIDTAC ; exact_no_check c])) gls
      with UserError("SubstInHyp",_) -> tclIDTAC gls)
   with UserError ("find_eq_data_decompose",_)->  
-    errorlabstrm "rewrite_in" [< 'sTR"No equality here" >] 
+    errorlabstrm "rewrite_in" (str"No equality here") 
       
 let subst eqn cls gls =
   match cls with
@@ -1358,6 +1279,7 @@ let substConcl_LR_tac =
       (function 
 	 | [Command eqn] -> 
 	     (fun gls ->  substConcl_LR (pf_interp_constr gls eqn)  gls)
+	 | [Constr c] -> substConcl_LR c
 	 | _ -> assert false)
   in 
   fun eqn  -> gentac [Command eqn] 
@@ -1537,7 +1459,7 @@ let sub_list lref i_s i_e =
     else if (i>=i_s) & (i<i_e) then
       sub_list_rec (l@[List.nth lref i]) (i+1)
     else
-      anomalylabstrm "Equality.sub_list" [<'sTR "Out of range">]
+      anomalylabstrm "Equality.sub_list" (str "Out of range")
   in
   sub_list_rec [] i_s
 
@@ -1592,8 +1514,8 @@ type hint_base =
 let explicit_hint_base gl = function 
   | By_name id -> 
       begin match rules_of_base id with
-	| [] -> errorlabstrm "autorewrite" [<'sTR ("Base "^(string_of_id id)^
-						   " does not exist")>]
+	| [] -> errorlabstrm "autorewrite" (str ("Base "^(string_of_id id)^
+						   " does not exist"))
 	| lbs -> lbs
       end 
   | Explicit lbs -> 
@@ -1647,7 +1569,7 @@ let autorewrite lbases ltacstp opt_step ltacrest opt_rest depth_step gls =
     let cmd = ref cmod
     and wrn = ref warn in
     if !cmd=depth_step then begin
-      wARNING [<'sTR ((string_of_int cglob)^" rewriting(s) carried out") >];
+      msg_warning (str ((string_of_int cglob)^" rewriting(s) carried out"));
       cmd := 0;
       wrn := true
     end;
@@ -1764,7 +1686,7 @@ let autorewrite lbases ltacstp opt_step ltacrest opt_rest depth_step gls =
     in
     let (gl,lvalid)=
       let (gl_res,lvalid_res,warn)=iterative_rew 0 0 (0,0,false) [g] [] in
-      if warn then mSGNL [<>];
+      if warn then msgnl (mt ());
       (gl_res,lvalid_res)
     in
     let validation_fun=
@@ -1799,7 +1721,7 @@ let autorewrite lbases ltacstp opt_step ltacrest opt_rest depth_step gls =
   and int_arg=function
     | [(Integer n)] -> n
     | _ -> anomalylabstrm "dyn_autorewrite" 
-	  [<'sTR "Bad call of int_arg (not an INTEGER)">]
+	  (str "Bad call of int_arg (not an INTEGER)")
   and list_args_rest (lstep,evstep) (ostep,evostep) (lrest,evrest)
     (orest,evorest) (depth,evdepth) = function
       | [] -> (lstep,ostep,lrest,orest,depth)
@@ -1833,13 +1755,13 @@ let autorewrite lbases ltacstp opt_step ltacrest opt_rest depth_step gls =
 		 (orest,evorest) (dth,true) tail
              else
                errorlabstrm "dyn_autorewrite" 
-		 [<'sTR "Depth value lower or equal to 0">])
+		 (str "Depth value lower or equal to 0"))
 	  else
             anomalylabstrm "dyn_autorewrite" 
-	      [<'sTR "Bad call of list_args_rest">]
+	      (str "Bad call of list_args_rest")
       | _ -> 
 	  anomalylabstrm "dyn_autorewrite" 
-	    [<'sTR "Bad call of list_args_rest">]
+	    (str "Bad call of list_args_rest")
   and list_args = function
     | (Redexp (s,lbases))::tail ->
 	if s = "BaseList" then
@@ -1852,10 +1774,10 @@ let autorewrite lbases ltacstp opt_step ltacrest opt_rest depth_step gls =
 	     ostep (if lrest=[] then None else Some lrest) orest depth)
 	else
           anomalylabstrm "dyn_autorewrite" 
-	    [<'sTR "Bad call of list_args (not a BaseList tagged REDEXP)">]
+	    (str "Bad call of list_args (not a BaseList tagged REDEXP)")
     | _ ->
 	anomalylabstrm "dyn_autorewrite" 
-	  [<'sTR "Bad call of list_args (not a REDEXP)">]
+	  (str "Bad call of list_args (not a REDEXP)")
   in
   list_args largs*)
 
