@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: equality.mli,v 1.9 2001/03/15 13:38:43 filliatr Exp $ i*)
+(*i $Id: equality.mli,v 1.11 2001/11/06 13:05:45 clrenard Exp $ i*)
 
 (*i*)
 open Names
@@ -57,9 +57,6 @@ val conditional_rewrite_in :
 val abstract_replace : 
   constr * constr -> constr * constr -> constr -> constr -> bool -> tactic
 
-(* Only for internal use *)
-val unsafe_replace : constr -> constr -> tactic
-
 val replace   : constr -> constr -> tactic
 val h_replace : constr -> constr -> tactic
 
@@ -90,7 +87,7 @@ val dEq : clause -> tactic
 val dEqThen : (int -> tactic) -> clause -> tactic
 
 val make_iterated_tuple : 
-  env -> 'a evar_map -> (constr * constr) -> (constr * constr) 
+  env -> evar_map -> (constr * constr) -> (constr * constr) 
     -> constr * constr * constr
 
 val subst : constr -> clause -> tactic
@@ -98,7 +95,7 @@ val hypSubst : identifier -> clause -> tactic
 val revSubst : constr -> clause -> tactic
 val revHypSubst : identifier -> clause -> tactic
 
-val discriminable : env -> 'a evar_map -> constr -> constr -> bool
+val discriminable : env -> evar_map -> constr -> constr -> bool
 
 (***************)
 (* AutoRewrite *)
