@@ -6,18 +6,20 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: inv.ml,v 1.26 2001/03/28 15:11:24 barras Exp $ *)
+(* $Id: inv.ml,v 1.33 2001/12/13 09:03:11 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Term
+open Termops
 open Global
 open Sign
 open Environ
-open Inductive
+open Inductiveops
 open Printer
-open Reduction
+open Reductionops
 open Retyping
 open Tacmach
 open Proof_type
@@ -69,7 +71,7 @@ let dest_match_eq gls eqn =
 	  pf_matches gls (Coqlib.build_coq_idT_pattern ()) eqn
         with PatternMatchingFailure ->
 	  errorlabstrm "dest_match_eq" 
-	    [< 'sTR "no primitive equality here" >]))
+	    (str "no primitive equality here")))
     
 (* Environment management *)
 let push_rels vars env =
@@ -88,12 +90,12 @@ let make_inv_predicate env sigma ind id status concl =
     match status with
       | NoDep ->
 	  (* We push the arity and leave concl unchanged *)
-	  let hyps_arity,_ = get_arity indf in
+	  let hyps_arity,_ = get_arity env indf in
 	  (hyps_arity,concl)
       | Dep dflt_concl ->
 	  if not (dependent (mkVar id) concl) then
 	    errorlabstrm "make_inv_predicate"
-              [< 'sTR "Current goal does not depend on "; pr_id id >];
+              (str "Current goal does not depend on " ++ pr_id id);
           (* We abstract the conclusion of goal with respect to
              realargs and c to * be concl in order to rewrite and have
              c also rewritten when the case * will be done *)
@@ -188,21 +190,24 @@ let rec dependent_hyps id idlist sign =
   let rec dep_rec =function
     | [] -> []
     | (id1::l) -> 
-	let id1ty = snd (lookup_named id1 sign) in  
-	if occur_var id (body_of_type id1ty) then id1::dep_rec l else dep_rec l
+	let (_,_,id1ty) = lookup_named id1 sign in  
+	if occur_var (Global.env()) id (body_of_type id1ty)
+        then id1::dep_rec l
+        else dep_rec l
   in 
   dep_rec idlist 
 
 let generalizeRewriteIntros tac depids id gls = 
   let dids = dependent_hyps id depids (pf_env gls) in
-  (tclTHEN (bring_hyps (List.map in_some dids))
+  (tclTHEN (bring_hyps dids)
      (tclTHEN tac
 	(introsReplacing dids)))
   gls
 
 let var_occurs_in_pf gl id =
-  occur_var id (pf_concl gl) or
-  List.exists (fun (_,t) -> occur_var id t) (pf_hyps_types gl)
+  let env = pf_env gl in
+  occur_var env id (pf_concl gl) or
+  List.exists (fun (_,t) -> occur_var env id t) (pf_hyps_types gl)
 
 let split_dep_and_nodep idl gl =
   (List.filter (var_occurs_in_pf gl) idl,
@@ -222,39 +227,39 @@ let dest_eq gls t =
    If it can discriminate then the goal is proved, if not tries to use it as
    a rewrite rule. It erases the clause which is given as input *)
 
-let projectAndApply thin cls depids gls =
-  let subst_hyp_LR cls = tclTRY(hypSubst (out_some cls)  None) in
-  let subst_hyp_RL cls = tclTRY(revHypSubst (out_some cls) None) in
+let projectAndApply thin id depids gls =
+  let subst_hyp_LR id = tclTRY(hypSubst id None) in
+  let subst_hyp_RL id = tclTRY(revHypSubst id None) in
   let subst_hyp gls = 
-    let orient_rule cls = 
-      let (t,t1,t2) = dest_eq gls (clause_type cls gls) in
+    let orient_rule id = 
+      let (t,t1,t2) = dest_eq gls (pf_get_hyp_typ gls id) in
       match (kind_of_term (strip_outer_cast t1),
 	     kind_of_term (strip_outer_cast t2)) with
-        | IsVar id1, _ -> generalizeRewriteIntros (subst_hyp_LR cls) depids id1
-        | _, IsVar id2 -> generalizeRewriteIntros (subst_hyp_RL cls) depids id2
-        | _ -> subst_hyp_RL cls
+        | Var id1, _ -> generalizeRewriteIntros (subst_hyp_LR id) depids id1
+        | _, Var id2 -> generalizeRewriteIntros (subst_hyp_RL id) depids id2
+        | _ -> subst_hyp_RL id
     in 
     onLastHyp orient_rule gls 
   in
-  let (t,t1,t2) = dest_eq gls (clause_type cls gls)  in
+  let (t,t1,t2) = dest_eq gls (pf_get_hyp_typ gls id)  in
   match (thin, kind_of_term (strip_outer_cast t1), kind_of_term (strip_outer_cast t2)) with
-    | (true, IsVar id1,  _) -> generalizeRewriteIntros
-          (tclTHEN (subst_hyp_LR cls) (clear_clause cls)) depids id1 gls
-    | (false, IsVar id1,  _) ->
-        generalizeRewriteIntros (subst_hyp_LR cls) depids id1 gls
-    | (true, _ , IsVar id2) -> generalizeRewriteIntros
-          (tclTHEN (subst_hyp_RL cls) (clear_clause cls)) depids id2 gls
-    | (false, _ , IsVar id2) ->
-        generalizeRewriteIntros (subst_hyp_RL cls) depids id2 gls
+    | (true, Var id1,  _) -> generalizeRewriteIntros
+          (tclTHEN (subst_hyp_LR id) (clear_clause id)) depids id1 gls
+    | (false, Var id1,  _) ->
+        generalizeRewriteIntros (subst_hyp_LR id) depids id1 gls
+    | (true, _ , Var id2) -> generalizeRewriteIntros
+          (tclTHEN (subst_hyp_RL id) (clear_clause id)) depids id2 gls
+    | (false, _ , Var id2) ->
+        generalizeRewriteIntros (subst_hyp_RL id) depids id2 gls
     | (true, _, _) ->
         let deq_trailer neqns =
           tclDO neqns
             (tclTHEN intro (tclTHEN subst_hyp (onLastHyp clear_clause)))
         in 
-	(tclTHEN (tclTRY (dEqThen deq_trailer cls)) (clear_clause cls)) gls
+	(tclTHEN (tclTRY (dEqThen deq_trailer (Some id))) (clear_one id)) gls
     | (false, _, _) -> 
         let deq_trailer neqns = tclDO neqns (tclTHEN intro subst_hyp) in 
-	(tclTHEN (dEqThen deq_trailer cls) (clear_clause cls)) gls
+	(tclTHEN (dEqThen deq_trailer (Some id)) (clear_one id)) gls
 
 (* Inversion qui n'introduit pas les hypotheses, afin de pouvoir les nommer
    soi-meme (proposition de Valerie). *)
@@ -269,24 +274,23 @@ let case_trailer_gene othin neqns ba gl =
                   (tclDO neqns
                      (tclTHEN intro
                         (onLastHyp
-                           (fun cls ->
-                              tclTRY (projectAndApply thin cls depids)))))
+                           (fun id ->
+                              tclTRY (projectAndApply thin id depids)))))
 		  (tclTHEN
-                     (onCL (compose List.rev (afterHyp (out_some last))) 
-			bring_hyps)
-                     (onCL (afterHyp (out_some last)) clear_clauses))))
+                     (onHyps (compose List.rev (afterHyp last)) bring_hyps)
+                     (onHyps (afterHyp last) clear))))
       | None -> tclIDTAC
   in
   (tclTHEN (tclDO neqns intro)
-     (tclTHEN (bring_hyps (List.map in_some nodepids))
-	(tclTHEN (clear_clauses (List.map in_some nodepids))
-	   (tclTHEN (onCL (compose List.rev (nLastHyps neqns)) bring_hyps)
-	      (tclTHEN (onCL (nLastHyps neqns) clear_clauses)
+     (tclTHEN (bring_hyps nodepids)
+	(tclTHEN (clear_clauses nodepids)
+	   (tclTHEN (onHyps (compose List.rev (nLastHyps neqns)) bring_hyps)
+	      (tclTHEN (onHyps (nLastHyps neqns) clear_clauses)
 		 (tclTHEN rewrite_eqns
 		    (tclMAP (fun id ->
-			       (tclORELSE (clear_clause (Some id))
-				  (tclTHEN (bring_hyps [Some id])
-				     (clear_clause (Some id)))))
+			       (tclORELSE (clear_clause id)
+				  (tclTHEN (bring_hyps [id])
+				     (clear_one id))))
 		       depids)))))))
   gl
 
@@ -300,28 +304,28 @@ let case_trailer othin neqns ba gl =
   let rewrite_eqns =
     match othin with
       | Some thin ->
-          (tclTHEN (onCL (compose List.rev (nLastHyps neqns)) bring_hyps)
-             (tclTHEN (onCL (nLastHyps neqns) clear_clauses)
+          (tclTHEN (onHyps (compose List.rev (nLastHyps neqns)) bring_hyps)
+             (tclTHEN (onHyps (nLastHyps neqns) clear_clauses)
 		(tclTHEN
 		   (tclDO neqns
                       (tclTHEN intro
 			 (onLastHyp
-			    (fun cls -> 
-			       tclTRY (projectAndApply thin cls depids)))))
+			    (fun id -> 
+			       tclTRY (projectAndApply thin id depids)))))
 		   (tclTHEN (tclDO (List.length nodepids) intro)
 		      (tclMAP (fun id -> 
-				 tclTRY (clear_clause (Some id))) depids)))))
+				 tclTRY (clear_clause id)) depids)))))
       | None -> tclIDTAC
   in
   (tclTHEN (tclDO neqns intro)
-     (tclTHEN (bring_hyps (List.map in_some nodepids))
-	(tclTHEN (clear_clauses (List.map in_some nodepids))
+     (tclTHEN (bring_hyps nodepids)
+	(tclTHEN (clear_clauses nodepids)
 	   rewrite_eqns)))
   gl
 
 let collect_meta_variables c = 
   let rec collrec acc c = match kind_of_term c with
-    | IsMeta mv -> mv::acc
+    | Meta mv -> mv::acc
     | _ -> fold_constr collrec acc c
   in 
   collrec [] c
@@ -331,10 +335,10 @@ let check_no_metas clenv ccl =
     let metas = List.map (fun n -> Intmap.find n clenv.namenv)
 		  (collect_meta_variables ccl) in
     errorlabstrm "res_case_then" 
-      [< 'sTR ("Cannot find an instantiation for variable"^
-	       (if List.length metas = 1 then " " else "s "));
+      (str ("Cannot find an instantiation for variable"^
+	       (if List.length metas = 1 then " " else "s ")) ++
 	 prlist_with_sep pr_coma pr_id metas
-	 (* ajouter "in "; prterm ccl mais il faut le bon contexte *) >]
+	 (* ajouter "in " ++ prterm ccl mais il faut le bon contexte *))
 
 let res_case_then gene thin indbinding id status gl =
   let env = pf_env gl and sigma = project gl in
@@ -350,7 +354,7 @@ let res_case_then gene thin indbinding id status gl =
     try find_rectype env sigma ccl
     with Induc ->
       errorlabstrm "res_case_then"
-	[< 'sTR ("The type of "^(string_of_id id)^" is not inductive") >] in
+	(str ("The type of "^(string_of_id id)^" is not inductive")) in
   let (elim_predicate,neqns) =
     make_inv_predicate env sigma indt id status (pf_concl gl) in
   let (cut_concl,case_tac) =
@@ -365,9 +369,9 @@ let res_case_then gene thin indbinding id status gl =
   (tclTHENS
      (cut_intro cut_concl)
      [onLastHyp
-        (fun cls->
+        (fun id ->
            (tclTHEN (applyUsing
-                       (applist(mkVar (out_some cls),
+                       (applist(mkVar id,
                                 list_tabulate
                                   (fun _ -> mkMeta(Clenv.new_meta())) neqns)))
               Auto.default_auto));
@@ -378,22 +382,22 @@ let res_case_then gene thin indbinding id status gl =
 (* Error messages of the inversion tactics *)
 let not_found_message ids =
   if List.length ids = 1 then
-    [<'sTR "the variable"; 'sPC ; 'sTR (string_of_id (List.hd ids)) ; 'sPC;
-      'sTR" was not found in the current environment" >]
+    (str "the variable" ++ spc () ++ str (string_of_id (List.hd ids)) ++ spc () ++
+      str" was not found in the current environment")
   else 
-    [<'sTR "the variables ["; 
-      'sPC ; prlist (fun id -> [<'sTR (string_of_id id) ; 'sPC >]) ids;
-      'sTR" ] were not found in the current environment" >]
+    (str "the variables [" ++ 
+      spc () ++ prlist (fun id -> (str (string_of_id id) ++ spc ())) ids ++
+      str" ] were not found in the current environment")
 
 let dep_prop_prop_message id =
   errorlabstrm "Inv"
-    [< 'sTR "Inversion on "; pr_id id ;
-       'sTR " would needs dependent elimination Prop-Prop" >]
+    (str "Inversion on " ++ pr_id id ++
+       str " would needs dependent elimination Prop-Prop")
  
 let not_inductive_here id =
   errorlabstrm "mind_specif_of_mind"
-    [< 'sTR "Cannot recognize an inductive predicate in "; pr_id id ;
-       'sTR ". If there is one, may be the structure of the arity or of the type of constructors is hidden by constant definitions." >]
+    (str "Cannot recognize an inductive predicate in " ++ pr_id id ++
+       str ". If there is one, may be the structure of the arity or of the type of constructors is hidden by constant definitions.")
 
 (* Noms d'errreurs obsoltes ?? *)
 let wrap_inv_error id = function
@@ -408,15 +412,15 @@ let inv gene com status id =
   let inv_tac = res_case_then gene com [] id status in
   let tac =
     if com = Some true (* if Inversion_clear, clear the hypothesis *) then 
-      tclTHEN inv_tac (tclTRY (clear_clause (Some id)))
+      tclTHEN inv_tac (tclTRY (clear_clause id))
     else 
       inv_tac
   in 
   fun gls -> try tac gls with e -> wrap_inv_error id e
 
-let hinv_kind = Identifier (id_of_string "HalfInversion")
-let inv_kind = Identifier (id_of_string "Inversion")
-let invclr_kind = Identifier (id_of_string "InversionClear")
+let hinv_kind = Quoted_string "HalfInversion"
+let inv_kind = Quoted_string "Inversion"
+let invclr_kind = Quoted_string "InversionClear"
 
 let com_of_id id =
   if id = hinv_kind then None
@@ -428,20 +432,22 @@ let (half_inv_tac, inv_tac, inv_clear_tac) =
   let gentac =
     hide_tactic "Inv"
       (function
-	 | [ic; Identifier id] -> inv false (com_of_id ic) NoDep id
-	 | _ -> anomaly "Inv called with bad args")
+	 | ic :: [id_or_num] ->
+	     tactic_try_intros_until (inv false (com_of_id ic) NoDep) id_or_num
+	 | l -> bad_tactic_args "Inv" l)
   in
   ((fun id -> gentac [hinv_kind; Identifier id]),
    (fun id -> gentac [inv_kind; Identifier id]),
    (fun id -> gentac [invclr_kind; Identifier id]))
 
+
 (* Inversion without intros. No vernac entry yet! *)
 let named_inv =
   let gentac =
     hide_tactic "NamedInv"
       (function
 	 | [ic; Identifier id] -> inv true (com_of_id ic) NoDep id
-	 | _ -> anomaly "NamedInv called with bad args")
+	 | l -> bad_tactic_args "NamedInv" l)
   in 
   (fun ic id -> gentac [ic; Identifier id])
 
@@ -450,8 +456,10 @@ let (half_dinv_tac, dinv_tac, dinv_clear_tac) =
   let gentac =
     hide_tactic "DInv"
       (function
-	 | [ic; Identifier id] -> inv false (com_of_id ic) (Dep None) id
-	 | _ -> anomaly "DInv called with bad args")
+	 | ic :: [id_or_num] ->
+	     tactic_try_intros_until (inv false (com_of_id ic) (Dep None))
+	       id_or_num
+	 | l -> bad_tactic_args "DInv" l)
   in
   ((fun id -> gentac [hinv_kind; Identifier id]),
    (fun id -> gentac [inv_kind; Identifier id]),
@@ -462,13 +470,17 @@ let (half_dinv_with, dinv_with, dinv_clear_with) =
   let gentac =
     hide_tactic "DInvWith"
       (function
-	 | [ic; Identifier id; Command com] ->
-             fun gls -> 
-	       inv false (com_of_id ic)
-		 (Dep (Some (pf_interp_constr gls com))) id gls
-	 | [ic; Identifier id; Constr c] ->
-             fun gls -> inv false (com_of_id ic) (Dep (Some c)) id gls
-	 | _ -> anomaly "DInvWith called with bad args")
+	 | [ic; id_or_num; Command com] ->
+	     tactic_try_intros_until
+               (fun id gls -> 
+		  inv false (com_of_id ic)
+		    (Dep (Some (pf_interp_constr gls com))) id gls)
+	       id_or_num
+	 | [ic; id_or_num; Constr c] ->
+	     tactic_try_intros_until
+               (fun id gls -> inv false (com_of_id ic) (Dep (Some c)) id gls)
+	       id_or_num
+	 | l -> bad_tactic_args "DInvWith" l)
   in
   ((fun id c -> gentac [hinv_kind; Identifier id; Constr c]),
    (fun id c -> gentac [inv_kind; Identifier id; Constr c]),
@@ -490,7 +502,7 @@ let invIn com id ids gls =
       (tclTHEN (tclDO nb_of_new_hyp intro) (introsReplacing ids)) gls
   in
   try 
-    (tclTHEN (bring_hyps (List.map in_some ids))
+    (tclTHEN (bring_hyps ids)
        (tclTHEN (inv false com NoDep id)
           (intros_replace_ids)))
     gls
@@ -500,17 +512,18 @@ let invIn_tac =
   let gentac =
     hide_tactic "InvIn"
       (function
-	 | (com::(Identifier id)::hl) ->
+	 | (com::(Identifier id)::hl as ll) ->
 	     let hl' =
-	       List.map (function
-			   | Identifier id -> id
-			   | _ -> anomaly "InvIn called with bas args") hl 
+	       List.map
+		 (function
+		    | Identifier id -> id
+		    | _ -> bad_tactic_args "InvIn" ll) hl 
 	     in
              invIn (com_of_id com) id hl'
-	 | _ -> anomaly "InvIn called with bad args")
+	 | ll -> bad_tactic_args "InvIn" ll)
   in 
   fun com id hl ->
     gentac
-      ((Identifier (id_of_string com))
+      ((Identifier com)
        ::(Identifier id)
        ::(List.map (fun id -> (Identifier id)) hl))
