@@ -6,18 +6,20 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: leminv.ml,v 1.23 2001/03/28 15:11:24 barras Exp $ *)
+(* $Id: leminv.ml,v 1.31 2001/12/13 09:03:11 filliatr Exp $ *)
 
 open Pp
 open Util
 open Names
+open Nameops
 open Term
+open Termops
 open Sign
 open Evd
 open Printer
-open Reduction
+open Reductionops
 open Declarations
-open Inductive
+open Inductiveops
 open Environ
 open Tacmach
 open Proof_trees
@@ -30,15 +32,16 @@ open Wcclausenv
 open Tacticals
 open Tactics
 open Inv
+open Safe_typing
 
 let not_work_message = "tactic fails to build the inversion lemma, may be because the predicate has arguments that depend on other arguments"
 
 let no_inductive_inconstr env constr =
-  [< 'sTR "Cannot recognize an inductive predicate in "; 
-     prterm_env env constr;
-     'sTR "."; 'sPC; 'sTR "If there is one, may be the structure of the arity";
-     'sPC; 'sTR "or of the type of constructors"; 'sPC;
-     'sTR "is hidden by constant definitions." >]
+  (str "Cannot recognize an inductive predicate in " ++ 
+     prterm_env env constr ++
+     str "." ++ spc () ++ str "If there is one, may be the structure of the arity" ++
+     spc () ++ str "or of the type of constructors" ++ spc () ++
+     str "is hidden by constant definitions.")
 
 (* Inversion stored in lemmas *)
 
@@ -83,14 +86,15 @@ let no_inductive_inconstr env constr =
 
  *)
  
-let thin_ids (hyps,vars) =
+let thin_ids env (hyps,vars) =
   fst
     (List.fold_left
        (fun ((ids,globs) as sofar) (id,c,a) ->
           if List.mem id globs then
 	    match c with
-	      | None -> (id::ids,(global_vars a)@globs)
-	      | Some body -> (id::ids,(global_vars body)@(global_vars a)@globs)
+	      | None -> (id::ids,(global_vars env a)@globs)
+	      | Some body ->
+                  (id::ids,(global_vars env body)@(global_vars env a)@globs)
           else sofar)
        ([],vars) hyps)
 
@@ -130,14 +134,14 @@ let max_prefix_sign lid sign =
 *)
 let rec add_prods_sign env sigma t =
   match kind_of_term (whd_betadeltaiota env sigma t) with
-    | IsProd (na,c1,b) ->
-	let id = Environ.id_of_name_using_hdchar env t na in
+    | Prod (na,c1,b) ->
+	let id = id_of_name_using_hdchar env t na in
 	let b'= subst1 (mkVar id) b in
-        add_prods_sign (Environ.push_named_assum (id,c1) env) sigma b'
-    | IsLetIn (na,c1,t1,b) ->
-	let id = Environ.id_of_name_using_hdchar env t na in
+        add_prods_sign (push_named_decl (id,None,c1) env) sigma b'
+    | LetIn (na,c1,t1,b) ->
+	let id = id_of_name_using_hdchar env t na in
 	let b'= subst1 (mkVar id) b in
-        add_prods_sign (Environ.push_named_def (id,c1,t1) env) sigma b'
+        add_prods_sign (push_named_decl (id,Some c1,t1) env) sigma b'
     | _ -> (env,t)
 
 (* [dep_option] indicates wether the inversion lemma is dependent or not.
@@ -167,19 +171,22 @@ let compute_first_inversion_scheme env sigma ind sort dep_option =
       pty,goal
     else
       let i = mkAppliedInd ind in
-      let ivars = global_vars i in
+      let ivars = global_vars env i in
       let revargs,ownsign =
 	fold_named_context
 	  (fun env (id,_,_ as d) (revargs,hyps) ->
-             if List.mem id ivars then ((mkVar id)::revargs,add_named_decl d hyps)
-	     else (revargs,hyps))
-          env ([],[]) in
+             if List.mem id ivars then 
+	       ((mkVar id)::revargs,add_named_decl d hyps)
+	     else 
+	       (revargs,hyps))
+          env ~init:([],[]) 
+      in
       let pty = it_mkNamedProd_or_LetIn (mkSort sort) ownsign in
       let goal = mkArrow i (applist(mkVar p, List.rev revargs)) in
       (pty,goal)
   in
   let npty = nf_betadeltaiota env sigma pty in
-  let extenv = push_named_assum (p,npty) env in
+  let extenv = push_named_decl (p,None,npty) env in
   extenv, goal
 
 (* [inversion_scheme sign I]
@@ -201,17 +208,15 @@ let inversion_scheme env sigma t sort dep_option inv_op =
   in
   assert 
     (list_subset 
-       (global_vars invGoal) 
+       (global_vars env invGoal) 
        (ids_of_named_context (named_context invEnv)));
   (*
     errorlabstrm "lemma_inversion"
-    [< 'sTR"Computed inversion goal was not closed in initial signature" >];
+    (str"Computed inversion goal was not closed in initial signature");
   *)
   let invSign = named_context invEnv in
-  let pfs = mk_pftreestate (mk_goal (mt_ctxt Intset.empty) invSign invGoal) in
-  let pfs =
-    solve_pftreestate (tclTHEN intro 
-			 (onLastHyp (compose inv_op out_some))) pfs in
+  let pfs = mk_pftreestate (mk_goal invSign invGoal) in
+  let pfs = solve_pftreestate (tclTHEN intro (onLastHyp inv_op)) pfs in
   let (pfterm,meta_types) = extract_open_pftreestate pfs in
   let global_named_context = Global.named_context () in
   let ownSign =
@@ -219,13 +224,13 @@ let inversion_scheme env sigma t sort dep_option inv_op =
       (fun env (id,_,_ as d) sign ->
          if mem_named_context id global_named_context then sign
 	 else add_named_decl d sign)
-      invEnv empty_named_context 
+      invEnv ~init:empty_named_context 
   in
   let (_,ownSign,mvb) =
     List.fold_left
       (fun (avoid,sign,mvb) (mv,mvty) ->
          let h = next_ident_away (id_of_string "H") avoid in
-	 (h::avoid, add_named_assum (h,mvty) sign, (mv,mkVar h)::mvb))
+	 (h::avoid, add_named_decl (h,None,mvty) sign, (mv,mkVar h)::mvb))
       (ids_of_context invEnv, ownSign, [])
       meta_types 
   in
@@ -238,8 +243,10 @@ let add_inversion_lemma name env sigma t sort dep inv_op =
   let invProof = inversion_scheme env sigma t sort dep inv_op in
   let _ = 
     declare_constant name
-    (ConstantEntry { const_entry_body = invProof; const_entry_type = None }, 
-     NeverDischarge,false)
+    (ConstantEntry { const_entry_body = invProof;
+                     const_entry_type = None;
+                     const_entry_opaque = false }, 
+     NeverDischarge)
   in ()
 
 (* open Pfedit *)
@@ -252,14 +259,14 @@ let inversion_lemma_from_goal n na id sort dep_option inv_op =
   let gl = nth_goal_of_pftreestate n pts in
   let t = pf_get_hyp_typ gl id in
   let env = pf_env gl and sigma = project gl in
-  let fv = global_vars t in
+  let fv = global_vars env t in
 (* Pourquoi ??? 
   let thin_ids = thin_ids (hyps,fv) in
   if not(list_subset thin_ids fv) then
     errorlabstrm "lemma_inversion"
-      [< 'sTR"Cannot compute lemma inversion when there are" ; 'sPC ;
-	 'sTR"free variables in the types of an inductive" ; 'sPC ;
-	 'sTR"which are not free in its instance" >]; *)
+      (str"Cannot compute lemma inversion when there are" ++ spc () ++
+	 str"free variables in the types of an inductive" ++ spc () ++
+	 str"which are not free in its instance"); *)
   add_inversion_lemma na env sigma t sort dep_option inv_op
    
 open Vernacinterp
@@ -270,7 +277,7 @@ let _ =
     (function
        | [VARG_NUMBER n; VARG_IDENTIFIER na; VARG_IDENTIFIER id] ->
 	   fun () ->
-	     inversion_lemma_from_goal n na id prop false inv_clear_tac
+	     inversion_lemma_from_goal n na id mk_Prop false inv_clear_tac
        | _ -> bad_vernac_args "MakeInversionLemmaFromHyp")
 
 let add_inversion_lemma_exn na com comsort bool tac =
@@ -298,7 +305,7 @@ let _ =
     (function
        | [VARG_NUMBER n; VARG_IDENTIFIER na; VARG_IDENTIFIER id] ->
 	   fun () ->
-	     inversion_lemma_from_goal n na id prop false half_inv_tac
+	     inversion_lemma_from_goal n na id mk_Prop false half_inv_tac
        | _ -> bad_vernac_args "MakeSemiInversionLemmaFromHyp")
 
 let _ = 
@@ -345,18 +352,22 @@ let lemInv id c gls =
  *)
     |  UserError (a,b) -> 
 	 errorlabstrm "LemInv" 
-	   [< 'sTR "Cannot refine current goal with the lemma "; 
-	      prterm_env (Global.env()) c  >] 
+	   (str "Cannot refine current goal with the lemma " ++ 
+	      prterm_env (Global.env()) c) 
 
 let useInversionLemma =
   let gentac =
     hide_tactic "UseInversionLemma"
       (function
-	 | [Identifier id; Command com] ->
-             fun gls -> lemInv id (pf_interp_constr gls com) gls
-	 | [Identifier id; Constr c] ->
-             fun gls -> lemInv id c gls
-	 | _  -> anomaly "useInversionLemma")
+	 | [id_or_num; Command com] ->
+	     tactic_try_intros_until
+               (fun id gls -> lemInv id (pf_interp_constr gls com) gls)
+	       id_or_num
+	 | [id_or_num; Constr c] ->
+	     tactic_try_intros_until
+               (fun id gls -> lemInv id c gls)
+	       id_or_num
+	 | l  -> bad_vernac_args "useInversionLemma" l)
   in 
   fun id c -> gentac [Identifier id;Constr c]
 
@@ -369,8 +380,7 @@ let lemInvIn id c ids gls =
       (tclTHEN (tclDO nb_of_new_hyp intro) (intros_replacing ids)) gls
   in 
 (*  try *)
-    ((tclTHEN (tclTHEN (bring_hyps (List.map in_some ids))
-		 (lemInv id c))
+    ((tclTHEN (tclTHEN (bring_hyps ids) (lemInv id c))
         (intros_replace_ids)) gls)
 (*  with Not_found -> errorlabstrm "LemInvIn" (not_found_message ids)
     |  UserError(a,b) -> errorlabstrm "LemInvIn" b  
@@ -380,21 +390,21 @@ let useInversionLemmaIn =
   let gentac =
     hide_tactic "UseInversionLemmaIn"
       (function
-	 | ((Identifier id)::(Command com)::hl) ->
+	 | ((Identifier id)::(Command com)::hl as ll) ->
 	     fun gls ->
 	       lemInvIn id (pf_interp_constr gls com)
                  (List.map
 		    (function
 		       | (Identifier id) -> id
-		       | _ -> anomaly "UseInversionLemmaIn") hl) gls
-	 | ((Identifier id)::(Constr c)::hl) ->
+		       | _ -> bad_vernac_args "UseInversionLemmaIn" ll) hl) gls
+	 | ((Identifier id)::(Constr c)::hl as ll) ->
 	     fun gls ->
 	       lemInvIn id c
                  (List.map
 		    (function
 		       | (Identifier id) -> id
-		       | _ -> anomaly "UseInversionLemmaIn") hl) gls
-	 | _ -> anomaly "UseInversionLemmaIn")
+		       | _ -> bad_vernac_args "UseInversionLemmaIn" ll) hl) gls
+	 | ll -> bad_vernac_args "UseInversionLemmaIn" ll)
   in 
   fun id c hl ->
     gentac ((Identifier id)::(Constr c)
