@@ -6,12 +6,13 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: tactics.mli,v 1.21 2001/03/28 15:11:25 barras Exp $ i*)
+(*i $Id: tactics.mli,v 1.31 2001/11/12 15:56:10 clrenard Exp $ i*)
 
 (*i*)
 open Names
 open Term
 open Environ
+open Sign
 open Tacmach
 open Proof_type
 open Reduction
@@ -20,21 +21,17 @@ open Evar_refiner
 open Clenv
 open Tacred
 open Tacticals
+open Nametab
 (*i*)
 
 (* Main tactics. *)
 
 (*s General functions. *)
 
-val type_clenv_binding : walking_constraints ->
+val type_clenv_binding : named_context sigma ->
   constr * constr -> constr substitution  -> constr
 
-(*i
-(* [force_reference c] fails if [c] is not a reference *)
-val force_reference   : constr -> constr
-i*)
-
-val string_head       : constr -> string
+val string_of_inductive : constr -> string
 val head_constr       : constr -> constr list
 val head_constr_bound : constr -> constr list -> constr list
 val bad_tactic_args : string -> tactic_arg list -> 'a
@@ -59,6 +56,7 @@ val dyn_mutual_cofix : tactic_arg list -> tactic
 (*s Introduction tactics. *)
 
 val next_global_ident_away : identifier -> identifier list -> identifier
+val fresh_id : identifier list -> identifier -> goal sigma -> identifier
 
 val intro                : tactic
 val introf               : tactic
@@ -69,7 +67,7 @@ val dyn_intro_move       : tactic_arg list -> tactic
 
 val intro_replacing      : identifier -> tactic
 val intro_using          : identifier -> tactic
-val intro_using_warning  : identifier -> tactic
+val intro_mustbe         : identifier -> tactic
 val intros_using         : identifier list -> tactic
 val intro_erasing        : identifier -> tactic
 val intros_replacing     : identifier list -> tactic
@@ -87,6 +85,21 @@ val dyn_intros_until     : tactic_arg list -> tactic
 
 val intros_clearing      : bool list -> tactic
 
+(* Assuming a tactic [tac] depending on an hypothesis identifier,
+   [tactic_try_intros_until tac arg] first assumes that arg denotes a
+   quantified hypothesis (denoted by name or by index) and try to
+   introduce it in context before to apply [tac], otherwise assume the
+   hypothesis is already in context and directly apply [tac] *)
+val tactic_try_intros_until : (identifier,tactic_arg) parse_combinator
+
+(* Assuming a tactic [tac] depending on an hypothesis identifier,
+   [hide_ident_or_numarg_tactic str tac] registers a tactic which
+   compose [tac] with "Intros Until" and returns a tactic which
+   behaves as [tac] (without implicit "Intros until") but hiding the
+   implementation under the name [str] *)
+
+val hide_ident_or_numarg_tactic : identifier hide_combinator
+
 (*s Exact tactics. *)
 
 val assumption         : tactic
@@ -100,41 +113,41 @@ val dyn_exact_check  : tactic_arg list -> tactic
 
 (*s Reduction tactics. *)
 
-type 'a tactic_reduction = env -> enamed_declarations -> constr -> constr
+type tactic_reduction = env -> evar_map -> constr -> constr
 
-val reduct_in_hyp     : 'a tactic_reduction -> identifier -> tactic
-val reduct_option     : 'a tactic_reduction -> identifier option -> tactic
-val reduct_in_concl   : 'a tactic_reduction -> tactic
+val reduct_in_hyp     : tactic_reduction -> hyp_location -> tactic
+val reduct_option     : tactic_reduction -> hyp_location option -> tactic
+val reduct_in_concl   : tactic_reduction -> tactic
 val change_in_concl   : constr -> tactic
 
-val change_in_hyp     : constr -> identifier -> tactic
-val change_option     : constr -> identifier option -> tactic
+val change_in_hyp     : constr -> hyp_location -> tactic
+val change_option     : constr -> hyp_location option -> tactic
 val red_in_concl      : tactic
-val red_in_hyp        : identifier        -> tactic
-val red_option        : identifier option -> tactic
+val red_in_hyp        : hyp_location        -> tactic
+val red_option        : hyp_location option -> tactic
 val hnf_in_concl      : tactic
-val hnf_in_hyp        : identifier        -> tactic
-val hnf_option        : identifier option -> tactic
+val hnf_in_hyp        : hyp_location        -> tactic
+val hnf_option        : hyp_location option -> tactic
 val simpl_in_concl    : tactic
-val simpl_in_hyp      : identifier        -> tactic
-val simpl_option      : identifier option -> tactic
+val simpl_in_hyp      : hyp_location        -> tactic
+val simpl_option      : hyp_location option -> tactic
 val normalise_in_concl: tactic
-val normalise_in_hyp  : identifier        -> tactic
-val normalise_option  : identifier option -> tactic
+val normalise_in_hyp  : hyp_location        -> tactic
+val normalise_option  : hyp_location option -> tactic
 val unfold_in_concl   : (int list * Closure.evaluable_global_reference) list
   -> tactic
 val unfold_in_hyp     : 
-  (int list * Closure.evaluable_global_reference) list -> identifier -> tactic
+  (int list * Closure.evaluable_global_reference) list -> hyp_location -> tactic
 val unfold_option     : 
-  (int list * Closure.evaluable_global_reference) list -> identifier option
+  (int list * Closure.evaluable_global_reference) list -> hyp_location option
     -> tactic
-val reduce            : red_expr -> identifier list -> tactic
+val reduce            : red_expr -> hyp_location list -> tactic
 val dyn_reduce        : tactic_arg list -> tactic
 val dyn_change        : tactic_arg list -> tactic
 
 val unfold_constr     : global_reference -> tactic
 val pattern_option    : 
-  (int list * constr * constr) list -> identifier option -> tactic
+  (int list * constr * constr) list -> hyp_location option -> tactic
 
 (*s Modification of the local context. *)
 
@@ -142,8 +155,11 @@ val clear         : identifier list -> tactic
 val clear_one     : identifier -> tactic
 val dyn_clear     : tactic_arg list -> tactic
 
-val clear_clauses : clause list -> tactic
-val clear_clause  : clause -> tactic
+val clear_body    : identifier list -> tactic
+val dyn_clear_body : tactic_arg list -> tactic
+
+val clear_clauses : identifier list -> tactic
+val clear_clause  : identifier -> tactic
 
 val new_hyp       : int option ->constr -> constr substitution -> tactic
 val dyn_new_hyp   : tactic_arg list -> tactic
@@ -155,7 +171,7 @@ val dyn_move_dep  : tactic_arg list -> tactic
 
 val apply_type : constr -> constr list -> tactic
 val apply_term : constr -> constr list -> tactic
-val bring_hyps : clause list -> tactic
+val bring_hyps : identifier list -> tactic
 
 val apply                 : constr      -> tactic
 val apply_without_reduce  : constr      -> tactic
@@ -188,6 +204,7 @@ val dyn_case              : tactic_arg list -> tactic
 val destruct              : identifier -> tactic
 val destruct_nodep        : int -> tactic
 val dyn_destruct          : tactic_arg list -> tactic
+val dyn_new_destruct      : tactic_arg list -> tactic
 
 (*s Eliminations giving the type instead of the proof. *)
 
@@ -252,7 +269,9 @@ val cut_intro                   : constr -> tactic
 val cut_replacing               : identifier -> constr -> tactic
 val cut_in_parallel             : constr list -> tactic
 val dyn_cut                     : tactic_arg list -> tactic
+val dyn_true_cut                : tactic_arg list -> tactic
 val dyn_lettac                  : tactic_arg list -> tactic
+val dyn_forward                 : tactic_arg list -> tactic
 
 val generalize                  : constr list -> tactic
 val dyn_generalize              : tactic_arg list -> tactic 
