@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: wcclausenv.mli,v 1.11 2001/03/28 15:11:25 barras Exp $ i*)
+(*i $Id: wcclausenv.mli,v 1.14 2001/11/19 15:44:30 clrenard Exp $ i*)
 
 (*i*)
 open Names
@@ -32,13 +32,10 @@ type arg_binder =
 
 type arg_bindings = (arg_binder * constr) list
 
-type wc = walking_constraints
+type wc = named_context sigma
 
 val clenv_constrain_with_bindings : arg_bindings -> wc clausenv -> wc clausenv
 
-val add_prod_rel : 'a evar_map -> constr * env -> constr * env
-val add_prods_rel : 'a evar_map -> constr * env -> constr * env
-
 (*i**
 val add_prod_sign : 
   'a evar_map -> constr * types signature -> constr * types signature
@@ -47,22 +44,7 @@ val add_prods_sign :
   'a evar_map -> constr * types signature -> constr * types signature
 **i*)
 
-val res_pf_THEN : 
-  (wc -> tactic) -> wc clausenv -> (wc clausenv -> tactic) -> tactic
-
-(* This behaves as [res_pf_THEN] but the tactic applied then takes
-   also the subgoal number (starting from 1) as argument *)
-val res_pf_THEN_i : 
-  (wc -> tactic) -> wc clausenv -> (wc clausenv -> int -> tactic) -> tactic
-
 val elim_res_pf_THEN_i : 
   (wc -> tactic) -> wc clausenv -> (wc clausenv -> int -> tactic) -> tactic
 
-val mk_clenv_using : wc -> constr -> wc clausenv
-
 val applyUsing : constr -> tactic
-
-val clenv_apply_n_times : int -> wc clausenv -> wc clausenv
-
-
-
