@@ -6,12 +6,14 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: coqtop.ml,v 1.25 2001/04/19 07:39:08 filliatr Exp $ *)
+(* $Id: coqtop.ml,v 1.34 2001/12/13 09:03:12 filliatr Exp $ *)
 
 open Pp
 open Util
 open System
 open Options
+open Names
+open Nameops
 open States
 open Toplevel
 open Coqinit
@@ -44,10 +46,10 @@ let outputstate = ref ""
 let set_outputstate s = outputstate:=s
 let outputstate () = if !outputstate <> "" then extern_state !outputstate
 
-let set_include d p = push_include (d,Names.dirpath_of_string p)
-let set_rec_include d p = push_rec_include (d,Names.dirpath_of_string p)
-let set_default_include d = set_include d Nametab.default_root
-let set_default_rec_include d = set_rec_include d Nametab.default_root
+let set_include d p = push_include (d,p)
+let set_rec_include d p = push_rec_include (d,p)
+let set_default_include d = set_include d Nameops.default_root_prefix
+let set_default_rec_include d = set_rec_include d Nameops.default_root_prefix
  
 let load_vernacular_list = ref ([] : string list)
 let add_load_vernacular s =
@@ -60,17 +62,25 @@ let load_vernacular () =
 let load_vernacular_obj = ref ([] : string list)
 let add_vernac_obj s = load_vernacular_obj := s :: !load_vernacular_obj
 let load_vernac_obj () = 
-  List.iter
-    (fun s -> Library.load_module (Filename.basename s) (Some s))
-    (List.rev !load_vernacular_obj)
+  List.iter Library.read_module_from_file (List.rev !load_vernacular_obj)
 
 let require_list = ref ([] : string list)
 let add_require s = require_list := s :: !require_list
 let require () =
   List.iter
-    (fun s -> Library.require_module None (Filename.basename s) (Some s) false)
+    (fun s -> 
+      let qid = Nametab.make_short_qualid (id_of_string (Filename.basename s)) in
+      Library.require_module None qid  (Some s) false)
     (List.rev !require_list)
 
+let compile_list = ref ([] : (bool * string) list)
+let add_compile verbose s =
+  set_batch_mode ();
+  Options.make_silent true; 
+  compile_list := (verbose,s) :: !compile_list
+let compile_files () =
+  List.iter (fun (v,f) -> Vernac.compile v f) (List.rev !compile_list)
+
 (* Re-exec Coq in bytecode or native code if necessary. [s] is either
    ["byte"] or ["opt"]. Notice that this is possible since the nature of
    the toplevel has already been set in [Mltop] by the main file created
@@ -82,11 +92,11 @@ let re_exec s =
     let prog = Sys.argv.(0) in
     let newprog = 
       let dir = Filename.dirname prog in
-      let com = "coqtop." ^ s in
+      let com = "coqtop." ^ s ^ Coq_config.exec_extension in
       if dir <> "." then Filename.concat dir com else com 
     in
     Sys.argv.(0) <- newprog;
-    Unix.execvp newprog Sys.argv
+    Unix.handle_unix_error Unix.execvp newprog Sys.argv
   end
 
 (*s Parsing of the command line.
@@ -101,7 +111,7 @@ let usage () =
   flush stderr ;
   exit 1
 
-let warning s = wARNING [< 'sTR s >]
+let warning s = msg_warning (str s)
 
 let parse_args () =
   let rec parse = function
@@ -110,7 +120,7 @@ let parse_args () =
     | ("-I"|"-include") :: d :: rem -> set_default_include d; parse rem
     | ("-I"|"-include") :: []       -> usage ()
 
-    | "-R" :: d :: p :: rem -> set_rec_include d p; parse rem
+    | "-R" :: d :: p :: rem ->set_rec_include d (dirpath_of_string p);parse rem
     | "-R" :: ([] | [_]) -> usage ()
 
     | "-q" :: rem -> no_load_rc (); parse rem
@@ -121,6 +131,7 @@ let parse_args () =
 
     | "-batch" :: rem -> set_batch_mode (); parse rem
     | "-boot" :: rem -> boot := true; no_load_rc (); parse rem	     
+    | "-quality" :: rem -> term_quality := true; no_load_rc (); parse rem
     | "-outputstate" :: s :: rem -> set_outputstate s; parse rem
     | "-outputstate" :: []       -> usage ()
 
@@ -144,6 +155,12 @@ let parse_args () =
     | "-require" :: f :: rem -> add_require f; parse rem
     | "-require" :: []       -> usage ()
 
+    | "-compile" :: f :: rem -> add_compile false f; parse rem
+    | "-compile" :: []       -> usage ()
+
+    | "-compile-verbose" :: f :: rem -> add_compile true f; parse rem
+    | "-compile-verbose" :: []       -> usage ()
+
     | "-unsafe" :: f :: rem -> add_unsafe f; parse rem
     | "-unsafe" :: []       -> usage ()
 
@@ -181,9 +198,9 @@ let parse_args () =
 	try
 	  Stream.empty s; exit 1
 	with Stream.Failure ->
-	  mSGNL (Errors.explain_exn e); exit 1
+	  msgnl (Errors.explain_exn e); exit 1
       end
-    | e -> begin mSGNL (Errors.explain_exn e); exit 1 end
+    | e -> begin msgnl (Errors.explain_exn e); exit 1 end
 
 
 (* To prevent from doing the initialization twice *)
@@ -205,14 +222,19 @@ let start () =
       require ();
       load_rcfile();
       load_vernacular ();
-      outputstate ()
+      compile_files ();
+      outputstate ();
     with e ->
       flush_all();
       if not !batch_mode then message "Error during initialization :";
-      mSGNL (Toplevel.print_toplevel_error e);
+      msgnl (Toplevel.print_toplevel_error e);
       exit 1
   end;
   if !batch_mode then (flush_all(); Profile.print_profile ();exit 0);
-  Toplevel.loop()
+  Toplevel.loop();
+(* Initialise and launch the Ocaml toplevel *)
+  Coqinit.init_ocaml_path();
+  Mltop.ocaml_toploop();
+  exit 1
 
 (* [Coqtop.start] will be called by the code produced by coqmktop *)
