@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: himsg.mli,v 1.12 2001/03/15 13:38:58 filliatr Exp $ i*)
+(*i $Id: himsg.mli,v 1.13 2001/11/05 16:48:29 barras Exp $ i*)
 
 (*i*)
 open Pp
@@ -21,7 +21,7 @@ open Logic
 
 (* This module provides functions to explain the type errors. *)
 
-val explain_type_error : path_kind -> env -> type_error -> std_ppcmds
+val explain_type_error : env -> type_error -> std_ppcmds
 
 val explain_pretype_error : env -> pretype_error -> std_ppcmds
 
