@@ -6,13 +6,15 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(*i $Id: usage.mli,v 1.4 2001/03/15 13:38:59 filliatr Exp $ i*)
+(*i $Id: usage.mli,v 1.5 2001/05/11 10:31:34 filliatr Exp $ i*)
 
 (*s Prints the version number on the standard output and exits (with 0). *)
 
 val version : unit -> 'a
 
-(*s Prints the usage on the error output. *)
+(*s Prints the usage on the error output, preceeded by a user-provided message. *)
+val print_usage : string -> unit
 
+(*s Prints the usage on the error output. *)
 val print_usage_coqtop : unit -> unit
 val print_usage_coqc : unit -> unit
