@@ -6,7 +6,7 @@
 (*         *       GNU Lesser General Public License Version 2.1       *)
 (***********************************************************************)
 
-(* $Id: vernac.ml,v 1.11 2001/04/09 15:01:30 courant Exp $ *)
+(* $Id: vernac.ml,v 1.19 2001/12/13 09:03:13 filliatr Exp $ *)
 
 (* Parsing of vernacular. *)
 
@@ -100,17 +100,10 @@ let rec vernac interpfun input =
   let rec interp com =
     match com with
       | Node (_,"LoadFile",[Str(_, verbosely); Str(_,fname)]) ->
-          let verbosely = verbosely = "Verbose" in
-          raw_load_vernac_file verbosely (make_suffix fname ".v")
+          let verbosely = (verbosely = "Verbose") in
+          let _ = read_vernac_file verbosely (make_suffix fname ".v") in
+          ()
             
-(*      | Node (_,"CompileFile",[Str(_,verbosely); Str(_,only_spec);
-			       Str (_,mname); Str(_,fname)]) ->
-          let verbosely = verbosely = "Verbose" in
-          let only_spec = only_spec = "Specification" in
-          States.with_heavy_rollback (* to roll back in case of error *)
-            (raw_compile_module verbosely only_spec mname)
-            (make_suffix fname ".v")
-*)	    
       | Node(_,"VernacList",l) ->
 	  List.iter interp l
 
@@ -118,8 +111,8 @@ let rec vernac interpfun input =
 	  let tstart = System.get_time() in
           interp v;
 	  let tend = System.get_time() in
-          mSGNL [< 'sTR"Finished transaction in " ;
-                   System.fmt_time_difference tstart tend >]
+          msgnl (str"Finished transaction in " ++
+                   System.fmt_time_difference tstart tend)
 	    
       | _ -> if not !just_parsing then interpfun com
   in 
@@ -127,22 +120,6 @@ let rec vernac interpfun input =
     interp com
   with e -> 
     raise (DuringCommandInterp (Ast.loc com, e))
-    
-and raw_load_vernac_file verbosely s =
-  let _ = read_vernac_file verbosely s in ()
-
-(*
-and raw_compile_module verbosely only_spec mname file =
-  assert false; (* I bet this code is never used in practice. Judicael. *)
-  Library.import_module mname; (* ??? *)
-  let lfname = read_vernac_file verbosely file in
-  let base = Filename.chop_suffix lfname ".v" in
-  Pfedit.check_no_pending_proofs ();
-  if only_spec then
-    failwith ".vi not yet implemented"
-  else 
-    Discharge.save_module_to mname base
-*)
 
 and read_vernac_file verbosely s =
   let interpfun =
@@ -163,14 +140,34 @@ and read_vernac_file verbosely s =
       | _ -> raise_with_file fname e
 
 (* raw_do_vernac : char Stream.t -> unit
- * parses and executes one command of the vernacular char stream *)
+ * parses and executes one command of the vernacular char stream.
+ * Marks the end of the command in the lib_stk to make vernac undoing
+ * easier. *)
 
 let raw_do_vernac po =
-  vernac (States.with_heavy_rollback Vernacinterp.interp) (po,None)
+  vernac (States.with_heavy_rollback Vernacinterp.interp) (po,None);
+  Lib.mark_end_of_command()
 
 (* Load a vernac file. Errors are annotated with file and location *)
 let load_vernac verb file =
   try 
-    raw_load_vernac_file verb file
+    let _ = read_vernac_file verb file in ()
   with e -> 
     raise_with_file file e
+
+(* Compile a vernac file (f is assumed without .v suffix) *)
+let compile verbosely f =
+  try
+    let s = Filename.basename f in
+    let m = Names.id_of_string s in
+    let _,longf = find_file_in_path (Library.get_load_path ()) (f^".v") in
+    let ldir0 = Library.find_logical_path (Filename.dirname longf) in
+    let ldir = Nameops.extend_dirpath ldir0 m in
+    Termops.set_module ldir; (* Just for universe naming *)
+    Lib.start_module ldir;
+    let _ = load_vernac verbosely longf in
+    let mid = Lib.end_module m in
+    assert (mid = ldir);
+    Library.save_module_to ldir (longf^"o")
+  with e ->
+    raise_with_file f e
